#!/bin/bash

# First get the list of all the command names
COMMAND_NAMES=`grep -h -i "@command:" ../../src/kvirc/kvi_commands.cpp | sed -e 's/[	 ]*@command:[ ]*//g'`
COMMAND_LINKS=""

for a_name in $COMMAND_NAMES; do
	LOWER_NAME=`echo "$a_name" | sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`
	echo "Generating ../../data/help/commands/$LOWER_NAME.kvihelp"
	DOCUMENT_DATA=`grep -h -i -A 150 "@command:[ ]*$a_name[ 	]*\$" ../../src/kvirc/kvi_commands.cpp`
	# Now strip the first line and the lines after the end of the comment
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e '/@command:[ ]*[a-zA-Z_]*/ d'`
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e '/\*\// q'`
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e '/\*\// d'`
	# Strip empty lines and kill leading tabs
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | grep -h "[a-zA-Z0-9{}]"`
 	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e 's/^[	 ]*//'`
 	# Extract the "@syntax:" part
 	SYNTAX_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 100 "@syntax:"`
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e '/@syntax:/ d'`
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e '/@[a-zA-Z]*:/ d'`
 	# Replace eventual < and > in the syntax
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e 's/</\&lt;/g'`
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e 's/>/\&gt;/g'`
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e 's/\&lt;br&gt;/<br>/g'`
 	SYNTAX_DATA=`echo "$SYNTAX_DATA" | sed -e 's/@tab@/    /g'`
 	# Extract the "@description:" part
 	DESCRIPTION_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 150 "@description:"`
 	DESCRIPTION_DATA=`echo "$DESCRIPTION_DATA" | sed -e '/@description:/ d'`
 	DESCRIPTION_DATA=`echo "$DESCRIPTION_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	DESCRIPTION_DATA=`echo "$DESCRIPTION_DATA" | sed -e '/@[a-zA-Z]*:/ d'` 	
 	# Extract the "@short:" part
 	SHORT_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 100 "@short:"`
 	SHORT_DATA=`echo "$SHORT_DATA" | sed -e '/@short:/ d'`
 	SHORT_DATA=`echo "$SHORT_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	SHORT_DATA=`echo "$SHORT_DATA" | sed -e '/@[a-zA-Z]*:/ d'` 	
    	# Extract the "@examples:" part
 	EXAMPLES_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 100 "@examples:"`
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e '/@examples:/ d'`
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e '/@[a-zA-Z]*:/ d'`
 	# Replace eventual @tab@ signs
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e 's/@tab@/    /g'`
	# Extract the "@seealso:" part
 	SEEALSO_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 100 "@seealso:"`
 	SEEALSO_DATA=`echo "$SEEALSO_DATA" | sed -e '/@seealso:/ d'`
 	SEEALSO_DATA=`echo "$SEEALSO_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	SEEALSO_DATA=`echo "$SEEALSO_DATA" | sed -e '/@[a-zA-Z]*:/ d'`
 	# Create the main document
 	DOCUMENT="<doctitle>$a_name command</doctitle>
<docbody>
<hr>
"
if test -n "$SYNTAX_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Syntax</docsubtitle>
$SYNTAX_DATA<br>
"
fi
if test -n "$DESCRIPTION_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Description</docsubtitle>
$DESCRIPTION_DATA<br>
"
fi
if test -n "$EXAMPLES_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Examples</docsubtitle>
$EXAMPLES_DATA<br>
"
fi
if test -n "$SEEALSO_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>See also</docsubtitle>
$SEEALSO_DATA<br>
"
fi
DOCUMENT="$DOCUMENT<hr>
<a href=\"index.kvihelp\">Main index</a> , <a href=\"commands.kvihelp\">Command index</a>
</docbody>
"
echo "$DOCUMENT" > ../../data/help/commands/$LOWER_NAME.kvihelp

# Update the command links

if test -n "$COMMAND_LINKS"; then
	COMMAND_LINKS="$COMMAND_LINKS
<a href=\"$LOWER_NAME.kvihelp\">$a_name</a>"
else
	COMMAND_LINKS="<a href=\"$LOWER_NAME.kvihelp\">$a_name</a>"
fi

if test -n "$SHORT_DATA"; then
	COMMAND_LINKS="$COMMAND_LINKS : $SHORT_DATA<br>"
else
	COMMAND_LINKS="$COMMAND_LINKS<br>"
fi

done

# Prepare the main command index

COMMAND_LINKS=`echo "$COMMAND_LINKS" | sort`

COMMAND_INDEX="<doctitle>Supported commands</doctitle>
<docbody>
<hr>
$COMMAND_LINKS
<hr>
<a href=\"index.kvihelp\">Main index</a>
</docbody>
"
echo "Generating ../../data/help/commands/commands.kvihelp"
echo "$COMMAND_INDEX" > ../../data/help/commands/commands.kvihelp

# Now regenerate the Makefile.am

LS_OUTPUT=`ls ../../data/help/commands/*.kvihelp | sed -e 's/^..\/..\/data\/help\/commands\///'`

FILES_OUTPUT=""
for a_file in $LS_OUTPUT; do
	if test -n "$FILES_OUTPUT"; then
		FILES_OUTPUT="$FILES_OUTPUT \\
			$a_file"
	else
		FILES_OUTPUT="$a_file"
	fi
done

MAKEFILE_TEXT="datadir= \$(helpdir)
data_DATA= $FILES_OUTPUT
EXTRA_DIST= \$(data_DATA)
"
echo "Generating ../../data/help/commands/Makefile.am"
echo "$MAKEFILE_TEXT" > ../../data/help/commands/Makefile.am
