#!/bin/bash

# First get the list of all the event names
EVENT_NAMES=`grep -h -i "@event:" ../../src/kvirc/kvi_event.cpp | sed -e 's/[	 ]*@event:[ ]*//g'`
EVENT_LINKS=""

for a_name in $EVENT_NAMES; do
	LOWER_NAME=`echo "$a_name" | sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`
	echo "Generating ../../data/help/events/$LOWER_NAME.kvihelp"
	DOCUMENT_DATA=`grep -h -i -A 100 "@event:[ ]*$a_name" ../../src/kvirc/kvi_event.cpp`
	# Now strip the first line and the lines after the end of the comment
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e '/@event:[ ]*[a-zA-Z_]*/ d'`
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e '/\*\// q'`
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e '/\*\// d'`
	# Strip empty lines and kill leading tabs
	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | grep -h "[a-zA-Z0-9{}]"`
 	DOCUMENT_DATA=`echo "$DOCUMENT_DATA" | sed -e 's/^[	 ]*//'`
 	# Extract the "@parameters:" part
 	PARAMETERS_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 50 "@parameters:"`
 	PARAMETERS_DATA=`echo "$PARAMETERS_DATA" | sed -e '/@parameters:/ d'`
 	PARAMETERS_DATA=`echo "$PARAMETERS_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	PARAMETERS_DATA=`echo "$PARAMETERS_DATA" | sed -e '/@[a-zA-Z]*:/ d'`
 	# Replace eventual < and > in the parameters
 	PARAMETERS_DATA=`echo "$PARAMETERS_DATA" | sed -e 's/</\&lt;/g'`
 	PARAMETERS_DATA=`echo "$PARAMETERS_DATA" | sed -e 's/>/\&gt;/g'`
 	# Extract the "@short:" part
 	SHORT_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 50 "@short:"`
 	SHORT_DATA=`echo "$SHORT_DATA" | sed -e '/@short:/ d'`
 	SHORT_DATA=`echo "$SHORT_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	SHORT_DATA=`echo "$SHORT_DATA" | sed -e '/@[a-zA-Z]*:/ d'` 	 	
 	# Extract the "@description:" part
 	DESCRIPTION_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 50 "@description:"`
 	DESCRIPTION_DATA=`echo "$DESCRIPTION_DATA" | sed -e '/@description:/ d'`
 	DESCRIPTION_DATA=`echo "$DESCRIPTION_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	DESCRIPTION_DATA=`echo "$DESCRIPTION_DATA" | sed -e '/@[a-zA-Z]*:/ d'` 	
 	# Extract the "@window:" part
 	WINDOW_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 50 "@window:"`
 	WINDOW_DATA=`echo "$WINDOW_DATA" | sed -e '/@window:/ d'`
 	WINDOW_DATA=`echo "$WINDOW_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	WINDOW_DATA=`echo "$WINDOW_DATA" | sed -e '/@[a-zA-Z]*:/ d'` 	
    	# Extract the "@examples:" part
 	EXAMPLES_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 50 "@examples:"`
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e '/@examples:/ d'`
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e '/@[a-zA-Z]*:/ d'`
 	# Replace eventual @tab@ signs
 	EXAMPLES_DATA=`echo "$EXAMPLES_DATA" | sed -e 's/@tab@/    /g'`
    	# Extract the "@seealso:" part
 	SEEALSO_DATA=`echo "$DOCUMENT_DATA" | grep -h -i -A 50 "@seealso:"`
 	SEEALSO_DATA=`echo "$SEEALSO_DATA" | sed -e '/@seealso:/ d'`
 	SEEALSO_DATA=`echo "$SEEALSO_DATA" | sed -e '/@[a-zA-Z]*:/ q'`
 	SEEALSO_DATA=`echo "$SEEALSO_DATA" | sed -e '/@[a-zA-Z]*:/ d'`
 	# Create the main document
 	DOCUMENT="<doctitle>$a_name event</doctitle>
<docbody>
<hr>
"
if test -n "$PARAMETERS_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Parameters</docsubtitle>
$PARAMETERS_DATA<br>
"
fi
if test -n "$WINDOW_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Window</docsubtitle>
$WINDOW_DATA<br>
"
fi
if test -n "$DESCRIPTION_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Description</docsubtitle>
$DESCRIPTION_DATA<br>
"
fi
if test -n "$EXAMPLES_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>Examples</docsubtitle>
$EXAMPLES_DATA<br>
"
fi
if test -n "$SEEALSO_DATA"; then
	DOCUMENT="$DOCUMENT<docsubtitle>See also</docsubtitle>
$SEEALSO_DATA<br>
"
fi
DOCUMENT="$DOCUMENT<hr>
<a href=\"index.kvihelp\">Main index</a> , <a href=\"events.kvihelp\">Event index</a>
</docbody>
"

echo "$DOCUMENT" > ../../data/help/events/$LOWER_NAME.kvihelp

if test -n "$EVENT_LINKS"; then
	EVENT_LINKS="$EVENT_LINKS
<a href=\"$LOWER_NAME.kvihelp\">$a_name</a>"
else
	EVENT_LINKS="<a href=\"$LOWER_NAME.kvihelp\">$a_name</a>"
fi

if test -n "$SHORT_DATA"; then
	EVENT_LINKS="$EVENT_LINKS : $SHORT_DATA<br>"
else
	EVENT_LINKS="$EVENT_LINKS<br>"
fi

done


# Prepare the main events index

EVENT_LINKS=`echo "$EVENT_LINKS" | sort`

EVENT_INDEX="<doctitle>Supported events</doctitle>
<docbody>
<hr>
$EVENT_LINKS
<hr>
<a href=\"index.kvihelp\">Main index</a>
</docbody>
"
echo "Generating ../../data/help/events/events.kvihelp"
echo "$EVENT_INDEX" > ../../data/help/events/events.kvihelp

# Now regenerate the Makefile.am

LS_OUTPUT=`ls ../../data/help/events/*.kvihelp | sed -e 's/^..\/..\/data\/help\/events\///'`

FILES_OUTPUT=""
for a_file in $LS_OUTPUT; do
	if test -n "$FILES_OUTPUT"; then
		FILES_OUTPUT="$FILES_OUTPUT \\
			$a_file"
	else
		FILES_OUTPUT="$a_file"
	fi
done

MAKEFILE_TEXT="datadir= \$(helpdir)
data_DATA= $FILES_OUTPUT
EXTRA_DIST= \$(data_DATA)
"
echo "Generating ../../data/help/events/Makefile.am"
echo "$MAKEFILE_TEXT" > ../../data/help/events/Makefile.am
