#ifndef _KVI_EXPREVAL_H_INCLUDED_
#define _KVI_EXPREVAL_H_INCLUDED_

//
//   File : kvi_expreval.h (/usr/build/NEW_kvirc/kvirc/src/kvilib/kvi_expreval.h)
//   Last major modification : Sun Feb 21 1999 15:05:24 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998-1999 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_string.h"

class KviExprTreeNode
{
public:
	enum NodeType{ Operand, BinaryOperator, UnaryOperator };
public:
	KviExprTreeNode(NodeType t,long v);
	~KviExprTreeNode();
public:
	int      m_type;

	long m_value;
private:
	KviExprTreeNode * m_left;
	KviExprTreeNode * m_right;
	KviExprTreeNode * m_parent;
public:
	KviExprTreeNode * left(){ return m_left; };
	KviExprTreeNode * right(){ return m_right; };
	void setLeft(KviExprTreeNode * node);
	void setRight(KviExprTreeNode * node);
	long firstBinaryOperator();
	KviExprTreeNode * parentWithPrecedenceLowerThan(long operValue);
//	void dump();
	long calculate();
};

// absolute precedence (~operand part)
#define OP_NOT -3          /* !  */
#define OP_BITNOT -2       /* ~  */
#define OP_NEGATE -1       /* -  */
// high precedence
#define OP_BITAND 0        /* &  */
#define OP_BITOR 1         /* |  */
#define OP_SHR 2           /* >> */
#define OP_SHL 3           /* << */
#define OP_POWER 4         /* ^  */
#define OP_MUL 5           /* *  */
#define OP_DIV 6           /* /  */
#define OP_MOD 7           /* %  */
#define OP_ADD 8           /* +  */
#define OP_SUB 9           /* -  */
#define OP_GT 10           /* >  */
#define OP_LT 11           /* <  */
#define OP_EQ 12           /* == */
#define OP_GE 13           /* >= */
#define OP_LE 14           /* <= */
#define OP_NE 15           /* != */
#define OP_AND 16          /* && */
#define OP_OR 17           /* || */
// low precedence

// Kvi Algebraic Expression Evaluator

#define KVI_EXPR_OK 0
#define KVI_EXPR_MISSING_CLOSING_PARENTHESIS 1
#define KVI_EXPR_UNEXPECTED_CHAR 2
#define KVI_EXPR_INTERNAL_ERROR 3
#define KVI_EXPR_UNKNOWN_OPERATOR 4
#define KVI_EXPR_PARENTHESIS_MISMATCH 5

class KviAlgExpr
{
public:
	KviAlgExpr(const char *pData);
	~KviAlgExpr();
private:
	const char * m_ptr;
	KviExprTreeNode * m_root;
	int m_error;
public:
	int evaluate(KviStr &buffer);
	int evaluate(long * valuePtr);
private:
	KviExprTreeNode * evaluateExpression();
	KviExprTreeNode * evaluateOperand();
	KviExprTreeNode * evaluateSimpleOperand();
	KviExprTreeNode * evaluateOperator();
	KviExprTreeNode * evaluateExpression_RightOperand(KviExprTreeNode *curTopOperator);
	void skipSpaces(){ while(*m_ptr && (*m_ptr < 33))m_ptr++; };
};

#endif //!_KVI_EXPREVAL_H_INCLUDED_
