//
//   File : kvi_hof.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_hof.cpp)
//   Last major modification : Sat Apr 3 1999 17:19:37 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998-1999 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CHECK_RANGE_
#include "kvi_debug.h"

#include "kvi_hof.h"
#include "kvi_app.h"
#include "kvi_ircview.h"
#include "kvi_input.h"
#include "kvi_frame.h"
#include "kvi_listbox.h"
#include "kvi_locale.h"
#include "kvi_options.h"
//#include "kvi_uparser.h"

//#include "kvi_fileutils.h"

//#include "kvi_ircsocket.h"
//#include "kvi_chaneditors.h"

//#include "kvi_event.h"

#include <qpalette.h>
#include <qdatetime.h>
//#include <qcursor.h>
#include <qlist.h>
#include <qvaluelist.h>
#include <qnamespace.h>
#include <qtimer.h>

#include <stdlib.h> //for rand()

//Declared in kvi_app.cpp and managed by KviApp class
extern QPixmap * g_pWindowIcon[KVI_WND_NUM_ICONS];

typedef struct KviHallOfFameEntry
{
	const char * nick;
	const char * user;
	const char * host;
	const char * whois;
};

#define KVI_HALL_OF_FAME_ENTRIES 7

static KviHallOfFameEntry hallOfFameEntries[KVI_HALL_OF_FAME_ENTRIES]=
{
//	La_For<La_Forge-> aahha, nono, famo O`Brien@IrcNet =)
//[02:50:54] <MalboroLi> guori unoge
	{
		"Pragma" , "nobody" , "pragmatico.net" ,
		"Pragma is Szymon Stefanek\nPragma is the main developer of the KVirc project"
	},
	{
//MalboroLi!ciao@<irc.net> Tommaso
		"MalboroLi" , "ght" , "unina.it" ,
		"MalboroLi is a hard beta tester"
	},
	{
		"BuTi" , "buti" , "something.*.at",
		"BuTi is a co-developer"
	},
	{
		"Drosha" , "dron" , "ilm.net",
		"Drosha is Andrew Frolov\nDrosha is 5449333e+14 secons idle"
	},
	{
		"sigmaya" , "solaris" , "solaris.net" ,
		"sigmaya is the main solaris tester"
	},
	{
		"Arter|o" , "arte" , "arterion.wav" ,
		"Arter|o is Arter|o"
	},
	{
//Diabl0!Diabl0@Irc.Net		"Diabl0" , "diab" , "louis.cypher.net" ,
		"Diabl0 is Louis Cypher"
	}
};

//============ KviHallOfFame ============//

KviHallOfFame::KviHallOfFame(KviFrame *lpFrm,const char * name)
:KviWindow(name,KVI_WND_TYPE_HALLOFFAME,lpFrm)
{

	m_pTopSplitter = new QSplitter(QSplitter::Horizontal,this);
	m_pTopicLabel  = new QLabel(__tr("The twilight zone"),m_pTopSplitter);
	m_pTopicLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pTopicLabel->setTextFormat(Qt::PlainText);
	m_pModeLabel   = new QLabel("ZxWstn",m_pTopSplitter);
	m_pModeLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pModeLabel->setTextFormat(Qt::PlainText);
	m_pUsersLabel  = new QLabel("8 [x:7 o:7 v:7 b:13392 e:148922]",m_pTopSplitter);
	m_pUsersLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pUsersLabel->setTextFormat(Qt::PlainText);
	m_pSplitter    = new QSplitter(QSplitter::Horizontal,this);
	m_pView        = new KviIrcView(m_pSplitter,lpFrm,this);
	m_pListBox     = new KviListBox(m_pSplitter,this,lpFrm->m_pUserParser,lpFrm);
	connect(m_pListBox,SIGNAL(rightClicked()),this,SLOT(listBoxRightClicked()));

	m_pInput       = new KviInput(this,lpFrm->m_pUserParser,m_pView,m_pListBox);
	// reasonable default for the splitters
	QValueList<int> l;
	l.append(80); //%
	l.append(20); //%
	m_pSplitter->setSizes(l);

	QValueList<int> l2;
	l2.append(60); //%
	l2.append(20); //%
	l2.append(20); //%
	m_pTopSplitter->setSizes(l2);

	setFocusHandler(m_pInput);

	applyLabelsOptions();
	connect(m_pView,SIGNAL(contextPopupRequested(KviIrcView *)),this,SLOT(viewRightClicked(KviIrcView *)));

	m_szLocalNick = m_pFrm->m_global.szCurrentNick;
	m_szLocalMask = m_pFrm->m_global.szCurrentMaskFromServer;
	if(m_szLocalNick.isEmpty())m_szLocalNick = __tr("me");
	if(m_szLocalMask.isEmpty())m_szLocalMask = __tr("me!me@localhost");

	startTheShow();

}

void KviHallOfFame::handleUserText(const KviStr &str)
{
	m_pFrm->outputPrivmsg(this,KVI_OUT_NONE,"Pragma","Pragma!nobody@pragmatico.net","Silence plz...we're working");
}

void KviHallOfFame::startTheShow()
{
	// Join all
	for(int i=0;i<KVI_HALL_OF_FAME_ENTRIES;i++)join(i,true);
	// Join the user
	KviStr tmp = m_szLocalMask;
	tmp.prepend("* ");
	KviIrcUser u(tmp.ptr(),0x01);
	m_pListBox->join(u,false,false,false);
	output(KVI_OUT_JOIN,__tr("%s [%s@%s] has joined %s"),m_szLocalNick.ptr(),u.username(),u.host(),KVI_HALLOFFAME_WINDOW_NAME);
	
	m_pListBox->repaint();

	QTimer::singleShot(2000 + (rand() % 120000),this,SLOT(randomJoin()));
}

void KviHallOfFame::join(int idx,bool bPragma)
{
	KviIrcUser u;
	KviStr tmp = "# ";
	tmp.append(hallOfFameEntries[idx].nick);
	u.setNick(tmp.ptr());
	u.setUsername(hallOfFameEntries[idx].user);
	u.setHost(hallOfFameEntries[idx].host);
	m_pListBox->join(u,bPragma,false,true);
	if(!bPragma){
		output(KVI_OUT_JOIN,__tr("%s [%s@%s] has joined %s"),hallOfFameEntries[idx].nick,
			hallOfFameEntries[idx].user,hallOfFameEntries[idx].host,KVI_HALLOFFAME_WINDOW_NAME);
	}
	QTimer::singleShot(500 + (rand() % 60000),this,SLOT(massOp()));
	QTimer::singleShot(5000 + (rand() % 180000),this,SLOT(randomPart()));
}

void KviHallOfFame::randomJoin()
{
	if(m_pListBox->count() < 8){
		// someone is still missing
		int entry = rand() % KVI_HALL_OF_FAME_ENTRIES;
		KviStr tmp = "# ";
		tmp.append(hallOfFameEntries[entry].nick);
		while(m_pListBox->findUser(tmp.ptr())){
			entry ++;
			if(entry == KVI_HALL_OF_FAME_ENTRIES)entry = 0;
			tmp = "# ";
			tmp.append(hallOfFameEntries[entry].nick);
		}
		join(entry,false);
	} else QTimer::singleShot(500 + (rand() % 60000),this,SLOT(randomPart()));
}

#define KVI_HALL_OF_FAME_QUIT_MESSAGES 10

static char * quitReason[KVI_HALL_OF_FAME_QUIT_MESSAGES]=
{
	"Segmentation fault",
	"Ping timeout",
	"No route to host",
	"Cya all!",
	"I'll go sleep",
	"No reson",
	"Netsplit",
	"Gogo",
	"1",
	"?????"
};

#define KVI_HALL_OF_FAME_PART_MESSAGES 4

static char * partReason[KVI_HALL_OF_FAME_PART_MESSAGES]=
{
	"Segmentation fault",
	"Ping timeout",
	"This channel sucks",
	"Adios"
};

void KviHallOfFame::randomPart()
{
	if(m_pListBox->count() > 2){
		int entry = 1 + (rand() % (KVI_HALL_OF_FAME_ENTRIES - 1));
		KviStr tmp = "# ";
		tmp.append(hallOfFameEntries[entry].nick);
		KviIrcUser *u = m_pListBox->findUser(tmp.ptr());
		if(!u){
			QTimer::singleShot(500 + (rand() % 30000),this,SLOT(randomPart()));
			QTimer::singleShot(500 + (rand() % 60000),this,SLOT(randomJoin()));
			return;
		}
		if(rand() % 2)output(KVI_OUT_PART,__tr("%s [%s!%s] has left %s [%s]"),u->nick(),u->username(),u->host(),KVI_HALLOFFAME_WINDOW_NAME,partReason[rand() % KVI_HALL_OF_FAME_PART_MESSAGES]);
		else output(KVI_OUT_QUIT,__tr("%s [%s!%s] has quit irc [%s]"),u->nick(),u->username(),u->host(),quitReason[rand() % KVI_HALL_OF_FAME_QUIT_MESSAGES]);
		m_pListBox->part(u->nick());
	}
	QTimer::singleShot(500 + (rand() % 30000),this,SLOT(randomJoin()));
}

void KviHallOfFame::massOp()
{
	KviIrcUser * u = m_pListBox->firstOp();
	if(!u){
		outputNoFmt(KVI_OUT_OP,"Pragma magically sets mode +ooo Pragma");
		m_pListBox->op("# Pragma",1);
		u = m_pListBox->findUser("# Pragma");
		if(!u)return;
	}
	QList<KviStr> l;
	l.setAutoDelete(true);
	for(KviIrcUser * us=m_pListBox->firstUser();us;us=m_pListBox->nextUser()){
		if(!m_pListBox->isOp(us->nick()) && (*(us->nick()) != '*')){
			l.append(new KviStr(us->nick()));
		}
	}
	while(l.first()){
		m_pListBox->op(l.first()->ptr(),1);
		output(KVI_OUT_OP,"%s [%s!%s] sets mode +o %s",u->nick(),u->username(),u->host(),l.first()->ptr());
		l.removeFirst();
	}
	QTimer::singleShot(500 + (rand() % 30000),this,SLOT(randomJoin()));
}

//============ ~KviHallOfFame ============//

KviHallOfFame::~KviHallOfFame()
{
}

void KviHallOfFame::saveProperties()
{
	KviWindowProperty p;
	p.rect = externalGeometry();
	p.isDocked = isAttacched();
	QValueList<int> l(m_pSplitter->sizes());
	if(l.count() >= 1)p.splitWidth1 = *(l.at(0));
	if(l.count() >= 2)p.splitWidth2 = *(l.at(1));
	QValueList<int> l2(m_pTopSplitter->sizes());
	if(l2.count() >= 1)p.topSplitWidth1 = *(l2.at(0));
	if(l2.count() >= 2)p.topSplitWidth2 = *(l2.at(1));
	if(l2.count() >= 3)p.topSplitWidth3 = *(l2.at(2));
	p.timestamp = m_pView->timestamp();
	p.imagesVisible = m_pView->imagesVisible();
	p.isMaximized = isAttacched() && isMaximized();
	g_pOptions->m_pWinPropertiesList->setProperty(caption(),&p);
}

void KviHallOfFame::setProperties(KviWindowProperty *p)
{
	QValueList<int> l;
	l.append(p->splitWidth1);
	l.append(p->splitWidth2);
	m_pSplitter->setSizes(l);
	QValueList<int> l2;
	l2.append(p->topSplitWidth1);
	l2.append(p->topSplitWidth2);
	l2.append(p->topSplitWidth3);
	m_pTopSplitter->setSizes(l2);
	m_pView->setTimestamp(p->timestamp);
	m_pView->setShowImages(p->imagesVisible);
}

void KviHallOfFame::viewRightClicked(KviIrcView *)
{
#warning "View right clicked ... and now ?"
//	g_pChannelPopup->doPopup(this,SLOT(viewPopupClicked(const KviStr &)));
}

void KviHallOfFame::listBoxRightClicked()
{
#warning "List box right clicked ... and now ?"
//	g_pUserlistPopup->doPopup(this,SLOT(userlistPopupClicked(const KviStr &)));
}

//=============== applyOptions ================//

void KviHallOfFame::applyLabelsOptions()
{
	if(!g_pOptions->m_pixLabelsBack->isNull()){
		m_pTopicLabel->setBackgroundPixmap(*(g_pOptions->m_pixLabelsBack));
		m_pModeLabel->setBackgroundPixmap(*(g_pOptions->m_pixLabelsBack));
		m_pUsersLabel->setBackgroundPixmap(*(g_pOptions->m_pixLabelsBack));
	} else {
		m_pTopicLabel->setBackgroundColor(g_pOptions->m_clrLabelsBack);
		m_pModeLabel->setBackgroundColor(g_pOptions->m_clrLabelsBack);
		m_pUsersLabel->setBackgroundColor(g_pOptions->m_clrLabelsBack);
	}

	QPalette pal(m_pTopicLabel->palette());
	pal.setColor(QPalette::Normal,QColorGroup::Text,g_pOptions->m_clrLabelsInactiveFore);

	m_pTopicLabel->setPalette(pal);
	m_pModeLabel->setPalette(pal);
	m_pUsersLabel->setPalette(pal);

//	setTopicChangeEnabled(isMeOp() || (!isModeActive('t')));
//	setModeChangeEnabled(isMeOp());

	m_pTopicLabel->setFont(g_pOptions->m_fntLabels);
	m_pModeLabel->setFont(g_pOptions->m_fntLabels);
	m_pUsersLabel->setFont(g_pOptions->m_fntLabels);
}

void KviHallOfFame::applyOptions()
{
	m_pView->setFont(g_pOptions->m_fntView);
	m_pView->setShowImages(g_pOptions->m_bShowImages,false);
	m_pView->setTimestamp(g_pOptions->m_bTimestamp);
	m_pView->setMaxBufferSize(g_pOptions->m_iViewMaxBufferSize);
	applyLabelsOptions();
}

//================ myIconPtr =================//

QPixmap * KviHallOfFame::myIconPtr()
{
	return g_pWindowIcon[KVI_WND_ICON_CHANNEL];
}

//================ resizeEvent ===============//

void KviHallOfFame::resizeEvent(QResizeEvent *)
{
	int inputSize = m_pInput->heightHint();
	int labelSize = m_pUsersLabel->sizeHint().height();
	m_pTopSplitter->setGeometry(0,0,width(),labelSize);
	m_pSplitter->setGeometry(0,labelSize,width(),height() - (inputSize+labelSize));
	m_pInput->setGeometry(0,height() - inputSize,width(),inputSize);
}

#include "m_kvi_hof.moc"
