//
//   File : kvi_style.cpp (/usr/build/KVIrc/kvirc/src/kvilib/kvi_style.cpp)
//   Last major modification : Wed Jul 21 1999 16:41:02 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CLASS_NAME_ "KviStyle"
#define _KVI_DEBUG_CHECK_RANGE_
#include "kvi_debug.h"

#include "kvi_style.h"

#include <qnamespace.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qpalette.h>
#include <qpushbutton.h>

KviStyle::KviStyle()
:QPlatinumStyle()
{
}

KviStyle::~KviStyle()
{
	__enter("~KviStyle");
}
/*
void KviStyle::fillRectWithPoints(QPainter *p,int x,int y,int w,int h,bool bSunken,
	const QColor &light,const QColor &mid,const QColor &dark)
{
	QPointArray pnt(1);
	int x1 = x+w;
	int y1 = y+h;
	int idx = 0;
	for(int i=x;i<x1;i+=5){
		for(int j=y;j<y1;j+=5){
			pnt.putPoints(idx,1,i,j);
			idx++;
		}
	}
	p->setPen(bSunken ? dark : light);
	p->drawPoints(pnt);
	pnt.translate(1,0);
	p->setPen(mid);
	p->drawPoints(pnt);
	pnt.translate(-1,1);
	p->drawPoints(pnt);
	pnt.translate(1,0);
	p->setPen(bSunken ? light : dark);
	p->drawPoints(pnt);
}


void KviStyle::drawButton(QPainter *p,int x,int y,int w,int h,const QColorGroup &g,
		 bool sunken,const QBrush * fill)
{
	QPlatinumStyle::drawButton(p,x,y,w,h,g,sunken,fill);
	fillRectWithPoints(p,x+3,y+3,w-6,h-6,sunken,
		g.light(),g.mid(),g.dark());
}

void KviStyle::drawBevelButton(QPainter *p,int x,int y,int w,int h,const QColorGroup &g,
		 bool sunken,const QBrush * fill)
{
	QPlatinumStyle::drawBevelButton(p,x,y,w,h,g,sunken,fill);
	fillRectWithPoints(p,x+3,y+3,w-6,h-6,sunken,
		g.light(),g.mid(),g.dark());
}
*/

void KviStyle::lightenColor(QColor &clr,int factor)
{
	int r,g,b;
	clr.rgb(&r,&g,&b);
	r += factor;
	g += factor;
	b += factor;
	clr.setRgb((r < 255) ? r : 255,(g < 255) ? g : 255,(b < 255) ? b : 255);
}

void KviStyle::darkenColor(QColor &clr,int factor)
{
	int r,g,b;
	clr.rgb(&r,&g,&b);
	r -= factor;
	g -= factor;
	b -= factor;
	clr.setRgb((r > 0) ? r : 0,(g > 0) ? g : 0,(b > 0) ? b : 0);
}
void KviStyle::doShadows(QPainter *p,int x,int y,int w,int h,
			const QColorGroup &g,bool sunken=false,const QBrush *fill= 0)
{
	if (w < 5 || h < 5)return;
    QPen oldPen = p->pen();

	if(fill){
		QBrush oldBrush = p->brush();
		p->setBrush( *fill );
		p->setPen( NoPen );
		p->drawRect(x,y,w,h);
		p->setBrush(oldBrush);
	}
	QColor c(fill ? fill->color() : g.button());

	int step = (g.button().red() + g.button().green() + g.button().blue());
	if(step < 382)step = 765 - step;
	step = step / 20;

	QPointArray a(3);

	if(sunken){
		for(int i=2;i>0;i--){
			a.setPoint(0,x+i,y+h-i);
			a.setPoint(1,x+i,y+i);
			a.setPoint(2,x+w-i,y+i);
			lightenColor(c,step);
			p->setPen(c);
			p->drawPolyline(a);
		}
		a.setPoint(0,x,y+h);
		a.setPoint(1,x,y);
		a.setPoint(2,x+w,y);
		darkenColor(c,step);
		p->setPen(c);
		p->drawPolyline(a);

		c = (fill ? fill->color() : g.button());

		for(int i=2;i>=0;i--){
			a.setPoint(0,x+i,y+h-i);
			a.setPoint(1,x+w-i,y+h-i);
			a.setPoint(2,x+w-i,y+i);
			darkenColor(c,step);
			p->setPen(c);
			p->drawPolyline(a);
		}
	} else {
		for(int i=4;i>1;i--){
			a.setPoint(0,x+i,y+h-i);
			a.setPoint(1,x+i,y+i);
			a.setPoint(2,x+w-i,y+i);
			lightenColor(c,step);
			p->setPen(c);
			p->drawPolyline(a);
		}
		for(int i=1;i>=0;i--){
			a.setPoint(0,x+i,y+h-i);
			a.setPoint(1,x+i,y+i);
			a.setPoint(2,x+w-i,y+i);
			darkenColor(c,step);
			p->setPen(c);
			p->drawPolyline(a);
		}
		c = (fill ? fill->color() : g.button());
		for(int i=4;i>=0;i--){
			a.setPoint(0,x+i,y+h-i);
			a.setPoint(1,x+w-i,y+h-i);
			a.setPoint(2,x+w-i,y+i);
			darkenColor(c,step);
			p->setPen(c);
			p->drawPolyline(a);
		}
	}

	p->setPen( oldPen );
}

void KviStyle::drawBevelButton( QPainter *p, int x, int y, int w, int h,
			const QColorGroup &g, bool sunken,const QBrush *fill)
{
	doShadows(p,x,y,w,h,g,sunken,fill);
}



void KviStyle::drawPushButton( QPushButton* btn, QPainter *p)
{
    QColorGroup g = btn->colorGroup();
    int x, y, x2, y2;

    btn->rect().coords( &x, &y, &x2, &y2 );	// get coordinates
    int w = x2 + 1;
    int h = y2 + 1;


	doShadows(p,x,y,w,h,g,btn->isDown(),&g.brush( QColorGroup::Button ));


/*
    p->setPen( g.foreground() );
    p->setBrush( QBrush(g.button(),NoBrush) );

    bool clearButton = TRUE;
    if ( btn->isDown() ) {
	if ( btn->isDefault() ) {
	    p->setPen( g.shadow() );
	    p->drawRect( x1, y1, x2-x1+1, y2-y1+1 );
	    p->setPen( g.dark() );
	    p->drawRect( x1+1, y1+1, x2-x1-1, y2-y1-1 );
	} else {
	    drawButton( p, x1, y1, w, h, g, TRUE,
			&g.brush( QColorGroup::Button ) );
	}
    } else {
	if ( btn->isDefault() ) {
	    p->setPen( g.shadow() );
	    p->drawRect( x1, y1, w, h );
	    x1++; y1++;
	    x2--; y2--;
	}
	if ( btn->isToggleButton() && btn->isOn() && btn->isEnabled() ) {
	    QBrush fill(g.light(), Dense4Pattern );
	    drawButton( p, x1, y1, x2-x1+1, y2-y1+1, g, TRUE, &fill );
	    clearButton = FALSE;
	} else {
	    drawButton( p, x1, y1, x2-x1+1, y2-y1+1, g, btn->isOn(),
			&g.brush( QColorGroup::Button ) );
	}
    }
    if ( clearButton ) {
	if (btn->isDown())
	    p->setBrushOrigin(p->brushOrigin() + QPoint(1,1));
	p->fillRect( x1+2, y1+2, x2-x1-3, y2-y1-3,
		     g.brush( QColorGroup::Button ) );
	if (btn->isDown())
	    p->setBrushOrigin(p->brushOrigin() - QPoint(1,1));
    }
    if ( btn->isMenuButton() ) {
	int dx = (y2-y1) / 3;
	drawArrow( p, DownArrow, FALSE,
		    x2 - dx, y1, dx, y2 - y1,
		    g, btn->isEnabled() );
    }

    if ( p->brush().style() != NoBrush )
	p->setBrush( NoBrush );
*/
}
#include "m_kvi_style.moc"
