//
//   File : kvi_dcc_iputils.cpp (/usr/build/NEW_kvirc/kvirc/src/kvidccslave/kvi_dcc_iputils.cpp)
//   Last major modification : Mon Apr 26 1999 23:42:12 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1998-1999 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "../common/kvi_settings.h"

#include "kvi_dcc_error.h"

#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>   /* for timeval */
#include <sys/select.h>
#include <netinet/in.h> //in_addr
#include <netdb.h>
#include <arpa/inet.h>  //inet_ntoa depends on this one.


//Check for inet aton
#ifndef HAVE_INET_ATON

#warning "Your system lacks the inet_aton function,"
#warning "or you're trying to compile this file without"
#warning "the config.h created by the configure script."
#warning "Using own internal implementation of inet_aton."

#include <ctype.h>

// Need own inet_aton implementation

// 
// Check whether "cp" is a valid ascii representation
// of an Internet address and convert to a binary address.
// Returns 1 if the address is valid, 0 if not.
// This replaces inet_addr, the return value from which
// cannot distinguish between failure and a local broadcast address.
//
// Original code comes from the ircd source.
//

bool kvi_dcc_stringIpToBinaryIp(const char *szIp,struct in_addr *address)
{
	register unsigned long val;
	register int base, n;
	register char c;
	unsigned int parts[4];
	register unsigned int *pp = parts;
	c = *szIp;
	for(;;){
		// Collect number up to ``.''.
		// Values are specified as for C:
		// 0x=hex, 0=octal, isdigit=decimal.
		if(!isdigit(c))return false;
		val  = 0;
		base = 10;
		if(c == '0'){
			c = *++szIp;
			if((c == 'x')||(c == 'X'))base = 16, c = *++szIp;
			else base = 8;
		}
		for (;;) {
			if(isascii(c) && isdigit(c)) {
				val = (val * base) + (c - '0');
				c   = *++szIp;
			} else if (base == 16 && isascii(c) && isxdigit(c)) {
				val = (val << 4) | (c + 10 - (islower(c) ? 'a' : 'A'));
				c   = *++szIp;
			} else break;
		}
		if(c == '.'){
			// Internet format:
			//	a.b.c.d
			//	a.b.c	(with c treated as 16 bits)
			//	a.b	(with b treated as 24 bits)
			if(pp >= (parts + 3)) return false;
			*pp++ = val;
			c     = *++szIp;
		} else break;
	}
	// Check for trailing characters.
	if ((c != '\0') && (!isascii(c) || !isspace(c)))return false;
	// Concact the address according to
	// the number of parts specified.
	n = pp - parts + 1;
	switch (n) {
	case 0: return false; // initial nondigit
	case 1:	break;        // a -- 32 bits 
	case 2:               // a.b -- 8.24 bits
		if(val > 0xffffff) return false;
		val |= parts[0] << 24;
		break;
	case 3:               // a.b.c -- 8.8.16 bits
		if(val > 0xffff)return false;
		val |= (parts[0] << 24) | (parts[1] << 16);
		break;
	case 4:               // a.b.c.d -- 8.8.8.8 bits
		if(val > 0xff)return false;
		val |= (parts[0] << 24) | (parts[1] << 16) | (parts[2] << 8);
		break;
	}
	if(address)address->s_addr = htonl(val);
	return true;
}

#else //HAVE_INET_ATON

bool kvi_dcc_stringIpToBinaryIp(const char *szIp,struct in_addr *address)
{
	return (inet_aton(szIp,address) != 0);
}

#endif //HAVE_INET_ATON


//Check for inet_ntoa
#ifndef HAVE_INET_NTOA

#warning "Your system lacks the inet_ntoa function,"
#warning "or you're trying to compile this file without"
#warning "the config.h created by the configure script."
#warning "Using own internal implementation of inet_ntoa."

//
// Original code comes from the ircd source.
//

bool kvi_binaryIpToString(struct in_addr in,char *ipBuffer);
{
	unsigned char *s = (unsigned char *)in;
	int	a,b,c,d;
	a = (int)*s++;
	b = (int)*s++;
	c = (int)*s++;
	d = (int)*s;
	sprintf(ipBuffer,"%d.%d.%d.%d",a,b,c,d); // Can be overflow ?...
	return true;
}

#else //HAVE_INET_NTOA

bool kvi_binaryIpToString(struct in_addr in,char *ipBuffer)
{
	char * ptr = inet_ntoa(in);
	if(!ptr)return false;
	strcpy(ipBuffer,ptr);
	return true;
}

#endif //HAVE_INET_NTOA

extern void * kvi_dcc_memmove(void *dst_ptr,const void *src_ptr,int len);

int kvi_dcc_resolveHost(const char *szHost,struct in_addr * address)
{
	if(!kvi_dcc_stringIpToBinaryIp(szHost,address)){
		struct hostent *hostEntry = gethostbyname(szHost);
		if(!hostEntry){
			switch(h_errno){
				case HOST_NOT_FOUND:
					return KVI_DCC_ERROR_HostNotFound;
					break;
				case NO_ADDRESS:
					return KVI_DCC_ERROR_ValidNameButNoIpAddress;
					break;
				case NO_RECOVERY:
					return KVI_DCC_ERROR_UnrecoverableDNSError;
					break;
				case TRY_AGAIN:
					return KVI_DCC_ERROR_TemporaneousDNSFault;
					break;
				default:
					return KVI_DCC_ERROR_CantResolveHostname;
					break;
			}
			return false;
		}
		kvi_dcc_memmove((void *)address,(void *)&(hostEntry->h_addr),sizeof(in_addr));
	}
	return KVI_DCC_Success;
}


int kvi_dcc_connect(struct in_addr addr,unsigned short port,int * fdRet)
{
	int sock=socket(AF_INET,SOCK_STREAM,0);
	if(sock<0)return KVI_DCC_ERROR_SocketCallFailed;
	if(fcntl(sock, F_SETFL, O_NONBLOCK) < 0){
		close(sock);
		return KVI_DCC_ERROR_CantSetNonBlockingIO;
	}
	struct sockaddr_in sain;
	sain.sin_family = AF_INET;
	sain.sin_port   = htons(port);
	sain.sin_addr   = addr;
	if(connect(sock,(struct sockaddr*)(&sain),sizeof(sain))<0){
		if(errno != EINPROGRESS){
			close(sock);
			switch(errno){
				case ECONNREFUSED:
					return KVI_DCC_ERROR_ConnectionRefused;
					break;
				case ENETUNREACH:
					return KVI_DCC_ERROR_NetworkIsUnreachable;
					break;
				case ETIMEDOUT:
					return KVI_DCC_ERROR_ConnectionTimedOut;
					break;
				default:
					return KVI_DCC_ERROR_CantConnectToRemoteHost;
					break;
			}
		}
	}
	*fdRet = sock;
	return KVI_DCC_Success;
}

int kvi_dcc_waitForConnection(int sock)
{
	struct timeval m_tm;
	fd_set m_fd;
	for(;;){
		m_tm.tv_sec = 0;
		m_tm.tv_usec = 1;
		FD_ZERO(&m_fd);
		FD_SET(sock,&m_fd);
		if(select(sock+1,0,&m_fd,0,&m_tm)){
			int nSockErr;
			ksize_t iSize=sizeof(int);
			if(getsockopt(sock,SOL_SOCKET,SO_ERROR,(void *)&nSockErr,&iSize)==-1)nSockErr=EBADF;
			if(nSockErr != 0){
				switch(nSockErr){
					case ECONNREFUSED:
						return KVI_DCC_ERROR_ConnectionRefused;
						break;
					case ENETUNREACH:
						return KVI_DCC_ERROR_NetworkIsUnreachable;
						break;
					case ETIMEDOUT:
						return KVI_DCC_ERROR_ConnectionTimedOut;
						break;
					default:
						return KVI_DCC_ERROR_CantConnectToRemoteHost;
						break;
				}
			} else return KVI_DCC_Success;
		} else usleep(200);
	}
}

int kvi_dcc_listen(int *fdRet,unsigned int *portRet)
{
	struct sockaddr_in sockAddress;
	// Do a bzero
	int siz = sizeof(sockAddress);
	char * tmp = (char *)&sockAddress;
	for(int i=0;i<siz;i++){
		*tmp = 0;
		tmp++;
	}
//	bzero((char *)&sockAddress, sizeof(sockAddress));
	sockAddress.sin_family = AF_INET;
	sockAddress.sin_port   = htons(0);
	sockAddress.sin_addr.s_addr = INADDR_ANY;
	int aSock;
	if((aSock = socket(AF_INET,SOCK_STREAM,0)) < 0)return KVI_DCC_ERROR_SocketCallFailed;
	if((bind(aSock,(struct sockaddr *) &sockAddress,sizeof(sockAddress))<0)||(listen(aSock,100)<0) ) {
		close(aSock);
		return KVI_DCC_ERROR_CantSetupListeningSocket;
	}
	if(fcntl(aSock, F_SETFL, O_NONBLOCK) < 0){
		close(aSock);
		return KVI_DCC_ERROR_CantSetNonBlockingIO;
	}
	ksize_t size=sizeof(sockAddress);
	getsockname(aSock, (struct sockaddr*)&sockAddress, &size );
	*portRet = ntohs(sockAddress.sin_port);
	*fdRet = aSock;
	return KVI_DCC_Success;
}

int kvi_dcc_waitForAccept(int sock,int *fdRet,unsigned int *portRet,char *ipBuffer)
{	
	struct sockaddr_in sockAddress;
	// Do a bzero
	int size = sizeof(sockAddress);
	char * tmp = (char *)&sockAddress;
	for(int i=0;i<size;i++){
		*tmp = 0;
		tmp++;
	}
	for(;;){
		//connected ,accept it*/
		ksize_t size=sizeof(sockAddress);
		*fdRet=accept(sock, (struct sockaddr*)&sockAddress, &size);
		if(*fdRet != -1){
//			fprintf(stderr,"CConnected to %s on port %u",inet_ntoa(sockAddress.sin_addr),sockAddress.sin_port);
			if(!kvi_binaryIpToString(sockAddress.sin_addr,ipBuffer))strcpy(ipBuffer,"Unknown host");
			*portRet = sockAddress.sin_port;
			return KVI_DCC_Success;
		} else {
			if(errno != EWOULDBLOCK)return KVI_DCC_ERROR_CantAcceptIncomingConnection;
			usleep(200);
		}
	}
}
