//
//   File : kvi_colordialog.cpp (/usr/build/NEW_kvirc/kvirc/src/kvilib/kvi_colordialog.cpp)
//   Last major modification : Sun Mar 21 1999 19:20:16 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include <qlayout.h>
#include <qpushbutton.h>
#include <qpainter.h>
#include <qnamespace.h>

#include <math.h>

#include "kvi_locale.h"
#include "kvi_colordialog.h"

KviBaseColorSelector::KviBaseColorSelector(QWidget *parent)
:QFrame(parent)
{
	setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_x = m_y = 1;
	m_pMemPix = new QImage(300,255,32);
	createGradient();
	QSize s(300+2,255+2);
	setFixedSize(s);
}

KviBaseColorSelector::~KviBaseColorSelector()
{
	delete m_pMemPix;
}

void KviBaseColorSelector::createGradient()
{	
	int i,j;
	for(i = 0;i<255 ; i++){
		QRgb * pDkLine = (QRgb *)m_pMemPix->scanLine(i);
		for(j=0;j<50;j++)pDkLine[j]    = qRgb(255,0,j*5);
		for(j=50;j<100;j++)pDkLine[j]  = qRgb((255 - ((j-50) * 5)),0,255);
		for(j=100;j<150;j++)pDkLine[j] = qRgb(0,(j-100)*5,255);
		for(j=150;j<200;j++)pDkLine[j] = qRgb(0,255,(255-((j-150)*5)));
		for(j=200;j<250;j++)pDkLine[j] = qRgb((j-200) * 5,255,0);
		for(j=250;j<300;j++)pDkLine[j] = qRgb(255,(255-((j-250)*5)),0);
	}
	for(i = 0; i<128; i++){
		unsigned char * pDkLine = m_pMemPix->scanLine(127-i);
		for(j=0;j<300;j++){
			int idx = j*4;
			pDkLine[idx]   -= ((pDkLine[idx]*i)/127);
			pDkLine[idx+1] -= ((pDkLine[idx+1]*i)/127);
			pDkLine[idx+2] -= ((pDkLine[idx+2]*i)/127);
			pDkLine[idx+3] -= ((pDkLine[idx+2]*i)/127);
		}
	}
	for(int k = 128;k<255;k++){
		unsigned char * pDkLine = m_pMemPix->scanLine(k);
		for(j=0;j<300;j++){
			i=k-128;
			int idx = j*4;
			pDkLine[idx]   += (((255-pDkLine[idx])*i)/126);
			pDkLine[idx+1] += (((255-pDkLine[idx+1])*i)/126);
			pDkLine[idx+2] += (((255-pDkLine[idx+2])*i)/126);
			pDkLine[idx+3] += (((255-pDkLine[idx+2])*i)/126);
		}
	}
}

void KviBaseColorSelector::mousePressEvent(QMouseEvent *e)
{
	m_x = e->pos().x();
	m_y = e->pos().y();
	if(m_x < 1)m_x = 1;
	if(m_y < 1)m_y = 1;
	if(m_y > 255)m_y = 255;
	if(m_x > 300)m_x = 300;
	repaint(false);
	QRgb * rgb = (QRgb *)m_pMemPix->scanLine(m_y-1);
	emit colorSelected(rgb[m_x-1]);
}

void KviBaseColorSelector::mouseMoveEvent(QMouseEvent *e)
{
	QPainter p(this);
	p.setRasterOp(Qt::NotROP);
	p.drawLine(m_x,1,m_x,255);
	p.drawLine(1,m_y,300,m_y);
	m_x = e->pos().x();
	m_y = e->pos().y();
	if(m_x < 1)m_x = 1;
	if(m_y < 1)m_y = 1;
	if(m_y > 255)m_y = 255;
	if(m_x > 300)m_x = 300;
	p.drawLine(m_x,1,m_x,255);
	p.drawLine(1,m_y,300,m_y);
	QRgb * rgb = (QRgb *)m_pMemPix->scanLine(m_y-1);
	emit colorSelected(rgb[m_x-1]);
}

void KviBaseColorSelector::drawContents(QPainter *p)
{
	p->drawImage(1,1,*m_pMemPix);
	p->setRasterOp(Qt::NotROP);
	p->drawLine(m_x,1,m_x,255);
	p->drawLine(1,m_y,300,m_y);
}

KviShadeColorSelector::KviShadeColorSelector(QWidget * parent)
:QFrame(parent)
{
	setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_y = 128;
	m_pMemPix = new QImage(10,255,32);
	QSize s(10+2,255+2);
	setFixedSize(s);
}

KviShadeColorSelector::~KviShadeColorSelector()
{
	delete m_pMemPix;
}

void KviShadeColorSelector::drawContents(QPainter *p)
{
	int i,k,j;
	for(i = 0; i<128; i++){
		QRgb * pLine = (QRgb * )m_pMemPix->scanLine(127-i);
		for(j=0;j<10;j++){
			pLine[j]   = qRgb(m_curBaseColor.red() - ((m_curBaseColor.red()*i)/127),
								m_curBaseColor.green() - ((m_curBaseColor.green()*i)/127),
								m_curBaseColor.blue() - ((m_curBaseColor.blue()*i)/127));
		}
	}
	for(k = 128;k<255;k++){
		QRgb * pLine = (QRgb * )m_pMemPix->scanLine(k);
		for(j=0;j<10;j++){
			i=k-128;
			pLine[j]   = qRgb(m_curBaseColor.red() + (((255-m_curBaseColor.red())*i)/126),
								m_curBaseColor.green() + (((255-m_curBaseColor.green())*i)/126),
								m_curBaseColor.blue() + (((255-m_curBaseColor.blue())*i)/126));
		}
	}
	p->drawImage(1,1,*m_pMemPix);
	p->setRasterOp(Qt::NotROP);
	p->drawLine(1,m_y,10,m_y);
}

void KviShadeColorSelector::mousePressEvent(QMouseEvent *e)
{
	m_y = e->pos().y();
	if(m_y < 1)m_y = 1;
	if(m_y > 255)m_y = 255;
	repaint(false);
	QRgb * rgb = (QRgb *)m_pMemPix->scanLine(m_y-1);
	emit colorSelected(rgb[0]);
}

void KviShadeColorSelector::mouseMoveEvent(QMouseEvent *e)
{
	QPainter p(this);
	p.setRasterOp(Qt::NotROP);
	p.drawLine(1,m_y,10,m_y);
	m_y = e->pos().y();
	if(m_y < 1)m_y = 1;
	if(m_y > 255)m_y = 255;
	p.drawLine(1,m_y,10,m_y);
	QRgb * rgb = (QRgb *)m_pMemPix->scanLine(m_y-1);
	emit colorSelected(rgb[0]);
}

void KviShadeColorSelector::setBaseColor(QRgb rgb)
{
	m_y = 128;
	m_curBaseColor.setRgb(rgb);
	repaint(false);
	emit colorSelected(rgb);
}

KviColorDialog::KviColorDialog(const QColor &color)
:QDialog(0,"",true)
{
	setCaption(__tr("Choose a color..."));

	QGridLayout *g=new QGridLayout(this,3/*rows*/,5/*cols*/,4,3);

	KviShadeColorSelector *s=new KviShadeColorSelector(this);
	g->addWidget(s,0,0);
	s->setBaseColor(color.rgb());
	connect(s,SIGNAL(colorSelected(QRgb)),this,SLOT(baseColorSelected(QRgb)));

	KviBaseColorSelector *b=new KviBaseColorSelector(this);
	g->addMultiCellWidget(b,0,0,1,4);
	connect(b,SIGNAL(colorSelected(QRgb)),s,SLOT(setBaseColor(QRgb)));

	m_pSample = new QLabel(this);
	m_pSample->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pSample->setBackgroundColor(color);
	g->addMultiCellWidget(m_pSample,1,1,0,1);

	m_pRed = new QLabel(this);
	m_pRed->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pRed->setMinimumWidth(70);
	g->addWidget(m_pRed,1,2);

	m_pGreen = new QLabel(this);
	m_pGreen->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pGreen->setMinimumWidth(70);
	g->addWidget(m_pGreen,1,3);

	m_pBlue = new QLabel(this);
	m_pBlue->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pBlue->setMinimumWidth(70);
	g->addWidget(m_pBlue,1,4);


	QPushButton *pOk=new QPushButton(__tr("Ok"),this);
	g->addWidget(pOk,2,4);
	pOk->setDefault(true);
	QPushButton *pCancel=new QPushButton(__tr("Cancel"),this);
	g->addWidget(pCancel,2,3);

	connect(pOk,SIGNAL(clicked()),this,SLOT(accept()));
	connect(pCancel,SIGNAL(clicked()),this,SLOT(reject()));

	baseColorSelected(color.rgb());
}

KviColorDialog::~KviColorDialog()
{
}

void KviColorDialog::baseColorSelected(QRgb rgb)
{
	m_pSample->setBackgroundColor(QColor(rgb));
	QString num;
	num.setNum(qBlue(rgb));
	num.prepend("B:");
	m_pBlue->setText(num);
	num.setNum(qRed(rgb));
	num.prepend("R:");
	m_pRed->setText(num);
	num.setNum(qGreen(rgb));
	num.prepend("G:");
	m_pGreen->setText(num);
}

#include "m_kvi_colordialog.moc"
