//
//   File : kvi_fontdialog.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_fontdialog.cpp)
//   Last major modification : Thu Mar 18 1999 03:52:26 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "kvi_fontdialog.h"

#include "kvi_string.h"
#include "kvi_locale.h"

#include <qlayout.h>
#include <qlist.h>
#include <qpushbutton.h>
#include <qfontinfo.h>

#include <X11/Xlib.h>

// Avoid name clashes...
#undef TrueColor


KviFontDialog::KviFontDialog(const QFont &font)
:QDialog(0,"",true)
{
	setCaption(__tr("Choose a font..."));
	m_curFont = font;

	QGridLayout *g=new QGridLayout(this,6,4,4,4);

	QLabel * lab = new QLabel(__tr("Selected font:"),this);
	g->addWidget(lab,0,0);

	m_pFamilyCombo=new QComboBox(false,this);
	g->addWidget(m_pFamilyCombo,1,0);

	m_pSizeCombo=new QComboBox(false,this);
	g->addWidget(m_pSizeCombo,2,0);

	m_pCharSetCombo=new QComboBox(false,this);
	g->addWidget(m_pCharSetCombo,3,0);

	QFrame * sep=new QFrame(this);
	sep->setFrameStyle(QFrame::VLine|QFrame::Sunken);
	g->addMultiCellWidget(sep,0,3,1,1);

	lab = new QLabel(__tr("Actual font:"),this);
	g->addMultiCellWidget(lab,0,0,2,3);

	m_pActualFamily = new QLabel(this);
	m_pActualFamily->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	g->addMultiCellWidget(m_pActualFamily,1,1,2,3);
	m_pActualPointSize = new QLabel(this);
	m_pActualPointSize->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	g->addMultiCellWidget(m_pActualPointSize,2,2,2,3);
	m_pActualCharset = new QLabel(this);
	m_pActualCharset->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	g->addMultiCellWidget(m_pActualCharset,3,3,2,3);

	m_pSampleLabel=new QLineEdit(this);
	m_pSampleLabel->setText("Nonsense in BASIC");

	g->addMultiCellWidget(m_pSampleLabel,4,4,0,3);

	QPushButton *pOk=new QPushButton(__tr("Ok"),this);
	g->addWidget(pOk,5,3);
	pOk->setDefault(true);
	QPushButton *pCancel=new QPushButton(__tr("Cancel"),this);
	g->addWidget(pCancel,5,2);
	
	fillFamilyCombo();
	fillSizeCombo();
	fillCharSetCombo();

	describeCurrentFont();

	connect(m_pFamilyCombo,SIGNAL(activated(const QString &)),this,SLOT(familyChange(const QString &)));
	connect(m_pSizeCombo,SIGNAL(activated(int)),this,SLOT(sizeChange(int)));
	connect(m_pCharSetCombo,SIGNAL(activated(int)),this,SLOT(charSetChange(int)));

	connect(pOk,SIGNAL(clicked()),this,SLOT(accept()));
	connect(pCancel,SIGNAL(clicked()),this,SLOT(reject()));
}

KviFontDialog::~KviFontDialog()
{
}

void KviFontDialog::fillFamilyCombo()
{
	m_pFamilyCombo->clear();
	int numFonts;

	char ** ppFontNames = XListFonts(x11Display(), "*", 32767, &numFonts);
	if(!ppFontNames){
		//ooops XListFonts failed
		debug(__tr("Can not list available fonts"));
		return;
	}
	char ** ppFontNames_saved = ppFontNames;

	QList<KviStr> fList;
	fList.setAutoDelete(true);

	KviStr szFontEntry,szFamily;
	KviStr * pFont;

	for(int i=0;i<numFonts;i++){
		if (**ppFontNames != '-'){ 
			// kill the aliased fonts that do not start with a dash
			ppFontNames++;
			continue;
		}
		szFontEntry = *ppFontNames;
		szFontEntry.cutLeft(1); //kill the first dash
		szFontEntry.getToken(szFamily,'-'); //kill the first token (between the first and second dash)
		szFontEntry.getToken(szFamily,'-'); //and extract the next one
		if(szFamily.isEmpty()){
			ppFontNames++;
			continue;
		}
		bool bFound = false;
		for(pFont=fList.first();pFont && !bFound;pFont=fList.next()){
			if(kvi_strEqualCI(pFont->ptr(),szFamily.ptr()))bFound=true;
		}
		if(!bFound){
			fList.append(new KviStr(szFamily));
		}
		ppFontNames++;
	}
	int idx = 0;
	int curOne = 0;
	KviStr szCurOne = m_curFont.family();
	for(pFont=fList.first();pFont;pFont=fList.next()){
		m_pFamilyCombo->insertItem(pFont->ptr(),-1);
		if(kvi_strEqualCI(pFont->ptr(),szCurOne.ptr()))curOne = idx;
		idx++;
	}
	m_pFamilyCombo->setCurrentItem(curOne);
	XFreeFontNames(ppFontNames_saved);
}

void KviFontDialog::familyChange(const QString &family)
{
	m_curFont.setFamily(family);
	describeCurrentFont();
}

int font_sizes_table[40]=
{
	2, 3, 4, 5, 6, 7, 8, 9,10,11,
	12,13,14,15,16,17,18,19,20,21,
	22,23,24,25,26,27,28,29,30,31,
	32,34,36,38,40,44,48,52,56,64
};

void KviFontDialog::fillSizeCombo()
{
	for(int i=0;i<40;i++){
		QString num;
		num.setNum(font_sizes_table[i]);
		m_pSizeCombo->insertItem(num);
		if(m_curFont.pointSize() == font_sizes_table[i])m_pSizeCombo->setCurrentItem(i);
	}
}

void KviFontDialog::sizeChange(int index)
{
	if((index < 0)||(index > 39))index = 0;
	m_pSizeCombo->setCurrentItem(index);
	m_curFont.setPointSize(font_sizes_table[index]);
	describeCurrentFont();
}


void KviFontDialog::describeCurrentFont()
{
	m_pSampleLabel->setFont(m_curFont);
	QFontInfo fi(m_curFont);
	QString tmp(__tr("Family : "));
	tmp+=fi.family();
	m_pActualFamily->setText(tmp);
	tmp=__tr("Charset : ");
	tmp+=getCharset(fi.charSet());
	m_pActualCharset->setText(tmp);
	tmp.setNum(fi.pointSize());
	tmp.prepend(__tr("Point size : "));
	m_pActualPointSize->setText(tmp);
}


typedef struct charSetTableEntry
{
	QFont::CharSet identifier;
	const char * name;
};

charSetTableEntry csTable[17]=
{
	{ QFont::ISO_8859_1 , "ISO_8859_1 (Latin1)" },
	{ QFont::ISO_8859_2 , "ISO_8859_2 (Latin2)" },
	{ QFont::ISO_8859_3 , "ISO_8859_3 (Latin3)" },
	{ QFont::ISO_8859_4 , "ISO_8859_4 (Latin4)" },
	{ QFont::ISO_8859_5 , "ISO_8859_5"          },
	{ QFont::ISO_8859_6 , "ISO_8859_6"          },
	{ QFont::ISO_8859_7 , "ISO_8859_7"          },
	{ QFont::ISO_8859_8 , "ISO_8859_8"          },
	{ QFont::ISO_8859_9 , "ISO_8859_9"          },
	{ QFont::KOI8R      , "KOI8R"               },
	{ QFont::Set_Ja     , "Set_Ja"              },
	{ QFont::Set_Ko     , "Set_Ko"              },
	{ QFont::Set_Th_TH  , "Set_Th_TH"           },
	{ QFont::Set_Zh     , "Set_Zh"              },
	{ QFont::Set_Zh_TW  , "Set_Zh_TW"           },
	{ QFont::Unicode    , "Unicode"             },
	{ QFont::AnyCharSet , "Any"                 }
};


QString KviFontDialog::getCharset(QFont::CharSet cs)
{
	QString ret(__tr("Unknown"));
	for(int i = 0;i< 17;i++){
		if(csTable[i].identifier==cs){
			ret = csTable[i].name;
			break;
		}
	}
	return ret;
}

void KviFontDialog::fillCharSetCombo()
{
	QFont::CharSet cs = m_curFont.charSet();

	for(int i = 0;i< 17;i++){
		m_pCharSetCombo->insertItem(csTable[i].name,-1);
		if(cs == csTable[i].identifier)m_pCharSetCombo->setCurrentItem(i);
	}
}

void KviFontDialog::charSetChange(int index)
{
	if(index < 0 || index > 16)index = 0;
	m_curFont.setCharSet(csTable[index].identifier);
	describeCurrentFont();
}

/*
KviFontDialog::KviFontDialog(const QFont &font)
:QFontDialog(0,"",true)
{
	setFont(font);
}

KviFontDialog::~KviFontDialog(){}

QFont KviFontDialog::getCurrentFont()
{
	return font();
}
*/
#include "m_kvi_fontdialog.moc"
