//
//   File : kvi_mime.cpp (/usr/build/NEW_kvirc/kvirc/src/kvilib/kvi_mime.cpp)
//   Last major modification : Mon May 10 1999 17:34:49 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "kvi_mime.h"
#include "kvi_config.h"
#include "kvi_locale.h"

#include <qregexp.h>
#include <qfile.h>

KviMimeManager::KviMimeManager()
{
	m_pMimeList = new QList<KviMimeType>;
	m_pMimeList->setAutoDelete(true);
	m_pDefault = new KviMimeType;
	m_pDefault->mimeName = "Unknown";
	m_pDefault->magicBytes = "";
	m_pDefault->fileMask = "*";
	m_pDefault->commandline = "xedit %";
	m_pDefault->savePath = "";
}

KviMimeManager::~KviMimeManager()
{
	delete m_pMimeList;
	delete m_pDefault;
}

void KviMimeManager::save(KviConfig * cfg)
{
	cfg->writeEntry("MimeCount",m_pMimeList->count());
	int i=0;
	for(KviMimeType * m=m_pMimeList->first();m;m=m_pMimeList->next()){
		KviStr key(KviStr::Format,"Mime_%d_Name",i);
		cfg->writeEntry(key.ptr(),m->mimeName.ptr());
		key.sprintf("Mime_%d_Magic",i);
		cfg->writeEntry(key.ptr(),m->magicBytes.ptr());
		key.sprintf("Mime_%d_FileMask",i);
		cfg->writeEntry(key.ptr(),m->fileMask.ptr());
		key.sprintf("Mime_%d_Commandline",i);
		cfg->writeEntry(key.ptr(),m->commandline.ptr());
		key.sprintf("Mime_%d_SavePath",i);
		cfg->writeEntry(key.ptr(),m->savePath.ptr());
		key.sprintf("Mime_%d_RemoteExecSafe",i);
		cfg->writeEntry(key.ptr(),m->remoteExecSafe);
		i++;
	}
}

void KviMimeManager::load(KviConfig * cfg)
{
	int count = cfg->readIntEntry("MimeCount",0);
	for(int i=0;i<count;i++){
		KviMimeType * t = new KviMimeType;
		KviStr key(KviStr::Format,"Mime_%d_Name",i);
		t->mimeName = cfg->readEntry(key.ptr(),"Unknown");
		key.sprintf("Mime_%d_Magic",i);
		t->magicBytes = cfg->readEntry(key.ptr(),"");
		key.sprintf("Mime_%d_FileMask",i);
		t->fileMask = cfg->readEntry(key.ptr(),"*");
		key.sprintf("Mime_%d_Commandline",i);
		t->commandline = cfg->readEntry(key.ptr(),"xedit %");
		key.sprintf("Mime_%d_SavePath",i);
		t->savePath = cfg->readEntry(key.ptr(),"");
		key.sprintf("Mime_%d_RemoteExecSafe",i);
		t->remoteExecSafe = cfg->readBoolEntry(key.ptr(),false);
		inSort(t);
	}
}


void KviMimeManager::inSort(KviMimeType * m)
{
	// Order the mimetypes in a way that we always find the correct match
	// first sort by the number of wildcards inside : the one that has less wildcards , comes first.
	// then longer fileMasks (*.tar.gz)
	// then shorter ones  (*.gz) (*)
	// The mimes with a magic go before similar mimes without
	uint idx = 0;
	m->fileMask.stripWhiteSpace();
	m->magicBytes.stripWhiteSpace();
	m->savePath.stripWhiteSpace();
	KviStr strippedMaskM = m->fileMask;
	strippedMaskM.replaceAll('*',""); //remove all wildcards
	KviStr strippedMaskT;
	for(KviMimeType * t = m_pMimeList->first();t;t = m_pMimeList->next()){
		if(m->fileMask.contains('*') <= t->fileMask.contains('*')){
			strippedMaskT = t->fileMask;
			strippedMaskT.replaceAll('*',"");
			// the item that we're inserting has less wildcards than the one in the list
			if(strippedMaskM.len() >= strippedMaskT.len()){
				// the item that we're inserting is longer than the one in the list
				if(m->magicBytes.len() >= t->magicBytes.len()){
					//the item that we're inserting has a longer magic than the one in the list
					m_pMimeList->insert(idx,m);
					return;
				} // else has shorter magic ... goes after ... skip one item
			} // else has shorter file mask ... goes after ... skip one item
		} // else has more wildcards ... goes after ... skip one item
		idx++;
	}
	m_pMimeList->append(m);
}

bool KviMimeManager::checkMagicMatch(KviStr &magic,const char *fileName)
{
	QFile f(fileName);
	if(!f.exists())return true; // matched

	if(!f.open(IO_ReadOnly)){
		debug(__tr("Mime match : Could not open file %s"),fileName);
		return true; //!!!
	}

    char buffer[16];
    int readLen = f.readBlock(buffer,16);

	f.close();

	if(readLen <= 0){
		debug(__tr("Mime match : Zero length file %s"),fileName);
		return true;
	}

	// Ugly hack
    for(int i=0;i<readLen;i++){
		if(buffer[i]=='\0')buffer[i]='\01';
	}

	buffer[readLen-1]='\0';

	QRegExp rexp(QString(magic.ptr()));
	return (rexp.match(buffer) == 0);
}

KviMimeType * KviMimeManager::findMatch(const char *fileName,bool bDoMagicCheck)
{
	KviStr fName(fileName);
	int lastIdx = fName.findLastIdx("/");
	if(lastIdx != -1)fName.cutLeft(lastIdx + 1);
	for(KviMimeType * m=m_pMimeList->first();m;m=m_pMimeList->next()){
		// If no magic checks have to be done , get rid of the
		// totally wild fileMasks (*)....
		QRegExp rExp(QString(m->fileMask.ptr()),false,true);
		int len = 0;
		if(rExp.match(QString(fName.ptr()),0,&len) == 0){
			if(len == fName.len()){
				// matched...
				if(bDoMagicCheck){
					// now check the first bytes of the file...
					m->magicBytes.stripWhiteSpace();
					if(m->magicBytes.len() <= 0)return m; //no magic to be used
					if(checkMagicMatch(m->magicBytes,fileName))return m;
				} else return m; //no magic check has to be done.
			}
		}
	}
	return m_pDefault;
}

