//
//   File : kvi_selectors.cpp (/usr/build/NEW_kvirc/kvirc/kvilib/kvi_selectors.cpp)
//   Last major modification : Sat Jan 16 1999 14:57:02 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "kvi_selectors.h"

#include "kvi_buttons.h"

#include "kvi_locale.h"

#include <qvalidator.h>
#include <qlayout.h>
#include <qfiledialog.h>

//
// String value selector
//

KviStringSelector::KviStringSelector(QWidget *parent,QTabDialog * tab,KviStr *valuePtr,
		const char * label,bool bEnabled,const char * szDefault,int defInputWidth)
:QWidget(parent)
{
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	m_pLabel   = new QLabel(label,this);
	m_pInput   = new QLineEdit(this);
	QString val(valuePtr->ptr());
	m_pInput->setText(val);
	m_szDefault = szDefault;
	m_iDefInputWidth = defInputWidth;
	setMinimumSize(sizeHint());
	setEnabled(bEnabled);
}

KviStringSelector::~KviStringSelector()
{}
void KviStringSelector::applyButtonPressed()
{
	KviStr tmp = m_pInput->text();
	if(tmp.hasData()) *m_valuePtr = tmp;
	else {
		if(m_szDefault.isEmpty()) *m_valuePtr = "";
		else *m_valuePtr = m_szDefault.ptr();
	}
}
void KviStringSelector::setNotEnabled(bool bNotEnabled)
{ setEnabled(!bNotEnabled); }
void KviStringSelector::setEnabled(bool bEnabled)
{
	m_pInput->setEnabled(bEnabled);
	m_pLabel->setEnabled(bEnabled);
}
QSize KviStringSelector::sizeHint()
{
	QSize sz=m_pLabel->sizeHint();
	sz.setWidth(sz.width()+m_iDefInputWidth); //at least 100 pixels for each input line
	sz.setHeight(m_pInput->sizeHint().height());
	return sz;
}

void KviStringSelector::resizeEvent(QResizeEvent *)
{
	m_pLabel->setGeometry(0,0,width()-(m_iDefInputWidth +1),height());
	m_pInput->setGeometry(width()-m_iDefInputWidth,0,m_iDefInputWidth,height());
}

//
// Char value selector
//

KviCharSelector::KviCharSelector(QWidget *parent,QTabDialog * tab,char *valuePtr,
		const char * label,bool bEnabled,int defInputWidth)
:QWidget(parent)
{
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	m_pLabel   = new QLabel(label,this);
	m_pInput   = new QLineEdit(this);
	QString val;
	val += *valuePtr;
	m_pInput->setText(val);
	m_iDefInputWidth = defInputWidth;
	setMinimumSize(sizeHint());
	setEnabled(bEnabled);
}

KviCharSelector::~KviCharSelector()
{}
void KviCharSelector::applyButtonPressed()
{
	KviStr tmp = m_pInput->text();
	tmp.stripWhiteSpace();
	if(tmp.hasData()) *m_valuePtr = *tmp.ptr();
	else *m_valuePtr = '/';
}
void KviCharSelector::setNotEnabled(bool bNotEnabled)
{ setEnabled(!bNotEnabled); }
void KviCharSelector::setEnabled(bool bEnabled)
{
	m_pInput->setEnabled(bEnabled);
	m_pLabel->setEnabled(bEnabled);
}
QSize KviCharSelector::sizeHint()
{
	QSize sz=m_pLabel->sizeHint();
	sz.setWidth(sz.width()+m_iDefInputWidth); //at least 100 pixels for each input line
	sz.setHeight(m_pInput->sizeHint().height());
	return sz;
}

void KviCharSelector::resizeEvent(QResizeEvent *)
{
	m_pLabel->setGeometry(0,0,width()-(m_iDefInputWidth +1),height());
	m_pInput->setGeometry(width()-m_iDefInputWidth,0,m_iDefInputWidth,height());
}


//
// Bool value selector
//

KviBoolSelector::KviBoolSelector(QWidget *parent,QTabDialog * tab,bool *valuePtr,const char * label,bool bEnabled)
:QCheckBox(label,parent)
{
	setEnabled(bEnabled);
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	setChecked(*valuePtr);
	setMinimumSize(sizeHint());
}
KviBoolSelector::~KviBoolSelector()
{}
void KviBoolSelector::applyButtonPressed()
{ *m_valuePtr = isChecked(); }
void KviBoolSelector::setNotEnabled(bool bNotEnabled)
{ setEnabled(!bNotEnabled); }


//
// Numeric value selector
//

KviIntegerSelector::KviIntegerSelector(QWidget *parent,QTabDialog * tab,
		void * valuePtr,KviIntegerValueType valueType,const char * label,
		bool bEnabled,int min,int max,int iDefault,int defInputWidth)
:QWidget(parent)
{
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	m_pLabel   = new QLabel(label,this);
	m_pInput   = new QLineEdit(this);
	QIntValidator *v = new QIntValidator(this);
	v->setRange(min,max);
	m_pInput->setValidator(v);
	KviStr szNum;
	switch(valueType){
		case Int:    szNum.setNum(*((int *)valuePtr));   break;
		case UInt:   szNum.setNum(*((unsigned int *)valuePtr));  break;
		case UShort: szNum.setNum(*((unsigned short *)valuePtr)); break;
		default: szNum.setNum(*((short *)valuePtr)); break;
	}
	m_pInput->setText(szNum.ptr());
	m_min      = min;
	m_max      = max;
	m_valType  = valueType;
	m_iDefInputWidth = defInputWidth;
	m_szDefault.setNum(iDefault);
	setMinimumSize(sizeHint());
	setEnabled(bEnabled);
}

KviIntegerSelector::~KviIntegerSelector()
{}
void KviIntegerSelector::setEnabled(bool bEnabled)
{
	m_pInput->setEnabled(bEnabled);
	m_pLabel->setEnabled(bEnabled);
	QWidget::setEnabled(bEnabled);
}
void KviIntegerSelector::applyButtonPressed()
{
	if(m_pInput->text()){
		KviStr szT = m_pInput->text();
		bool bOk = false;
		//Adjust the value
		int value = szT.toInt(&bOk);
		if(!bOk)szT = m_szDefault.ptr();
		if(value < m_min)szT.setNum(m_min);
		if(value > m_max)szT.setNum(m_max);
		switch(m_valType){
			case Int:
				(*((int *)m_valuePtr)) = szT.toInt(&bOk);
				if(!bOk)(*((int *)m_valuePtr)) = m_szDefault.toInt(&bOk);
				break;
			case UInt:
				(*((unsigned int *)m_valuePtr)) = szT.toUInt(&bOk);
				if(!bOk)(*((unsigned int *)m_valuePtr)) = m_szDefault.toUInt(&bOk);
				break;
			case UShort:
				(*((unsigned short *)m_valuePtr)) = szT.toUShort(&bOk);
				if(!bOk)(*((unsigned short *)m_valuePtr)) = m_szDefault.toUShort(&bOk);
				break;
			default:
				(*((short *)m_valuePtr)) = szT.toShort(&bOk);
				if(!bOk)(*((short *)m_valuePtr)) = m_szDefault.toShort(&bOk);
				break;
		}
	}
}
void KviIntegerSelector::setNotEnabled(bool bNotEnabled)
{ setEnabled(!bNotEnabled); }

QSize KviIntegerSelector::sizeHint()
{
	QSize sz=m_pLabel->sizeHint();
	sz.setWidth(sz.width()+m_iDefInputWidth); //at least 100 pixels for each input line
	sz.setHeight(m_pInput->sizeHint().height());
	return sz;
}

void KviIntegerSelector::resizeEvent(QResizeEvent *)
{
	m_pLabel->setGeometry(0,0,width()-(m_iDefInputWidth +1),height());
	m_pInput->setGeometry(width()-m_iDefInputWidth,0,m_iDefInputWidth,height());
}


//
// KviColorSelector
//

KviColorSelector::KviColorSelector(QWidget *parent,QTabDialog * tab,QColor *valuePtr,
		const char * label,int defInputWidth)
:QWidget(parent)
{
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	m_pLabel   = new QLabel(label,this);
	m_pClrButton = new KviColorButton(this,*valuePtr);
	m_iDefInputWidth = defInputWidth;
	setMinimumSize(sizeHint());
}
KviColorSelector::~KviColorSelector()
{}
void KviColorSelector::applyButtonPressed()
{
	*m_valuePtr = m_pClrButton->getSelectedColor();
}
void KviColorSelector::setColor(const QColor &clr)
{
	m_pClrButton->setColor(clr);
}
void KviColorSelector::setEnabled(bool bEnabled)
{
	m_pClrButton->setEnabled(bEnabled);
	m_pLabel->setEnabled(bEnabled);
}

const QColor & KviColorSelector::getColor()
{
	return m_pClrButton->getSelectedColor();
}
QSize KviColorSelector::sizeHint()
{
	QSize sz=m_pLabel->sizeHint();
	sz.setWidth(sz.width()+m_iDefInputWidth);
	sz.setHeight(m_pLabel->sizeHint().height());
	if(sz.height() < 24)sz.setHeight(24);
	return sz;
}
void KviColorSelector::resizeEvent(QResizeEvent *)
{
	m_pLabel->setGeometry(0,0,width()-(m_iDefInputWidth +1),height());
	m_pClrButton->setGeometry(width()-m_iDefInputWidth,0,m_iDefInputWidth,height());
}

//
// KviFontSelector
//

KviFontSelector::KviFontSelector(QWidget *parent,QTabDialog * tab,QFont *valuePtr,
		const char * label,int defInputWidth)
:QWidget(parent)
{
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	m_pLabel   = new QLabel(label,this);
	QFont tmp(*valuePtr);
	m_pFntButton = new KviFontButton(this,tmp);
	m_iDefInputWidth = defInputWidth;
	setMinimumSize(sizeHint());
}
KviFontSelector::~KviFontSelector()
{}
void KviFontSelector::applyButtonPressed()
{
	*m_valuePtr = m_pFntButton->getSelectedFont();
}
QFont  KviFontSelector::getFont()
{
	return m_pFntButton->getSelectedFont();
}
QSize KviFontSelector::sizeHint()
{
	QSize sz=m_pLabel->sizeHint();
	sz.setWidth(sz.width()+m_iDefInputWidth);
	sz.setHeight(m_pLabel->sizeHint().height());
	if(sz.height() < 24)sz.setHeight(24);
	return sz;
}
void KviFontSelector::resizeEvent(QResizeEvent *)
{
	m_pLabel->setGeometry(0,0,width()-(m_iDefInputWidth +1),height());
	m_pFntButton->setGeometry(width()-m_iDefInputWidth,0,m_iDefInputWidth,height());
}


//
// KviFontSelector
//

KviImageSelector::KviImageSelector(QWidget *parent,QTabDialog * tab,KviStr *valuePtr,
		const char * label)
:QWidget(parent)
{
	connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	m_valuePtr = valuePtr;
	m_pCheckBox   = new QCheckBox(label,this);
	m_pCheckBox->setChecked(!valuePtr->isEmpty());
	m_pImgButton = new KviImageButton(this,valuePtr->ptr());
	connect(m_pCheckBox,SIGNAL(toggled(bool)),m_pImgButton,SLOT(setEnabled(bool)));
	m_pImgButton->setEnabled(!valuePtr->isEmpty());
	setMinimumSize(sizeHint());
}
KviImageSelector::~KviImageSelector()
{}
void KviImageSelector::applyButtonPressed()
{
	if(m_pCheckBox->isChecked()){
		const char *ptr = m_pImgButton->getSelectedImagePath();
		*m_valuePtr = ptr ? ptr : "";
	} else *m_valuePtr = ""; //no image
}
const QPixmap * KviImageSelector::getImage()
{
	if(m_pCheckBox->isChecked())return m_pImgButton->pixmap();
	else return 0;
}
QSize KviImageSelector::sizeHint()
{
	QSize sz=m_pCheckBox->sizeHint();
	sz.setHeight(sz.height() + 100);
	return sz;
}
void KviImageSelector::resizeEvent(QResizeEvent *)
{
	int hght = m_pCheckBox->sizeHint().height();
	m_pCheckBox->setGeometry(0,0,width(),hght);
	m_pImgButton->setGeometry(0,hght+1,width(),height() - (hght+1));
}



	

KviFileSelector::KviFileSelector(QWidget * parent,QTabDialog * dlg,KviStr *valuePtr,bool bEnabled)
:QWidget(parent)
{
	connect(dlg,SIGNAL(applyButtonPressed()),this,SLOT(applyButtonPressed()));
	QGridLayout * g= new QGridLayout(this,1,2,0,2);
//	m_pLabel = new QLabel(label,this);
//	g->addWidget(m_pLabel,0,0);
	m_valuePtr = valuePtr;
	m_pValueLabel = new QLabel(valuePtr->ptr(),this);
	m_pValueLabel->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	g->addWidget(m_pValueLabel,0,0);
	m_pButton = new QPushButton(__tr("Change..."),this);
	g->addWidget(m_pButton,0,1);
	connect(m_pButton,SIGNAL(clicked()),this,SLOT(changeFile()));
	setMinimumSize(sizeHint());
	setEnabled(bEnabled);
}

KviFileSelector::~KviFileSelector()
{
}

QSize KviFileSelector::sizeHint()
{
	QSize sz=m_pValueLabel->sizeHint();
	sz.setWidth(sz.width() + m_pButton->sizeHint().width() + 4);
	return sz;
}

void KviFileSelector::applyButtonPressed()
{
	*m_valuePtr = m_pValueLabel->text();
}

void KviFileSelector::changeFile()
{
	QString s = QFileDialog::getOpenFileName(m_pValueLabel->text(),QString::null,this);
	if(!s.isNull())
	{
		m_pValueLabel->setText(s);
	}
}

void KviFileSelector::setEnabled(bool bEnabled)
{
	m_pValueLabel->setEnabled(bEnabled);
	m_pButton->setEnabled(bEnabled);
	QWidget::setEnabled(bEnabled);
}

#include "m_kvi_selectors.moc"
