#ifndef _KVI_SELECTORS_H_INCLUDED_
#define _KVI_SELECTORS_H_INCLUDED_

//
//   File : kvi_selectors.h (/usr/build/NEW_kvirc/kvirc/kvilib/kvi_selectors.h)
//   Last major modification : Sat Jan 16 1999 14:56:21 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_string.h"

#include <qwidget.h>
#include <qtabdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qcolor.h>
#include <qfont.h>
#include <qpushbutton.h>

/**
* @short KviStr value changer for tab-dialogs
* @class KviStringSelector
*/

class KviStringSelector : public QWidget
{
	Q_OBJECT
public:
	KviStringSelector(QWidget *parent,QTabDialog *dlg,KviStr *valuePtr,
		const char * label,bool bEnabled = true,const char *szDefault = 0,int defInputWidth = 300);
	~KviStringSelector();
public slots:
	void setNotEnabled(bool bNotEnabled);
private:
	KviStr *m_valuePtr;
	QLabel  *m_pLabel;
	QLineEdit *m_pInput;
	KviStr m_szDefault;
	int     m_iDefInputWidth;
protected:
	virtual void resizeEvent(QResizeEvent *);
public:
	virtual QSize sizeHint();
public slots:
	virtual void setEnabled(bool bEnabled);
protected slots:
	void applyButtonPressed();
};

class KviCharSelector : public QWidget
{
	Q_OBJECT
public:
	KviCharSelector(QWidget *parent,QTabDialog *dlg,char *valuePtr,
		const char * label,bool bEnabled = true,int defInputWidth = 300);
	~KviCharSelector();
public slots:
	void setNotEnabled(bool bNotEnabled);
private:
	char *m_valuePtr;
	QLabel  *m_pLabel;
	QLineEdit *m_pInput;
	int     m_iDefInputWidth;
protected:
	virtual void resizeEvent(QResizeEvent *);
public:
	virtual QSize sizeHint();
public slots:
	virtual void setEnabled(bool bEnabled);
protected slots:
	void applyButtonPressed();
};


class KviIntegerSelector : public QWidget
{
	Q_OBJECT
public:
	enum KviIntegerValueType { Int,UInt,Short,UShort };
	KviIntegerSelector(QWidget *parent,QTabDialog *dlg,void *valuePtr,KviIntegerValueType valueType,
		const char *label,bool bEnabled = true,int min = 0,int max = 65535,int iDefault = 0,int defInputWidth = 300);
	~KviIntegerSelector();
public slots:
	void setNotEnabled(bool bNotEnabled);
public:
	void * m_valuePtr;
	QLabel *m_pLabel;
	QLineEdit *m_pInput;
	int m_min;
	int m_max;
	int m_iDefInputWidth;
	KviStr m_szDefault;
	KviIntegerValueType m_valType;
protected:
	virtual void resizeEvent(QResizeEvent *);
	virtual void setEnabled(bool bEnabled);
public:
	virtual QSize sizeHint();
protected slots:
	void applyButtonPressed();
};

/**
* @short bool value changer for tab-dialogs
* @class KviBoolSelector
*/

class KviBoolSelector : public QCheckBox
{
	Q_OBJECT
public:
	KviBoolSelector(QWidget *parent,QTabDialog *dlg,bool *valuePtr,const char *label,bool bEnabled = true);
	~KviBoolSelector();
public slots:
	void setNotEnabled(bool bNotEnabled);
private:
	bool *m_valuePtr;
protected slots:
	void applyButtonPressed();
};

class KviColorButton;

class KviColorSelector : public QWidget
{
	Q_OBJECT
public:
	KviColorSelector(QWidget *parent,QTabDialog *dlg,QColor *valuePtr,
		const char * label,int defInputWidth = 100);
	~KviColorSelector();
private:
	QColor *m_valuePtr;
	QLabel  *m_pLabel;
	KviColorButton *m_pClrButton;
	int     m_iDefInputWidth;
protected:
	virtual void resizeEvent(QResizeEvent *);
public:
	void setColor(const QColor &clr);
	const QColor &getColor();
	virtual QSize sizeHint();
	virtual void setEnabled(bool bEnabled);
protected slots:
	void applyButtonPressed();
};

class KviFontButton;

class KviFontSelector : public QWidget
{
	Q_OBJECT
public:
	KviFontSelector(QWidget *parent,QTabDialog *dlg,QFont *fnt,
		const char *label,int defInputWidth = 100);
	~KviFontSelector();
private:
	QFont *m_valuePtr;
	QLabel  *m_pLabel;
	KviFontButton *m_pFntButton;
	int     m_iDefInputWidth;
protected:
	virtual void resizeEvent(QResizeEvent *);
public:
	virtual QSize sizeHint();
	QFont getFont();
protected slots:
	void applyButtonPressed();
};

class KviImageButton;

class KviImageSelector : public QWidget
{
	Q_OBJECT
public:
	KviImageSelector(QWidget *parent,QTabDialog *dlg,KviStr *valuePtr,
		const char *label);
	~KviImageSelector();
private:
	KviStr *m_valuePtr;
	QCheckBox *m_pCheckBox;
	KviImageButton *m_pImgButton;
protected:
	virtual void resizeEvent(QResizeEvent *);
public:
	const QPixmap *getImage();
	virtual QSize sizeHint();
protected slots:
	void applyButtonPressed();
};

class KviFileSelector : public QWidget
{
	Q_OBJECT
public:
	KviFileSelector(QWidget * parent,QTabDialog * dlg,KviStr *valuePtr,bool bEnabled = true);
	~KviFileSelector();
private:
	KviStr * m_valuePtr;
	QLabel * m_pValueLabel;
	QPushButton * m_pButton;
public:
	virtual QSize sizeHint();
public slots:
	virtual void setEnabled(bool bEnabled);
protected slots:
	void applyButtonPressed();
	void changeFile();
};

#endif //!_KVI_SELECTORS_H_INCLUDED_
