//
//   File : kvi_app.cpp (/usr/build/NEW_kvirc/kvirc/kvirc/kvi_app.cpp)
//   Last major modification : Thu Dec 24 1998 04:05:26 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

//#define _KVI_DEBUG_CHECK_RANGE_
//#define _KVI_DEBUG_CLASS_NAME_ "KviApp"
#include "kvi_debug.h"

#include "kvi_app.h"
#include "kvi_locale.h" 
#include "kvi_frame.h"
#include "kvi_options.h"
#include "kvi_colorwin.h"
#include "kvi_window.h"

#include "kvi_mdi.h"
#include "kvi_imagelib.h"
#include "kvi_fileutils.h"
#include "kvi_malloc.h"
#include "kvi_msgbox.h"
#include "kvi_splash.h"
#include "kvi_varcache.h"
#include "kvi_helpqml.h"
#include "kvi_ircserver.h"

#include <qpopupmenu.h>

#include <stdlib.h> //for getenv

#include "kvi_dlguser.h"
#include "kvi_dlgserver.h"
#include "kvi_dlggui.h"
#include "kvi_dlgscript.h"
#include "kvi_dlgoptions.h"
#include "kvi_dlgregusers.h"
#include "kvi_dlgdcc.h"
#include "kvi_dlgloadscript.h"

#include "kvi_scriptwizard.h"

#include "kvi_tipotd.h"
#include "kvi_about.h"


#include "kvi_alias.h"
#include "kvi_event.h"

#include "kvi_helpwindow.h"
#include "kvi_dirbrowser.h"

#include "kvi_popupmenu.h"

#include "kvi_jpegio.h"

#include "kvi_thread.h"

#include "kvi_mime.h"
#include "kvi_process.h"

#include "kvi_console.h"
#include "kvi_utoolbar.h"

#include "kvi_uparser.h"
#include "kvi_config.h"
#include "kvi_wmdock.h"
#include "kvi_menubar.h"

#include "kvi_ircview.h"

#ifdef COMPILE_PLUGIN_SUPPORT
	#include "kvi_plugin.h"
#endif

#include <qtextstream.h>
#include <qdir.h>
#include <qtimer.h>
#include <qpalette.h>

#include <qmotifstyle.h>
#include <qplatinumstyle.h>
#include <qcdestyle.h>
#include <qwindowsstyle.h>
#include <qfileinfo.h>

#include <X11/Xlib.h>

//#include "kvi_style.h"

#define _KVI_APP_CPP_
// Global application pointer
// Auto declared as extern in kvi_app.h
KviApp *g_pApp;

KviThreadEventDispatcher *g_pThreadEventDispatcher;

//Other global stuff..
//Need to explicitly declare it as extern in other files
KviColorWindow *g_pColorWindow;
QPopupMenu     *g_pInputPopup;
QPopupMenu     *g_pTaskBarPopup;
QPopupMenu     *g_pViewSelectPopup;
QPopupMenu     *g_pWindowPopup;
QPopupMenu     *g_pBookmarksPopup;

//Various pixmaps...handled in loadPixmaps() and destroyPixmaps();
QPixmap * g_pWindowIcon[KVI_WND_NUM_ICONS];
QPixmap * g_pListBoxOpPixmap;
QPixmap * g_pListBoxVoicePixmap;
//Stuff used by KviToolBar
QPixmap *g_pReadyStatePixmap;
QPixmap *g_pBusyStatePixmap;
QPixmap *g_pConnectedStatePixmap;
QPixmap *g_pSysTrayBigIcons[4];
QPixmap *g_pFileIcons[KVI_FILE_ICONS_NUM_IMAGES];
QPixmap *g_pBrowserIcons[KVI_BROWSER_ICONS_NUM_IMAGES];
QPixmap *g_pHelpIcons[KVI_HELP_ICONS_NUM_IMAGES];
//Used by KviIrcView
QPixmap *g_pixViewOut[KVI_OUT_NUM_IMAGES];
//Used here and in KviOptions
KviSplashScreen *g_pSplashScreen;
//Created and destroyed here , loaded and saved by KviOptions
KviAliasManager *g_pAliasManager;
KviEventManager *g_pEventManager;
KviVariableCache *g_pVarCache;

KviHelpWindow *g_pHelpWindow;
KviDirBrowser *g_pDirBrowser;

KviPopupMenu *g_pChannelPopup;
KviPopupMenu *g_pUserlistPopup;
KviPopupMenu *g_pConsolePopup;
KviPopupMenu *g_pNotifylistPopup;
KviPopupMenu *g_pQueryPopup;
KviPopupMenu *g_pDccChatPopup;

KviUserToolBarTemplate *g_pUserToolBarTemplate;

KviConfig * g_pUserConfig;

#ifdef COMPILE_PLUGIN_SUPPORT
	KviPluginManager *g_pPluginManager;
#endif

Display * g_display = 0;

// IRC VIEW GLOBALS

GC g_ircviewGC;
QPixmap * g_pIrcViewMemBuffer;
HANDLE g_hIrcViewMemBuffer;
QList<KviIrcView> * g_pIrcViewWidgetList;

void g_setSplashScreenText(const char *text)
{
	if(g_pSplashScreen)g_pSplashScreen->setText(text);
}

void g_setSplashScreenProgress(int progress)
{
	if(g_pSplashScreen)g_pSplashScreen->setProgress(progress);
}

QStyle * g_getMotifStyle()
{
	return new QMotifStyle();
}

QStyle * g_getWindowsStyle()
{
	return new QWindowsStyle();
}

QStyle * g_getCDEStyle()
{
	return new QCDEStyle();
}

QStyle * g_getPlatinumStyle()
{
	return new QPlatinumStyle();
}

//============ KviApp ============//

KviApp::KviApp(int argc,char ** argv)
:KviAppBaseClass(argc,argv,"kvirc")
{
	m_startupTime = QDateTime::currentDateTime();
//	__enter("KviApp");

#ifdef COMPILE_JPEG_SUPPORT
	kvi_initJpegIo();
#endif

//#ifdef COMPILE_IGNORE_SIGALARM
//	kvi_threadIgnoreSigalarm();
//#endif
	kvi_threadInitialize();
	m_pFrameList = new QList<KviFrame>;
	m_pFrameList->setAutoDelete(false);

	m_pDestroyList = new QList<QObject>;
	m_pDestroyList->setAutoDelete(true);

	m_pStyleList = new QList<KviStyleEntry>;
	m_pStyleList->setAutoDelete(true);

	m_pConfigFile = 0;
	m_pCreateConfig = false;
//  Will be set by parseArgs in main() (before this->setup());
//	debug("argc: %d, argv: %s",argc, argv[1]);
//	if(argc > 1)
//		m_pConfigFile = argv[1];

	//Do not call anything else that could use the g_pApp pointer.
	//It is not set yet here!
	m_pUserOptionsDialog   = 0;
	m_pServerOptionsDialog = 0;
	m_pGuiOptionsDialog    = 0;
	m_pMiscOptionsDialog   = 0;
	m_pScriptDialog        = 0;
	m_pRegUsersDialog      = 0;
	m_pTipOfTheDayDialog   = 0;
	m_pAboutDialog		   = 0;
	m_pDccOptionsDialog    = 0;
	m_pScriptWizard        = 0;
	m_pWMDockWidget        = 0;
	g_pSplashScreen        = 0;
	g_pHelpWindow          = 0;
	g_pDirBrowser          = 0;
	m_bGuiUpdatePending    = false;
	m_savedAppPalette      = palette();
	m_szLastGUIStyle       = "Motif";

	g_pUserToolBarTemplate = 0;
//#warning "find a better solution for g_pUserToolBar    ?!"
	// Create is as soon as possible...
	g_pThreadEventDispatcher = new KviThreadEventDispatcher();
//	debug("Matching AAA!AAA@203.121.23.255 and *!*@*.net : %d",kvi_matchWildExpr("*!*@*.net","AAA!AAA@203.121.23.255"));

}

//============ ~KviApp ============//

KviApp::~KviApp()
{
//	__enter("~KviApp");

	// ### Save stuff
	saveKVircDirectory();
	// Kill the dialogs if needed
	if(m_pUserOptionsDialog)delete m_pUserOptionsDialog;
	if(m_pServerOptionsDialog)delete m_pServerOptionsDialog;
	if(m_pGuiOptionsDialog)delete m_pGuiOptionsDialog;
	if(m_pMiscOptionsDialog)delete m_pMiscOptionsDialog;
	if(m_pScriptDialog)delete m_pScriptDialog;
	if(m_pRegUsersDialog)delete m_pRegUsersDialog;
	if(m_pTipOfTheDayDialog)delete m_pTipOfTheDayDialog;
	if(m_pAboutDialog)delete m_pAboutDialog;
	if(m_pDccOptionsDialog)delete m_pDccOptionsDialog;
	if(m_pScriptWizard)delete m_pScriptWizard;
	if(m_pWMDockWidget)delete m_pWMDockWidget;
	if(g_pHelpWindow)delete g_pHelpWindow;
	if(g_pDirBrowser)delete g_pDirBrowser;
	//There should be no remaining windows now in the list...
	__range_valid(m_pFrameList->isEmpty());
	delete m_pFrameList;
	m_pFrameList = 0;
	// no more frames = no more widgets = no more paint requests
	XFreeGC(g_display,g_ircviewGC);
	delete g_pIrcViewMemBuffer;
	__range_valid(g_pIrcViewWidgetList->isEmpty());
	delete g_pIrcViewWidgetList;

	emptyDestroyList();
	delete m_pDestroyList;
	// Kill globals

	delete g_pOptions;
	g_pOptions = 0;
	delete g_pUserConfig;
	delete g_pVarCache;
	delete g_pAliasManager; // this must be killed AFTER g_pOptions
#ifdef COMPILE_PLUGIN_SUPPORT
	// this must be killed BEFORE g_pEventManager (will unload all the plugins and unregister handles)
	// must be also killed AFTER g_pOptions
	setStyle(new QMotifStyle()); //hehe...QApplication dies if the style set is in a plugin :)
	delete g_pPluginManager;
#endif
	delete g_pEventManager; // this must be killed AFTER g_pOptions
	delete g_pColorWindow;
	delete g_pInputPopup;
	delete g_pTaskBarPopup;
	delete g_pViewSelectPopup;
	delete g_pChannelPopup;
	delete g_pConsolePopup;
	delete g_pUserlistPopup;
	delete g_pNotifylistPopup;
	delete g_pQueryPopup;
	delete g_pDccChatPopup;
	delete g_pWindowPopup;
	delete g_pBookmarksPopup;
	delete g_pUserToolBarTemplate;
	destroyPixmaps();
	destroySplashScreen(); //make sure that it is destroyed
	// Delete is as late as possible...
	kvi_destroyLocale();
	delete g_pThreadEventDispatcher;
	unregisterStylesFor(0); // crash test :)
	delete m_pStyleList;

//	__leave("~KviApp");
}

void KviApp::registerStyle(const char *name,void * plugin_handle,QStyle * (*style_provider_routine)())
{
	KviStyleEntry * e = new KviStyleEntry;
	e->name = name;
	e->plugin_handle = plugin_handle;
	e->get_style_object = style_provider_routine;
	m_pStyleList->append(e);
	if(m_pGuiOptionsDialog){
		delete m_pGuiOptionsDialog; // need to refresh it
		m_pGuiOptionsDialog = 0;
	}
}

void KviApp::unregisterStyle(const char *name,void * plugin_handle)
{
	KviStyleEntry * e = 0;
	for(e=m_pStyleList->first();e;e=m_pStyleList->next()){
		if(kvi_strEqualCI(e->name.ptr(),name)){
			if(plugin_handle == e->plugin_handle){
				if(g_pOptions){ // options may be zero when closing down the app
					// not closing down :)
					if(kvi_strEqualCI(e->name.ptr(),g_pOptions->m_szApplicationGUIStyle.ptr())){
						// This is the current style....mmmmh
						// The object code may be not available anymore ...
						g_pOptions->m_szApplicationGUIStyle = "Motif";
						setApplicationStyle();
					}
				} // else g_pOptions already deleted :)
				m_pStyleList->removeRef(e);
				return;
			}
		}
	}
	if(m_pGuiOptionsDialog){
		delete m_pGuiOptionsDialog; // need to refresh it
		m_pGuiOptionsDialog = 0;
	}
}

void KviApp::unregisterStylesFor(void * plugin_handle)
{
	QList<KviStyleEntry> l;
	l.setAutoDelete(false);
	KviStyleEntry * e = 0;
	for(e=m_pStyleList->first();e;e=m_pStyleList->next()){
		if(plugin_handle == e->plugin_handle)l.append(e);
	}
	while(l.first()){
		unregisterStyle(l.first()->name.ptr(),plugin_handle);
		l.removeFirst();
	}
	if(m_pGuiOptionsDialog){
		delete m_pGuiOptionsDialog; // need to refresh it
		m_pGuiOptionsDialog = 0;
	}
}

void KviApp::slot_safeQuit()
{
	// Do a safe quit...
	// Each frame is self-deleted in the close event
	// and removes itself from m_pFrameList in the destructor.
	while(!m_pFrameList->isEmpty())m_pFrameList->first()->close();
	// When the last frame is closed , the app should exit automatically...
}

void KviApp::destroyLater(QObject * o)
{
	m_pDestroyList->append(o);
	QTimer::singleShot(100,this,SLOT(emptyDestroyList()));
}

void KviApp::emptyDestroyList()
{
//	__enter("emptyDestroyList");
	while(!m_pDestroyList->isEmpty())m_pDestroyList->removeFirst();
}

//============ setup ============//

void KviApp::setup()
{
	loadDirectories();

	createSplashScreen();
	g_setSplashScreenProgress(0);
	g_setSplashScreenText(__tr("Loading icons..."));

	g_display = g_pSplashScreen->x11Display();
	// IRCVIEW STUFF
	g_ircviewGC = XCreateGC(g_display,g_pSplashScreen->handle(),0,0);
	g_pIrcViewWidgetList = new QList<KviIrcView>;
	g_pIrcViewWidgetList->setAutoDelete(false);

	g_pIrcViewMemBuffer = new QPixmap(16,16); // just dummy 16x16 pixmap by now
	g_hIrcViewMemBuffer = g_pIrcViewMemBuffer->handle();

	loadPixmaps();                         //0-13%
	//Create globals

	g_setSplashScreenText(__tr("Preparing to parse config files..."));
	// The KviPopups MUST be created before KviOptions (and destroyed after)
	g_pChannelPopup        = new KviPopupMenu();
	g_pConsolePopup        = new KviPopupMenu();
	g_pUserlistPopup       = new KviPopupMenu();
	g_pNotifylistPopup     = new KviPopupMenu();
	g_pQueryPopup          = new KviPopupMenu();
	g_pDccChatPopup        = new KviPopupMenu();

	g_pUserToolBarTemplate = new KviUserToolBarTemplate();

	g_pVarCache=new KviVariableCache();
	openUserConfig();
	g_pAliasManager=new KviAliasManager(); // this MUST be created BEFORE the g_pOptions
	g_pEventManager=new KviEventManager(); // Same as the above

#ifdef COMPILE_PLUGIN_SUPPORT
	g_pPluginManager=new KviPluginManager(); // this MUST be created AFTER g_pEventManager
#endif

	g_pOptions=new KviOptions();           //14-83% //memory allocation eats really much time...
	// Set the fonts for the popups created before g_pOptions
	g_pChannelPopup->setFont(g_pOptions->m_fntApplication);
	g_pConsolePopup->setFont(g_pOptions->m_fntApplication);
	g_pUserlistPopup->setFont(g_pOptions->m_fntApplication);
	g_pNotifylistPopup->setFont(g_pOptions->m_fntApplication);
	g_pQueryPopup->setFont(g_pOptions->m_fntApplication);
	g_pDccChatPopup->setFont(g_pOptions->m_fntApplication);
	// Set the fonts for the rest of the widgets...
	setFont(g_pOptions->m_fntApplication); // Do not repaint widgets

	if(g_pOptions->m_bSetGlobalPaletteAtStartup){
		QColorGroup cg(
			g_pOptions->m_clrNormalGlobalForeground,
			g_pOptions->m_clrNormalGlobalButton,
			g_pOptions->m_clrNormalGlobalLight,
			g_pOptions->m_clrNormalGlobalDark,
			g_pOptions->m_clrNormalGlobalMid,
			g_pOptions->m_clrNormalGlobalText,
			g_pOptions->m_clrNormalGlobalText.light(150),
			g_pOptions->m_clrNormalGlobalBase,
			g_pOptions->m_clrNormalGlobalButton
		);
		QColorGroup cgdis(
			g_pOptions->m_clrDisabledGlobalForeground,
			g_pOptions->m_clrNormalGlobalButton,
			g_pOptions->m_clrNormalGlobalLight,
			g_pOptions->m_clrNormalGlobalDark,
			g_pOptions->m_clrNormalGlobalMid,
			g_pOptions->m_clrDisabledGlobalText,
			g_pOptions->m_clrNormalGlobalText.light(150),
			g_pOptions->m_clrDisabledGlobalBase,
			g_pOptions->m_clrNormalGlobalButton
		);
		QPalette pal(cg,cgdis,cg);	
		g_pChannelPopup->setPaletteRecursive(pal);
		g_pConsolePopup->setPaletteRecursive(pal);
		g_pUserlistPopup->setPaletteRecursive(pal);
		g_pNotifylistPopup->setPaletteRecursive(pal);
		g_pQueryPopup->setPaletteRecursive(pal);
		g_pDccChatPopup->setPaletteRecursive(pal);
		setPalette(pal); //Do not repaint widgets
	}

	g_pColorWindow=new KviColorWindow();   //84-88%
	g_pInputPopup=new QPopupMenu();
	g_pTaskBarPopup=new QPopupMenu();
	g_pViewSelectPopup=new QPopupMenu();
	g_pWindowPopup=new QPopupMenu();

	g_pBookmarksPopup=new QPopupMenu();
	connect(g_pBookmarksPopup,SIGNAL(activated(int)),this,SLOT(bookmarkSelected(int)));

	g_setSplashScreenProgress(88);
	g_setSplashScreenText(__tr("Booting the nuclear reactor..."));
	createQMLProvider();
	m_szLastHelpDocument = KVI_HELP_QML_MAIN_TOPIC_FILE;

	g_setSplashScreenProgress(90);

	KviFrame * frm = new KviFrame();       // 89 - 100 % Do not show yet...

	g_setSplashScreenProgress(100);
	g_setSplashScreenText(__tr("Go!"));

	destroySplashScreen();

	registerStyle("Motif",0,g_getMotifStyle);
	registerStyle("CDE",0,g_getCDEStyle);
	registerStyle("Platinum",0,g_getPlatinumStyle);
	registerStyle("Windows",0,g_getWindowsStyle);

	setApplicationStyle();
	// call it this way to ensure that the frame is inserted correctly
	__range_valid(frm == m_pFrameList->first());
	frm->show();

	if(g_pOptions->m_bShowTipOfTheDayAtStartup)slot_doTipOfTheDay();
}

void KviApp::connectAfterStartup(const char * server,const char * port)
{
	__range_valid(m_pFrameList->first()); // we must be called after setup()
	g_pOptions->m_pServerManager->iWantThisServerToBeCurrent(server ? server : "irc.openprojects.net",port ? port : "6667");
	// Jump out of this handler , call it from the main event loop...
	m_pFrameList->first()->doAsyncServerCommandCall(); //will run in exec() loop
}

void KviApp::setApplicationStyle()
{
	if(kvi_strEqualCI(m_szLastGUIStyle.ptr(),g_pOptions->m_szApplicationGUIStyle.ptr()))return;
	QStyle * s = 0;
	for(KviStyleEntry * e=m_pStyleList->first();e;e=m_pStyleList->next()){
		if(kvi_strEqualCI(g_pOptions->m_szApplicationGUIStyle.ptr(),e->name.ptr())){
			// got the style 
			s = e->get_style_object();
		}
	}
	if(!s){
		g_pOptions->m_szApplicationGUIStyle = "Motif";
		s = new QMotifStyle();
	}
	m_szLastGUIStyle = g_pOptions->m_szApplicationGUIStyle;
	setStyle(s);
}

//=========== createSplashScreen ============//

void KviApp::createSplashScreen()
{

	KviStr szImage;
	g_pSplashScreen = 0;
	if(findImage(szImage,KVI_SPLASH_IMAGE_NAME)){
		QPixmap pix(szImage.ptr());
		if(!pix.isNull()){
			g_pSplashScreen = new KviSplashScreen(pix);
		}
	}
	if(!g_pSplashScreen){
		debug("Could not load the splash screen image %s",szImage.ptr());
		g_pSplashScreen = new KviSplashScreen(); //no splash image?
	}
	g_pSplashScreen->show();
}

//=========== destroySplashScreen ============//

void KviApp::destroySplashScreen()
{
	if(g_pSplashScreen)delete g_pSplashScreen;
	g_pSplashScreen = 0;
}

//=============================================================================
//
// ### Load settings and create globals ###
//
//=============================================================================

void KviApp::createQMLProvider()
{
	KviHelpQMLProvider * p = new KviHelpQMLProvider();
	KviStr szProviderSearchPath;
	QStringList sl;
	getGlobalKVircDirectory(szProviderSearchPath,Help,0);
//	__debug_1arg("Setting default QML provider search path to %s",szProviderSearchPath.ptr());
	sl.append(QString(szProviderSearchPath.ptr()));
//	getLocalKVircDirectory(szProviderSearchPath,Help,0);
//	sl.append(QString(szProviderSearchPath.ptr()));
	p->setFilePath(sl);
//#warning "Check if this is deleted somewhere in Qt"
	QMimeSourceFactory::setDefaultFactory(p);

	KviHelpQMLStyleSheet * s = new KviHelpQMLStyleSheet(this);
	QStyleSheet::setDefaultSheet(s);
}

void KviApp::loadPixmaps()
{
	// This function should set progress from 0 to 21 %
	KviStr szP1;
	KviStr szP2;
	getLocalKVircDirectory(szP1,Pics,KVI_WIN_ICONS_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_WIN_ICONS_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib0(szP1.ptr(),szP2.ptr(),16,16);
	g_setSplashScreenProgress(1);
	if(!lib0.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_WIN_ICONS_IMAGE_LIBRARY_NAME);
	for(int i=0;i<KVI_WND_NUM_ICONS;i++)g_pWindowIcon[i]=new QPixmap(lib0.getImage(i));

	//Fist toolBar icon (three state pixmap)
	getLocalKVircDirectory(szP1,Pics,KVI_CONNECT_STATE_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_CONNECT_STATE_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib1(szP1.ptr(),szP2.ptr(),32,32);
	g_setSplashScreenProgress(2);
	if(!lib1.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_CONNECT_STATE_IMAGE_LIBRARY_NAME);
	g_pReadyStatePixmap = new QPixmap(lib1.getImage(0));
	g_pBusyStatePixmap  = new QPixmap(lib1.getImage(1));
	g_pConnectedStatePixmap = new QPixmap(lib1.getImage(2));

	//List box : two images
	getLocalKVircDirectory(szP1,Pics,KVI_LISTBOX_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_LISTBOX_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib2(szP1.ptr(),szP2.ptr(),16,16);
	g_setSplashScreenProgress(3);
	if(!lib2.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_LISTBOX_IMAGE_LIBRARY_NAME);
	g_pListBoxOpPixmap = new QPixmap(lib2.getImage(0));
	g_pListBoxVoicePixmap  = new QPixmap(lib2.getImage(1));

	//Small image (view icons) library...16x16
	getLocalKVircDirectory(szP1,Pics,KVI_OUT_ICONS_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_OUT_ICONS_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib3(szP1.ptr(),szP2.ptr(),16,16);
	g_setSplashScreenProgress(4);
	if(!lib3.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_OUT_ICONS_IMAGE_LIBRARY_NAME);
	for(int i=0;i<KVI_OUT_NUM_IMAGES;i++)g_pixViewOut[i]=new QPixmap(lib3.getImage(i));
	g_setSplashScreenProgress(5);

	// Systray icons
	getLocalKVircDirectory(szP1,Pics,KVI_SYSTRAY_ICONS_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_SYSTRAY_ICONS_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib4(szP1.ptr(),szP2.ptr(),32,32);
	g_setSplashScreenProgress(6);
	if(!lib4.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_SYSTRAY_ICONS_IMAGE_LIBRARY_NAME);
	for(int i=0;i<4;i++)g_pSysTrayBigIcons[i]=new QPixmap(lib4.getImage(i));
	g_setSplashScreenProgress(7);

	// File icons
	getLocalKVircDirectory(szP1,Pics,KVI_FILE_ICONS_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_FILE_ICONS_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib5(szP1.ptr(),szP2.ptr(),16,16);
	g_setSplashScreenProgress(8);
	if(!lib5.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_FILE_ICONS_IMAGE_LIBRARY_NAME);
	for(int i=0;i<KVI_FILE_ICONS_NUM_IMAGES;i++)g_pFileIcons[i]=new QPixmap(lib5.getImage(i));
	g_setSplashScreenProgress(9);

	// Browser icons
	getLocalKVircDirectory(szP1,Pics,KVI_BROWSER_ICONS_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_BROWSER_ICONS_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib6(szP1.ptr(),szP2.ptr(),32,32);
	g_setSplashScreenProgress(10);
	if(!lib6.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_BROWSER_ICONS_IMAGE_LIBRARY_NAME);
	for(int i=0;i<KVI_BROWSER_ICONS_NUM_IMAGES;i++)g_pBrowserIcons[i]=new QPixmap(lib6.getImage(i));
	g_setSplashScreenProgress(11);

	// Help icons
	getLocalKVircDirectory(szP1,Pics,KVI_HELP_ICONS_IMAGE_LIBRARY_NAME);
	getGlobalKVircDirectory(szP2,Pics,KVI_HELP_ICONS_IMAGE_LIBRARY_NAME);
	KviImageLibrary lib7(szP1.ptr(),szP2.ptr(),32,32);
	g_setSplashScreenProgress(12);
	if(!lib7.libraryLoaded())warningBox(__tr("The image library %s can not be found."),KVI_HELP_ICONS_IMAGE_LIBRARY_NAME);
	for(int i=0;i<KVI_HELP_ICONS_NUM_IMAGES;i++)g_pHelpIcons[i]=new QPixmap(lib7.getImage(i));
	g_setSplashScreenProgress(13);
}

//=============================================================================
//
// ### Save settings ###
//
//=============================================================================

void KviApp::saveKVircDirectory()
{
	// Here we save the local directory path
#ifdef COMPILE_WITH_KDE
	// In KDE we use the application config file
	if(!m_pConfigFile){	// not if user supplied a config file
		KConfig * cfg = getConfig();
		if(cfg && (getConfigState()!=APPCONFIG_NONE)&&
			(g_pApp->getConfigState()!=APPCONFIG_READONLY))
		{
			cfg->setGroup("LocalKVircDirectory");
			cfg->writeEntry("LocalKVircDirectory",m_szLocalKVircDir.ptr());
			cfg->sync();
			return;
		}
	}
#endif
	// In NON-KDE we use $HOME/.kvirc-VERSION.rc
	KviStr szF(QDir::homeDirPath());
	if(m_pConfigFile)
		if(*m_pConfigFile != '/') {//assume $HOME/ if not absolute
			szF+="/";
			szF+=m_pConfigFile;
		} else
			szF=m_pConfigFile;
	else {
		szF+="/";
		szF+=KVI_HOME_CONFIG_FILE_NAME;
	}

	QFile f(szF.ptr());
	if(f.open(IO_WriteOnly|IO_Truncate)){
		QTextStream t(&f);
		t << m_szLocalKVircDir.ptr();
		f.close();
		__debug_1arg("KVirc local directory saved to %s",szF.ptr());
	} else {
		warningBox(__tr("Failed writing the kvirc home directory path to %s.\n"\
			"You will need to do it manually , otherwise KVirc will restart with\n"\
			"the setup!"),szF.ptr());
	}
}

//=============================================================================
//
// ### Main frames handling ###
//
//=============================================================================

void KviApp::createNewFrame()
{
	//Creates a new toplevel window.
	KviFrame *frm=new KviFrame();
	frm->show();
}

void KviApp::insertNewFrame(KviFrame *frm)
{
	//KviFrame calls this in the constructor.
	m_pFrameList->append(frm);
}

void KviApp::removeFrame(KviFrame *frm)
{
	//KviFrame calls this in the destructor.
	m_pFrameList->removeRef(frm);
	//If the last window has been closed , die :)
	if(m_pFrameList->isEmpty())quit();
}

//=============================================================================
//
// ### Application exits : destroy global stuff ###
//
//=============================================================================

void KviApp::destroyPixmaps()
{
	for(int i=0;i<KVI_WND_NUM_ICONS;i++)delete g_pWindowIcon[i];
	for(int j=0;j<KVI_OUT_NUM_IMAGES;j++)delete g_pixViewOut[j];
	for(int k=0;k<4;k++)delete g_pSysTrayBigIcons[k];
	for(int l=0;l<KVI_FILE_ICONS_NUM_IMAGES;l++)delete g_pFileIcons[l];
	for(int m=0;m<KVI_BROWSER_ICONS_NUM_IMAGES;m++)delete g_pBrowserIcons[m];
	for(int n=0;n<KVI_HELP_ICONS_NUM_IMAGES;n++)delete g_pHelpIcons[n];
	delete g_pReadyStatePixmap;
	delete g_pBusyStatePixmap;
	delete g_pConnectedStatePixmap;
	delete g_pListBoxOpPixmap;
	delete g_pListBoxVoicePixmap;
}

//=============================================================================
//
// ### Global utilities ###
//
//=============================================================================

//=============== warningBox ===============//

void KviApp::warningBox(const char *fmt,...)
{
	//realloc 256 bytes
	char *buf=(char *)kvi_malloc(256);
	va_list list;
	va_start(list,fmt);
	//print...with max 256 chars
	int len=kvi_vsnprintf(buf,256,fmt,list);
	va_end(list);
	//check if we failed
	if(len < 0){
		//yes , failed....
		int ilen=256;
		do{ //we failed , so retry with 256 more chars
			ilen+=256;
			//realloc
			buf=(char *)kvi_realloc(buf,ilen);	
			//print...
			va_start(list,fmt);
			len=kvi_vsnprintf(buf,ilen,fmt,list);
			va_end(list);
		} while(len < 0);
	}
	//done...
	//now m_len is the length of the written string not including the terminator...
	//perfect! :)
	buf=(char *)kvi_realloc(buf,len+1);
	kvi_warningBox(__tr("KVirc : WARNING"),buf);
	kvi_free(buf);
}

//============ getGlobalKVircDirectory ================//

void KviApp::getGlobalKVircDirectory(KviStr &szData,KVircSubdir dir,const char *appendFile)
{
	szData=m_szGlobalKVircDir.ptr();
	switch(dir){
		case None          :                              ; break;
		case Pics          : szData+="/pics"              ; break;
		case Plugins       : szData+="/plugins"           ; break;
		case ConfigPlugins : szData+="/config/plugins"    ; break;
		case Help          :
			szData+="/help";
			{
				// Localized help
				KviStr tmp = kvi_getLocaleName();
				tmp.prepend('/');
				tmp.prepend(szData);
				if(kvi_directoryExists(tmp.ptr()))szData = tmp;
				else {
					tmp = szData; tmp.append("/en");
					if(kvi_directoryExists(tmp.ptr()))szData = tmp;
				}
			}
		break;
		case HelpEN     : szData+="/help/en"  ; break;
		case HelpNoIntl : szData+="/help"     ; break;
		case Log        : __debug("WARNING Global log directory requested!"); break;
		case Incoming   : __debug("WARNING Global incoming directory requested!"); break;
		case Trash      : __debug("WARNING Global trash directory requested!"); break;
		case Config     : szData+="/config"   ; break; //Read only...!!!
		case Audio      : szData+="/audio"    ; break;
		case Scripts    : szData+="/scripts"  ; break;
	}
	if(appendFile){
		szData.append('/');
		szData.append(appendFile);
	}
}

//============ getLocalKVircDirectory ================//

void KviApp::getLocalKVircDirectory(KviStr &szData,KVircSubdir dir,const char *appendFile,bool bCreateIfNeeded)
{
	szData=m_szLocalKVircDir.ptr();
	switch(dir){
		case None          :                           ; break;
		case Pics          : szData+="/pics"           ; break;
		case Config        : szData+="/config"         ; break;
		case ConfigPlugins : szData+="/config/plugins" ; break;
		case Log           : szData+="/log"            ; break;
		case Incoming      : szData+="/incoming"       ; break;
		case Help          : szData+="/help"           ; break;
		case HelpEN        : szData+="/help"           ; break;
		case HelpNoIntl    : szData+="/help"           ; break;
		case Audio         : szData+="/audio"          ; break;
		case Scripts       : szData+="/scripts"        ; break;
		case Plugins       : szData+="/plugins"        ; break;
		case Trash         : szData+="/trash"          ; break;
	}
	if(bCreateIfNeeded)kvi_makeDir(szData.ptr());
	if(appendFile){
		szData.append('/');
		szData.append(appendFile);
	}
}

bool KviApp::trashFile(const char *filename)
{
	QFileInfo fi(filename);
	if(!fi.exists())return false;

	KviStr lastPart = filename;
	int idx = lastPart.findLastIdx('/');
	if(idx != -1)lastPart.cutLeft(idx + 1);
	lastPart.append(".bak_");
	QDateTime tm(QDateTime::currentDateTime());
	lastPart.append(tm.toString());
	lastPart.replaceAll(' ',"_");

	KviStr trashFile;
	getLocalKVircDirectory(trashFile,Trash,lastPart.ptr(),true);

	fi.setFile(trashFile.ptr());
	while(fi.exists()){
		trashFile.append(".rnm");
		fi.setFile(trashFile.ptr());
	}
	QDir d;
	return d.rename(QString(filename),QString(trashFile.ptr()));
}


void KviApp::openUserConfig()
{
	KviStr tmp;
	getLocalKVircDirectory(tmp,Config,"kvi.userconfig.conf");
	g_pUserConfig = new KviConfig(tmp.ptr());
}

#ifdef COMPILE_PLUGIN_SUPPORT
	void KviApp::getPluginConfigFilePath(KviStr &buffer,const char *plugin_name)
	{
		KviStr fName(KviStr::Format,"kvi.%s.conf",plugin_name);
		getLocalKVircDirectory(buffer,ConfigPlugins,fName.ptr());
	}
#endif

//============= getChannelDumpLogFileName ==============//

void KviApp::getChannelDumpLogFileName(KviStr &str)
{
	unsigned int logN = 0;
	KviStr fPrefix="kick_";
	do{
		KviStr fName(KviStr::Format,"%s%u.log",fPrefix.ptr(),logN);
		getLocalKVircDirectory(str,Log,fName.ptr());
		logN++;
		if(logN > 9999){
			// Time to clear logs! :)
			fPrefix+="x";
			logN = 0;
		}
	} while(kvi_fileExists(str.ptr()));
}

void KviApp::getDefaultDccSaveFilePath(KviStr &path,const char *filename)
{
	KviMimeType * m = g_pOptions->m_pMimeManager->findMatch(filename,false);
	// the mime manager always returns something
	path = "";
	if(m->savePath.hasData()){
		// check the path validity
		if(*(m->savePath.ptr()) == '/'){
			// at least a valid dir...
			if(kvi_makeDir(m->savePath.ptr())){
				// Ok...the directory was made
				path = m->savePath;
			}
		}
	}
	if(path.isEmpty())getLocalKVircDirectory(path,Incoming,0,true);
	if(path.isEmpty())path = QDir::homeDirPath(); //last resource...
}

//============= findUserFile =================//

bool KviApp::findUserFile(KviStr &szRetPath,const char *filename)
{
	if(kvi_isAbsolutePath(filename)){ szRetPath=filename; return kvi_fileExists(filename); }
	KviStr szP;

	// Check in the mime save directory
	KviMimeType * m = g_pOptions->m_pMimeManager->findMatch(filename,false);

	if(m){
		if(m->savePath.hasData()){
			szP = m->savePath;
			if(*(szP.ptr() + szP.len()) != '/')szP.append('/');
			szP.append(filename);
			if(kvi_fileExists(szP.ptr())){
				szRetPath = szP.ptr();
				return true;
			}
		}
	}

	getLocalKVircDirectory(szP,Incoming,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	getLocalKVircDirectory(szP,Pics,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	getLocalKVircDirectory(szP,Audio,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	getLocalKVircDirectory(szP,Log,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	getGlobalKVircDirectory(szP,Pics,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	getGlobalKVircDirectory(szP,Audio,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	// Last resort ...
	szP = QDir::homeDirPath();
	if(*(szP.ptr() + szP.len()) != '/')szP.append('/');
	szP.append(filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath = szP.ptr(); return true; }

	szRetPath = filename;
	return false;
}

//============= findImage =================//

bool KviApp::findImage(KviStr &szRetPath,const char *filename)
{
	// Find an user file...
	if(kvi_isAbsolutePath(filename)){
		szRetPath=filename;
		return kvi_fileExists(filename);
	}

	KviStr szP;
	getLocalKVircDirectory(szP,Pics,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	getGlobalKVircDirectory(szP,Pics,filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath=szP.ptr(); return true; }
	// Last resort
	szP = QDir::homeDirPath();
	if(*(szP.ptr() + szP.len()) != '/')szP.append('/');
	szP.append(filename);
	if(kvi_fileExists(szP.ptr())){ szRetPath = szP.ptr(); return true; }
	return false;
}

bool KviApp::getReadOnlyConfigPath(KviStr &buffer,const char *config_name)
{
	// Take a look in the local directory....
	getLocalKVircDirectory(buffer,KviApp::Config,config_name);
	if(!kvi_fileExists(buffer.ptr())){
		// No saved config yet... check for defaults
		KviStr tmp;
		g_pApp->getGlobalKVircDirectory(tmp,KviApp::Config,config_name);
		if(!kvi_fileExists(tmp.ptr())){
			// No defaults...no such config file at all.
			return false;
		} else buffer = tmp; // get the defaults this time
	} // else file exists...
	return true;
}

void KviApp::raiseTopLevelWidget(QWidget * ptr)
{
	if(ptr->hasFocus())return;
	if(ptr->isActiveWindow())return;
	if(g_pOptions->m_bDialogShowHackForBrainDamagedWM)ptr->hide(); //enlightenment works fine with raise , but does not like hide()
	processEvents();
	ptr->show();
	processEvents();
	ptr->raise();
	processEvents();
	ptr->setActiveWindow();
	processEvents();
	ptr->setFocus();
}
/*
void KviApp::executeFile(const char *filePath)
{
	KviMimeType * m = g_pOptions->m_pMimeManager->findMatch(filePath);
	if(m){
		if(m->commandline.hasData()){
			executeFileWithCommand(m->commandline.ptr(),filePath);
		}
	}
}
*/
void KviApp::executeFileWithCommand(const char *commandline,const char *filePath,KviFrame * callerFrame)
{
/* FIXME:  "If filePath has spaces inside ?" */
	if(!callerFrame)callerFrame = m_pFrameList->first();
	__range_valid(callerFrame);
	KviStr cmd = commandline;
	cmd.stripWhiteSpace();
	cmd.replaceAll('%',filePath);
	if(kvi_strEqualCIN("INTERNAL ",cmd.ptr(),9)){
		// internal viewer
		cmd.cutLeft(9);
		cmd.stripWhiteSpace();
		callerFrame->m_pUserParser->parseCommand(cmd.ptr(),callerFrame->m_pConsole);
		return;
	}
	KviProcess proc;
	if(!proc.run(cmd.ptr())){
		g_pApp->warningBox(__tr("Could not execute command\n%s"),cmd.ptr());
	}
}

//=============================================================================
//
// ### Option dialogs ###
//
//=============================================================================

void KviApp::slot_doUserOptionsDialog()
{
	if(m_pUserOptionsDialog){
		raiseTopLevelWidget(m_pUserOptionsDialog);
//		delete m_pUserOptionsDialog;
//		m_pUserOptionsDialog = 0;
	} else {
		m_pUserOptionsDialog = new KviDlgOptUser();
		connect(m_pUserOptionsDialog,SIGNAL(finished(bool)),this,SLOT(slot_userOptionsDialogFinished(bool)));
		m_pUserOptionsDialog->show();
	}
}

void KviApp::slot_userOptionsDialogFinished(bool bAccepted)
{
//	__debug("Destroying user options dialog");
	__range_valid(m_pUserOptionsDialog);
	delete m_pUserOptionsDialog;
	m_pUserOptionsDialog = 0;
}

void KviApp::slot_doServerOptionsDialog()
{
	if(m_pServerOptionsDialog)raiseTopLevelWidget(m_pServerOptionsDialog);
	else {
		m_pServerOptionsDialog = new KviDlgOptServer();
		connect(m_pServerOptionsDialog,SIGNAL(finished(bool)),this,SLOT(slot_serverOptionsDialogFinished(bool)));
		m_pServerOptionsDialog->show();
	}
}

void KviApp::slot_serverOptionsDialogFinished(bool bAccepted)
{
//	__debug("Destroying server options dialog");
	__range_valid(m_pServerOptionsDialog);
	delete m_pServerOptionsDialog;
	m_pServerOptionsDialog = 0;
}

void KviApp::slot_doScriptDialog()
{
	if(m_pScriptDialog)raiseTopLevelWidget(m_pScriptDialog);
	else {
		m_pScriptDialog = new KviDlgScript();
		connect(m_pScriptDialog,SIGNAL(finished(bool)),this,SLOT(slot_scriptDialogFinished(bool)));
		m_pScriptDialog->show();
	}
}

void KviApp::slot_scriptDialogFinished(bool)
{
//	__debug("Destroying script dialog");
	__range_valid(m_pScriptDialog);
	delete m_pScriptDialog;
	m_pScriptDialog = 0;
	for(KviFrame *f = m_pFrameList->first();f;f= m_pFrameList->next()){
		f->updateUserToolBar();
	}
	g_pOptions->saveEvents();
	g_pOptions->savePopups();
	g_pOptions->saveAliases();
	g_pOptions->saveToolBar();
}

void KviApp::slot_toggleHelpWindow()
{
	if(g_pHelpWindow){
		raiseTopLevelWidget(g_pHelpWindow);
	} else {
		g_pHelpWindow = new KviHelpWindow();
		g_pHelpWindow->show();
		g_pHelpWindow->setFocus();
	}
}

void KviApp::requestHelpOn(const char *topic)
{
	slot_toggleHelpWindow();
	g_pHelpWindow->setDocument(topic);
}

void KviApp::requestHelpSearchOn(const char *topic)
{
	slot_toggleHelpWindow();
	g_pHelpWindow->doExactSearchForString(topic);
}

void KviApp::helpWindowCloseRequest()
{
	__range_valid(g_pHelpWindow);
	if(g_pHelpWindow){
		delete g_pHelpWindow;
		g_pHelpWindow = 0;
	}
}

void KviApp::slot_toggleDirBrowser()
{
	if(g_pDirBrowser){
		raiseTopLevelWidget(g_pDirBrowser);
//		delete g_pDirBrowser;
//		g_pDirBrowser = 0;
	} else {
		g_pDirBrowser = new KviDirBrowser();
		g_pDirBrowser->show();
		g_pDirBrowser->setFocus();
	}
}

void KviApp::dirBrowserCloseRequest()
{
	__range_valid(g_pDirBrowser);
	if(g_pDirBrowser){
		delete g_pDirBrowser;
		g_pDirBrowser = 0;
	}
}

void KviApp::appendCurrentBookmarksToBookmarksMenu()
{
	for(KviStr * s = g_pOptions->m_pDirBookmarks->first();s;s = g_pOptions->m_pDirBookmarks->next()){
		g_pBookmarksPopup->insertItem(QString(s->ptr()));
	}
}

void KviApp::bookmarkSelected(int id)
{
	KviStr tmp=g_pBookmarksPopup->text(id);
	if(!g_pDirBrowser)slot_toggleDirBrowser();

	if(kvi_strEqualCSN("/",tmp.ptr(),1)){
		// A path item
		g_pDirBrowser->openDirectory(tmp.ptr());
	} else {
		// Non path item , "Add to bookmarks"
		// Add to bookmarks the current path
		// Avoid double entries
		g_pDirBrowser->getCurrentPath(tmp);
		for(KviStr * s = g_pOptions->m_pDirBookmarks->first();s;s = g_pOptions->m_pDirBookmarks->next()){
			if(kvi_strEqualCS(s->ptr(),tmp.ptr()))return;
		}
		if(g_pOptions->m_pDirBookmarks->count()  == KVI_DIRBROWSER_MAX_BOOKMARKS){
			g_pOptions->m_pDirBookmarks->removeFirst();
		}
		g_pOptions->m_pDirBookmarks->append(new KviStr(tmp));
	}
}

void KviApp::slot_doMiscOptionsDialog()
{
	if(m_pMiscOptionsDialog){
		raiseTopLevelWidget(m_pMiscOptionsDialog);
//		delete m_pMiscOptionsDialog;
//		m_pMiscOptionsDialog = 0;
	} else {
		m_pMiscOptionsDialog = new KviDlgOptMisc();
		connect(m_pMiscOptionsDialog,SIGNAL(finished(bool)),this,SLOT(slot_miscOptionsDialogFinished(bool)));
		m_pMiscOptionsDialog->show();
	}
}

void KviApp::slot_miscOptionsDialogFinished(bool bAccepted)
{
	delete m_pMiscOptionsDialog;
	m_pMiscOptionsDialog = 0;
	if(bAccepted){
		for(KviFrame *f = m_pFrameList->first();f;f= m_pFrameList->next()){
			f->updateCaption();
//#warning "Do not update ALL the options here...just the iViewMaxBufferSize"
			f->applyOptions();
//			f->applyToolbarOptions();
		}
		g_pOptions->fillHighlightWordsList();
	}
}

void KviApp::slot_doGuiOptionsDialog()
{
	if(m_pGuiOptionsDialog){
		raiseTopLevelWidget(m_pGuiOptionsDialog);
//		delete m_pGuiOptionsDialog;
//		m_pGuiOptionsDialog = 0;
	} else {
		m_pGuiOptionsDialog = new KviDlgOptGui();
		connect(m_pGuiOptionsDialog,SIGNAL(finished(bool)),this,SLOT(slot_guiOptionsDialogFinished(bool)));
		m_pGuiOptionsDialog->show();
	}
}


void KviApp::slot_guiOptionsDialogFinished(bool bAccepted)
{
	__range_valid(m_pGuiOptionsDialog);
	if(bAccepted){
		KviCachedGuiPixmaps p;
		m_pGuiOptionsDialog->getCachedGuiPixmaps(&p);
		updateGuiWithCache(&p);
	}
	delete m_pGuiOptionsDialog;
	m_pGuiOptionsDialog = 0;
}

void KviApp::triggerUpdateGui()
{
	// Avoid multiple updates...
	if(!m_bGuiUpdatePending){
		m_bGuiUpdatePending = true;
		QTimer::singleShot(50,this,SLOT(updateGui()));
	}
}

//extern void qt_set_x11_resources( const char*, const char*,const char*, const char*);

void KviApp::updateGui()
{
	updateGuiWithCache(0);
}

void KviApp::updateGuiWithCache(struct KviCachedGuiPixmaps *p)
{
	m_bGuiUpdatePending = false;
	bool bWasHelpWindow = false;
	if(g_pHelpWindow){
		bWasHelpWindow = true;
		delete g_pHelpWindow;
	}

	setApplicationStyle();

	if(!(font() == g_pOptions->m_fntApplication)){
		setFont(g_pOptions->m_fntApplication,true);
	}

	if(g_pOptions->m_bSetGlobalPaletteAtStartup){
		QColorGroup cg(
			g_pOptions->m_clrNormalGlobalForeground,
			g_pOptions->m_clrNormalGlobalButton,
			g_pOptions->m_clrNormalGlobalLight,
			g_pOptions->m_clrNormalGlobalDark,
			g_pOptions->m_clrNormalGlobalMid,
			g_pOptions->m_clrNormalGlobalText,
			g_pOptions->m_clrNormalGlobalText.light(150),
			g_pOptions->m_clrNormalGlobalBase,
			g_pOptions->m_clrNormalGlobalButton
		);
		QColorGroup cgdis(
			g_pOptions->m_clrDisabledGlobalForeground,
			g_pOptions->m_clrNormalGlobalButton,
			g_pOptions->m_clrNormalGlobalLight,
			g_pOptions->m_clrNormalGlobalDark,
			g_pOptions->m_clrNormalGlobalMid,
			g_pOptions->m_clrDisabledGlobalText,
			g_pOptions->m_clrNormalGlobalText.light(150),
			g_pOptions->m_clrDisabledGlobalBase,
			g_pOptions->m_clrNormalGlobalButton
		);
		QPalette pal(cg,cgdis,cg);	
		setPalette(pal,true); //repaint widgets this time
	} else {
		if(!(palette() == m_savedAppPalette))setPalette(m_savedAppPalette,true);
	}

	g_pOptions->calculateInputFontVariables();
	g_pOptions->calculateListBoxFontVariables();

	g_pOptions->loadListBoxBackgroundImage(p ? p->pListBoxPix : 0);
	g_pOptions->loadInputBackgroundImage(p ? p->pInputPix : 0);
	g_pOptions->loadOutputBackgroundImage(p ? p->pViewPix : 0);
	g_pOptions->loadMdiBackgroundImage(p ? p->pMdiPix : 0);
	g_pOptions->loadLabelsBackgroundImage(p ? p->pLabelsPix : 0);
	// Update the help window style
	KviHelpQMLStyleSheet * s = new KviHelpQMLStyleSheet(this);
	QStyleSheet::setDefaultSheet(s);

	if(bWasHelpWindow){
		g_pHelpWindow = new KviHelpWindow();
		g_pHelpWindow->show();
		g_pHelpWindow->setFocus();
	}
	//Force update to all windows
	for(KviFrame *f = m_pFrameList->first();f;f= m_pFrameList->next()){
		f->applyOptions();
		f->applyToolbarOptions();
	}
}

void KviApp::slot_doRegUsersDialog()
{
	if(m_pRegUsersDialog)raiseTopLevelWidget(m_pRegUsersDialog);
	else {
		m_pRegUsersDialog = new KviDlgRegUsers();
		connect(m_pRegUsersDialog,SIGNAL(finished(bool)),this,SLOT(slot_regUsersDialogFinished(bool)));
		m_pRegUsersDialog->show();
	}
}

void KviApp::slot_regUsersDialogFinished(bool bAccepted)
{
//	__debug("Destroying reg users dialog");
	__range_valid(m_pRegUsersDialog);
	delete m_pRegUsersDialog;
	m_pRegUsersDialog = 0;
	restartNotifyLists();
}

void KviApp::slot_doScriptWizard()
{
	if(m_pScriptWizard)raiseTopLevelWidget(m_pScriptWizard);
	else {
		m_pScriptWizard = new KviScriptWizard();
		connect(m_pScriptWizard,SIGNAL(finished(bool)),this,SLOT(slot_scriptWizardFinished(bool)));
		m_pScriptWizard->show();
	}
}

void KviApp::slot_scriptWizardFinished(bool bAccepted)
{
	__range_valid(m_pScriptWizard);
	KviSaveScriptStruct s;
	s.pWnd = m_pFrameList->first()->activeWindow();
	if(bAccepted)m_pScriptWizard->fillSaveScriptStruct(&s);
	delete m_pScriptWizard;
	m_pScriptWizard = 0;
	if(bAccepted){
		if(saveScript(&s))s.pWnd->output(KVI_OUT_INTERNAL,__tr("Script successfully saved"));
		else s.pWnd->output(KVI_OUT_ERROR,__tr("Failed to save the script"));
		if(s.pExtraFilesList)delete s.pExtraFilesList;
	}
}


void KviApp::slot_doDccOptionsDialog()
{
	if(m_pDccOptionsDialog){
		raiseTopLevelWidget(m_pDccOptionsDialog);
	} else {
		m_pDccOptionsDialog = new KviDlgOptDcc();
		connect(m_pDccOptionsDialog,SIGNAL(finished(bool)),this,SLOT(slot_dccOptionsDialogFinished(bool)));
		m_pDccOptionsDialog->show();
	}
}

void KviApp::slot_dccOptionsDialogFinished(bool bAccepted)
{
//	__debug("Destroying dcc options dialog");
	__range_valid(m_pDccOptionsDialog);
	delete m_pDccOptionsDialog;
	m_pDccOptionsDialog = 0;
}

void KviApp::restartNotifyLists()
{
	for(KviFrame *f = m_pFrameList->first();f;f= m_pFrameList->next()){
		f->updateNotifyOrWatchList();
	}
}

void KviApp::slot_doTipOfTheDay()
{
	if(!m_pTipOfTheDayDialog)
	{
		m_pTipOfTheDayDialog = new KviTipOfTheDay();
		m_pTipOfTheDayDialog->show();
		connect(m_pTipOfTheDayDialog,SIGNAL(closed()),this,SLOT(slot_TipOfTheDayClosed()));
	} else {
		m_pTipOfTheDayDialog->raise();
		m_pTipOfTheDayDialog->setFocus();
		m_pTipOfTheDayDialog->hide();
		m_pTipOfTheDayDialog->show();
	}
}

void KviApp::slot_TipOfTheDayClosed()
{
//	__debug("Destroying Tip of the Day Dialog");
	delete m_pTipOfTheDayDialog;
	m_pTipOfTheDayDialog = 0;
}

void KviApp::slot_doAboutDialog()
{
	if(!m_pAboutDialog)
	{
		m_pAboutDialog = new KviDlgAbout();
		m_pAboutDialog->show();
		m_pAboutDialog->setFixedSize(m_pAboutDialog->size());
		connect(m_pAboutDialog,SIGNAL(closed()),this,SLOT(slot_AboutDialogClosed()));
	} else {
		m_pAboutDialog->raise();
		m_pAboutDialog->setFocus();
//		m_pTipOfTheDayDialog->hide();
//		m_pTipOfTheDayDialog->show();
	}
}
void KviApp::slot_AboutDialogClosed()
{
//	__debug("Destroying About Dialog");
	delete m_pAboutDialog;
	m_pAboutDialog = 0;
}

void KviApp::slot_doLoadScriptDialog()
{
	if(!m_pLoadScriptDialog){
		m_pLoadScriptDialog = new KviDlgLoadScript();
		m_pLoadScriptDialog->show();
		connect(m_pLoadScriptDialog,SIGNAL(finished(bool)),this,SLOT(slot_loadScriptDialogFinished(bool)));
	} else {
		raiseTopLevelWidget(m_pLoadScriptDialog);
	}
}

void KviApp::slot_loadScriptDialogFinished(bool bAccepted)
{
	if(bAccepted){
		KviFrame * f = m_pFrameList->first();
		KviStr scriptName;
		m_pLoadScriptDialog->getSelectedScriptFileName(scriptName);
		if(scriptName.hasData()){
			KviStr tmp(KviStr::Format,"PARSE %s",scriptName.ptr());
			f->m_pUserParser->parseCommand(tmp.ptr(),f->m_pConsole);
		}
	}
	delete m_pLoadScriptDialog;
	m_pLoadScriptDialog = 0;
}

///////////////////////////////////////////////////////

void KviApp::slot_saveSettings()
{
	g_pOptions->save();
}

///////////////////////////////////////////////////////

void KviApp::adjustPopupMenusForWMDockWidget()
{
	for(KviFrame * f = m_pFrameList->first();f;f = m_pFrameList->next()){
		f->m_pMenuBar->wmDockWidgetStateChanged(m_pWMDockWidget != 0);
	}
}

void KviApp::wmDockWidgetDestroyed()
{
	if(m_pWMDockWidget){
		m_pWMDockWidget = 0;
		showAllFrames();
		adjustPopupMenusForWMDockWidget();
	}
}

void KviApp::slot_createWMDockWidget()
{
	if(!m_pWMDockWidget)m_pWMDockWidget = new KviWMDockWidget();
	adjustPopupMenusForWMDockWidget();
}

void KviApp::slot_destroyWMDockWidget()
{
	if(m_pWMDockWidget)delete m_pWMDockWidget;
	m_pWMDockWidget = 0;
	adjustPopupMenusForWMDockWidget();
}

void KviApp::globalHighlight(bool bAltColor)
{
	if(m_pWMDockWidget){
		if(!m_pWMDockWidget->framesVisible()){
			m_pWMDockWidget->highlight(bAltColor);
		}
	}
}

void KviApp::hideAllFrames()
{
	for(KviFrame * f=m_pFrameList->first();f;f=m_pFrameList->next())f->hide();
	if(m_pWMDockWidget)m_pWMDockWidget->setFramesVisible(false);
}

void KviApp::showAllFrames()
{
	// This does not work with enlightenment :)
	for(KviFrame * f=m_pFrameList->first();f;f=m_pFrameList->next()){
		if(g_pOptions->m_bDialogShowHackForBrainDamagedWM){
			f->hide(); //enlightenment works fine with raise , but does not like hide()
			processEvents();
		}
		f->showNormal();
		f->show();
		f->raise();
	}
	if(m_pWMDockWidget)m_pWMDockWidget->setFramesVisible(true);
}

#ifdef COMPILE_WITH_KDE
	KviAppBaseClass::KviAppBaseClass(int argc,char ** argv,const char *appname)
	:KApplication(argc,argv,appname){}
	#include "m_kvi_appbase_kde.moc"
#else
	KviAppBaseClass::KviAppBaseClass(int argc,char ** argv,const char *)
	:QApplication(argc,argv){}
	#include "m_kvi_appbase_qt.moc"
#endif

KviAppBaseClass::~KviAppBaseClass(){}

#include "m_kvi_app.moc"
