//
//   File : kvi_chaneditors.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_chaneditors.cpp)
//   Last major modification : Sat May 1 1999 01:42:53 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#include "kvi_chaneditors.h"
#include "kvi_channel.h"
#include "kvi_locale.h"
#include "kvi_app.h"

#include <qlayout.h>

KviTopicEditor::KviTopicEditor(QWidget * parent)
:QLineEdit(parent)
{
	setFocusPolicy(QWidget::ClickFocus);
}
KviTopicEditor::~KviTopicEditor()
{
}
void KviTopicEditor::focusOutEvent(QFocusEvent *)
{
	hide();
}
void KviTopicEditor::connectMe(QObject * receiver,const char *slot)
{
	disconnect(this,SIGNAL(returnPressed()),0,0);
	connect(this,SIGNAL(returnPressed()),receiver,slot);
}

//imnpst
KviModeDescriptionEntry KviModeEditor::modeTable[6]=
{
	{ 'i' , _i19n_("i - Invite only")      },
	{ 'm' , _i19n_("m - Moderated")        },
	{ 'n' , _i19n_("n - No external msgs") },
	{ 'p' , _i19n_("p - Private")          },
	{ 's' , _i19n_("s - Secret")           },
	{ 't' , _i19n_("t - Topic limited")    }
};

KviModeEditor::KviModeEditor(QWidget * parent)
:QFrame(parent)
{
	setFrameStyle(QFrame::WinPanel|QFrame::Raised);
	setFocusPolicy(QWidget::ClickFocus);
	QGridLayout * g = new QGridLayout(this,7,1,2,0);
	for(int i=0;i<6;i++){
		m_pCheck[i] = new QCheckBox(_i18n_translateNoText(modeTable[i].description),this);
		g->addWidget(m_pCheck[i],i,0);
		m_pCheck[i]->setFocusPolicy(QWidget::ClickFocus);
		m_pCheck[i]->setFocusProxy(this);
	}
	m_pCommit = new QPushButton(__tr("Commit"),this);
	g->addWidget(m_pCommit,6,0);
	m_pCommit->setFocusPolicy(QWidget::ClickFocus);
	m_pCommit->setFocusProxy(this);
	connect(m_pCommit,SIGNAL(clicked()),this,SLOT(commitChanges()));
}

KviModeEditor::~KviModeEditor()
{
}
void KviModeEditor::focusOutEvent(QFocusEvent *)
{
	hide();
}
void KviModeEditor::commitChanges()
{
	hide();
	KviStr minus = "-";
	KviStr plus  = "+";
	for(int i=0;i<6;i++){
		if(m_bInitState[i]){ //It was set at the beginning
			if(!m_pCheck[i]->isChecked()){ //and was unset
				minus += modeTable[i].flag;
			}
		} else { //It was NOT set at the beginning
			if(m_pCheck[i]->isChecked()){ //and now is set
				plus += modeTable[i].flag;
			}
		}
	}
	KviStr tmp;
	if(minus.len() > 1)tmp+=minus;
	if(plus.len() > 1)tmp+=plus;
	emit modeChanged(tmp.ptr());
}

void KviModeEditor::connectMe(QObject * receiver,const char *slot)
{
	disconnect(this,SIGNAL(modeChanged(const char *)),0,0);
	connect(this,SIGNAL(modeChanged(const char *)),receiver,slot);
}
void KviModeEditor::setMode(KviStr &flags)
{
	for(int i=0;i<6;i++){
		m_bInitState[i] = flags.findFirstIdx(modeTable[i].flag) != -1;
		m_pCheck[i]->setChecked(m_bInitState[i]);
	}
}
void KviModeEditor::doEdit(KviStr &flags)
{
	setMode(flags);
	show();
	setFocus();
}


KviMaskEditor::KviMaskEditor(QWidget * parent)
:QFrame(parent)
{
	setFrameStyle(QFrame::WinPanel|QFrame::Raised);
	setFocusPolicy(QWidget::ClickFocus);

	QGridLayout *g = new QGridLayout(this,5,1,2,0);

	m_pMaskBox = new QListView(this);
	m_pMaskBox->setFocusPolicy(QWidget::ClickFocus);
	m_pMaskBox->setFocusProxy(this);
	m_pMaskBox->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	m_pMaskBox->addColumn(QString(__tr("Active")));
	m_pMaskBox->addColumn(QString(__tr("Set by")));
	m_pMaskBox->addColumn(QString(__tr("Set at")));
	m_pMaskBox->setMultiSelection(true);
	m_pMaskBox->setAllColumnsShowFocus(true);
	g->addWidget(m_pMaskBox,0,0);

	m_pRemoveMask  = new QPushButton(__tr("Remove"),this);
	m_pRemoveMask->setFocusPolicy(QWidget::ClickFocus);
	m_pRemoveMask->setFocusProxy(this);
	connect(m_pRemoveMask,SIGNAL(clicked()),this,SLOT(removeMaskClicked()));
	g->addWidget(m_pRemoveMask,1,0);

//	m_pExcBox = new QListView(this);
//	m_pExcBox->setFocusPolicy(QWidget::ClickFocus);
//	m_pExcBox->setFocusProxy(this);
//	m_pExcBox->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
//	m_pExcBox->addColumn(QString(__tr("Ban exception")));
//	m_pExcBox->addColumn(QString(__tr("Set by")));
//	m_pExcBox->addColumn(QString(__tr("Set at")));
//	m_pExcBox->setMultiSelection(true);
//	m_pExcBox->setAllColumnsShowFocus(true);
//	g->addWidget(m_pExcBox,2,0);

//	m_pUnexc  = new QPushButton(__tr("Remove"),this);;
//	m_pUnexc->setFocusPolicy(QWidget::ClickFocus);
//	m_pUnexc->setFocusProxy(this);
//	connect(m_pUnexc,SIGNAL(clicked()),this,SLOT(unexcClicked()));
//	g->addWidget(m_pUnexc,3,0);

	m_pCommit = new QPushButton(__tr("Commit"),this);
	m_pCommit->setFocusPolicy(QWidget::ClickFocus);
	m_pCommit->setFocusProxy(this);
	connect(m_pCommit,SIGNAL(clicked()),this,SLOT(commitChanges()));
	g->addWidget(m_pCommit,4,0);

	hide();
}

KviMaskEditor::~KviMaskEditor()
{
}

void KviMaskEditor::focusOutEvent(QFocusEvent *)
{
//	debug("FOCUS OUT EVENT");
	hide();
}

//void KviMaskEditor::focusInEvent(QFocusEvent *)
//{
//	debug("FOCUS IN EVENT");
//	emit gotFocus(); //horrible workaround
//}

void KviMaskEditor::connectMe(QObject * receiver,const char *slot)
{
	disconnect(this,SIGNAL(masksChanged(const char *)),0,0);
	connect(this,SIGNAL(masksChanged(const char *)),receiver,slot);
}

void KviMaskEditor::doEdit(QList<KviBanMask> * maskList,const char *masksLabel,bool canCommit)
{
	setMasks(maskList,masksLabel);
	m_pCommit->setEnabled(canCommit);
	m_pRemoveMask->setEnabled(canCommit && (m_pMaskBox->childCount() > 0));
//	m_pUnexc->setEnabled(canCommit && (m_pExcBox->childCount() > 0));
	m_szMasksRemoved = "";
//	m_szExcsRemoved = "";
//	debug("################ SHOWING ##############");
	show();
//	debug("################# SHOWN ###############");
	setFocus();
//	debug("################ setFocus #############");
//	g_pApp->processEvents();
//	if(hasFocus())debug("AND I HAVE IT NOW"}
}
void KviMaskEditor::setMasks(QList<KviBanMask> * maskList,const char *masksLabel)
{
	m_pMaskBox->clear();
	m_pMaskBox->setColumnText(0,QString(masksLabel));
	QListViewItem *i = 0;
	for(KviBanMask * b=maskList->first();b;b=maskList->next()){
		i = new QListViewItem(m_pMaskBox,QString(b->mask.ptr()),QString(b->setBy.ptr()),QString(b->setAt.ptr()));
	}
//	m_pExcBox->clear();
//	for(KviBanMask * e=excList->first();e;e=excList->next()){
//		QListViewItem *i = new QListViewItem(m_pExcBox,QString(e->mask.ptr()),QString(e->setBy.ptr()),QString(e->setAt.ptr()));
//	}
//#warning "Do a trick to hide the Exception box and button if there are no exceptions (or are not supported)"
//#warning "Why not use splitters ???????"
}

void KviMaskEditor::removeMaskClicked()
{
	QListViewItem * it = m_pMaskBox->firstChild();
	while(it){
		if(it->isSelected()){
			if(m_szMasksRemoved.hasData())m_szMasksRemoved += " ";
			m_szMasksRemoved+=it->text(0);
			QListViewItem * tmp = it;
			it = it->nextSibling();
			delete tmp;
		} else it = it->nextSibling();
	}
	m_pRemoveMask->setEnabled(m_pMaskBox->childCount() > 0);
}

/*void KviMaskEditor::unexcClicked()
{
	QListViewItem * it = m_pExcBox->firstChild();
	while(it){
		if(it->isSelected()){
			if(m_szExcsRemoved.hasData())m_szExcsRemoved += " ";
			m_szExcsRemoved+=it->text(0);
			QListViewItem * tmp = it;
			it = it->nextSibling();
			delete tmp;
		} else it = it->nextSibling();
	}
	m_pUnexc->setEnabled(m_pExcBox->childCount() > 0);
}*/

void KviMaskEditor::commitChanges()
{
	hide();
	if(m_szMasksRemoved.hasData()){
		emit masksChanged(m_szMasksRemoved.ptr());
	} // else simply abort....
}

#include "m_kvi_chaneditors.moc"
