//
//   File : kvi_dirbrowser.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_dirbrowser.cpp)
//   Last major modification : Sun Apr 25 1999 13:58:06 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CHECK_RANGE_
#define _KVI_DEBUG_CLASS_NAME_ "KviDirStuff"
#include "kvi_debug.h"
#include "kvi_dirbrowser.h"

#include "kvi_app.h"
#include "kvi_options.h"
#include "kvi_locale.h"
#include "kvi_defines.h"
#include "kvi_mime.h"

#include <qpixmap.h>
#include <qfileinfo.h>
#include <qdragobject.h>
#include <qtooltip.h>
#include <qpopupmenu.h>

//Declared in kvi_app.cpp and managed by KviApp class
extern QPixmap * g_pWindowIcon[KVI_WND_NUM_ICONS];
extern QPixmap * g_pFileIcons[KVI_FILE_ICONS_NUM_IMAGES];
extern QPixmap * g_pBrowserIcons[KVI_BROWSER_ICONS_NUM_IMAGES];

extern QPopupMenu * g_pBookmarksPopup;

KviDirView::KviDirView(KviDirBrowser *parent)
:QListView(parent)
{
	m_pParent = parent;
	m_bClickedInside = false;
	setAcceptDrops(true); //It will make the cursor change to a stop sign!
}

KviDirView::~KviDirView()
{
}

void KviDirView::checkForDrag()
{
	__enter("checkForDrag");

	QString path = m_pParent->m_pDir->path();
	if(path.right(1) != "/")path+="/";

	QStrList *list = new QStrList(true); //Deep copies thnx
	list->setAutoDelete(true);

	QListViewItem *it = firstChild();
	QListViewItem *first = 0;

	KviStr fName;
	while(it){
		if(it->isSelected()){
			fName = path;
			KviStr tmp(it->text(0));
			fName.append(tmp);
			QFileInfo fi(QString(fName.ptr()));
			if(fi.exists() && fi.isReadable() && fi.isFile() && (!fi.isSymLink())){
				if(first == 0)first = it;
				fName.prepend("file:");
				list->append(fName.ptr());
			}
		}
		it = it->nextSibling();
	}
	// And start dragging
	if(!list->isEmpty()){
		__debug_1arg("Starting drag for %u files",list->count());
		QUrlDrag *drag = new QUrlDrag(*list,viewport());
		QPixmap pm = *(first->pixmap(0));
		debug("Pixmap %d , %d",pm.width(),pm.height());
		drag->setPixmap(pm,QPoint(8,8));
		drag->dragMove();
	}
	delete list;
}

void KviDirView::contentsMouseMoveEvent(QMouseEvent *e)
{
	if(m_bClickedInside){
		m_bClickedInside = false;
		checkForDrag();
	} else QListView::contentsMouseMoveEvent(e);
}

void KviDirView::enterEvent(QEvent *e)
{
	QListView::enterEvent(e);
	m_bClickedInside = false;
}

void KviDirView::leaveEvent(QEvent *e)
{
	QListView::leaveEvent(e);
	m_bClickedInside = false;
}

void KviDirView::contentsMousePressEvent(QMouseEvent *e)
{
	// Allow drag
	if(e->button() & LeftButton){
		// Multiselection ?
		if(e->state() & ShiftButton)setMultiSelection(true);
		else {
			if(isMultiSelection()){
				setUpdatesEnabled(false);
				clearSelection();
				setUpdatesEnabled(true);
				setMultiSelection(false);
			}
		}
	}
	QListView::contentsMousePressEvent(e); //do the default job here...
	m_bClickedInside = true;
}

void KviDirView::contentsMouseReleaseEvent(QMouseEvent *e)
{
	QListView::contentsMouseReleaseEvent(e);
	m_bClickedInside = false;
}

KviDirBrowser::KviDirBrowser()
:QWidget(0,"dir_browser",WStyle_StaysOnTop)
{
	setIcon(*(g_pWindowIcon[KVI_WND_ICON_DIRBROWSER]));

	m_pView = new KviDirView(this);
	m_pView->addColumn(__tr("Name"));
	m_pView->addColumn(__tr("Type"));
	m_pView->addColumn(__tr("Size"));
	m_pView->setSorting(-1); //Disable QListView sorting...QDir does that...
	m_pView->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	m_pView->setAllColumnsShowFocus(true);

	connect(m_pView,SIGNAL(doubleClicked(QListViewItem *)),this,SLOT(itemDoubleClicked(QListViewItem *)));
	connect(m_pView,SIGNAL(currentChanged(QListViewItem *)),this,SLOT(currentItemChanged(QListViewItem *)));

	m_pCdUp = new QPushButton(this);
	m_pCdUp->setPixmap(*g_pBrowserIcons[KVI_BROWSER_ICON_CDUP]);
	QToolTip::add(m_pCdUp,QString(__tr("Cd up")));
	connect(m_pCdUp,SIGNAL(clicked()),this,SLOT(cdUp()));
	m_pReload = new QPushButton(this);
	m_pReload->setPixmap(*g_pBrowserIcons[KVI_BROWSER_ICON_RELOAD]);
	QToolTip::add(m_pReload,QString(__tr("Reload")));
	connect(m_pReload,SIGNAL(clicked()),this,SLOT(reload()));
	m_pCdHome = new QPushButton(this);
	m_pCdHome->setPixmap(*g_pBrowserIcons[KVI_BROWSER_ICON_HOME]);
	QToolTip::add(m_pCdHome,QString(__tr("Home")));
	connect(m_pCdHome,SIGNAL(clicked()),this,SLOT(cdHome()));
	m_pCdKVirc = new QPushButton(this);
	m_pCdKVirc->setPixmap(*g_pBrowserIcons[KVI_BROWSER_ICON_KVIRCHOME]);
	QToolTip::add(m_pCdKVirc,QString(__tr("KVirc home")));
	connect(m_pCdKVirc,SIGNAL(clicked()),this,SLOT(cdKVirc()));
//	m_pBookmark = new QPushButton(this);
//	m_pBookmark->setPixmap(*g_pBrowserIcons[KVI_BROWSER_ICON_BOOKMARK]);
//	QToolTip::add(m_pBookmark,QString(__tr("Add to bookmarks")));
//	connect(m_pBookmark,SIGNAL(clicked()),this,SLOT(bookmarkCurrentDir()));
	m_pGoBookmark = new QPushButton(this);
	m_pGoBookmark->setPixmap(*g_pBrowserIcons[KVI_BROWSER_ICON_BOOKMARK]);
	QToolTip::add(m_pGoBookmark,QString(__tr("Show bookmarks")));
	connect(m_pGoBookmark,SIGNAL(clicked()),this,SLOT(showBookmarks()));

	m_pStatus = new QLabel(__tr("None"),this);

	setGeometry(g_pOptions->m_rectDirBrowserGeometry);
	m_pDir = new QDir(QString(g_pOptions->m_szLastBrowsedDirectory.ptr()));

	m_pDir->setSorting(QDir::Name|QDir::DirsFirst|QDir::IgnoreCase);
	m_pDir->setFilter(QDir::All|QDir::Hidden);

//	m_pBookmarkMenu = new QPopupMenu(this);
//	

	fillView();
}

KviDirBrowser::~KviDirBrowser()
{
	if(g_pOptions->m_bUseHackedFrameGeometry){
		g_pOptions->m_rectDirBrowserGeometry.moveTopLeft(frameGeometry().topLeft());
		g_pOptions->m_rectDirBrowserGeometry.setSize(size());
	} else g_pOptions->m_rectDirBrowserGeometry = geometry();
	delete m_pDir;
}

void KviDirBrowser::fillView()
{
	if((!m_pDir->exists()) || (!m_pDir->isReadable())){
		KviStr tmp=m_pDir->path();
		g_pApp->warningBox(__tr("Could not read directory %s"),tmp.ptr());
		m_pDir->setPath(QDir::homeDirPath()); //reasonable default ?
	}
	const QFileInfoList * fi = m_pDir->entryInfoList();
	if(!fi){
		debug("ooops...something went wrong with Qt...the directory exists but has no QFileInfoList ?");
		return;
	}
	m_pView->setUpdatesEnabled(false);
	m_pView->clear();
	QFileInfoListIterator it(*fi);
	QFileInfo *i;
	while((i = it.current())){
		QString size;
		size.setNum(i->size());
		if(i->isDir()){
			QListViewItem *item = new QListViewItem(m_pView,i->fileName(),__tr("Directory"),size);
			item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_DIRECTORY]));
		} else if(i->isSymLink()){
			QString name = i->fileName();
			name +=" -> ";
			name += i->readLink();
			QListViewItem * item = new QListViewItem(m_pView,name,__tr("Symbolic Link"),size);
			item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_SYMLINK]));
		} else {
			KviStr tmp=i->extension(false);
			if(kvi_strEqualCI(tmp.ptr(),"gz")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Compressed"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_ARCHIVE]));
			} else if(kvi_strEqualCI(tmp.ptr(),"bz2")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Compressed"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_ARCHIVE]));
			} else if(kvi_strEqualCI(tmp.ptr(),"zip")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Compressed"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_ARCHIVE]));
			} else if(kvi_strEqualCIN(tmp.ptr(),"htm",3)){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Html"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_HTML]));
			} else if(kvi_strEqualCI(tmp.ptr(),"mp3")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Audio"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_AUDIO]));
			} else if(kvi_strEqualCI(tmp.ptr(),"wav")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Audio"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_AUDIO]));
			} else if(kvi_strEqualCI(tmp.ptr(),"mid")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Audio"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_AUDIO]));
			} else if(kvi_strEqualCI(tmp.ptr(),"png")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Image"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_IMAGE]));
			} else if(kvi_strEqualCI(tmp.ptr(),"gif")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Image"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_IMAGE]));
			} else if(kvi_strEqualCI(tmp.ptr(),"jpg")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Image"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_IMAGE]));
			} else if(kvi_strEqualCI(tmp.ptr(),"bmp")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Image"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_IMAGE]));
			} else if(kvi_strEqualCI(tmp.ptr(),"log")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Log file"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_LOG]));
			} else if(kvi_strEqualCI(tmp.ptr(),"kvihelp")){
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("Help file"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_HELP]));
			} else {
				QListViewItem * item = new QListViewItem(m_pView,i->fileName(),__tr("File"),size);
				item->setPixmap(0,*(g_pFileIcons[KVI_FILE_ICON_FILE]));
			}
		}
		++it;
	}
	m_pView->setUpdatesEnabled(true);
	m_pView->repaint();
	g_pOptions->m_szLastBrowsedDirectory = m_pDir->path();
	setCaption(m_pDir->path());
}

void KviDirBrowser::currentItemChanged(QListViewItem *i)
{
	if(!i){
		m_pStatus->setText("");
		return;
	}

	QString fName = m_pDir->path();
	if(fName.right(1) != "/")fName+="/";
	fName+=i->text(0);

	QFileInfo fi(fName);

	if(fi.isSymLink()){
		m_pStatus->setText(__tr("Symbolic link"));
		return;
	}

	if(fi.isDir()){
		m_pStatus->setText(__tr("Directory (double click to cd in)"));
		return;
	}

	if(!fi.exists()){
		m_pStatus->setText(__tr("Opps ? ... please click the refresh button"));
		return;
	}

	KviStr tmp = fi.absFilePath();
	KviMimeType * m = g_pOptions->m_pMimeManager->findMatch(tmp.ptr());
	if(m){
		KviStr tmp2(KviStr::Format,__tr("%s (double click : %s)"),m->mimeName.ptr(),m->commandline.ptr());
		m_pStatus->setText(QString(tmp2.ptr()));
	} else m_pStatus->setText(__tr("Unknown mime type"));
}

void KviDirBrowser::itemDoubleClicked(QListViewItem *i)
{
	QString fName = m_pDir->path();
	if(fName.right(1) != "/")fName+="/";
	fName+=i->text(0);

	QFileInfo fi(fName);
	if(fi.exists() && fi.isReadable()){
		if(fi.isDir()){
			m_pDir->setPath(fName);
			fillView();
		} else {
			KviStr tmp = fi.absFilePath();
			KviMimeType * m = g_pOptions->m_pMimeManager->findMatch(tmp.ptr());
			if(m){
				if(m->commandline.hasData()){
					g_pApp->executeFileWithCommand(m->commandline.ptr(),tmp.ptr());
				}
			}
		}
	}
}



void KviDirBrowser::cdUp()
{
	m_pDir->cdUp();
	fillView();
}

void KviDirBrowser::reload()
{
	m_pDir->setPath(m_pDir->path());
	fillView();
}

void KviDirBrowser::cdHome()
{
	m_pDir->setPath(QDir::homeDirPath());
	fillView(); //assume that $HOME is always readable....
}


//void KviDirBrowser::bookmarkCurrentDir()
//{

//}

void KviDirBrowser::cdKVirc()
{
	KviStr tmp;
	g_pApp->getLocalKVircDirectory(tmp,KviApp::None,0,true);
	m_pDir->setPath(QString(tmp.ptr()));
	fillView();
}

void KviDirBrowser::showBookmarks()
{
	g_pBookmarksPopup->clear();
	g_pBookmarksPopup->insertItem(QString(__tr("Add to bookmarks")));
	g_pBookmarksPopup->insertSeparator();
	g_pApp->appendCurrentBookmarksToBookmarksMenu();

	QPoint pnt(m_pGoBookmark->pos());
	pnt.setY(pnt.y() + m_pGoBookmark->height());
	g_pBookmarksPopup->popup(mapToGlobal(pnt));
	
}

void KviDirBrowser::openDirectory(const char *path)
{
	m_pDir->setPath(QString(path));
	fillView();
}

void KviDirBrowser::getCurrentPath(KviStr &buffer)
{
	buffer = m_pDir->path();
}

//============== closeEvent ================//

void KviDirBrowser::closeEvent(QCloseEvent *e)
{
	e->ignore(); //we ignore the event , and then close later if needed.
	g_pApp->dirBrowserCloseRequest();
}

//=============== resizeEvent ===============//

void KviDirBrowser::resizeEvent(QResizeEvent *)
{
	int hght = m_pCdUp->sizeHint().height();
	m_pCdUp->setGeometry(0,0,hght,hght);
	m_pReload->setGeometry(hght,0,hght,hght);
	m_pCdHome->setGeometry(hght*2,0,hght,hght);
	m_pCdKVirc->setGeometry(hght*3,0,hght,hght);
//	m_pBookmark->setGeometry(hght*4,0,hght,hght);
	m_pGoBookmark->setGeometry(hght*4,0,hght,hght);
	int statHght = m_pStatus->sizeHint().height();
	m_pStatus->setGeometry(0,height()-statHght,width(),statHght);
	m_pView->setGeometry(0,hght,width(),height()-(hght+statHght));
}

#include "m_kvi_dirbrowser.moc"
