//
//   File : kvi_dlgdcc.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_dlgdcc.cpp)
//   Last major modification : Thu Jun 24 1999 17:51:07 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_app.h"
#include "kvi_options.h"
#include "kvi_defines.h"
#include "kvi_dlgdcc.h"
#include "kvi_locale.h"
#include "kvi_selectors.h"

#include <qlayout.h>
#include <qframe.h>

KviDlgOptDcc::KviDlgOptDcc()
:QTabDialog()
{
	setCaption(_i18n_("Dcc options"));

	//
	// General options tab...
	//

	QWidget * tab = new QWidget(this);
	QGridLayout *g = new QGridLayout(tab,6,1,10,4);

	KviBoolSelector * b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreDccRequests),__tr("Ignore all DCC requests"));
	g->addWidget(b1,0,0);

	QFrame *r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,1,0);

	KviBoolSelector * b = new KviBoolSelector(tab,this,&(g_pOptions->m_bNotifyAllDccRequestsInConsole),__tr("Notify all requests in console"),!g_pOptions->m_bIgnoreDccRequests);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setNotEnabled(bool)));
	g->addWidget(b,2,0);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,3,0);

	b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseUserDefinedIpForDccRequests),__tr("Use the following local IP address for outgoing dcc requests"));
	g->addWidget(b1,4,0);

	KviStringSelector * s = new KviStringSelector(tab,this,&(g_pOptions->m_szDccLocalIpAddress),__tr("Local IP address to force:"),g_pOptions->m_bUseUserDefinedIpForDccRequests);
	connect(b1,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,5,0);

//	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bBindToUserDefinedIpForDccRequests),__tr("Bind the listening socket to this IP address"),g_pOptions->m_bUseUserDefinedIpForDccRequests);
//	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));
//	g->addWidget(b,6,0);

	g->activate();

	addTab(tab,__tr("General options"));

	//
	// Chat tab...
	//

	tab = new QWidget(this);
	g = new QGridLayout(tab,2,1,10,4);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoAcceptDccChat),__tr("Auto accept"),!g_pOptions->m_bIgnoreDccRequests);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setNotEnabled(bool)));
	g->addWidget(b,0,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCreateIconifiedDccChat),__tr("Create iconified"));
//	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setNotEnabled(bool)));
	g->addWidget(b,1,0);

	
	g->activate();

	addTab(tab,__tr("Chat"));

	//
	// Send tab...
	//

	tab = new QWidget(this);
	g = new QGridLayout(tab,5,2,10,4);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoAcceptDccSend),__tr("Auto accept"),!g_pOptions->m_bIgnoreDccRequests);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setNotEnabled(bool)));
	g->addWidget(b,0,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bEnableResumeOnAutoAccept),__tr("Enable resume on auto-accept"));
	g->addWidget(b,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCreateIconifiedDccSend),__tr("Create iconified"));
	g->addWidget(b,1,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoCloseDccSendOnSuccess),__tr("Auto close on success"));
	g->addWidget(b,1,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bNotifyDccSendCompletionInConsole),__tr("Notify completion in console"));
	g->addWidget(b,2,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bReplaceSpacesInDccSendFileNames),__tr("Replace spaces with underscores in filenames"));
	g->addWidget(b,2,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnorePortNumberInResumeRequests),__tr("Ignore port number in resume requests"));
	g->addWidget(b,3,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseFastDccSend),__tr("Use fast send (aka send ahead)"));
	g->addWidget(b,3,1);

	KviIntegerSelector * i = new KviIntegerSelector(tab,this,&(g_pOptions->m_iDccSendBlockSize),
			KviIntegerSelector::Int,__tr("Packet size (1-4096 bytes)"),true,1,KVI_DCC_SEND_MAX_BLOCK_SIZE,512,100);
	g->addMultiCellWidget(i,4,4,0,1);

	g->activate();

	addTab(tab,__tr("Send"));

	//
	// Voice tab...
	//

	tab = new QWidget(this);
	g = new QGridLayout(tab,3,1,10,4);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoAcceptDccVoice),__tr("Auto accept"),!g_pOptions->m_bIgnoreDccRequests);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setNotEnabled(bool)));
	g->addWidget(b,0,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCreateIconifiedDccVoice),__tr("Create iconified"));
	g->addWidget(b,1,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_b090CompatibleVoiceCompression),__tr("KVIrc 0.9.0 compatible compression (Simulate 0.9.0 bug)"));
	g->addWidget(b,2,0);
	
	g->activate();

	addTab(tab,__tr("Voice"));

	setCancelButton(__tr("Cancel"));
	setDefaultButton(QString(__tr("What is this ?")));
	connect(this,SIGNAL(defaultButtonPressed()),g_pApp,SLOT(slot_whatIsThisRequest()));

}

KviDlgOptDcc::~KviDlgOptDcc()
{
}

void KviDlgOptDcc::done(int r)
{
	QTabDialog::done(r);
	emit finished((r == Accepted));
}

void KviDlgOptDcc::closeEvent(QCloseEvent *)
{
	emit finished(false);
}

#include "m_kvi_dlgdcc.moc"
