//
//   File : kvi_dlggui.cpp (/usr/build/NEW_kvirc/kvirc/kvirc/kvi_dlggui.cpp)
//   Last major modification : Thu Jan 21 1999 18:27:11 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CHECK_RANGE_
#define _KVI_DEBUG_CLASS_NAME_ "KviDlgOptGui"
#include "kvi_debug.h"

#define _KVI_DLGGUI_CPP_

#include "kvi_app.h"
#include "kvi_dlggui.h"
#include "kvi_locale.h"
#include "kvi_options.h"
//#include "kvi_quickhelp.h"

#include "kvi_selectors.h"

#include <qlayout.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qpainter.h>
#include <qlabel.h>
#include <qstyle.h>

// Declared in kvi_options.cpp
extern QRgb defaultMircRgb[16];

// Declared in kvi_app.cpp and managed by KviApp class
extern QPixmap * g_pixViewOut[KVI_OUT_NUM_IMAGES];

// This must match the order of the KVI_OUT_* constants order 
// (declared in kvi_defines.h)
// used also in kvi_dlgoptions.cpp!!!
const char * messageTypeString[KVI_OUT_NUM_IMAGES + 2]=
{
	_i19n_("Normal text"),
	_i19n_("Internal messages"),
	_i19n_("Internal errors"),
	_i19n_("KVirc logo text"),
	_i19n_("'Echoed' text"),
	_i19n_("Socket messages"),
	_i19n_("Server informations"),
	_i19n_("Unhandled server replies"),
	_i19n_("Server message of the day"),
	_i19n_("Unhandled error messages"),
	_i19n_("Nickname error messages"),
	_i19n_("Ping pongs"),
	_i19n_("Join messages"),
	_i19n_("Topic changes"),
	_i19n_("Time messages"),
	_i19n_("Internal desync notices"),
	_i19n_("Names reply"),
	_i19n_("Part messages"),
	_i19n_("Kick messages"),
	_i19n_("Quit messages"),
	_i19n_("Quit on split messages"),
	_i19n_("Split warnings"),
	_i19n_("Own user mode changes"),
	_i19n_("Op messages"),
	_i19n_("Deop messages"),
	_i19n_("Voice messages"),
	_i19n_("Devoice messages"),
	_i19n_("Ban messages"),
	_i19n_("Unban messages"),
	_i19n_("Ban exception messages"),
	_i19n_("Ban unexception messages"),
	_i19n_("Key (mode +k) messages"),
	_i19n_("Limit (mode +l) messages"),
	_i19n_("Other channel mode messages"),
	_i19n_("Who replies"),
	_i19n_("Private messages not handled"),
	_i19n_("Ctcp requests"),
	_i19n_("Ctcp replies"),
	_i19n_("Ctcp errors"),
	_i19n_("Raw text to server"),
	_i19n_("Own channel and query text"),
	_i19n_("Spam warnings"),
	_i19n_("Nick changes"),
	_i19n_("Flood warnings"),
	_i19n_("Actions"),
	_i19n_("Notices"),
	_i19n_("Dns results"),
	_i19n_("Notify list changes"),
	_i19n_("Highlighted text"),
	_i19n_("Invite messages"),
	_i19n_("DCC Info"),
	_i19n_("DCC Warnings"),
	_i19n_("DCC Errors"),
	_i19n_("Help text"),
	_i19n_("Process stdin"),
	_i19n_("Process stdout"),
	_i19n_("Process stderr"),
	_i19n_("Directory entries of /DIR"),
	_i19n_("File entries of /DIR"),
	_i19n_("Plugin output"),
	_i19n_("Happy text"),
	_i19n_("Unhappy text"),
	_i19n_("Angry text"),
	_i19n_("Kill text"),
	_i19n_("Killed text"),
	_i19n_("CTCP Multimedia"),
	//New message types go BEFORE THIS LINE!
	_i19n_("Selected text"),
	_i19n_("URL Highlight (foreground only)")
};

//-----------------------------------------------------------------------------
// Mirc colors preview widget
//-----------------------------------------------------------------------------

KviMircColorPreview::KviMircColorPreview(KviDlgOptGui *dlg,QWidget *parent)
:QTableView(parent)
{
	m_pGuiDlg    = dlg;
	m_pFm        = new QFontMetrics(font());
	m_pMainPopup = new QPopupMenu();
	m_pBackPopup = new QPopupMenu();
	m_pForePopup = new QPopupMenu();
	m_rowToEdit  = -1;

	for(int i=0;i<16;i++)m_pColorPixmap[i]=new QPixmap(80,16);

	setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
	setNumCols(1);
	setNumRows(KVI_OUT_NUM_IMAGES + 2);
	setCellWidth(width());
	setCellHeight(18);

	updatePixmaps();

	connect(m_pForePopup,SIGNAL(activated(int)),this,SLOT(forePopupActivated(int)));
	connect(m_pBackPopup,SIGNAL(activated(int)),this,SLOT(backPopupActivated(int)));

	m_pMainPopup->insertItem("Foreground",m_pForePopup,1);
	m_pMainPopup->insertItem("Background",m_pBackPopup,2);

	setTableFlags(tableFlags() | Tbl_autoVScrollBar | Tbl_snapToGrid);
}

KviMircColorPreview::~KviMircColorPreview()
{
	delete m_pFm;
	delete m_pForePopup;
	delete m_pBackPopup;
	delete m_pMainPopup;

	for(int i=0;i<16;i++)
		delete m_pColorPixmap[i];
}

void KviMircColorPreview::updatePixmaps()
{
	for(int i=0;i<16;i++)
		m_pColorPixmap[i]->fill(m_pGuiDlg->m_pMircColorSelector[i]->getColor());
}

void KviMircColorPreview::setupPainter(QPainter *p)
{
//	if(!p->font().isCopyOf(m_pGuiDlg->m_pViewFontSelector->getFont())){
//	setFont(m_pGuiDlg->m_pViewFontSelector->getFont());
	p->setFont(m_pGuiDlg->m_pViewFontSelector->getFont());
	//recreate the Font Metrics....maybe it is not the best way...
	delete m_pFm;
	m_pFm = new QFontMetrics(p->font());
//	}

	m_brush.setColor(m_pGuiDlg->m_pViewBackColorSelector->getColor());

	if(m_pGuiDlg->m_pViewBackImageSelector->getImage()){
		m_brush.setPixmap(*(m_pGuiDlg->m_pViewBackImageSelector->getImage()));
	} else m_brush.setStyle(SolidPattern);
}

void KviMircColorPreview::paintCell(QPainter *p,int row,int)
{
	__range_valid(row < (KVI_OUT_NUM_IMAGES + 2));
	__range_valid(row >= 0); //Huh ? :)

	int cW = cellWidth(0);
	int strWidth = m_pFm->width(_i18n_translateNoText(messageTypeString[row]));
	if(strWidth > (cW - 24))strWidth = (cW - 24);
/* FIXME "Set brush origin here ?" */
	p->fillRect(0,0,cW,cellHeight(row),m_brush);

	if(row < KVI_OUT_NUM_IMAGES){
		p->drawPixmap(2,2,*(g_pixViewOut[row]));
		if(m_pGuiDlg->m_cViewOutTypeBack[row] < 16){
			QBrush br(m_pGuiDlg->m_pMircColorSelector[m_pGuiDlg->m_cViewOutTypeBack[row]]->getColor());
			p->setBrush(br);
			p->fillRect(20,1,strWidth + 4,cellHeight(row),br);
		}
	} else if(row == KVI_OUT_NUM_IMAGES && m_pGuiDlg->m_cViewOutSeleBack < 16){
		QBrush br(m_pGuiDlg->m_pMircColorSelector[m_pGuiDlg->m_cViewOutSeleBack]->getColor());
		p->setBrush(br);
		p->fillRect(20,1,strWidth + 4,cellHeight(row),br);
	}
	if(m_rowToEdit == row){
		p->setPen(red);
		p->setBrush(NoBrush);
		p->drawRect(0,0,cW-5,cellHeight(row));
	}
	__range_valid(m_pGuiDlg->m_cViewOutTypeFore[row] < 16);
	if(row < KVI_OUT_NUM_IMAGES){
		p->setPen(m_pGuiDlg->m_pMircColorSelector[m_pGuiDlg->m_cViewOutTypeFore[row]]->getColor());
	} else if(row == KVI_OUT_NUM_IMAGES){
		p->setPen(m_pGuiDlg->m_pMircColorSelector[m_pGuiDlg->m_cViewOutSeleFore]->getColor());
	} else p->setPen(m_pGuiDlg->m_pMircColorSelector[m_pGuiDlg->m_cViewOutUrlFore]->getColor());
	p->drawText(24,1,strWidth,cellHeight(row),AlignCenter,_i18n_translateNoText(messageTypeString[row]));
}

void KviMircColorPreview::resizeEvent(QResizeEvent *e)
{
	QTableView::resizeEvent(e);
	setCellWidth(width());
}

void KviMircColorPreview::fillBackPopup(int /*row*/)
{
	m_pBackPopup->clear();
	for(int i=0;i<16;i++)m_pBackPopup->insertItem(*m_pColorPixmap[i],i);
	m_pBackPopup->insertItem(_i18n_("Transparent"),16);
}

void KviMircColorPreview::fillForePopup(int /*row*/)
{
	m_pForePopup->clear();
	for(int i=0;i<16;i++)m_pForePopup->insertItem(*m_pColorPixmap[i],i);
}

void KviMircColorPreview::forePopupActivated(int itemid)
{
	__range_valid(m_rowToEdit <= KVI_OUT_NUM_IMAGES + 1);
	__range_valid(m_rowToEdit >= 0);//Huh ? :)
	__range_valid(itemid >= 0);
	__range_valid(itemid <= 15);
	if((m_rowToEdit >= 0) && (itemid >= 0) && (itemid <= 15)){
		if(m_rowToEdit < KVI_OUT_NUM_IMAGES)m_pGuiDlg->m_cViewOutTypeFore[m_rowToEdit] = itemid;
		else if(m_rowToEdit == KVI_OUT_NUM_IMAGES)m_pGuiDlg->m_cViewOutSeleFore = itemid;
		else m_pGuiDlg->m_cViewOutUrlFore = itemid;
		int row=m_rowToEdit;
		m_rowToEdit = -1;
		updateCell(row,0,false);
	}
}

void KviMircColorPreview::backPopupActivated(int itemid)
{
	__range_valid(m_rowToEdit <= KVI_OUT_NUM_IMAGES);
	__range_valid(m_rowToEdit >= 0);//Huh ? :)
	__range_valid(itemid >= 0);
	__range_valid(itemid <= 16);
	if((m_rowToEdit >= 0) && (itemid >= 0) && (itemid <= 16)){
		if(m_rowToEdit < KVI_OUT_NUM_IMAGES)m_pGuiDlg->m_cViewOutTypeBack[m_rowToEdit] = (itemid < 16 ? itemid : KVI_TRANSPARENT);
		else m_pGuiDlg->m_cViewOutSeleBack = (itemid < 16 ? itemid : KVI_TRANSPARENT);
		int row = m_rowToEdit;
		m_rowToEdit = -1;
		updateCell(row,0,false);
	}
}

void KviMircColorPreview::mousePressEvent(QMouseEvent *e)
{
	int clickedRow = findRow(e->pos().y());
	if(m_rowToEdit != -1){
		m_rowToEdit = -1;
		repaint(false);
	}
	if(clickedRow >= 0){
		updatePixmaps();
		fillBackPopup(clickedRow);
		fillForePopup(clickedRow);
		m_rowToEdit = clickedRow;
		updateCell(clickedRow,0,false);
		m_pMainPopup->setItemEnabled(2,clickedRow <= KVI_OUT_NUM_IMAGES);
		m_pMainPopup->popup(mapToGlobal(e->pos()));
	}
}

//-----------------------------------------------------------------------------
// Main GUI options dialog
//-----------------------------------------------------------------------------

KviDlgOptGui::KviDlgOptGui()
:QTabDialog()
{
	// Local copy of data
	for(int j = 0;j < KVI_OUT_NUM_IMAGES;j++){
		m_cViewOutTypeBack[j] = g_pOptions->m_cViewOutTypeBack[j];
		m_cViewOutTypeFore[j] = g_pOptions->m_cViewOutTypeFore[j];
		//Sanity check
		__range_valid((m_cViewOutTypeBack[j] < 16)||(m_cViewOutTypeBack[j] == KVI_TRANSPARENT));
		__range_valid(m_cViewOutTypeFore[j] < 16);
	}
	m_cViewOutSeleBack = g_pOptions->m_cViewOutSeleBack;
	m_cViewOutSeleFore = g_pOptions->m_cViewOutSeleFore;
	m_cViewOutUrlFore  = g_pOptions->m_cViewOutUrlFore;
	//Sanity check
	__range_valid((m_cViewOutSeleBack < 16)||(m_cViewOutSeleBack == KVI_TRANSPARENT));
	__range_valid(m_cViewOutSeleFore < 16);
	__range_valid(m_cViewOutUrlFore < 16);


	setCaption(_i18n_("GUI options"));

	//
	// Mdi tab
	//
	QWidget *tab = new QWidget(this);
	QGridLayout *g = new QGridLayout(tab,11,1,10,4);

	KviColorSelector *s = new KviColorSelector(tab,this,&(g_pOptions->m_clrMdiCaptionActiveBack),_i18n_("Active caption background color"));
	g->addWidget(s,0,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrMdiCaptionActiveFore),_i18n_("Active caption foreground color"));
	g->addWidget(s,1,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrMdiCaptionInactiveBack),_i18n_("Inactive caption background color"));
	g->addWidget(s,2,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrMdiCaptionInactiveFore),_i18n_("Inactive caption foreground color"));
	g->addWidget(s,3,0);
	KviFontSelector *f = new KviFontSelector(tab,this,&(g_pOptions->m_fntMdiCaption),_i18n_("Caption font"));
	g->addWidget(f,4,0);
	QFrame *r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,5,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrMdiBack),_i18n_("Main window background color"));
	g->addWidget(s,6,0);
	m_pMdiBackImageSelector = new KviImageSelector(tab,this,&(g_pOptions->m_szMdiBack),_i18n_("Main window background image"));
	g->addWidget(m_pMdiBackImageSelector,7,0);
	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,8,0);
	KviBoolSelector *b = new KviBoolSelector(tab,this,&(g_pOptions->m_bMdiOpaqueMove),_i18n_("Opaque window movement"));
	g->addWidget(b,9,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bMdiOpaqueResize),_i18n_("Opaque window resize"));
	g->addWidget(b,10,0);


	g->activate();
	addTab(tab,_i18n_("MDI"));


	//
	// Input tab
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,8,1,10,4);

	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrInputBack),_i18n_("Background color"));
	g->addWidget(s,0,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrInputFore),_i18n_("Foreground color"));
	g->addWidget(s,1,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrInputSeleBack),_i18n_("Selection background color"));
	g->addWidget(s,2,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrInputSeleFore),_i18n_("Selection foreground color"));
	g->addWidget(s,3,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrInputCursor),_i18n_("Corsor and control marks"));
	g->addWidget(s,4,0);
	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntInput),_i18n_("Font"));
	g->addWidget(f,5,0);
	r = new QFrame(tab);

	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,6,0);
	m_pInputBackImageSelector = new KviImageSelector(tab,this,&(g_pOptions->m_szInputBack),_i18n_("Background image"));
	g->addWidget(m_pInputBackImageSelector,7,0);

	g->activate();
	addTab(tab,_i18n_("Input"));


	//
	// ListBox tab
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,11,1,10,4);

	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrListBoxBack),_i18n_("Background color"));
	g->addWidget(s,0,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrListBoxOp),_i18n_("Op foreground color"));
	g->addWidget(s,1,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrListBoxVoice),_i18n_("Voice foreground color"));
	g->addWidget(s,2,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrListBoxNormal),_i18n_("Normal foreground color"));
	g->addWidget(s,3,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrListBoxSeleBack),_i18n_("Selection background color"));
	g->addWidget(s,4,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrListBoxSeleFore),_i18n_("Selection foreground color"));
	g->addWidget(s,5,0);
	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntListBox),_i18n_("Font"));
	g->addWidget(f,6,0);
	//Sep
	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,7,0);
	m_pListBoxBackImageSelector = new KviImageSelector(tab,this,&(g_pOptions->m_szListBoxBack),_i18n_("Background image"));
	g->addWidget(m_pListBoxBackImageSelector,8,0);
	//Sep
	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,9,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bListBoxShowImages),_i18n_("Show op & voice icons"));
	g->addWidget(b,10,0);

	g->activate();
	addTab(tab,_i18n_("List box"));

	//
	// Help borwser tab
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,8,1,10,4);

	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrHelpBack),_i18n_("Background color"));
	g->addWidget(s,0,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrHelpFore),_i18n_("Foreground color"));
	g->addWidget(s,1,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrHelpTitle),_i18n_("Title foreground color"));
	g->addWidget(s,2,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrHelpSubtitle),_i18n_("Subtitle foreground color"));
	g->addWidget(s,3,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrHelpExample),_i18n_("Example foreground color"));
	g->addWidget(s,4,0);
	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntHelpNormal),_i18n_("Normal font"));
	g->addWidget(f,6,0);
	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntHelpFixed),_i18n_("Fixed font"));
	g->addWidget(f,7,0);
	g->activate();
	addTab(tab,_i18n_("Help browser"));

	//
	// Labels
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,6,1,10,4);

	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrLabelsBack),_i18n_("Background color"));
	g->addWidget(s,0,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrLabelsActiveFore),_i18n_("Active foreground color"));
	g->addWidget(s,1,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrLabelsInactiveFore),_i18n_("Inactive foreground color"));
	g->addWidget(s,2,0);
	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntLabels),_i18n_("Font"));
	g->addWidget(f,3,0);
	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,4,0);
	m_pLabelsBackImageSelector = new KviImageSelector(tab,this,&(g_pOptions->m_szLabelsBack),_i18n_("Background image"));
	g->addWidget(m_pLabelsBackImageSelector,5,0);

	g->activate();
	addTab(tab,_i18n_("Labels"));

	//
	// Output tab
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,8,1,10,4);

	m_pViewBackColorSelector = new KviColorSelector(tab,this,&(g_pOptions->m_clrViewBack),_i18n_("Background color"));
	g->addWidget(m_pViewBackColorSelector,0,0);
	m_pViewFontSelector = new KviFontSelector(tab,this,&(g_pOptions->m_fntView),__tr("Font"));
	g->addWidget(m_pViewFontSelector,1,0);
	//Sep
	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,2,0);
	m_pViewBackImageSelector = new KviImageSelector(tab,this,&(g_pOptions->m_szViewBack),_i18n_("Background image"));
	g->addWidget(m_pViewBackImageSelector,3,0);
	//Sep
	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,4,0);
	//Other stuff that could be moved to the general options dialog...
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowImages),_i18n_("Show message images"));
	g->addWidget(b,5,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bTimestamp),_i18n_("Timestamp output lines"));
	g->addWidget(b,6,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bWrapMargin),_i18n_("Add margin after first word wrap"));
	g->addWidget(b,7,0);

	g->activate();
	addTab(tab,_i18n_("Output"));


	//
	// Mirc colors tab
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,5,4,10,4);

	KviStr szNum;
	for(int i = 0;i < 16;i++){
		szNum.setNum(i);
		m_pMircColorSelector[i] = new KviColorSelector(tab,this,g_pOptions->m_pMircColor[i],szNum.ptr(),50);
		g->addWidget(m_pMircColorSelector[i],i / 4,i % 4);
	}
	QPushButton *pb = new QPushButton(_i18n_("Reset"),tab);
	pb->setMinimumSize(pb->sizeHint());
	g->addWidget(pb,4,0);
	connect(pb,SIGNAL(clicked()),this,SLOT(resetToMircColors()));

	pb = new QPushButton(_i18n_("Invert"),tab);
	pb->setMinimumSize(pb->sizeHint());
	g->addWidget(pb,4,1);
	connect(pb,SIGNAL(clicked()),this,SLOT(invertColors()));

	pb = new QPushButton(_i18n_("20% darker"),tab);
	pb->setMinimumSize(pb->sizeHint());
	g->addWidget(pb,4,2);
	connect(pb,SIGNAL(clicked()),this,SLOT(makeDarker()));

	pb = new QPushButton(_i18n_("20% lighter"),tab);
	pb->setMinimumSize(pb->sizeHint());
	g->addWidget(pb,4,3);
	connect(pb,SIGNAL(clicked()),this,SLOT(makeLighter()));

	g->activate();
	addTab(tab,_i18n_("mIrc Colors"));
	
	//
	// Mirc preview tab
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,2,1,10,4);

	QLabel *l=new QLabel(_i18n_("Click on a message line to change the colors"),tab);
	l->setMinimumSize(l->sizeHint());
	g->addWidget(l,0,0);

	m_pMircColorPreview = new KviMircColorPreview(this,tab);
	g->addWidget(m_pMircColorPreview,1,0);
	g->setRowStretch(1,1);

	g->activate();
	addTab(tab,_i18n_("Messages"));

	//
	// Other
	//
	tab = new QWidget(this);
	g = new QGridLayout(tab,16,2,10,4);

	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntApplication),_i18n_("Global application font"));
	g->addMultiCellWidget(f,0,0,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,1,1,0,1);

	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrSysTrayBack),_i18n_("System display background"));
	g->addMultiCellWidget(s,2,2,0,1);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrSysTrayFore),_i18n_("System display foreground"));
	g->addMultiCellWidget(s,3,3,0,1);
	f = new KviFontSelector(tab,this,&(g_pOptions->m_fntSysTray),_i18n_("System display font (face only)"));
	g->addMultiCellWidget(f,4,4,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,5,5,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseBigToolbarPixmaps),__tr("Use big toolbar images"));
	g->addWidget(b,6,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseTextToolbarComment),__tr("Show toolbar button text"));
	g->addWidget(b,6,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,7,7,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bSetGlobalPaletteAtStartup),_i18n_("Use private global color palette"));
	g->addMultiCellWidget(b,8,8,0,1);

	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalForeground),_i18n_("Foreground"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,9,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalButton),_i18n_("Background"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,10,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalLight),_i18n_("Light shadow"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,11,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalMid),_i18n_("Middle shadow"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,12,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalDark),_i18n_("Dark shadow"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,13,0);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalText),_i18n_("Text"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,9,1);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrNormalGlobalBase),_i18n_("Base"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,10,1);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrDisabledGlobalForeground),_i18n_("Disabled foreground"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,11,1);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrDisabledGlobalText),_i18n_("Disabled text"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,12,1);
	s = new KviColorSelector(tab,this,&(g_pOptions->m_clrDisabledGlobalBase),_i18n_("Disabled base"));
	s->setEnabled(g_pOptions->m_bSetGlobalPaletteAtStartup);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addWidget(s,13,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,14,14,0,1);

	l=new QLabel(_i18n_("Application GUI style"),tab);
	l->setMinimumSize(l->sizeHint());
	g->addWidget(l,15,0);

	m_pGUIStyleBox = new QComboBox(false,tab);
	g->addWidget(m_pGUIStyleBox,15,1);

	fillGuiStylesBox();

//	m_pGUIStyleBox->insertItem(QString(__tr("CDE")));
//	m_pGUIStyleBox->insertItem(QString(__tr("Motif")));
//	m_pGUIStyleBox->insertItem(QString(__tr("Platinum")));
//	m_pGUIStyleBox->insertItem(QString(__tr("Windows")));
//
//	switch(g_pOptions->m_iApplicationGUIStyle)
//	{
//		case KVI_APP_STYLE_MOTIF:    m_pGUIStyleBox->setCurrentItem(1);    break;
//		case KVI_APP_STYLE_CDE:      m_pGUIStyleBox->setCurrentItem(0);    break;
//		case KVI_APP_STYLE_WINDOWS:  m_pGUIStyleBox->setCurrentItem(3);    break;
//		case KVI_APP_STYLE_PLATINUM: m_pGUIStyleBox->setCurrentItem(2);    break;
//		default:                     m_pGUIStyleBox->setCurrentItem(0);    break;
//	}

	addTab(tab,_i18n_("Other"));

	setCancelButton(__tr("Cancel"));
	setDefaultButton(QString(__tr("What is this ?")));
	connect(this,SIGNAL(defaultButtonPressed()),g_pApp,SLOT(slot_whatIsThisRequest()));
}

KviDlgOptGui::~KviDlgOptGui()
{
}

void KviDlgOptGui::getCachedGuiPixmaps(struct KviCachedGuiPixmaps *p)
{
	p->pListBoxPix = m_pListBoxBackImageSelector->getImage();
	p->pMdiPix = m_pMdiBackImageSelector->getImage();
	p->pLabelsPix = m_pLabelsBackImageSelector->getImage();
	p->pViewPix = m_pViewBackImageSelector->getImage();
	p->pInputPix = m_pInputBackImageSelector->getImage();
}

//-----------------------------
// Mirc colors button handlers
//-----------------------------

void KviDlgOptGui::resetToMircColors()
{
	for(int i = 0;i< 16;i++)
		m_pMircColorSelector[i]->setColor(QColor(defaultMircRgb[i]));
}

void KviDlgOptGui::invertColors()
{
	for(int i = 0;i< 16;i++){
		QColor clr(m_pMircColorSelector[i]->getColor());
		clr.setRgb(255 - clr.red(),255 - clr.green(),255 - clr.blue());
		m_pMircColorSelector[i]->setColor(clr);
	}
}

void KviDlgOptGui::makeDarker()
{
	// Just for fun
	for(int i = 0;i < 16;i++)
		m_pMircColorSelector[i]->setColor(m_pMircColorSelector[i]->getColor().dark(120));
}

void KviDlgOptGui::makeLighter()
{
	// Just for fun
	for(int i = 0;i < 16;i++)
		m_pMircColorSelector[i]->setColor(m_pMircColorSelector[i]->getColor().light(120));
}

void KviDlgOptGui::fillGuiStylesBox()
{
	int idx = 0;
	int currentStyle = -1;
	for(KviStyleEntry * e=g_pApp->m_pStyleList->first();e;e=g_pApp->m_pStyleList->next()){
		m_pGUIStyleBox->insertItem(QString(e->name.ptr()));
		if(kvi_strEqualCI(e->name.ptr(),g_pOptions->m_szApplicationGUIStyle.ptr()))currentStyle = idx;
		idx++;
	}
	if(currentStyle != -1)m_pGUIStyleBox->setCurrentItem(currentStyle);
}

//
// Job terminated
//

void KviDlgOptGui::done(int r)
{
	QTabDialog::done(r);

	if(r == Accepted){
		//Commit the changes
		for(int j = 0;j < KVI_OUT_NUM_IMAGES;j++){
			//Sanity check
			__range_valid((m_cViewOutTypeBack[j] < 16)||(m_cViewOutTypeBack[j] == KVI_TRANSPARENT));
			__range_valid(m_cViewOutTypeFore[j] < 16);
			//Commit
			g_pOptions->m_cViewOutTypeBack[j] = m_cViewOutTypeBack[j];
			g_pOptions->m_cViewOutTypeFore[j] = m_cViewOutTypeFore[j];
		}
		//Sanity check
		__range_valid((m_cViewOutSeleBack < 16)||(m_cViewOutSeleBack == KVI_TRANSPARENT));
		__range_valid(m_cViewOutSeleFore < 16);
		__range_valid(m_cViewOutUrlFore < 16);
		//Commit
		g_pOptions->m_cViewOutSeleBack = m_cViewOutSeleBack;
		g_pOptions->m_cViewOutSeleFore = m_cViewOutSeleFore;
		g_pOptions->m_cViewOutUrlFore  = m_cViewOutUrlFore;

		KviStr save = g_pOptions->m_szApplicationGUIStyle;
		g_pOptions->m_szApplicationGUIStyle = m_pGUIStyleBox->currentText();
		if(g_pOptions->m_szApplicationGUIStyle.isEmpty()){
			g_pOptions->m_szApplicationGUIStyle = save;
		}
	}

	emit finished((r == Accepted));
}

void KviDlgOptGui::closeEvent(QCloseEvent *)
{
	// Finished without committing
	emit finished(false);
}

#include "m_kvi_dlggui.moc"
