#ifndef _KVI_DLGGUI_H_INCLUDED_
#define _KVI_DLGGUI_H_INCLUDED_

//
//   File : kvi_dlggui.h (/usr/build/NEW_kvirc/kvirc/kvirc/kvi_dlggui.h)
//   Last major modification : Thu Jan 21 1999 18:26:16 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <qtabdialog.h>
#include <qtableview.h>
#include <qbrush.h>
#include <qfontmetrics.h>
#include <qpopupmenu.h>
#include <qpixmap.h>
#include <qcombobox.h>

#include "kvi_defines.h"

class KviColorSelector;
class KviFontSelector;
class KviImageSelector;
class KviMircColorPreview;


struct KviCachedGuiPixmaps
{
	const QPixmap * pListBoxPix;
	const QPixmap * pInputPix;
	const QPixmap * pViewPix;
	const QPixmap * pMdiPix;
	const QPixmap * pLabelsPix;
};

class KviDlgOptGui : public QTabDialog
{
	friend class KviMircColorPreview;
	Q_OBJECT
public:
	KviDlgOptGui();
	~KviDlgOptGui();
private:
	KviColorSelector    *m_pMircColorSelector[16];
	KviFontSelector     *m_pViewFontSelector;
	KviColorSelector    *m_pViewBackColorSelector;
	KviImageSelector    *m_pViewBackImageSelector;
	KviImageSelector    *m_pInputBackImageSelector;
	KviImageSelector    *m_pListBoxBackImageSelector;
	KviImageSelector    *m_pMdiBackImageSelector;
	KviImageSelector    *m_pLabelsBackImageSelector;
	KviMircColorPreview *m_pMircColorPreview;
	char m_cViewOutTypeBack[KVI_OUT_NUM_IMAGES];
	char m_cViewOutTypeFore[KVI_OUT_NUM_IMAGES];
	char m_cViewOutSeleBack;
	char m_cViewOutSeleFore;
	char m_cViewOutUrlFore;
	QComboBox           *m_pGUIStyleBox;
public:
	void getCachedGuiPixmaps(struct KviCachedGuiPixmaps *p);
signals:
	void finished(bool bAccepted);
protected slots:
	virtual void done(int r);
	virtual void closeEvent(QCloseEvent *);
	void resetToMircColors();
	void invertColors();
	void makeDarker();
	void makeLighter();
private:
	void fillGuiStylesBox();
};

class KviMircColorPreview : public QTableView
{
	Q_OBJECT
public:
	KviMircColorPreview(KviDlgOptGui *dlg,QWidget *parent);
	~KviMircColorPreview();
private:
	int m_rowToEdit;
	KviDlgOptGui * m_pGuiDlg;
	QBrush m_brush;
	QFontMetrics *m_pFm;
	QPopupMenu *m_pMainPopup;
	QPopupMenu *m_pBackPopup;
	QPopupMenu *m_pForePopup;
	QPixmap *m_pColorPixmap[16];
protected:
	void updatePixmaps();
	void fillBackPopup(int row);
	void fillForePopup(int row);
	virtual void paintCell(QPainter *p,int row,int);
	virtual void setupPainter(QPainter *p);
	virtual void resizeEvent(QResizeEvent *e);
	virtual void mousePressEvent(QMouseEvent *e);
protected slots:
	void forePopupActivated(int itemid);
	void backPopupActivated(int itemid);
};

#endif //!_KVI_DLGGUI_H_INCLUDED_
