//
//   File : kvi_dlgoptions.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_dlgoptions.cpp)
//   Last major modification : Tue May 4 1999 18:02:42 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#define _KVI_DLGOPTIONS_CPP_

#include "kvi_dlgoptions.h"

#include "kvi_locale.h"

#include "kvi_defines.h"

#include "kvi_app.h"
#include "kvi_selectors.h"
#include "kvi_listview.h"
#include "kvi_mime.h"

#include "kvi_options.h"
#include "kvi_strsub.h"


#ifdef COMPILE_PLUGIN_SUPPORT

	#include "kvi_frame.h"
	#include "kvi_error.h"
	#include "kvi_console.h"
	#include "kvi_plugin.h"
	extern KviPluginManager * g_pPluginManager;

#endif

#include <qlayout.h>
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qheader.h>
#include <qpixmap.h>
#include <qfiledialog.h>
//#include <qpainter.h>


#include <unistd.h> // for usleep()

#include <qstyle.h>



KviDoubleRowTabBar::KviDoubleRowTabBar(QWidget * parent,const char * name)
:QTabBar(parent,name)
{
	m_pTabList = 0;
//	setShape(QTabBar::TriangularAbove);
}

KviDoubleRowTabBar::~KviDoubleRowTabBar()
{
	delete m_pTabList;
}

void KviDoubleRowTabBar::copyTabList(QList<QTab> * l)
{
	if(m_pTabList)delete m_pTabList;
	m_pTabList = new QList<QTab>;
	m_pTabList->setAutoDelete(false);
	for(QTab * t=l->last();t;t=l->prev())m_pTabList->append(t);
}

//#define KVI_DBLTABBAR_UPPER_ROW_CLIP 4

void KviDoubleRowTabBar::layoutTabs()
{
	if(!m_pTabList)copyTabList(tabList()); // HACK
	else {
		if(tabList()){
			if(m_pTabList->count() < tabList()->count())copyTabList(tabList());
		}
	}

	if(!m_pTabList)return;
    if(m_pTabList->isEmpty())return;

	int nTabsInARow = (m_pTabList->count() / 2);

	// calculate the maximum height
	QTab * t = m_pTabList->first();
	int hght = t->r.height();
	while((t = m_pTabList->next()) != 0)if(hght < t->r.height())hght = t->r.height();

	// get the tab bar metrics
    int hframe, vframe, overlap;
    style().tabbarMetrics( this, hframe, vframe, overlap );

    QFontMetrics fm = fontMetrics();

	// Yes...this is a horrible hack again
	t = tab(currentTab());
	int curIdx = m_pTabList->findRef(t);

    int y = ((curIdx <= nTabsInARow) ? hght : 0);
	int x = 0;
	int nTab = 0;
	int firstRowWidth = 0;

    for(t = m_pTabList->first(); t; t = m_pTabList->next()){
		int lw = fm.width( t->label );
		int iw = 0;
		int ih = 0;
		if(t->iconset != 0){
			iw = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 2;
			ih = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).height();
		}
		int h = QMAX( fm.height(), ih );

		h += vframe;
		t->r.setRect(x, y, lw + hframe + iw,QMAX(hght,h));
		x += t->r.width();
		if(y == hght){
			if((nTab == nTabsInARow) || (nTab == ((int)(m_pTabList->count() - 1)))){
				// This is the last tab of the lower row
				if(firstRowWidth > x)t->r.setWidth(t->r.width() + firstRowWidth - x);
			}
		}
		if(nTab == nTabsInARow){
			y = ((curIdx <= nTabsInARow) ? 0 : hght );
			firstRowWidth = x;
			x = 0;
		}
		
		nTab ++;
    }
}

void KviDoubleRowTabBar::paint( QPainter * p, QTab * t, bool selected ) const
{
    QRect r( t->r );

	QColor middark(colorGroup().button().dark(130));
	p->setPen(colorGroup().light());
	p->drawLine(r.left(),r.top(),r.left(),r.bottom() - (((r.left() != 0) && (r.top() != 0) && selected) ? 1 : 0));
	p->drawLine(r.left(),r.top(),r.right(),r.top());
	p->setPen(colorGroup().dark());
	p->drawLine(r.right(),r.top()+1,r.right(),r.bottom() - ((((r.left() != 0) && (r.top() == 0)) || (!selected)) ? 0 : 1));
	p->setPen(middark);
	p->drawPoint(r.right(),r.top());

	if(selected){
		p->drawLine(r.right() - 1,r.top() + 1,r.right() - 1,r.bottom() - (((r.left() != 0) && (r.top() == 0)) ? 0 : 1));
		if(style() != MotifStyle)p->setPen(colorGroup().midlight());
		else p->setPen(colorGroup().light());
		p->drawLine(r.left()+1,r.top()+1,r.left()+1,r.bottom() - (((r.left() != 0) && (r.top() != 0)) ? 1 : 0));
		p->drawLine(r.left()+1,r.top()+1,r.right()-1,r.top()+1);
	} else {
		QBrush br(middark,Dense5Pattern);
		p->fillRect(r.left() + 3,r.top() + 3,r.width() - 6,r.height() - ((r.top() == 0) ? 3 : 6),br);
		p->drawLine(r.right() - 1,r.top() + 1,r.right() - 1,r.bottom());
		p->setPen(colorGroup().midlight());
		p->drawLine(r.left()+1,r.top()+1,r.left()+1,r.bottom());
		p->drawLine(r.left()+1,r.top()+1,r.right()-1,r.top()+1);
		if(r.top() != 0){
			p->setPen(colorGroup().light());
			p->drawLine(r.left(),r.bottom() - 1,r.right(),r.bottom() - 1);
			if(style() != MotifStyle)p->setPen(colorGroup().midlight());
			p->drawLine(r.left()+((r.left() == 0) ? 1 : 0),r.bottom(),r.right(),r.bottom());
		}
	}
    p->setFont( font() );

    int iw = 0;
    int ih = 0;

    if ( t->iconset != 0 ) {
		iw = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).width() + 2;
		ih = t->iconset->pixmap( QIconSet::Small, QIconSet::Normal ).height();
    }

    int w;
	w = iw + p->fontMetrics().width( t->label ) + 4;
    int h;
	h = QMAX(p->fontMetrics().height() + 4, ih );

    paintLabel(p,QRect(r.left()+(r.width()-w)/2 - 3,r.top() + (r.height()-h)/2,
			  w, h ), t, t->id == keyboardFocusTab() );
}

void KviDoubleRowTabBar::setCurrentTab(int tab)
{
	QTabBar::setCurrentTab(tab);
	layoutTabs();
	repaint();
}

void KviDoubleRowTabBar::setCurrentTab(QTab * tab)
{
	QTabBar::setCurrentTab(tab);
	layoutTabs();
	repaint();
}

void KviDoubleRowTabBar::mouseReleaseEvent(QMouseEvent *e)
{
	QTabBar::mouseReleaseEvent(e);
	layoutTabs();
	repaint();
}

/*
IT LOOKS HORRIBLE ANYWAY....hehe...
void KviDoubleRowTabBar::paintEvent(QPaintEvent *e)
{
    QPainter p(this);
    p.setBrushOrigin(rect().bottomLeft());
    p.fillRect(0,0,width(),height(),QBrush(colorGroup().brush(QColorGroup::Background)));
	if(!m_pTabList)return;
    QTab * t = m_pTabList->first();
    do{
		if(t && t->r.intersects(e->rect())){
			if(t->r.y() == 0){
				p.setClipRect(t->r.x(),t->r.y(),t->r.width(),t->r.height() - KVI_DBLTABBAR_UPPER_ROW_CLIP);
				paint(&p,t,false);
			} else {
				p.setClipRect(t->r.x(),t->r.y(),t->r.width()+2,t->r.height());
				paint(&p,t,(t == tab(currentTab())));
			}
		}
		t = m_pTabList->next();
    } while ( t != 0 );

}
*/


// declared in kvi_dlggui.cpp
extern const char * messageTypeString[KVI_OUT_NUM_IMAGES + 2];
// Declared in kvi_app.cpp and managed by KviApp class
extern QPixmap * g_pixViewOut[KVI_OUT_NUM_IMAGES];

KviDlgOptMisc::KviDlgOptMisc()
:QTabDialog()
{
	setCaption(__tr("Misc options"));

	KviDoubleRowTabBar * tB = new KviDoubleRowTabBar(tabBar() ? ((QWidget *)tabBar()->parent()) : this);
	setTabBar(tB);

	// Messages 
	QWidget *tab = new QWidget(this);
	QGridLayout *g = new QGridLayout(tab,9,1,10,2);

	KviStringSelector *s = new KviStringSelector(tab,this,&(g_pOptions->m_szCaptionMessage),__tr("Caption text"));
	g->addWidget(s,0,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szPartMessage),__tr("Default part text"));
	g->addWidget(s,1,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szKickMessage),__tr("Default kick reason"));
	g->addWidget(s,2,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szQuitMessage),__tr("Default quit text"));
	g->addWidget(s,3,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szCtcpVersionAppendix),__tr("CTCP VERSION reply postfix"));
	g->addWidget(s,4,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szCtcpUserinfoAppendix),__tr("CTCP USERINFO reply postfix"));
	g->addWidget(s,5,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szCtcpClientinfoAppendix),__tr("CTCP CLIENTINFO reply postfix"));
	g->addWidget(s,6,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szCtcpSourceAppendix),__tr("CTCP SOURCE reply postfix"));
	g->addWidget(s,7,0);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szCtcpFingerReply),__tr("CTCP FINGER reply"));
	g->addWidget(s,8,0);

	addTab(tab,__tr("Messages"));

	// Output redirections
	tab = new QWidget(this);
	g = new QGridLayout(tab,9,3,10,2);

	QLabel * l = new QLabel(__tr("Redirect to console"),tab);
	g->addMultiCellWidget(l,0,0,0,2);

	KviBoolSelector * b = new KviBoolSelector(tab,this,&(g_pOptions->m_bJoinMsgsToConsole),__tr("Join messages"));
	g->addWidget(b,1,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bPartMsgsToConsole),__tr("Part messages"));
	g->addWidget(b,2,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bKickMsgsToConsole),__tr("Kick messages"));
	g->addWidget(b,3,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bQuitMsgsToConsole),__tr("Quit messages"));
	g->addWidget(b,4,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bNickMsgsToConsole),__tr("Nick changes"));
	g->addWidget(b,5,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bTopicMsgsToConsole),__tr("Topic changes"));
	g->addWidget(b,6,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bDesyncMsgsToConsole),__tr("Local desync warnings"));
	g->addWidget(b,7,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bServerInfoToConsole),__tr("Server info"));
	g->addWidget(b,8,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bSplitWarningsToConsole),__tr("Split warnings"));
	g->addWidget(b,1,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bServerNoticesToConsole),__tr("Server notices"));
	g->addWidget(b,2,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bDnsResultsToConsole),__tr("DNS results"));
	g->addWidget(b,3,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bInviteMessagesToConsole),__tr("Invite messages"));
	g->addWidget(b,4,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUnhandledServerErrorsToConsole),__tr("Unhandled server errors"));
	g->addWidget(b,5,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUnhandledNumericsToConsole),__tr("Unhandled numerics"));
	g->addWidget(b,6,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUnhandledLiteralsToConsole),__tr("Unhandled literals"));
	g->addWidget(b,7,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bNicknameErrorsToConsole),__tr("Nickname errors"));
	g->addWidget(b,8,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bNamesReplyToConsole),__tr("Names reply"));

	g->addWidget(b,1,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUserModeChangesToConsole),__tr("User mode changes"));
	g->addWidget(b,2,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bChannelModeChangesToConsole),__tr("Channel mode changes"));
	g->addWidget(b,3,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bWhoMsgsToConsole),__tr("Who replies"));
	g->addWidget(b,4,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bBanAndExceptionListToConsole),__tr("Ban, invite and exception list"));
	g->addWidget(b,5,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bWhoisRepliesToConsole),__tr("Whois replies"));
	g->addWidget(b,6,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bNotifyListChangesToConsole),__tr("Notify list changes"));
	g->addWidget(b,7,2);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bProcessOutputToConsole),__tr("Process output"));
	g->addWidget(b,8,2);


	addTab(tab,__tr("Redirection"));

	// Irc stuff
	tab = new QWidget(this);
	g = new QGridLayout(tab,9,2,10,2);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowChannelSyncTime),__tr("Show channel sync time"));
	g->addWidget(b,0,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowPingPong),__tr("Show server pings"));
	g->addWidget(b,1,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowOwnParts),__tr("Show own parts"));
	g->addWidget(b,2,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowOwnNickChanges),__tr("Show own nick changes in the console too"));
	g->addWidget(b,3,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bSkipMotd),__tr("Ignore MOTD"));
	g->addWidget(b,4,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bBrutalQuit),__tr("Force 'brutal' QUIT"));
	g->addWidget(b,5,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoRejoinOnKick),__tr("Auto rejoin channel when kicked"));
	g->addWidget(b,6,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bDumpChannelOnKick),__tr("Dump channel status to file when kicked"));
	g->addWidget(b,7,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoJoinChanOnInvite),__tr("Auto join channel when invited"));
	g->addWidget(b,8,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bRequestBanExceptionListOnJoin),__tr("Request ban and invite exception lists on join"));
	g->addWidget(b,0,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bRequestWhoAfterEndOfNames),__tr("Request WHO #channel after end of names"));
	g->addWidget(b,1,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowInternalWhoMessages),__tr("Show internal WHO replies"));
	g->addWidget(b,2,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowBanAndExceptionList),__tr("Show internal ban and invite exception lists replies"));
	g->addWidget(b,3,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseSmallAntispamOnPrivmsg),__tr("Filter common spam messages"));
	g->addWidget(b,4,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCreateQueryOnPrivmsg),__tr("Create QUERY windows on remote PRIVMSG"));
	g->addWidget(b,5,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCreateQueryOnNotice),__tr("Create QUERY windows on remote NOTICE"));
	g->addWidget(b,6,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCreateIconifiedQuery),__tr("Create minimized QUERY windows"));
	g->addWidget(b,7,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowExtendedServerInfo),__tr("Show extended server info"));
	g->addWidget(b,8,1);

	addTab(tab,__tr("Irc engine"));

	// Ctcp configuration
	tab = new QWidget(this);
	g = new QGridLayout(tab,14,2,10,2);

	KviBoolSelector * b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseAntiCtcpFlood),__tr("Enable CTCP flood protection:"));
	g->addMultiCellWidget(b1,0,0,0,1);

	KviIntegerSelector * i = new KviIntegerSelector(tab,this,&(g_pOptions->m_iMaxCtcpCount),KviIntegerSelector::Int,
		__tr("Allow max"),g_pOptions->m_bUseAntiCtcpFlood,1,65535,6,100);
	connect(b1,SIGNAL(toggled(bool)),i,SLOT(setEnabled(bool)));
	g->addWidget(i,1,0);

	i = new KviIntegerSelector(tab,this,&(g_pOptions->m_iMaxCtcpTime),KviIntegerSelector::Int,
		__tr("CTCP's in (sec.)"),g_pOptions->m_bUseAntiCtcpFlood,1,65535,10,100);
	connect(b1,SIGNAL(toggled(bool)),i,SLOT(setEnabled(bool)));
	g->addWidget(i,1,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCtcpFloodWarningsToConsole),__tr("Ctcp flood warnings to console"),g_pOptions->m_bUseAntiCtcpFlood);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));
	g->addMultiCellWidget(b,2,2,0,1);

	QFrame * r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,3,3,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCtcpRequestsToConsole),__tr("Private CTCP requests to console"));
	g->addMultiCellWidget(b,4,4,0,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bChannelCtcpRequestsToConsole),__tr("Channel CTCP requests to console"));
	g->addMultiCellWidget(b,5,5,0,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCtcpRepliesToConsole),__tr("CTCP replies to console"));
	g->addMultiCellWidget(b,6,6,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,7,7,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpPingRequests),__tr("Ignore CTCP PING"));
	g->addWidget(b,8,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpVersionRequests),__tr("Ignore CTCP VERSION"));
	g->addWidget(b,8,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpTimeRequests),__tr("Ignore CTCP TIME"));
	g->addWidget(b,9,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpUserinfoRequests),__tr("Ignore CTCP USERINFO"));
	g->addWidget(b,9,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpClientinfoRequests),__tr("Ignore CTCP CLIENTINFO"));
	g->addWidget(b,10,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpSourceRequests),__tr("Ignore CTCP SOURCE"));
	g->addWidget(b,10,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bIgnoreCtcpFingerRequests),__tr("Ignore CTCP FINGER"));
	g->addWidget(b,11,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bReplyErrmsgOnUnknownCtcp),__tr("Reply CTCP ERRMSG on unknown ctcp requests"));
	g->addWidget(b,11,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,12,12,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAppendRealUsernameToUserinfoReply),__tr("Append real username to the USERINFO reply"));
	g->addMultiCellWidget(b,13,13,0,1);


	addTab(tab,__tr("Ctcp"));

	// Ctcp Multimedia

	tab = new QWidget(this);
	g = new QGridLayout(tab,7,1,10,2);

	b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bListenToCtcpMultimedia),__tr("Enable processing for CTCP MULTIMEDIA (SOUND)"));
	g->addWidget(b1,0,0);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,1,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bPlayDefaultFileWhenFileNotFound),__tr("Play the following file when the requested one is not found:"),g_pOptions->m_bListenToCtcpMultimedia);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));
	g->addWidget(b,2,0);

	KviFileSelector * fs = new KviFileSelector(tab,this,&(g_pOptions->m_szCtcpMultimediaDefaultFileName),g_pOptions->m_bListenToCtcpMultimedia);
	connect(b1,SIGNAL(toggled(bool)),fs,SLOT(setEnabled(bool)));
	g->addWidget(fs,3,0);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,4,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bSendMultimediaRequestWhenFileNotFound),__tr("Send \"PRIVMSG nick :!nick <filename>\" request when the file is not found"),g_pOptions->m_bListenToCtcpMultimedia);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));
	g->addWidget(b,5,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bListenToMultimediaFileRequests),__tr("Accept \"PRIVMSG nick :!nick <filename>\" requests"));
	g->addWidget(b,6,0);

	addTab(tab,__tr("Ctcp Multimedia"));


	// Mime types
	tab = new QWidget(this);
	g = new QGridLayout(tab,2,2,10,2);

	m_pMimeView = new KviListView(tab);
	m_pMimeView->addColumn(QString(__tr("Name")));
	m_pMimeView->addColumn(QString(__tr("File mask")));
	m_pMimeView->addColumn(QString(__tr("Magic bytes")));
	m_pMimeView->addColumn(QString(__tr("Commandline")));
	m_pMimeView->addColumn(QString(__tr("Save path")));
	m_pMimeView->addColumn(QString(__tr("Remote exec safe")));
//	m_pMimeView->setMinimumSize(m_pMimeView->sizeHint());
	m_pMimeView->setSorting(-1);
	g->addMultiCellWidget(m_pMimeView,0,0,0,1);

	QPushButton *pb = new QPushButton(QString(__tr("Add new mime")),tab);
	connect(pb,SIGNAL(clicked()),this,SLOT(addNewMime()));
	g->addWidget(pb,1,0);

	pb = new QPushButton(QString(__tr("Remove selected")),tab);
	connect(pb,SIGNAL(clicked()),this,SLOT(deleteSelectedMimes()));
	g->addWidget(pb,1,1);

	QListViewItem * it;

	for(KviMimeType * m=g_pOptions->m_pMimeManager->m_pMimeList->first();m;m=g_pOptions->m_pMimeManager->m_pMimeList->next()){
		it = new QListViewItem(m_pMimeView,
				QString(m->mimeName.ptr()),QString(m->fileMask.ptr()),
				QString(m->magicBytes.ptr()),QString(m->commandline.ptr()),
				QString(m->savePath.ptr()),m->remoteExecSafe ? QString("true") : QString("false"));
	}

	addTab(tab,__tr("Mime types"));

	// Logging
	tab = new QWidget(this);
	g = new QGridLayout(tab,10,1,10,2);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoLogConsole),__tr("Auto enable logging in console window"));
	g->addWidget(b,0,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoLogChannels),__tr("Auto enable logging in channel windows"));
	g->addWidget(b,1,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoLogQueries),__tr("Auto enable logging in query windows"));
	g->addWidget(b,2,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoLogChats),__tr("Auto enable logging in dcc chat windows"));
	g->addWidget(b,3,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoLogSends),__tr("Auto enable logging in dcc send windows"));
	g->addWidget(b,4,0);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,5,0);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAppendServerNameToLogName),__tr("Append server name to log file name (when possible)"));
	g->addWidget(b,6,0);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addWidget(r,7,0);


	l = new QLabel(QString(__tr("Log message types : double click an item to toggle logging")),tab);
	g->addWidget(l,8,0);

	m_pLogMsgTypesView = new QListView(tab);
	m_pLogMsgTypesView->addColumn(QString(__tr("Message type")));
	m_pLogMsgTypesView->addColumn(QString(__tr("Logging enabled")));
//	m_pLogMsgTypesView->setMinimumSize(m_pMimeView->sizeHint());
	m_pLogMsgTypesView->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	m_pLogMsgTypesView->setSorting(-1);
	m_pLogMsgTypesView->header()->setClickEnabled(false);
	m_pLogMsgTypesView->setMultiSelection(false);
	m_pLogMsgTypesView->setAllColumnsShowFocus(true);

	it = 0;
	// DO NOT TRANSLATE YES AND NO HERE!
	for(int i=0;i< KVI_OUT_NUM_IMAGES;i++){
		it = new QListViewItem(m_pLogMsgTypesView,
			QString(_i18n_translateNoText(messageTypeString[i])),
			(g_pOptions->m_bLogMsgType[i] ? QString("yes") : QString("no")) );
		it->setPixmap(0,(*g_pixViewOut[i]));
//		it->setSelectable(false);
	}

	connect(m_pLogMsgTypesView,SIGNAL(doubleClicked(QListViewItem *)),
		this,SLOT(logMsgTypeViewItemDoubleClicked(QListViewItem *)));


	g->addWidget(m_pLogMsgTypesView,9,0);
	g->setRowStretch(9,2);

	addTab(tab,__tr("Logging"));

	// Internals
	tab = new QWidget(this);
	g = new QGridLayout(tab,9,2,10,2);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseHackedFrameGeometry),__tr("Save hacked frame geometry"));
	g->addWidget(b,0,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bDialogShowHackForBrainDamagedWM),__tr("'Hide/Show' hack for brain damaged WM"));
	g->addWidget(b,1,0);

	b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bWindowsRememberProprietes),__tr("Windows remember properties"));
	g->addWidget(b1,0,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bWindowsRememberDockState),__tr("Windows remember dock state"),g_pOptions->m_bWindowsRememberProprietes);
	g->addWidget(b,1,1);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bWindowsRememberMaximizeState),__tr("Windows remember maximize state"),g_pOptions->m_bWindowsRememberProprietes);
	g->addWidget(b,2,1);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,3,3,0,1);


	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bBeVerbose),__tr("Verbose mode"));
	g->addWidget(b,4,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAutoDisableBrokenEvents),__tr("Auto-disable broken events"));
	g->addWidget(b,5,0);
	b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bPedanticParser),__tr("Pedantic parser (stop even on non fatal errors)"));
	g->addWidget(b1,4,1);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bAvoidParserWarnings),__tr("Avoid parser warnings"),!g_pOptions->m_bPedanticParser);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setNotEnabled(bool)));
	g->addWidget(b,5,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,6,6,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bDisableUpdateUserList),__tr("Disable updates to the internal userlist"));
	g->addMultiCellWidget(b,7,7,0,1);

	i = new KviIntegerSelector(tab,this,&(g_pOptions->m_iUserListUpdateTimeoutInSecs),KviIntegerSelector::Int,
		__tr("Userlist update interval (seconds)"),!g_pOptions->m_bDisableUpdateUserList,1,65535,10,100);
	connect(b,SIGNAL(toggled(bool)),i,SLOT(setNotEnabled(bool)));
	g->addMultiCellWidget(i,8,8,0,1);


	addTab(tab,__tr("Internals"));


	tab = new QWidget(this);
	g = new QGridLayout(tab,9,2,10,2);

	s = new KviStringSelector(tab,this,&(g_pOptions->m_szStringToAddAfterCompletedNick),__tr("Nick/mask completion postfix"),true,0,100);
	g->addMultiCellWidget(s,0,0,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bApplyCompletionPostfixForFirstWordOnly),__tr("Apply completion postfix to first word only"));
	g->addWidget(b,1,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bCompletionReplaceWholeWord),__tr("Replace whole word"));
	g->addWidget(b,1,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,2,2,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bInputHistoryCursorAtEnd),__tr("Warp input cursor to end when browsing command history"));
	g->addMultiCellWidget(b,3,3,0,1);

	KviCharSelector * ch = new KviCharSelector(tab,this,&(g_pOptions->m_cPersonalCommandPrefix),__tr("Personal command prefix"),true,100);
	g->addMultiCellWidget(ch,4,4,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,5,5,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseStringSubstitution),__tr("String substitution:"));
	g->addMultiCellWidget(b,6,6,0,1);

	m_pStrSubView = new KviListView(tab);
	m_pStrSubView->addColumn(QString(__tr("String")));
	m_pStrSubView->addColumn(QString(__tr("Convert to")));
	m_pStrSubView->setMinimumHeight((m_pStrSubView->fontMetrics().height())*4);
	m_pStrSubView->setSorting(-1);
	m_pStrSubView->setEnabled(g_pOptions->m_bUseStringSubstitution);
	connect(b,SIGNAL(toggled(bool)),m_pStrSubView,SLOT(setEnabled(bool)));
	g->addMultiCellWidget(m_pStrSubView,7,7,0,1);

	pb = new QPushButton(QString(__tr("Add new string")),tab);
	pb->setEnabled(g_pOptions->m_bUseStringSubstitution);
	connect(b,SIGNAL(toggled(bool)),pb,SLOT(setEnabled(bool)));
	connect(pb,SIGNAL(clicked()),this,SLOT(addNewString()));
	g->addWidget(pb,8,0);


	pb = new QPushButton(QString(__tr("Remove selected")),tab);
	pb->setEnabled(g_pOptions->m_bUseStringSubstitution);
	connect(b,SIGNAL(toggled(bool)),pb,SLOT(setEnabled(bool)));
	connect(pb,SIGNAL(clicked()),this,SLOT(deleteSelectedStrings()));
	g->addWidget(pb,8,1);

	for(KviStrSubItem * m=g_pOptions->m_pStrSub->m_pList->first();m;m=g_pOptions->m_pStrSub->m_pList->next()){
		it = new QListViewItem(m_pStrSubView,
				QString(m->szOriginal.ptr()),
				QString(m->szSubstitute.ptr()));
	}

	addTab(tab,__tr("Commandline input"));

	// View
	tab = new QWidget(this);
	g = new QGridLayout(tab,14,2,10,2);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bUrlHighlighting),__tr("Highlight URLs in output windows"));
	g->addMultiCellWidget(b,0,0,0,1);

	s = new KviStringSelector(tab,this,&(g_pOptions->m_szBrowserCommandline),__tr("Browser commandline (%=filename)"),
		g_pOptions->m_bUrlHighlighting);
	g->addMultiCellWidget(s,1,1,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,2,2,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bWordHighlighting),__tr("Highlight text that contain specified words (PRIVMSG & NOTICE only)"));
	g->addMultiCellWidget(b,3,3,0,1);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szHighlightWords),__tr("Words to highlight (use space as separator)"),
		g_pOptions->m_bWordHighlighting);
	g->addMultiCellWidget(s,4,4,0,1);
	connect(b,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,5,5,0,1);

	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bDoNotShowThePopupOnTextSelected),__tr("When selecting text , do not show the menu : copy to clipboard instead"));
	g->addMultiCellWidget(b,6,6,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,7,7,0,1);

	i = new KviIntegerSelector(tab,this,&(g_pOptions->m_iViewMaxBufferSize),KviIntegerSelector::Int,
		__tr("View widget buffer size (32-65535)"),true,32,65535,KVI_IRCVIEW_MAX_LINES,100);
	g->addMultiCellWidget(i,8,8,0,1);

	r = new QFrame(tab);
	r->setFrameStyle(QFrame::Sunken | QFrame::HLine);
	r->setMinimumHeight(r->frameWidth() * 2);
	g->addMultiCellWidget(r,9,9,0,1);

	l = new QLabel(__tr("[PREFIX]nick[!user@host][POSTFIX]This is the extended output format"),tab);
	g->addMultiCellWidget(l,10,10,0,1);

	b1 = new KviBoolSelector(tab,this,&(g_pOptions->m_bUseExtendedPrivmsgView),__tr("Use extended privmsg nick formatting"));
	g->addWidget(b1,11,0);
	b = new KviBoolSelector(tab,this,&(g_pOptions->m_bShowUserAndHostInPrivmsgs),__tr("Show user and host in privmsgs"),g_pOptions->m_bUseExtendedPrivmsgView);
	connect(b1,SIGNAL(toggled(bool)),b,SLOT(setEnabled(bool)));
	g->addWidget(b,11,1);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szExtPrivmsgPrefix),__tr("Nick format prefix"),g_pOptions->m_bUseExtendedPrivmsgView,0,100);
	connect(b1,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addMultiCellWidget(s,12,12,0,1);
	s = new KviStringSelector(tab,this,&(g_pOptions->m_szExtPrivmsgPostfix),__tr("Nick format postfix"),g_pOptions->m_bUseExtendedPrivmsgView,0,100);
	connect(b1,SIGNAL(toggled(bool)),s,SLOT(setEnabled(bool)));
	g->addMultiCellWidget(s,13,13,0,1);

	addTab(tab,__tr("View"));

#ifdef COMPILE_PLUGIN_SUPPORT
	// Plugins
	tab = new QWidget(this);
	g = new QGridLayout(tab,8,3,10,2);

	m_pPluginBox = new QListBox(tab);
	m_pPluginBox->setMultiSelection(false);
	connect(m_pPluginBox,SIGNAL(highlighted(const QString &)),this,SLOT(pluginListItemSelected(const QString &)));
	g->addMultiCellWidget(m_pPluginBox,0,5,0,0);

	QPushButton *load = new QPushButton(__tr("Load plugin"),tab);
	connect(load,SIGNAL(clicked()),this,SLOT(pluginLoad()));
	g->addWidget(load,6,0);

	m_pUnloadPlugin = new QPushButton(__tr("Unload plugin"),tab);
	connect(m_pUnloadPlugin,SIGNAL(clicked()),this,SLOT(pluginUnload()));
	g->addWidget(m_pUnloadPlugin,7,0);

	l = new QLabel(__tr("Module name:"),tab);
	g->addWidget(l,0,1);
	m_pPluginModuleNameLabel = new QLabel(tab);
	m_pPluginModuleNameLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	g->addWidget(m_pPluginModuleNameLabel,0,2);

	l = new QLabel(__tr("Service name:"),tab);
	g->addWidget(l,1,1);
	m_pPluginServiceNameLabel = new QLabel(tab);
	m_pPluginServiceNameLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	g->addWidget(m_pPluginServiceNameLabel,1,2);

	l = new QLabel(__tr("Author:"),tab);
	g->addWidget(l,2,1);
	m_pPluginAuthorLabel = new QLabel(tab);
	m_pPluginAuthorLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	g->addWidget(m_pPluginAuthorLabel,2,2);

	l = new QLabel(__tr("Version:"),tab);
	g->addWidget(l,3,1);
	m_pPluginVersionLabel = new QLabel(tab);
	m_pPluginVersionLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	g->addWidget(m_pPluginVersionLabel,3,2);

	l = new QLabel(__tr("Description:"),tab);
	l->setAlignment(AlignTop);
	g->addWidget(l,4,1);
	m_pPluginDescriptionLabel = new QLabel(tab);
	m_pPluginDescriptionLabel->setFrameStyle(QFrame::Panel|QFrame::Sunken);
	m_pPluginDescriptionLabel->setAlignment(AlignLeft|AlignTop|WordBreak);
	g->addWidget(m_pPluginDescriptionLabel,4,2);

	g->setRowStretch(4,2);

	m_pPluginAutoloadCheck = new QCheckBox(__tr("Autoload on startup"),tab);
	m_pPluginAutoloadCheck->setMinimumSize(m_pPluginAutoloadCheck->sizeHint());
	connect(m_pPluginAutoloadCheck,SIGNAL(toggled(bool)),this,SLOT(pluginAutoloadToggled(bool)));
	g->addMultiCellWidget(m_pPluginAutoloadCheck,5,5,1,2);

	m_pConfigurePlugin = new QPushButton(__tr("Configure plugin"),tab);
	connect(m_pConfigurePlugin,SIGNAL(clicked()),this,SLOT(pluginConfigure()));
	g->addMultiCellWidget(m_pConfigurePlugin,6,6,1,2);

	m_pHelpOnPlugin = new QPushButton(__tr("Plugin help"),tab);
	connect(m_pHelpOnPlugin,SIGNAL(clicked()),this,SLOT(pluginHelp()));
	g->addMultiCellWidget(m_pHelpOnPlugin,7,7,1,2);

	fillPluginBox();

	addTab(tab,__tr("Plugins"));
#endif

	setCancelButton(__tr("Cancel"));
	setDefaultButton(QString(__tr("What is this ?")));
	connect(this,SIGNAL(defaultButtonPressed()),g_pApp,SLOT(slot_whatIsThisRequest()));

}

KviDlgOptMisc::~KviDlgOptMisc()
{
}

void KviDlgOptMisc::done(int r)
{
	QTabDialog::done(r);
	if(r == Accepted){
		// Commit the mimetypes...
		while(g_pOptions->m_pMimeManager->m_pMimeList->first())g_pOptions->m_pMimeManager->m_pMimeList->removeFirst();
		QListViewItem * it = m_pMimeView->firstChild();
		while(it){
			KviMimeType * m = new KviMimeType;
			m->mimeName = it->text(0);
			m->mimeName.stripWhiteSpace();
			m->fileMask = it->text(1);
			m->fileMask.stripWhiteSpace();
			m->magicBytes = it->text(2);
			m->magicBytes.stripWhiteSpace();
			m->commandline = it->text(3);
			m->commandline.stripWhiteSpace();
			m->savePath = it->text(4);
			m->savePath.stripWhiteSpace();
			KviStr tmp = it->text(5);
			m->remoteExecSafe = (kvi_strEqualCI("true",tmp.ptr()) || kvi_strEqualCI("1",tmp.ptr()));
			if(m->mimeName.hasData() && m->fileMask.hasData())g_pOptions->m_pMimeManager->inSort(m);
			else delete m; //incomplete...unuseful...
			it = it->nextSibling();
		}
		// Commit the StrSub changes...
		while(g_pOptions->m_pStrSub->m_pList->first())g_pOptions->m_pStrSub->m_pList->removeFirst();
		it = m_pStrSubView->firstChild();
		while(it){
			KviStrSubItem *item = new KviStrSubItem;
			item->szOriginal = it->text(0);
			item->szOriginal.stripWhiteSpace();
			item->szSubstitute = it->text(1);
			item->szSubstitute.stripWhiteSpace();
			if(item->szOriginal.hasData() && item->szSubstitute.hasData())g_pOptions->m_pStrSub->m_pList->append(item);
			else delete item; //incomplete...unuseful...
			it = it->nextSibling();
		}
		// Commit the log msg types changes...
		it = m_pLogMsgTypesView->firstChild();
		while(it){
			// Can not rely on sorting order of the QListView..:(((
			KviStr tmp = it->text(0);
			for(int i = 0;tmp.hasData() && (i < KVI_OUT_NUM_IMAGES);i++){
				if(kvi_strEqualCI(_i18n_translateNoText(messageTypeString[i]),tmp.ptr())){
					tmp = it->text(1);
					g_pOptions->m_bLogMsgType[i] = kvi_strEqualCIN("yes",tmp.ptr(),3);
					tmp = "";
				}
			}
			if(tmp.hasData()){
				debug(__tr("Warning : can not update logging state for message type %s"),tmp.ptr());
			}
			it = it->nextSibling();
		}
	}
	emit finished((r == Accepted));
}

void KviDlgOptMisc::closeEvent(QCloseEvent *)
{
	// Finished without committing
	emit finished(false);
}

void KviDlgOptMisc::addNewString()
{
	(void) new QListViewItem(m_pStrSubView,
			QString(__tr("*")),QString("*"),
			QString(""),QString(""),
			QString(""));
}

void KviDlgOptMisc::deleteSelectedStrings()
{
	QListViewItem * it = m_pStrSubView->firstChild();
	while(it){
		QListViewItem * tmp = it;
		it = it->nextSibling();
		if(tmp->isSelected())delete tmp;
	}
}

void KviDlgOptMisc::addNewMime()
{
	(void) new QListViewItem(m_pMimeView,
			QString(__tr("__Unnamed_")),QString("*"),
			QString(""),QString(""),
			QString(""),QString("false"));
}

void KviDlgOptMisc::deleteSelectedMimes()
{
	QListViewItem * it = m_pMimeView->firstChild();
	while(it){
		QListViewItem * tmp = it;
		it = it->nextSibling();
		if(tmp->isSelected())delete tmp;
	}
}

void KviDlgOptMisc::logMsgTypeViewItemDoubleClicked(QListViewItem *it)
{
	if(!it)return;
	KviStr state = it->text(1);
	if(kvi_strEqualCIN("yes",state.ptr(),3)){
		it->setText(1,QString("no"));
	} else it->setText(1,QString("yes"));
}
#ifdef COMPILE_PLUGIN_SUPPORT

void KviDlgOptMisc::fillPluginBox(unsigned int selectIdx)
{
	m_pPluginBox->clear();
	QList<KviPluginData> *l=g_pPluginManager->pluginList();
	for(KviPluginData *d=l->first();d;d=l->next()){
		m_pPluginBox->insertItem(QString(d->filename.ptr()));
	}
	if(m_pPluginBox->count() == 0){
		m_pPluginModuleNameLabel->setText("");
		m_pPluginVersionLabel->setText("");
		m_pPluginServiceNameLabel->setText("");
		m_pPluginDescriptionLabel->setText("");
		m_pPluginAuthorLabel->setText("");
		m_pUnloadPlugin->setEnabled(false);
		m_pConfigurePlugin->setEnabled(false);
		m_pPluginAutoloadCheck->setEnabled(false);
		m_pHelpOnPlugin->setEnabled(false);
	} else m_pPluginBox->setCurrentItem(selectIdx);
}

#endif

void KviDlgOptMisc::pluginListItemSelected(const QString &str)
{
#ifdef COMPILE_PLUGIN_SUPPORT
	KviStr tmp = str;
	KviPluginData * d = g_pPluginManager->findPlugin(tmp.ptr());
	if(!d){
		debug("Warning...could not find plugin %s",tmp.ptr());
		return;
	}
	m_pPluginModuleNameLabel->setText(d->description->module_name);
	m_pPluginServiceNameLabel->setText(d->description->service_name);
	m_pPluginAuthorLabel->setText(d->description->author);
	m_pPluginVersionLabel->setText(d->description->version);
	m_pPluginDescriptionLabel->setText(d->description->description);
	m_pUnloadPlugin->setEnabled(true);
	m_pConfigurePlugin->setEnabled(d->description->config_routine);
	m_pPluginAutoloadCheck->setEnabled(true);
	m_pPluginAutoloadCheck->setChecked(d->bAutoload);
	m_pHelpOnPlugin->setEnabled(d->description->help_routine);
#endif
}

void KviDlgOptMisc::pluginAutoloadToggled(bool bActive)
{
#ifdef COMPILE_PLUGIN_SUPPORT
	KviStr tmp = m_pPluginBox->text(m_pPluginBox->currentItem());
	if(tmp.isEmpty())return;
	KviPluginData * d = g_pPluginManager->findPlugin(tmp.ptr());
	if(!d){
		debug("Warning...could not find plugin %s",tmp.ptr());	
		return;
	}
	d->bAutoload = bActive;
#endif
}

void KviDlgOptMisc::pluginUnload()
{
#ifdef COMPILE_PLUGIN_SUPPORT
	KviStr tmp = m_pPluginBox->text(m_pPluginBox->currentItem());
	if(tmp.isEmpty())return;
	if(!g_pPluginManager->unloadPlugin(tmp.ptr())){
		debug("Warning...could not find plugin %s",tmp.ptr());	
		return;
	}
	fillPluginBox();
#endif
}

void KviDlgOptMisc::pluginLoad()
{
#ifdef COMPILE_PLUGIN_SUPPORT
	KviStr tmp;
	g_pApp->getGlobalKVircDirectory(tmp,KviApp::Plugins);
	KviStr fName = QFileDialog::getOpenFileName(tmp.ptr(),"*.so");
	if(fName.hasData()){
		KviPluginData * d= g_pPluginManager->loadPlugin(fName.ptr());
		if(!d){
			g_pApp->warningBox(__tr("Could not load plugin\n%s"),fName.ptr());
			return;
		}

		KviPluginCommandStruct plgcmd;

		plgcmd.app     = g_pApp;
		plgcmd.handle  = d->handle;
		plgcmd.params  = 0;
		plgcmd.frame   = g_pApp->m_pFrameList->first();
		plgcmd.error   = KVI_ERROR_NoError;
		plgcmd.console = plgcmd.frame->m_pConsole;
		plgcmd.window  = plgcmd.console;
		plgcmd.sock    = plgcmd.frame->m_pSocket;

		if(!g_pPluginManager->executeInitRoutine(d,&plgcmd)){
			g_pApp->warningBox(__tr("Could not execute init routine for plugin\n%s"),fName.ptr());
			g_pPluginManager->unloadPlugin(d->filename.ptr());
			return;
		}
		fillPluginBox(m_pPluginBox->count());
	}
#endif
}

void KviDlgOptMisc::pluginConfigure()
{
#ifdef COMPILE_PLUGIN_SUPPORT
	KviStr tmp = m_pPluginBox->text(m_pPluginBox->currentItem());
	if(tmp.isEmpty())return;
	KviPluginData * d = g_pPluginManager->findPlugin(tmp.ptr());
	if(!d){
		debug("Warning...could not find plugin %s",tmp.ptr());	
		return;
	}
	if(d->description->config_routine){
		d->description->config_routine();
	}
#endif
}

void KviDlgOptMisc::pluginHelp()
{
#ifdef COMPILE_PLUGIN_SUPPORT
	KviStr tmp = m_pPluginBox->text(m_pPluginBox->currentItem());
	if(tmp.isEmpty())return;
	KviPluginData * d = g_pPluginManager->findPlugin(tmp.ptr());
	if(!d){
		debug("Warning...could not find plugin %s",tmp.ptr());	
		return;
	}
	if(d->description->help_routine){
		d->description->help_routine();
	}
#endif
}

#include "m_kvi_dlgoptions.moc"
