//
//   File : kvi_dlgscript.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_dlgscript.cpp)
//   Last major modification : Sun Sep 26 1999 20:45 by Till Busch
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//   Copyright (C) 1999-2000 Till Busch (buti@geocities.com)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DLG_SCRIPT_CPP_

#define _KVI_DEBUG_CHECK_RANGE_
#include "kvi_debug.h"

#include "kvi_dlgscript.h"
#include "kvi_options.h"
#include "kvi_defines.h"
#include "kvi_app.h"
#include "kvi_locale.h"

#include "kvi_scripteditor.h"
#include "kvi_alias.h"
#include "kvi_event.h"
#include "kvi_popupmenu.h"
#include "kvi_utoolbar.h"

#include "kvi_frame.h"
#include "kvi_uparser.h"
#include "kvi_console.h"

#include "kvi_varcache.h"
#include "kvi_style.h"

#include <qlayout.h>
#include <qevent.h>
#include <qsplitter.h>
#include <qheader.h>
#include <qnamespace.h>
#include <qpixmap.h>

/* FIXME : "missing quickhelp for ToolBarTab" */
//#warning "Totally missing quick help"
// declared in kvi_app.cpp
extern KviAliasManager *g_pAliasManager;
extern KviEventManager *g_pEventManager;

extern KviVariableCache *g_pVarCache;

extern KviPopupMenu *g_pChannelPopup;
extern KviPopupMenu *g_pUserlistPopup;
extern KviPopupMenu *g_pNotifylistPopup;
extern KviPopupMenu *g_pQueryPopup;
extern KviPopupMenu *g_pConsolePopup;
extern KviPopupMenu *g_pDccChatPopup;

extern KviUserToolBarTemplate *g_pUserToolBarTemplate;

extern QPixmap *g_pixViewOut[KVI_OUT_NUM_IMAGES];

int g_iLastSelectedScriptTab = -1;
int g_iLastSelectedAlias = -1;
int g_iLastSelectedEvent = -1;
int g_iLastSelectedPopup = -1;

QValidator::State KviAliasNameValidator::validate(QString & str,int &) const
{
	KviStr s = str;
	const char * p=s.ptr();
	while(*p){
		if((!isalnum(*p))&&(*p != '.')&&(*p != '_'))return QValidator::Invalid;
		p++;
	}
	return QValidator::Acceptable;
}


/*
	@quickhelp: KviDlgScript
	@widget: Script dialog
		Here you can edit your script:<br>
		Edit your aliases in the <a href="qh_qwidget_aliasestab.kvihelp">aliases tab</a>.<br>
		Edit your event handlers in the <a href="qh_qwidget_eventstab.kvihelp">events tab</a>.<br>
		Edit your popups in the <a href="qh_qwidget_popupstab.kvihelp">popups tab</a>.<br>
*/


KviDlgScript::KviDlgScript()
:QTabDialog()
{
	// Local copy of data
	m_pAliasManager = new KviAliasManager;
	m_pAliasManager->setAliasList(g_pAliasManager);

	m_pEventManager = new KviEventManager;
	m_pEventManager->copyEventHandlers(g_pEventManager);

	m_pChannelPopup = new KviPopupMenu;
	m_pChannelPopup->copyFromPopup(g_pChannelPopup);
	m_pQueryPopup = new KviPopupMenu;
	m_pQueryPopup->copyFromPopup(g_pQueryPopup);
	m_pNotifylistPopup = new KviPopupMenu;
	m_pNotifylistPopup->copyFromPopup(g_pNotifylistPopup);
	m_pUserlistPopup = new KviPopupMenu;
	m_pUserlistPopup->copyFromPopup(g_pUserlistPopup);
	m_pConsolePopup = new KviPopupMenu;
	m_pConsolePopup->copyFromPopup(g_pConsolePopup);
	m_pDccChatPopup = new KviPopupMenu;
	m_pDccChatPopup->copyFromPopup(g_pDccChatPopup);
	m_editMode = None;

	setCaption(__tr("Script center"));

	//
	// Aliases tab...
	//

	/*
		@quickhelp: QWidget_AliasesTab
		@widget: Aliases tab
			Here you edit your aliases.<br>
			To create a new alias press 'New' and insert the alias name
			in the input that will appear.<br>
			Then you can edit your alias code in the lower editor.<br>
			Please note that you CAN create an alias with a name that
			is already used by a KVIrc command , but you will NOT be able to use it.<br>
	*/

	QWidget * tab1 = new QWidget(this,"aliasestab");
	QGridLayout *g = new QGridLayout(tab1,3,3,10,4);

	m_pAliasBox = new QComboBox(false,tab1);
	connect(m_pAliasBox,SIGNAL(activated(int)),this,SLOT(aliasSelected(int)));
	g->addMultiCellWidget(m_pAliasBox,0,0,0,2);

	m_pAliasNew = new QPushButton(__tr("New"),tab1);
	connect(m_pAliasNew,SIGNAL(clicked()),this,SLOT(newAlias()));
	g->addWidget(m_pAliasNew,1,0);

	m_pAliasRename = new QPushButton(__tr("Rename"),tab1);
	connect(m_pAliasRename,SIGNAL(clicked()),this,SLOT(renameCurrentAlias()));
	g->addWidget(m_pAliasRename,1,1);

	m_pAliasDelete = new QPushButton(__tr("Delete"),tab1);
	connect(m_pAliasDelete,SIGNAL(clicked()),this,SLOT(deleteCurrentAlias()));
	g->addWidget(m_pAliasDelete,1,2);

	m_pAliasEditor = new KviScriptEditor(tab1);
	m_pAliasEditor->setFocusPolicy(StrongFocus);
	g->addMultiCellWidget(m_pAliasEditor,2,2,0,2);
	g->setRowStretch(2,1);

	m_pAliasName = new QLineEdit(tab1);
	m_pAliasName->hide();
	m_pAliasName->installEventFilter(this);
	m_pAliasName->setValidator(new KviAliasNameValidator(m_pAliasName));
//  THE SIGNAL BELOW SIMPLY CAN NOT BE USED IN DIALOG BOXES ! (at least in the beta Qt2.0)
//	connect(m_pAliasName,SIGNAL(returnPressed()),this,SLOT(leaveEditMode()));

	g->activate();

	addTab(tab1,__tr("Aliases"));

	//
	// Events tab...
	//

	/*
		@quickhelp: QWidget_EventsTab
		@widget: Events tab
			Here you edit your event handlers.<br>
			To edit an event handler select its name in the left list box,
			and then enable it.<br>
	*/

	QWidget *tab2 = new QWidget(this,"eventstab");
	g = new QGridLayout(tab2,2,3,10,4);

	m_pEventBox = new QListBox(tab2);
	m_pEventBox->setMinimumWidth(130);
	connect(m_pEventBox,SIGNAL(highlighted(int)),this,SLOT(eventSelected(int)));
	g->addMultiCellWidget(m_pEventBox,0,2,0,0);

	m_pEventLabel = new QLabel(__tr("No event selected"),tab2);
	m_pEventLabel->setFrameStyle(QFrame::Sunken|QFrame::Panel);
	g->addWidget(m_pEventLabel,0,1);

	m_pEventEnable = new QCheckBox(__tr("Enabled"),tab2);
	connect(m_pEventEnable,SIGNAL(clicked()),this,SLOT(eventEnableClicked()));
	g->addWidget(m_pEventEnable,1,1);

	m_pEventEditor = new KviScriptEditor(tab2);
	g->addWidget(m_pEventEditor,2,1);

	g->setRowStretch(2,1);
	g->setColStretch(1,1);

	g->activate();

	addTab(tab2,__tr("Events"));

	//
	// Popups
	//

	/*
		@quickhelp: QWidget_PopupsTab
		@widget: Popups tab
			Here you can edit your popups.<br>
			In the upper list box you can select the popup to edit:<br>
			Channel : is shown when you right click in the channel window.<br>
			Query : is shown when you right click in a query window.<br>
			Console : is shown when you right click in the console window.<br>
			Notifylist : is shown when you right click on a nickname in the right pane of the console window (notify list).<br>
			Userlist : is shown when you right click on a nickname in the right pane of a channel window (user list box).<br>
			The lower tree view shows you the current popup.<br>
			A right mouse click in that tree view shows a menu that allows adding/removing
			items from the popup.<br>
			In the lower editor you edit the popup code associated with the selected popup item.<br>
			In the 'icon' line edit you can place a NUMBER that describes the icon to use with the popup (see <a href="echo.kvihelp">echo</a> -i option).<br>
			The 'icon' can be also an absolute or relative (to the kvirc/pics directory) file name.<br>
			Try right clicking in that edit , it will show a helper menu.<br>
			The 'name' line edit contains the current item name.<br>
			The test button allows you to test your popup in the current state: by clicking an item
			in that popup you will select it in the tree view.<br>
	*/

	QWidget *tab3 = new QWidget(this,"popupstab");
	g = new QGridLayout(tab3,4,3,10,4);

	QPushButton * b = new QPushButton(QString(__tr("Test")),tab3);
	g->addWidget(b,0,2);
	connect(b,SIGNAL(clicked()),this,SLOT(testCurrentPopup()));


	m_pPopupsBox = new QComboBox(false,tab3);
	m_pPopupsBox->insertItem(QString("Channel"));
	m_pPopupsBox->insertItem(QString("Console"));
	m_pPopupsBox->insertItem(QString("DccChat"));
	m_pPopupsBox->insertItem(QString("Notifylist"));
	m_pPopupsBox->insertItem(QString("Query"));
	m_pPopupsBox->insertItem(QString("Userlist"));
	g->addMultiCellWidget(m_pPopupsBox,0,0,0,1);

	connect(m_pPopupsBox,SIGNAL(activated(int)),this,SLOT(popupComboSelectionChanged(int)));

	QSplitter * splitter = new QSplitter(Qt::Vertical,tab3);

	m_pPopupView = new QListView(splitter);
	m_pPopupView->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	m_pPopupView->addColumn(QString("Item"));
	m_pPopupView->addColumn(QString("Type"));
	m_pPopupView->setRootIsDecorated(true);
	m_pPopupView->setMultiSelection(false);
	m_pPopupView->header()->setClickEnabled(false);
	m_pPopupView->setAllColumnsShowFocus(true);
	m_pPopupView->viewport()->installEventFilter(this); // Need to adjust the behaviour of mousePressEvent

	m_pPopupPopup = new QPopupMenu(this);

	connect(m_pPopupView,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(popupItemSelected(QListViewItem *)));
	connect(m_pPopupView,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),
			this,SLOT(popupViewRightButtonPressed(QListViewItem *,const QPoint &,int)));

	m_pPopupEditor = new KviScriptEditor(splitter);
	g->addMultiCellWidget(splitter,1,1,0,2);

	m_pIconPopup = new QPopupMenu(this);
	QPopupMenu *ppp=m_pIconPopup;
	connect(ppp,SIGNAL(activated(int)),this,SLOT(iconPopupActivated(int)));

	for(int i=0;i<KVI_OUT_NUM_IMAGES;i++){
		ppp->insertItem(*(g_pixViewOut[i]),i);
		if(((i % 10) == 0)&&(i != 0)){
			QPopupMenu * sub = new QPopupMenu(ppp);
			ppp->insertItem(__tr("More..."),sub);
			ppp = sub;
			connect(ppp,SIGNAL(activated(int)),this,SLOT(iconPopupActivated(int)));
		}
	}

	QLabel *l=new QLabel(QString("Icon:"),tab3);
	g->addWidget(l,2,0);

	m_pPopupIconEditor = new QLineEdit(tab3);
	m_pPopupIconEditor->installEventFilter(this);

	g->addMultiCellWidget(m_pPopupIconEditor,2,2,1,2);

	l = new QLabel(QString("Name:"),tab3);
	g->addWidget(l,3,0);

	m_pPopupNameEditor = new QLineEdit(tab3);
	g->addMultiCellWidget(m_pPopupNameEditor,3,3,1,2);

	g->setColStretch(1,10);

	g->activate();

	addTab(tab3,__tr("Popups"));
//========================
	//
	// ToolBar
	//
	/*
		@quickhelp: QWidget_ToolBarTab
		@widget: Toolbar Tab
			Here you can edit your toolbar.<br>
	*/

	QWidget *tab4 = new QWidget(this,"toolbartab");
	g = new QGridLayout(tab4,5,3,10,4);

	m_pToolBar = new KviUserToolBar((KviFrame *)(g_pApp->m_pFrameList->first()), tab4);
	m_pToolBar->copyFromTemplate(g_pUserToolBarTemplate, false);
//	QWidget *w = new QWidget(m_pToolBar,"bar");
	m_pToolBar->setStretchableWidget(new KviTransparentWidget(m_pToolBar));
	g->addMultiCellWidget(m_pToolBar,0,0,0,2);

	splitter = new QSplitter(Qt::Vertical,tab4);

	m_pToolBarView = new QListView(splitter);
	m_pToolBarView->setFrameStyle(QFrame::WinPanel|QFrame::Sunken);
	m_pToolBarView->addColumn(QString(__tr("Item")));
	m_pToolBarView->addColumn(QString(__tr("Type")));
	m_pToolBarView->setRootIsDecorated(true);
	m_pToolBarView->setMultiSelection(false);
	m_pToolBarView->header()->setClickEnabled(false);
	m_pToolBarView->setAllColumnsShowFocus(true);
	m_pToolBarView->viewport()->installEventFilter(this); // Need to adjust the behaviour of mousePressEvent

	m_pToolBarPopup = new QPopupMenu(this);

	connect(m_pToolBarView,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(toolButtonSelected(QListViewItem *)));
	connect(m_pToolBarView,SIGNAL(rightButtonPressed(QListViewItem *,const QPoint &,int)),
			this,SLOT(toolBarViewRightButtonPressed(QListViewItem *,const QPoint &,int)));

	m_pToolBarEditor = new KviScriptEditor(splitter);
	g->addMultiCellWidget(splitter,1,1,0,2);


	l=new QLabel(QString("Icon:"),tab4);
	g->addWidget(l,2,0);

	m_pToolBarIconEditor = new QLineEdit(tab4);
	m_pToolBarIconEditor->installEventFilter(this);

	g->addMultiCellWidget(m_pToolBarIconEditor,2,2,1,2);

	l = new QLabel(QString("Name:"),tab4);
	g->addWidget(l,3,0);

	m_pToolBarNameEditor = new QLineEdit(tab4);
	g->addMultiCellWidget(m_pToolBarNameEditor,3,3,1,2);

	l=new QLabel(QString("Statusbar:"),tab4);
	g->addWidget(l,4,0);

	m_pToolBarStatusBarEditor = new QLineEdit(tab4);
	g->addMultiCellWidget(m_pToolBarStatusBarEditor,4,4,1,2);


	g->setColStretch(1,10);

	g->activate();
	addTab(tab4,__tr("ToolBar"));

	/*
		@quickhelp: QWidget_VariablesTab
		@widget: Variables tab
			This tab shows an automatically generated script buffer.<br>
			It contains all the currently set global variables.<br>
			You can edit it to set the variable contents to match your test needs.<br>
			This script buffer will be executed only if you press OK in the dialog.<br>
			Please note that if you want to unset a variable , you have to
			remove the part after the '=' sign and NOT remove the variable name.<br>
	*/

	QWidget *tab5 = new QWidget(this,"variablestab");
	g = new QGridLayout(tab5,1,1,10,4);

	m_pVariablesEditor = new KviScriptEditor(tab5);
	g->addWidget(m_pVariablesEditor,0,0);
	fillVariablesEditor();
	m_bVariablesChanged = false;
	connect(m_pVariablesEditor,SIGNAL(textChanged()),this,SLOT(variablesTextChanged()));

	g->activate();
	addTab(tab5,__tr("Global variables"));


//============ Main Buttons ===================

	setCancelButton(__tr("Cancel"));
	setDefaultButton(QString(__tr("What is this ?")));
	connect(this,SIGNAL(defaultButtonPressed()),g_pApp,SLOT(slot_whatIsThisRequest()));

	//poor hack to catch the return keypress
	installEventFilter(this);

	m_bInitialSetupPhase = true;

	fillAliasesBox();
	selectAlias(g_iLastSelectedAlias);
	fillEventsBox();
	selectEvent(g_iLastSelectedEvent);

	m_pEditedMenu = 0;
	m_pEditedToolBar = 0;
	if((g_iLastSelectedPopup < 0)||(g_iLastSelectedPopup > 5))g_iLastSelectedPopup = 0;
	m_pPopupsBox->setCurrentItem(g_iLastSelectedPopup);
	popupComboSelectionChanged(g_iLastSelectedPopup);

	m_bInitialSetupPhase = false;

	setGeometry(g_pOptions->m_rectScriptDialog);

	switch(g_iLastSelectedScriptTab){
		case 0: showPage(tab1); break;
		case 1: showPage(tab2); break;
		case 2: showPage(tab3); break;
		case 3: showPage(tab4); break;
		case 4: showPage(tab5); break;
		default: /* nothing */  break;
	}
	startEditToolBar(m_pToolBar);
}

//============ ~KviDlgScript() ===================

KviDlgScript::~KviDlgScript()
{
	if(g_pOptions->m_bUseHackedFrameGeometry){
		g_pOptions->m_rectScriptDialog.moveTopLeft(frameGeometry().topLeft());
		g_pOptions->m_rectScriptDialog.setSize(size());
	} else g_pOptions->m_rectScriptDialog = geometry();

	delete m_pAliasManager;
	delete m_pEventManager;
	delete m_pConsolePopup;
	delete m_pChannelPopup;
	delete m_pQueryPopup;
	delete m_pUserlistPopup;
	delete m_pNotifylistPopup;
	delete m_pDccChatPopup;
	delete m_pIconPopup;
	delete m_pToolBar;
}

//============ KviDlgScript::done ===================

void KviDlgScript::done(int r)
{
	QTabDialog::done(r);
	// Copy the modified data
	if(r == Accepted){
		QWidget * w= currentPage();
		if(w){
			KviStr tmp = w->name();
			if(kvi_strEqualCI(tmp.ptr(),"aliasestab"))g_iLastSelectedScriptTab = 0;
			else if(kvi_strEqualCI(tmp.ptr(),"eventstab"))g_iLastSelectedScriptTab = 1;
			else if(kvi_strEqualCI(tmp.ptr(),"popupstab"))g_iLastSelectedScriptTab = 2;
			else if(kvi_strEqualCI(tmp.ptr(),"toolbartab"))g_iLastSelectedScriptTab = 3;
			else if(kvi_strEqualCI(tmp.ptr(),"variablestab"))g_iLastSelectedScriptTab = 4;
		} else g_iLastSelectedScriptTab = -1;

		saveLastSelectedAlias();
		saveLastSelectedEvent();
		if(m_pEditedMenu)saveMenu(m_pEditedMenu);
		if(m_pEditedToolBar)saveToolBar(m_pEditedToolBar);
		g_iLastSelectedPopup = m_pPopupsBox->currentItem();

		g_pChannelPopup->copyFromPopup(m_pChannelPopup);
		g_pQueryPopup->copyFromPopup(m_pQueryPopup);
		g_pNotifylistPopup->copyFromPopup(m_pNotifylistPopup);
		g_pUserlistPopup->copyFromPopup(m_pUserlistPopup);
		g_pConsolePopup->copyFromPopup(m_pConsolePopup);
		g_pDccChatPopup->copyFromPopup(m_pDccChatPopup);

		g_pUserToolBarTemplate->copyFromToolBar(m_pEditedToolBar);

		g_pAliasManager->setAliasList(m_pAliasManager);
		g_pEventManager->copyEventHandlers(m_pEventManager);
		g_pEventManager->disableEmptyHandlers();
		if(m_bVariablesChanged){
			KviStr tmp = m_pVariablesEditor->text();
			// any frame can parse this
			KviFrame * aFrame = g_pApp->m_pFrameList->first();
			if(aFrame)aFrame->m_pUserParser->parseCommand(tmp.ptr(),aFrame->m_pConsole);
		}
	}
	emit finished((r == Accepted));
}

//============ KviDlgScript::closeEvent ===================

void KviDlgScript::closeEvent(QCloseEvent *)
{
	emit finished(false);
}

//===========================================================================//
// ALIASES SECTION
//

void KviDlgScript::newAlias()
{
	saveLastSelectedAlias();
	m_editMode = NewAlias;
	enterEditMode("");
}

void KviDlgScript::renameCurrentAlias()
{
	saveLastSelectedAlias();
	m_editMode = RenameAlias;
	int curItem = m_pAliasBox->currentItem();
	KviAlias *a = m_pAliasManager->m_pAliasList->at(curItem);
	__range_valid(a);
	enterEditMode(a->szName.ptr());
}

void KviDlgScript::enterEditMode(const char *startText)
{
	m_pAliasName->setGeometry(m_pAliasBox->geometry());
	m_pAliasBox->hide();
	m_pAliasName->setText(startText);
	m_pAliasName->show();
	m_pAliasName->setFocus();
}

bool KviDlgScript::eventFilter(QObject * o,QEvent *e)
{
	// Poor hack
	if(o == m_pPopupIconEditor || o == m_pToolBarIconEditor){
		if(e->type() == QEvent::MouseButtonPress){
			if(((QMouseEvent *)e)->button() & RightButton){
				m_pIconPopup->popup(QCursor::pos());
				return true;
			}
		}
		return false;
	}

	if(o == this){
		if((e->type() == QEvent::Accel)||(e->type() == QEvent::KeyPress)){
			if(m_pAliasName->isVisible()){
				if(((QKeyEvent *)e)->key() == Qt::Key_Return){
					((QKeyEvent *)e)->accept();
					leaveEditMode();
					return true;
				}
			}
		}
		if(o == m_pAliasName){
			if(e->type() == QEvent::FocusOut)leaveEditMode();
			else return QTabDialog::eventFilter(o,e);
			return true;
		}
	} else if(o == m_pPopupView->viewport()){
		if(e->type() == QEvent::MouseButtonPress){
			if(m_pPopupView->firstChild() == 0){
				popupViewRightButtonPressed(0,m_pPopupView->mapToGlobal(((QMouseEvent *)e)->pos()),0);
				return true;
			}
		}
	} else if(o == m_pToolBarView->viewport()){
		if(e->type() == QEvent::MouseButtonPress){
			if(m_pToolBarView->firstChild() == 0){
				toolBarViewRightButtonPressed(0,m_pToolBarView->mapToGlobal(((QMouseEvent *)e)->pos()),0);
				return true;
			}
		}
	}
	return QTabDialog::eventFilter(o,e);
}

void KviDlgScript::leaveEditMode()
{
	if(m_editMode == None)return;
	m_pAliasName->hide();
	m_pAliasBox->show();
	KviAlias * a;
	if(m_editMode == NewAlias){
		a = new KviAlias;
		a->szName = m_pAliasName->text();
		if(a->szName.isEmpty())a->szName = __tr("_unnamed_");
		a->szBuffer = __tr("# Place here the alias code");
		while(m_pAliasManager->findAlias(a->szName.ptr()))a->szName+="_";
		m_pAliasManager->addAlias(a);
		fillAliasesBox();
		int index = m_pAliasManager->m_pAliasList->findRef(a);
		__range_valid(index != -1);
		g_iLastSelectedAlias = -1;
		selectAlias(index);
	} else {
		int curItem = m_pAliasBox->currentItem();
		a = m_pAliasManager->m_pAliasList->at(curItem);
		m_pAliasManager->m_pAliasList->setAutoDelete(false);
		m_pAliasManager->m_pAliasList->removeRef(a);
		m_pAliasManager->m_pAliasList->setAutoDelete(true);
		a->szName = m_pAliasName->text();
		if(a->szName.isEmpty())a->szName = __tr("_unnamed_");
		while(m_pAliasManager->findAlias(a->szName.ptr()))a->szName+="_";
		m_pAliasManager->addAlias(a);
		fillAliasesBox();
		int index = m_pAliasManager->m_pAliasList->findRef(a);
		__range_valid(index != -1);
//		g_iLastSelectedAlias = -1;
		selectAlias(index);
	}
	m_editMode = None;
}

void KviDlgScript::fillAliasesBox()
{
	m_pAliasBox->clear();
	for(KviAlias *a=m_pAliasManager->m_pAliasList->first();a;a=m_pAliasManager->m_pAliasList->next()){
		m_pAliasBox->insertItem(QString(a->szName.ptr()));
	}
	m_pAliasBox->setEnabled(m_pAliasBox->count() != 0);
}

void KviDlgScript::saveLastSelectedAlias()
{
	if((g_iLastSelectedAlias < 0)||(g_iLastSelectedAlias >= m_pAliasManager->count())||m_bInitialSetupPhase)return;
	KviAlias * a = m_pAliasManager->m_pAliasList->at(g_iLastSelectedAlias);
	a->szBuffer = m_pAliasEditor->text();
}

void KviDlgScript::selectAlias(int index)
{
	if(index < 0)index = 0; //Try with the first one
	if(index >= m_pAliasManager->count()){
		m_pAliasBox->setEnabled(false);
		m_pAliasDelete->setEnabled(false);
		m_pAliasRename->setEnabled(false);
		m_pAliasEditor->setEnabled(false);
		m_pAliasEditor->setText(QString(""));
		return;
	}
	m_pAliasBox->setCurrentItem(index);
	aliasSelected(index);
}

void KviDlgScript::aliasSelected(int index)
{
	saveLastSelectedAlias();
	m_pAliasBox->setEnabled(true);
	m_pAliasDelete->setEnabled(true);
	m_pAliasRename->setEnabled(true);
	m_pAliasEditor->setEnabled(true);
	__range_valid((index >= 0)&&(index < m_pAliasManager->count()));
	KviAlias * a = m_pAliasManager->m_pAliasList->at(index);
	__range_valid(a);
	m_pAliasEditor->setText(QString(a->szBuffer.ptr()));
	g_iLastSelectedAlias = index;
	m_pAliasEditor->setFocus();
}

void KviDlgScript::deleteCurrentAlias()
{
	int curAlias = m_pAliasBox->currentItem();
	if(curAlias < 0)return;
	__range_valid((curAlias >= 0)&&(curAlias < m_pAliasManager->count()));
	KviAlias * a = m_pAliasManager->m_pAliasList->at(curAlias);
#ifdef _KVI_DEBUG_CHECK_RANGE_
	KviStr check = m_pAliasBox->text(curAlias);
	__range_valid(check == a->szName);
#endif
	m_pAliasManager->m_pAliasList->removeRef(a);
	fillAliasesBox();
	g_iLastSelectedAlias = -1;
	selectAlias(-1);
}

//===========================================================================//
// EVENTS SECTION
//

void KviDlgScript::fillEventsBox()
{
	disconnect(m_pEventBox,SIGNAL(highlighted(int)),this,SLOT(eventSelected(int))); //HACK
	m_pEventBox->clear();
	m_pEventBox->setAutoUpdate(false);
	QString tmp;
	for(int i = 0;i< KVI_NUM_EVENTS;i++){
		tmp = m_pEventManager->m_eventDescriptor[i].eventName;
		if(m_pEventManager->m_bEventEnabled[i])tmp+=" *";
		m_pEventBox->insertItem(tmp);
	}
	m_pEventBox->setAutoUpdate(true);
	m_pEventBox->repaint(); //Damn ! here it fires the eventSelected(0)
	connect(m_pEventBox,SIGNAL(highlighted(int)),this,SLOT(eventSelected(int))); //HACK
}

void KviDlgScript::selectEvent(int index)
{
	if(index < 0)index = 0; //Try with the first one
	if(index >= KVI_NUM_EVENTS){
		m_pEventEditor->setText(QString(""));
		m_pEventEditor->setEnabled(false);
		m_pEventEnable->setEnabled(false);
		return;
	}
	m_pEventBox->setCurrentItem(index);
	m_pEventBox->centerCurrentItem();
	eventSelected(index);
}

void KviDlgScript::eventSelected(int index)
{
	saveLastSelectedEvent();
	__range_valid((index >= 0)&&(index < KVI_NUM_EVENTS));
	m_pEventEnable->setChecked(m_pEventManager->m_bEventEnabled[index]);
	m_pEventEnable->setEnabled(true);
	m_pEventEditor->setText(QString(m_pEventManager->m_szEventBuffer[index].ptr()));
	m_pEventEditor->setEnabled(m_pEventEnable->isChecked());
	QString params(__tr("Params:"));
	params+=m_pEventManager->m_eventDescriptor[index].eventParams;
	m_pEventLabel->setText(params);
	g_iLastSelectedEvent = index;
	m_pEventEditor->setFocus();
}

void KviDlgScript::saveLastSelectedEvent()
{
	if((g_iLastSelectedEvent < 0)||(g_iLastSelectedEvent >= KVI_NUM_EVENTS)||m_bInitialSetupPhase)return;
	m_pEventManager->m_szEventBuffer[g_iLastSelectedEvent] = m_pEventEditor->text();
	m_pEventManager->m_bEventEnabled[g_iLastSelectedEvent] = m_pEventEnable->isChecked();
}

void KviDlgScript::eventEnableClicked()
{
	if((g_iLastSelectedEvent < 0)||(g_iLastSelectedEvent >= KVI_NUM_EVENTS))return;
	m_pEventManager->m_bEventEnabled[g_iLastSelectedEvent] = m_pEventEnable->isChecked();
	fillEventsBox();
	selectEvent(g_iLastSelectedEvent);
}

//===================================================================================================
// POPUPS STUFF
//

KviPopupListViewItem::KviPopupListViewItem(PopupItemType t,KviPopupListViewItem * parent,const char * name,const char *type,const char *buffer,const char *icon)
:QListViewItem(parent,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}

KviPopupListViewItem::KviPopupListViewItem(PopupItemType t,KviPopupListViewItem * parent,KviPopupListViewItem * after,const char * name,const char *type,const char *buffer,const char *icon)
:QListViewItem(parent,after,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}
KviPopupListViewItem::KviPopupListViewItem(PopupItemType t,QListView * parent,const char *name,const char *type,const char *buffer,const char *icon)
:QListViewItem(parent,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}
KviPopupListViewItem::KviPopupListViewItem(PopupItemType t,QListView * parent,KviPopupListViewItem * after,const char *name,const char *type,const char *buffer,const char *icon)
:QListViewItem(parent,after,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}
KviPopupListViewItem::~KviPopupListViewItem()
{
}
void KviPopupListViewItem::tryLoadImage()
{
	m_icon.stripWhiteSpace();
	if(m_icon.hasData()){
		if(m_icon.isNum()){
			bool bOk=false;
			int imId = m_icon.toInt(&bOk);
			if(imId >= 0){
				imId = imId % KVI_OUT_NUM_IMAGES;
				setPixmap(0,*(g_pixViewOut[imId]));
			}
		} else {
			KviStr imagePath;
			if(g_pApp->findImage(imagePath,m_icon.ptr())){
				QPixmap pix(imagePath.ptr());
				if(!pix.isNull()){
					setPixmap(0,pix);
				} else debug("Unable to find image file %s",m_icon.ptr());
			}
		}
	}
}


void KviDlgScript::popupComboSelectionChanged(int index)
{
	switch(index){
		case 0: startEditPopup(m_pChannelPopup); break;
		case 1: startEditPopup(m_pConsolePopup); break;
		case 2: startEditPopup(m_pDccChatPopup); break;
		case 3: startEditPopup(m_pNotifylistPopup); break;
		case 4: startEditPopup(m_pQueryPopup); break;
		case 5: startEditPopup(m_pUserlistPopup); break;
		default: /* newer here */ break;
	}
}

void KviDlgScript::startEditPopup(KviPopupMenu *p)
{
	if(m_pEditedMenu)saveMenu(m_pEditedMenu);
	m_pEditedMenu = p;
	m_pPopupView->clear();
	m_pPopupView->setSorting(-1);
	appendChildPopup(0,p);
	m_pEditedPopupItem = 0;
	if(m_pPopupView->firstChild() == 0)disablePopupEditors();
	else m_pPopupView->setSelected(m_pPopupView->firstChild(),true);
}

void KviDlgScript::testCurrentPopup()
{
	if(m_pEditedMenu){
		saveMenu(m_pEditedMenu);
		m_pEditedMenu->doTestPopup(this,SLOT(testPopupSlot(int)));
	}
}

void KviDlgScript::testPopupSlot(int id)
{
	if(!selectPopupItemById(0,id))debug("oops...popup item not found in the list view ?");
}

bool KviDlgScript::selectPopupItemById(KviPopupListViewItem * par,int id)
{
	KviPopupListViewItem *it = (KviPopupListViewItem *)(par ? par->firstChild() : m_pPopupView->firstChild());
	while(it){
		if(it->m_type == KviPopupListViewItem::Item){
			if(it->m_id == id){
				// open all parents
				par = (KviPopupListViewItem *)it->parent();
				while(par){
					par->setOpen(true);
					par = (KviPopupListViewItem *)par->parent();
				}
				// select it
				m_pPopupView->setSelected(it,true);
				// and ensure it is visible
				m_pPopupView->ensureItemVisible(it);
				return true;
			}
		} else if(it->m_type == KviPopupListViewItem::Submenu){
			if(selectPopupItemById(it,id))return true;
		}
		it = (KviPopupListViewItem *)it->nextSibling();
	}
	return false;
}

void KviDlgScript::saveMenu(KviPopupMenu *p)
{
	if(m_pEditedPopupItem)savePopupItem(m_pEditedPopupItem); // do not forget to save the last one
	p->clearAll();
	saveSubmenu(p,0);
}

void KviDlgScript::saveSubmenu(KviPopupMenu *p,KviPopupListViewItem *par)
{
	KviPopupListViewItem *it = (KviPopupListViewItem *)(par ? par->firstChild() : m_pPopupView->firstChild());
	while(it){
		if(it->m_type == KviPopupListViewItem::Separator){
			p->insertSeparatorItem();
			it->m_id = 0;
		} else {
			if(it->m_type == KviPopupListViewItem::Item){
				KviStr tmp = it->text(0);
				tmp.stripWhiteSpace();
				if(tmp.isEmpty())tmp = __tr("Unnamed");
				it->m_id = p->insertNormalItem(tmp.ptr(),it->m_icon.ptr(),it->m_buffer.ptr());
			} else {
				KviPopupMenu *sub = new KviPopupMenu();
				saveSubmenu(sub,it);
				KviStr tmp = it->text(0);
				tmp.stripWhiteSpace();
				if(tmp.isEmpty())tmp = __tr("Unnamed");
				p->insertPopupItem(tmp.ptr(),it->m_icon.ptr(),sub);
				it->m_id = 0;
			}
		}
		it = (KviPopupListViewItem *)it->nextSibling();
	}
}

void KviDlgScript::disablePopupEditors()
{
	m_pPopupEditor->setText(QString(""));
	m_pPopupIconEditor->setText(QString(""));
	m_pPopupNameEditor->setText(QString(""));
	m_pPopupEditor->setEnabled(false);
	m_pPopupIconEditor->setEnabled(false);
	m_pPopupNameEditor->setEnabled(false);
}

void KviDlgScript::iconPopupActivated(int id)
{
	if(currentPage())
		if(kvi_strEqualCSN(((currentPage())->name()),"popupstab", 9)) {
			if((id >= 0)&&(id <= KVI_OUT_NUM_IMAGES) && m_pPopupIconEditor->isEnabled()){
				QString tmp;
				tmp.setNum(id);
				m_pPopupIconEditor->setText(tmp);
				if(m_pEditedPopupItem)savePopupItem(m_pEditedPopupItem);
				return;
			}
		} else {
			if((id >= 0)&&(id <= KVI_OUT_NUM_IMAGES) && m_pToolBarIconEditor->isEnabled()) {
				QString tmp;
				tmp.setNum(id);
				m_pToolBarIconEditor->setText(tmp);
				if(m_pEditedToolButton)saveToolButton(m_pEditedToolButton);
			}
		}
}

void KviDlgScript::appendChildPopup(KviPopupListViewItem * parent,KviPopupMenu *p)
{
	KviPopupListViewItem *it = 0;
	for(KviPopupMenuData * d = p->m_pDataList->first();d;d=p->m_pDataList->next()){
		if(d->menu != 0){
			KviStr tmp(p->text(d->id));
			if(parent){
				if(it){ // Insert after (Qt strangely sometimes inverts the insertion order)
					it = new KviPopupListViewItem(KviPopupListViewItem::Submenu,parent,it,
						tmp.ptr(),"submenu",0,d->image.ptr());
				} else {
					it = new KviPopupListViewItem(KviPopupListViewItem::Submenu,parent,
						tmp.ptr(),"submenu",0,d->image.ptr());
				}
			} else {
				if(it){
					it = new KviPopupListViewItem(KviPopupListViewItem::Submenu,m_pPopupView,it,
						tmp.ptr(),"submenu",0,d->image.ptr());
				} else {
					it = new KviPopupListViewItem(KviPopupListViewItem::Submenu,m_pPopupView,
						tmp.ptr(),"submenu",0,d->image.ptr());
				}
			}
			appendChildPopup(it,d->menu);
		} else {
			if((d->id == 0) && kvi_strEqualCS(d->buffer.ptr()," ")){
				if(parent){
					if(it){
						it = new KviPopupListViewItem(KviPopupListViewItem::Separator,parent,it,
							"----","separator",0,0);
					} else {
						it = new KviPopupListViewItem(KviPopupListViewItem::Separator,parent,
							"----","separator",0,0);
					}
				} else {
					if(it){
						it = new KviPopupListViewItem(KviPopupListViewItem::Separator,m_pPopupView,it,
							"----","separator",0,0);
					} else {
						it = new KviPopupListViewItem(KviPopupListViewItem::Separator,m_pPopupView,
							"----","separator",0,0);
					}
				}
			} else {
				KviStr tmp(p->text(d->id));
				if(parent){
					if(it){
						it = new KviPopupListViewItem(KviPopupListViewItem::Item,parent,it,
							tmp.ptr(),"item",d->buffer.ptr(),d->image.ptr());
					} else {
						it = new KviPopupListViewItem(KviPopupListViewItem::Item,parent,
							tmp.ptr(),"item",d->buffer.ptr(),d->image.ptr());
					}
				} else {
					if(it){
						it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,it,
							tmp.ptr(),"item",d->buffer.ptr(),d->image.ptr());
					} else {
						it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
							tmp.ptr(),"item",d->buffer.ptr(),d->image.ptr());
					}
				}
			}
		}
	}
}

void KviDlgScript::popupItemSelected(QListViewItem *i)
{
	if(m_pEditedPopupItem)savePopupItem(m_pEditedPopupItem);

	KviPopupListViewItem *it = (KviPopupListViewItem *)i;
	m_pEditedPopupItem = it;
	if(it == 0){
//		m_pPopupView->clearSelection();		// test -- Qt 2.0
		disablePopupEditors();
		return;
	}

	if(it->m_type == KviPopupListViewItem::Submenu){
		m_pPopupEditor->setText(QString(""));
		m_pPopupEditor->setEnabled(false);
		m_pPopupIconEditor->setText(QString(it->m_icon.ptr()));
		m_pPopupIconEditor->setEnabled(true);
		m_pPopupNameEditor->setText(it->text(0));
		m_pPopupNameEditor->setEnabled(true);
	} else if(it->m_type == KviPopupListViewItem::Separator){
		m_pPopupEditor->setText(QString(""));
		m_pPopupEditor->setEnabled(false);
		m_pPopupIconEditor->setText(QString(""));
		m_pPopupIconEditor->setEnabled(false);
		m_pPopupNameEditor->setText(QString(""));
		m_pPopupNameEditor->setEnabled(false);
	} else {
		m_pPopupEditor->setText(QString(it->m_buffer.ptr()));
		m_pPopupEditor->setEnabled(true);
		m_pPopupIconEditor->setText(QString(it->m_icon.ptr()));
		m_pPopupIconEditor->setEnabled(true);
		m_pPopupNameEditor->setText(it->text(0));
		m_pPopupNameEditor->setEnabled(true);
	}
}

void KviDlgScript::savePopupItem(KviPopupListViewItem * p)
{
	if(m_pEditedPopupItem->m_type == KviPopupListViewItem::Item){
		m_pEditedPopupItem->m_buffer = m_pPopupEditor->text();
		m_pEditedPopupItem->m_icon   = m_pPopupIconEditor->text();
		QString s = m_pPopupNameEditor->text();
		s.stripWhiteSpace();
		if(s.isEmpty())s = QString(__tr("Unnamed"));
		m_pEditedPopupItem->setText(0,s);
		m_pEditedPopupItem->tryLoadImage();
	} else if(m_pEditedPopupItem->m_type == KviPopupListViewItem::Submenu){
		m_pEditedPopupItem->m_icon   = m_pPopupIconEditor->text();
		QString s = m_pPopupNameEditor->text();
		s.stripWhiteSpace();
		if(s.isEmpty())s = QString(__tr("Unnamed"));
		m_pEditedPopupItem->setText(0,s);
		m_pEditedPopupItem->tryLoadImage();
	}
}

void KviDlgScript::popupViewRightButtonPressed(QListViewItem *i,const QPoint &p,int)
{
	if(i)m_pPopupView->setSelected(i,true);
	else {
		m_pPopupView->clearSelection();		// test -- Qt 2.0
		if(m_pEditedPopupItem)savePopupItem(m_pEditedPopupItem);
	}
	m_pEditedPopupItem = (KviPopupListViewItem *)i;

	if(m_pEditedPopupItem){
		m_pPopupPopup->clear();
		m_pPopupPopup->insertItem(QString(__tr("Remove this item")),this,SLOT(removeEditedPopupItem()));
		m_pPopupPopup->insertSeparator();
		m_pPopupPopup->insertItem(QString(__tr("Add empty item below")),this,SLOT(addEmptyPopupItem()));
		m_pPopupPopup->insertItem(QString(__tr("Add submenu below")),this,SLOT(addSubmenuItem()));
		m_pPopupPopup->insertItem(QString(__tr("Add separator below")),this,SLOT(addSeparatorItem()));
		if(m_pEditedPopupItem->parent() == m_pEditedPopupItem->itemAbove()){
			m_pPopupPopup->insertItem(QString(__tr("Add empty item above")),this,SLOT(addEmptyPopupItemAbove()));
			m_pPopupPopup->insertItem(QString(__tr("Add submenu above")),this,SLOT(addSubmenuItemAbove()));
			m_pPopupPopup->insertItem(QString(__tr("Add separator above")),this,SLOT(addSeparatorItemAbove()));
		}
		if(m_pEditedPopupItem->m_type == KviPopupListViewItem::Submenu){
			m_pPopupPopup->insertItem(QString(__tr("Add empty item inside")),this,SLOT(addEmptyPopupItemInside()));
			m_pPopupPopup->insertItem(QString(__tr("Add submenu inside")),this,SLOT(addSubmenuItemInside()));
		} else if(m_pEditedPopupItem->m_type == KviPopupListViewItem::Item){
			m_pPopupPopup->insertSeparator();
			m_pPopupPopup->insertItem(QString(__tr("Copy this item")),this,SLOT(copyItem()));
		}
		if(m_szClipboardItemName.hasData()){
			m_pPopupPopup->insertItem(QString(__tr("Paste item below")),this,SLOT(pasteItemBelow()));
			if(m_pEditedPopupItem->parent() == m_pEditedPopupItem->itemAbove())
				m_pPopupPopup->insertItem(QString(__tr("Paste item above")),this,SLOT(pasteItemAbove()));
			if(m_pEditedPopupItem->m_type == KviPopupListViewItem::Submenu)
				m_pPopupPopup->insertItem(QString(__tr("Paste item inside")),this,SLOT(pasteItemInside()));
		}
	} else {
		// At the end ?
		disablePopupEditors();
		m_pPopupPopup->clear();
		m_pPopupPopup->insertItem(QString(__tr("Add empty item")),this,SLOT(addEmptyPopupItem()));
		m_pPopupPopup->insertItem(QString(__tr("Add submenu")),this,SLOT(addSubmenuItem()));
		m_pPopupPopup->insertItem(QString(__tr("Add separator")),this,SLOT(addSeparatorItem()));
	}
	m_pPopupPopup->popup(p);
}

void KviDlgScript::removeEditedPopupItem()
{
	if(!m_pEditedPopupItem)return;
	QListViewItem * p = m_pEditedPopupItem->parent();
	KviPopupListViewItem * tmp = m_pEditedPopupItem;
	// Needs to be 0 before the delete (the QListViewItem destructor
	// triggers an update in the QListView that calls selectionChanged that
	// attempts to save the popup while it is being deleted!
	m_pEditedPopupItem = 0; 
	delete tmp;
	if(p)m_pPopupView->setSelected(p,true);
    else {
		if(m_pPopupView->firstChild())m_pPopupView->setSelected(m_pPopupView->firstChild(),true);
		else disablePopupEditors();
	}
	updateToolBar(m_pEditedToolBar, 0);
}

void KviDlgScript::copyItem()
{
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->m_type != KviPopupListViewItem::Item)return;
	m_szClipboardItemBuffer = m_pEditedPopupItem->m_buffer;
	m_szClipboardItemIcon   = m_pEditedPopupItem->m_icon;
	m_szClipboardItemName   = m_pEditedPopupItem->text(0);
	m_szClipboardItemSbar	= "";
}

void KviDlgScript::addEmptyPopupItem()
{
	KviPopupListViewItem *it;
	if(m_pEditedPopupItem){
		if(m_pEditedPopupItem->parent()){
			it = new KviPopupListViewItem(KviPopupListViewItem::Item,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
				m_pEditedPopupItem,"unnamed","item",__tr("# Place here the code for the new item"),"");
		} else {
			it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
				m_pEditedPopupItem,"unnamed","item",__tr("# Place here the code for the new item"),"");
		}
	} else {
		it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
			"unnamed","item",__tr("# Place here the code for the new item"),"");
	}
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::pasteItemBelow()
{
	KviPopupListViewItem *it;
	if(m_szClipboardItemName.isEmpty())return;
	if(m_pEditedPopupItem){
		if(m_pEditedPopupItem->parent()){
			it = new KviPopupListViewItem(KviPopupListViewItem::Item,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
				m_pEditedPopupItem,m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());
		} else {
			it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
				m_pEditedPopupItem,m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());
		}
	} else {
		it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
			m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());
	}
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::pasteItemInside()
{
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->m_type != KviPopupListViewItem::Submenu)return;
	if(m_szClipboardItemName.isEmpty())return;
	KviPopupListViewItem *it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pEditedPopupItem,
		m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::pasteItemAbove()
{
	KviPopupListViewItem *it;
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->parent() != m_pEditedPopupItem->itemAbove())return;
	if(m_szClipboardItemName.isEmpty())return;

	if(m_pEditedPopupItem->parent()){
		it = new KviPopupListViewItem(KviPopupListViewItem::Item,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
			0,m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());

	} else {
		it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
			0,m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());
	}
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addEmptyPopupItemAbove()
{
	KviPopupListViewItem *it;
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->parent() != m_pEditedPopupItem->itemAbove())return;
	if(m_pEditedPopupItem->parent()){
		it = new KviPopupListViewItem(KviPopupListViewItem::Item,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
			0,"unnamed","item",__tr("# Place here the code for the new item"),"");

	} else {
		it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pPopupView,
			0,"unnamed","item",__tr("# Place here the code for the new item"),"");
	}
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addSubmenuItemAbove()
{
	KviPopupListViewItem *sb;
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->parent() != m_pEditedPopupItem->itemAbove())return;
	if(m_pEditedPopupItem->parent()){
		sb = new KviPopupListViewItem(KviPopupListViewItem::Submenu,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
			0,"unnamed","submenu","","");
	} else {
		sb = new KviPopupListViewItem(KviPopupListViewItem::Submenu,m_pPopupView,
			0,"unnamed","submenu","","");
	}

	KviPopupListViewItem * it = new KviPopupListViewItem(KviPopupListViewItem::Item,sb,
		"unnamed","item",__tr("# Place here the code for the new item"),"");
	sb->setOpen(true);

	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addSeparatorItemAbove()
{
	KviPopupListViewItem * it;
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->parent() != m_pEditedPopupItem->itemAbove())return;
	if(m_pEditedPopupItem->parent()){
		it = new KviPopupListViewItem(KviPopupListViewItem::Separator,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
			0,"----","separator","","");
	} else {
		it = new KviPopupListViewItem(KviPopupListViewItem::Separator,m_pPopupView,
			0,"----","separator","","");
	}

	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addEmptyPopupItemInside()
{
	if(!m_pEditedPopupItem)return;
	if(m_pEditedPopupItem->m_type != KviPopupListViewItem::Submenu)return;
	KviPopupListViewItem *it = new KviPopupListViewItem(KviPopupListViewItem::Item,m_pEditedPopupItem,
		"unnamed","item",__tr("# Place here the code for the new item"),"");
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addSubmenuItemInside()
{
	if(!m_pEditedPopupItem)return;
	KviPopupListViewItem *sb = new KviPopupListViewItem(KviPopupListViewItem::Submenu,m_pEditedPopupItem,
		"unnamed","submenu","","");
	KviPopupListViewItem * it = new KviPopupListViewItem(KviPopupListViewItem::Item,sb,
		"unnamed","item",__tr("# Place here the code for the new item"),"");
	sb->setOpen(true);
	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addSubmenuItem()
{
	KviPopupListViewItem *sb;
	if(m_pEditedPopupItem){
		if(m_pEditedPopupItem->parent()){
			sb = new KviPopupListViewItem(KviPopupListViewItem::Submenu,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
				m_pEditedPopupItem,"unnamed","submenu","","");
		} else {
			sb = new KviPopupListViewItem(KviPopupListViewItem::Submenu,m_pPopupView,
				m_pEditedPopupItem,"unnamed","submenu","","");
		}
	} else {
		sb = new KviPopupListViewItem(KviPopupListViewItem::Submenu,m_pPopupView,
			"unnamed","submenu","","");
	}

	KviPopupListViewItem * it = new KviPopupListViewItem(KviPopupListViewItem::Item,sb,
		"unnamed","item",__tr("# Place here the code for the new item"),"");
	sb->setOpen(true);

	m_pPopupView->setSelected(it,true);
}

void KviDlgScript::addSeparatorItem()
{
	KviPopupListViewItem * it;
	if(m_pEditedPopupItem){
		if(m_pEditedPopupItem->parent()){
			it = new KviPopupListViewItem(KviPopupListViewItem::Separator,((KviPopupListViewItem *)m_pEditedPopupItem->parent()),
				m_pEditedPopupItem,"----","separator","","");
		} else {
			it = new KviPopupListViewItem(KviPopupListViewItem::Separator,m_pPopupView,
				m_pEditedPopupItem,"----","separator","","");
		}
	} else {
		it = new KviPopupListViewItem(KviPopupListViewItem::Separator,m_pPopupView,
			"separator","separator","","");
	}

	m_pPopupView->setSelected(it,true);
}


//===================================================================================================
// TOOLBAR STUFF
//

//==================== Constructor: Add ======================
KviToolBarListViewItem::KviToolBarListViewItem(ToolButtonType t,QListView * parent,const char *name,const char *type,const char *buffer,const char *icon, const char *sbar)
:QListViewItem(parent,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_sbar = sbar;
	m_id = 0;
	m_type = t;
	tryLoadImage();
}

//==================== Constructor: Add below =================
KviToolBarListViewItem::KviToolBarListViewItem(ToolButtonType t,QListView * parent,KviToolBarListViewItem * after,const char *name,const char *type,const char *buffer,const char *icon, const char *sbar)
:QListViewItem(parent,after,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_sbar = sbar;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}

//==================== Constructor: Add menu =================
KviToolBarListViewItem::KviToolBarListViewItem(ToolButtonType t,KviToolBarListViewItem * parent,const char * name,const char *type,const char *buffer,const char *icon, const char *sbar=0)
:QListViewItem(parent,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_sbar = sbar;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}

//==================== Constructor: Add menu below ============
KviToolBarListViewItem::KviToolBarListViewItem(ToolButtonType t,KviToolBarListViewItem * parent,KviToolBarListViewItem * after,const char * name,const char *type,const char *buffer,const char *icon, const char *sbar=0)
:QListViewItem(parent,after,QString(name),QString(type))
{
	if(buffer)m_buffer = buffer;
	if(icon)m_icon = icon;
	m_sbar = sbar;
	m_type = t;
	m_id = 0;
	tryLoadImage();
}

//==================== Destructor =================

KviToolBarListViewItem::~KviToolBarListViewItem()
{
}

//==================== tryLoadImage =================
void KviToolBarListViewItem::tryLoadImage()
{
	m_icon.stripWhiteSpace();
	if(m_icon.hasData()){
		if(m_icon.isNum()){
			bool bOk=false;
			int imId = m_icon.toInt(&bOk);
			if(imId >= 0){
				imId = imId % KVI_OUT_NUM_IMAGES;
				setPixmap(0,*(g_pixViewOut[imId]));
			}
		} else {
			KviStr imagePath;
			if(g_pApp->findImage(imagePath,m_icon.ptr())){
				QPixmap pix(imagePath.ptr());
				if(!pix.isNull()){
					setPixmap(0,pix);
				} else debug("Unable to find image file %s",m_icon.ptr());
			}
		}
	}
}

//==================== startEditToolBar =================
void KviDlgScript::startEditToolBar(KviUserToolBar *p)
{
	if(m_pEditedToolBar)saveToolBar(m_pEditedToolBar);
	m_pEditedToolBar = p;
	m_pToolBarView->clear();
	m_pToolBarView->setSorting(-1);
	appendToolButton(p);
//	m_pToolBarView->setSorting(-1);
	m_pEditedToolButton = 0;
	if(m_pToolBarView->firstChild() == 0)disableToolBarEditors();
	else m_pToolBarView->setSelected(m_pToolBarView->firstChild(),true);
}

//==================== selectToolButtonById =================
bool KviDlgScript::selectToolButtonById(KviToolBarListViewItem * par,int id)
{
	KviToolBarListViewItem *it = (KviToolBarListViewItem *)(par ? par->firstChild() : m_pToolBarView->firstChild());
	while(it){
		if(it->m_id == id){
			// open all parents
			par = (KviToolBarListViewItem *)it->parent();
			while(par){
				par->setOpen(true);
				par = (KviToolBarListViewItem *)par->parent();
			}
			// select it
			m_pToolBarView->setSelected(it,true);
			// and ensure it is visible
			m_pToolBarView->ensureItemVisible(it);
			return true;
		}
		it = (KviToolBarListViewItem *)it->nextSibling();
	}
	return false;
}

//==================== saveToolBar =================
void KviDlgScript::saveToolBar(KviUserToolBar *p)
{
	if(m_pEditedToolButton)saveToolButton(m_pEditedToolButton); // do not forget to save the last one
}

//==================== saveToolButton =================
void KviDlgScript::saveToolButton(KviToolBarListViewItem * p)
{
//	debug("saveToolButton: ");
	if(m_pEditedToolButton->m_type == KviToolBarListViewItem::Button || 
			m_pEditedToolButton->m_type == KviToolBarListViewItem::Menu ||
			m_pEditedToolButton->m_type == KviToolBarListViewItem::Item) {
		m_pEditedToolButton->m_buffer = m_pToolBarEditor->text();
		m_pEditedToolButton->m_icon   = m_pToolBarIconEditor->text();
		m_pEditedToolButton->m_sbar   = m_pToolBarStatusBarEditor->text();
		QString s = m_pToolBarNameEditor->text();
		s.stripWhiteSpace();
		if(s.isEmpty())s = QString(__tr("Unnamed"));
//		debug("    %s",s.ascii());
		m_pEditedToolButton->setText(0,s);
		m_pEditedToolButton->tryLoadImage();
	} else if(m_pEditedToolButton->m_type == KviToolBarListViewItem::Submenu) {
		m_pEditedToolButton->m_icon   = m_pToolBarIconEditor->text();
		QString s = m_pToolBarNameEditor->text();
		s.stripWhiteSpace();
		if(s.isEmpty())s = QString(__tr("Unnamed"));
		m_pEditedToolButton->setText(0,s);
		m_pEditedToolButton->tryLoadImage();
	}
		updateToolBar(m_pEditedToolBar, 0);
}

void KviDlgScript::saveToolBarMenu(KviPopupMenu *p,KviToolBarListViewItem *par)
{
	KviToolBarListViewItem *it = (KviToolBarListViewItem *)(par ? par->firstChild() : m_pToolBarView->firstChild());
	while(it){
		if(it->m_type == KviToolBarListViewItem::Separator){
			p->insertSeparatorItem();
			it->m_id = 0;
		} else {
			if(it->m_type == KviToolBarListViewItem::Item){
				debug("saving item: %s",it->text(0).ascii());
				KviStr tmp = it->text(0);
				tmp.stripWhiteSpace();
				if(tmp.isEmpty())tmp = __tr("Unnamed");
				it->m_id = p->insertNormalItem(tmp.ptr(),it->m_icon.ptr(),it->m_buffer.ptr());
			} else if(it->m_type == KviToolBarListViewItem::Menu) {
				debug("saving menu: %s",it->text(0).ascii());
				KviPopupMenu *sub = new KviPopupMenu();
				saveToolBarMenu(sub,it);
				KviStr tmp = it->text(0);
				tmp.stripWhiteSpace();
				if(tmp.isEmpty())tmp = __tr("Unnamed");
				p->insertPopupItem(tmp.ptr(),it->m_icon.ptr(),sub);
				it->m_id = 0;
			}
		}
		it = (KviToolBarListViewItem *)it->nextSibling();
	}
}

//==================== updateToolBar =================
void KviDlgScript::updateToolBar(KviUserToolBar *p,KviToolBarListViewItem *par)
{
	p->clearAll();
	KviToolBarListViewItem *it = (KviToolBarListViewItem *)(par ? par->firstChild() : m_pToolBarView->firstChild());
	while(it){
		if(it->m_type == KviToolBarListViewItem::Separator){
			p->insertSeparatorItem();
			it->m_id = 0;
		} else
		if(it->m_type == KviToolBarListViewItem::Button){
			KviStr tmp = it->text(0);
			tmp.stripWhiteSpace();
			if(tmp.isEmpty())tmp = __tr("Unnamed");
//			it->m_id = p->insertNormalItem(tmp.ptr(),it->m_sbar.ptr(),it->m_icon.ptr(),it->m_buffer.ptr());
			p->insertNormalItem(tmp.ptr(),it->m_sbar.ptr(),it->m_icon.ptr(),it->m_buffer.ptr());
		} else
		if(it->m_type == KviToolBarListViewItem::Menu){

			KviPopupMenu *m = new KviPopupMenu();
			saveToolBarMenu(m,it);

			KviStr tmp = it->text(0);
			tmp.stripWhiteSpace();
			if(tmp.isEmpty())tmp = __tr("Unnamed");
			p->insertNormalItem(tmp.ptr(),it->m_sbar.ptr(),it->m_icon.ptr(),it->m_buffer.ptr(),m);
		}
		it = (KviToolBarListViewItem *)it->nextSibling();
	}
	QWidget *w = new QWidget(p,"bar");
	p->setStretchableWidget(w);
	p->hide();
	p->show();
}


//==================== disableToolBarEditors =================
void KviDlgScript::disableToolBarEditors()
{
	m_pToolBarEditor->setText(QString(""));
	m_pToolBarIconEditor->setText(QString(""));
	m_pToolBarNameEditor->setText(QString(""));
	m_pToolBarStatusBarEditor->setText(QString(""));
	m_pToolBarEditor->setEnabled(false);
	m_pToolBarIconEditor->setEnabled(false);
	m_pToolBarNameEditor->setEnabled(false);
	m_pToolBarStatusBarEditor->setEnabled(false);
}

//==================== appendToolButton ========================
void KviDlgScript::appendToolButton(KviUserToolBar *p)
{
	KviToolBarListViewItem *it = 0;
	for(KviUserToolBarData * d = p->m_pDataList->first();d;d=p->m_pDataList->next()){
		if(it){
			switch (d->type) {
				case KviUserToolBar::Separator:
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,m_pToolBarView,it,
						"----","separator",0,0);
				break;
				case KviUserToolBar::Button:
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,it,
						d->text.ptr(),"button",d->buffer.ptr(),d->image.ptr(),d->sbar.ptr());
				break;
				case KviUserToolBar::Menu:
					{
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,m_pToolBarView,it,
						d->text.ptr(),"menu",d->buffer.ptr(),d->image.ptr(),d->sbar.ptr());
					appendToolBarMenu(it, d->menu);
					}
				break;
			}
		} else {
			switch (d->type) {
				case KviUserToolBar::Separator:
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,m_pToolBarView,
						"----","separator",0,0);
				break;
				case KviUserToolBar::Button:
					{
					KviStr tmp(d->text);
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
						tmp.ptr(),"button",d->buffer.ptr(),d->image.ptr(),d->sbar.ptr());
					}
				break;
				case KviUserToolBar::Menu:
					{
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,m_pToolBarView,
						d->text.ptr(),"menu",d->buffer.ptr(),d->image.ptr(),d->sbar.ptr());
					appendToolBarMenu(it, d->menu);
					}
				break;
			}
		}
	}
}

void KviDlgScript::appendToolBarMenu(KviToolBarListViewItem *parent, KviPopupMenu *p)
{
	KviToolBarListViewItem *it = 0;
	for(KviPopupMenuData * d = p->m_pDataList->first();d;d=p->m_pDataList->next()){
		if(d->menu != 0){
			KviStr tmp(p->text(d->id));
				if(it){ // Insert after (Qt strangely sometimes inverts the insertion order)
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,parent,it,
						tmp.ptr(),"submenu",0,d->image.ptr());
				} else {
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,parent,
						tmp.ptr(),"submenu",0,d->image.ptr());
				}
			appendToolBarMenu(it,d->menu);
		} else {
			if((d->id == 0) && kvi_strEqualCS(d->buffer.ptr()," ")){
				if(it){
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,parent,it,
						"----","separator",0,0);
				} else {
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,parent,
						"----","separator",0,0);
				}
			} else {
				KviStr tmp(p->text(d->id));
				if(it){
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,parent,it,
						tmp.ptr(),"item",d->buffer.ptr(),d->image.ptr());
				} else {
					it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,parent,
						tmp.ptr(),"item",d->buffer.ptr(),d->image.ptr());
				}
			}
		}
	}
}


//==================== toolButtonSelected ========================
void KviDlgScript::toolButtonSelected(QListViewItem *i)
{
	if(m_pEditedToolButton)saveToolButton(m_pEditedToolButton);

	KviToolBarListViewItem *it = (KviToolBarListViewItem *)i;
	m_pEditedToolButton = it;
	if(it == 0){
//		m_pToolBarView->clearSelection();		// test -- Qt 2.0
		disableToolBarEditors();
		return;
	}

	switch (it->m_type) {
		case KviToolBarListViewItem::Separator:
			m_pToolBarEditor->setText(QString(""));
			m_pToolBarEditor->setEnabled(false);
			m_pToolBarIconEditor->setText(QString(""));
			m_pToolBarIconEditor->setEnabled(false);
			m_pToolBarNameEditor->setText(QString(""));
			m_pToolBarNameEditor->setEnabled(false);
			m_pToolBarStatusBarEditor->setText(QString(""));
			m_pToolBarStatusBarEditor->setEnabled(false);
		break;					
		case KviToolBarListViewItem::Menu:
			m_pToolBarEditor->setText(QString(""));
			m_pToolBarEditor->setEnabled(false);
			m_pToolBarIconEditor->setText(QString(it->m_icon.ptr()));
			m_pToolBarIconEditor->setEnabled(true);
			m_pToolBarNameEditor->setText(it->text(0));
			m_pToolBarNameEditor->setEnabled(true);
			m_pToolBarStatusBarEditor->setText(QString(it->m_sbar.ptr()));
			m_pToolBarStatusBarEditor->setEnabled(true);
		break;					
		case KviToolBarListViewItem::Item:
			m_pToolBarEditor->setText(QString(it->m_buffer.ptr()));
			m_pToolBarEditor->setEnabled(true);
			m_pToolBarIconEditor->setText(QString(it->m_icon.ptr()));
			m_pToolBarIconEditor->setEnabled(true);
			m_pToolBarNameEditor->setText(it->text(0));
			m_pToolBarNameEditor->setEnabled(true);
			m_pToolBarStatusBarEditor->setText(QString(""));
			m_pToolBarStatusBarEditor->setEnabled(false);
		break;
		default:
			m_pToolBarEditor->setText(QString(it->m_buffer.ptr()));
			m_pToolBarEditor->setEnabled(true);
			m_pToolBarIconEditor->setText(QString(it->m_icon.ptr()));
			m_pToolBarIconEditor->setEnabled(true);
			m_pToolBarNameEditor->setText(it->text(0));
			m_pToolBarNameEditor->setEnabled(true);
			m_pToolBarStatusBarEditor->setText(QString(it->m_sbar.ptr()));
			m_pToolBarStatusBarEditor->setEnabled(true);
	}
}

//==================== toolBarViewRightButtonPressed ========================
void KviDlgScript::toolBarViewRightButtonPressed(QListViewItem *i,const QPoint &p,int)
{
	if(i)m_pToolBarView->setSelected(i,true);
	else {
		m_pToolBarView->clearSelection();		// test -- Qt 2.0
		if(m_pEditedToolButton)saveToolButton(m_pEditedToolButton);
	}
	m_pEditedToolButton = (KviToolBarListViewItem *)i;

	if(m_pEditedToolButton){
		m_pToolBarPopup->clear();
		m_pToolBarPopup->insertItem(QString(__tr("Remove this item")),this,SLOT(removeEditedToolButton()));
		m_pToolBarPopup->insertSeparator();
		m_pToolBarPopup->insertItem(QString(__tr("Add empty item below")),this,SLOT(addEmptyToolButton()));
		m_pToolBarPopup->insertItem(QString(__tr("Add separator below")),this,SLOT(addToolBarSeparatorItem()));
		m_pToolBarPopup->insertItem(QString(__tr("Add menu below")),this,SLOT(addToolBarMenu()));
		if(m_pEditedToolButton->parent() == m_pEditedToolButton->itemAbove()){
			m_pToolBarPopup->insertItem(QString(__tr("Add empty item above")),this,SLOT(addEmptyToolButtonAbove()));
			m_pToolBarPopup->insertItem(QString(__tr("Add separator above")),this,SLOT(addToolBarSeparatorItemAbove()));
			m_pToolBarPopup->insertItem(QString(__tr("Add menu above")),this,SLOT(addToolBarMenuAbove()));
		}
		if(m_pEditedToolButton->m_type == KviToolBarListViewItem::Button || m_pEditedToolButton->m_type == KviToolBarListViewItem::Item){
			m_pToolBarPopup->insertSeparator();
			m_pToolBarPopup->insertItem(QString(__tr("Copy this item")),this,SLOT(copyToolButton()));
		}
		if(m_szClipboardItemName.hasData()){
			m_pToolBarPopup->insertItem(QString(__tr("Paste item below")),this,SLOT(pasteToolButtonBelow()));
			if(m_pEditedToolButton->parent() == m_pEditedToolButton->itemAbove())
				m_pToolBarPopup->insertItem(QString(__tr("Paste item above")),this,SLOT(pasteToolButtonAbove()));
		}
	} else {
		// At the end ?
		disableToolBarEditors();
		m_pToolBarPopup->clear();
		m_pToolBarPopup->insertItem(QString(__tr("Add empty item")),this,SLOT(addEmptyToolButton()));
		m_pToolBarPopup->insertItem(QString(__tr("Add separator")),this,SLOT(addToolBarSeparatorItem()));
		m_pToolBarPopup->insertItem(QString(__tr("Add menu")),this,SLOT(addToolBarMenu()));
	}
	m_pToolBarPopup->popup(p);
}

//==================== removeEditedToolButton ========================
void KviDlgScript::removeEditedToolButton()
{
	if(!m_pEditedToolButton)return;
	QListViewItem * p = m_pEditedToolButton->parent();
	KviToolBarListViewItem * tmp = m_pEditedToolButton;
	// Needs to be 0 before the delete (the QListViewItem destructor
	// triggers an update in the QListView that calls selectionChanged that
	// attempts to save the popup while it is being deleted!
	m_pEditedToolButton = 0; 
	delete tmp;
	if(p)m_pToolBarView->setSelected(p,true);
    else {
		if(m_pToolBarView->firstChild())m_pToolBarView->setSelected(m_pToolBarView->firstChild(),true);
		else disableToolBarEditors();
	}
	updateToolBar(m_pEditedToolBar, 0);
}

//==================== copyToolButton ========================
void KviDlgScript::copyToolButton()
{
	if(!m_pEditedToolButton)return;
	if(m_pEditedToolButton->m_type != KviToolBarListViewItem::Button && m_pEditedToolButton->m_type != KviToolBarListViewItem::Item) return;
	m_szClipboardItemBuffer = m_pEditedToolButton->m_buffer;
	m_szClipboardItemIcon   = m_pEditedToolButton->m_icon;
	m_szClipboardItemName   = m_pEditedToolButton->text(0);
	m_szClipboardItemSbar	= m_pEditedToolButton->m_sbar;
}

//==================== addEmptyToolButton ========================
void KviDlgScript::addEmptyToolButton()
{
	KviToolBarListViewItem *it;
	if(m_pEditedToolButton){
		if(m_pEditedToolButton->parent())
			it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
				m_pEditedToolButton,"unnamed","item",__tr("# Place here the code for the new item"),"");
		else
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
			m_pEditedToolButton,"unnamed","button",__tr("# Place here the code for the new item"),"");
	} else {
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
			"unnamed","button",__tr("# Place here the code for the new item"),"");
	}
	m_pToolBarView->setSelected(it,true);
	updateToolBar(m_pEditedToolBar, 0);
}

void KviDlgScript::addToolBarMenu()
{
	KviToolBarListViewItem *sb;
	if(m_pEditedToolButton){
		if(m_pEditedToolButton->parent()){
			sb = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
				m_pEditedToolButton,"unnamed","submenu","","");
		} else {
			sb = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,m_pToolBarView,
				m_pEditedToolButton,"unnamed","menu","","");
		}
	} else {
		sb = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,m_pToolBarView,
			"unnamed","menu","","");
	}

	KviToolBarListViewItem * it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,sb,
		"unnamed","item",__tr("# Place here the code for the new item"),"");
	sb->setOpen(true);

	m_pToolBarView->setSelected(it,true);
	updateToolBar(m_pEditedToolBar, 0);
}

void KviDlgScript::addToolBarMenuAbove()
{
	KviToolBarListViewItem *sb;
	if(!m_pEditedToolButton)return;
	if(m_pEditedToolButton->parent() != m_pEditedToolButton->itemAbove())return; // ensure first item
	if(m_pEditedToolButton->parent())
		sb = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
			"unnamed","submenu","","");
	else
		sb = new KviToolBarListViewItem(KviToolBarListViewItem::Menu,m_pToolBarView,
			"unnamed","menu","","");

	KviToolBarListViewItem * it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,sb,
		"unnamed","item",__tr("# Place here the code for the new item"),"");
	sb->setOpen(true);

	m_pToolBarView->setSelected(it,true);
	updateToolBar(m_pEditedToolBar, 0);
}

//==================== pasteToolButtonBelow ========================
void KviDlgScript::pasteToolButtonBelow()
{
	KviToolBarListViewItem *it;
	if(m_szClipboardItemName.isEmpty())return;
	if(m_pEditedToolButton){
		if(m_pEditedToolButton->parent())
			it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
				m_pEditedToolButton,m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr());
		else
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
			m_pEditedToolButton,m_szClipboardItemName.ptr(),"button",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr(),m_szClipboardItemSbar.ptr());
	} else {
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
			m_szClipboardItemName.ptr(),"button",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr(),m_szClipboardItemSbar.ptr());
	}
	m_pToolBarView->setSelected(it,true);
}

//==================== pasteToolButtonAbove ========================
void KviDlgScript::pasteToolButtonAbove()
{
	KviToolBarListViewItem *it;
	if(!m_pEditedToolButton)return;
	if(m_pEditedToolButton->parent() != m_pEditedToolButton->itemAbove())return; // ensure first item
	if(m_szClipboardItemName.isEmpty())return;
	if(m_pEditedToolButton->parent())
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
			m_szClipboardItemName.ptr(),"item",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr(),"");
	else
	it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
		m_szClipboardItemName.ptr(),"button",m_szClipboardItemBuffer.ptr(),m_szClipboardItemIcon.ptr(),m_szClipboardItemSbar.ptr());

	m_pToolBarView->setSelected(it,true);
}

//==================== addEmptyToolButtonAbove ========================
void KviDlgScript::addEmptyToolButtonAbove()
{
	KviToolBarListViewItem *it;
	if(!m_pEditedToolButton)return;
	if(m_pEditedToolButton->parent() != m_pEditedToolButton->itemAbove())return; // ensure first item
	if(m_pEditedToolButton->parent())
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Item,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
			"unnamed","item",__tr("# Place here the code for the new item"),"","");
	else
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Button,m_pToolBarView,
			"unnamed","button",__tr("# Place here the code for the new item"),"","");
	m_pToolBarView->setSelected(it,true);
}

//==================== addToolBarSeparatorItemAbove ========================
void KviDlgScript::addToolBarSeparatorItemAbove()
{
	KviToolBarListViewItem * it;
	if(!m_pEditedToolButton)return;
	if(m_pEditedToolButton->parent() != m_pEditedToolButton->itemAbove())return; // ensure first item
	if(m_pEditedToolButton->parent())
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
			"----","separator","","","");
	else
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,m_pToolBarView,
			"----","separator","","","");
	m_pToolBarView->setSelected(it,true);
}

//==================== addToolBarSeparatorItem ========================
void KviDlgScript::addToolBarSeparatorItem()
{
	KviToolBarListViewItem * it;
	if(m_pEditedToolButton){
		if(m_pEditedToolButton->parent())
			it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,(KviToolBarListViewItem *)m_pEditedToolButton->parent(),
				m_pEditedToolButton,"----","separator",__tr("# Place here the code for the new item"),"");
		else
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,m_pToolBarView,
			m_pEditedToolButton,"----","separator","","");
	} else {
		it = new KviToolBarListViewItem(KviToolBarListViewItem::Separator,m_pToolBarView,
			"----","separator","","");
	}
	m_pToolBarView->setSelected(it,true);
}

//==================== Variables tab

void KviDlgScript::fillVariablesEditor()
{
	KviStr tmp;
	for(KviVariable * v=g_pVarCache->m_pVarList->first();v;v=g_pVarCache->m_pVarList->next()){
		tmp.append('%');
		tmp.append(v->szName);
		tmp.append(" = ");
		tmp.append(v->szValue);
		tmp.append('\n');
	}
	for(KviDictionary * d=g_pVarCache->m_pDictList->first();d;d=g_pVarCache->m_pDictList->next()){
		for(KviVariable * var=d->m_pVarCache->m_pVarList->first();var;var=d->m_pVarCache->m_pVarList->next()){
			tmp.append('%');
			tmp.append(d->szName);
			tmp.append('[');
			tmp.append(var->szName);
			tmp.append(']');
			tmp.append(" = ");
			tmp.append(var->szValue);
			tmp.append('\n');
		}
	}
	m_pVariablesEditor->setText(tmp.ptr());
}

void KviDlgScript::variablesTextChanged()
{
	m_bVariablesChanged = true;
}

#include "m_kvi_dlgscript.moc"
