#ifndef _KVI_DLGSCRIPT_H_INCLUDED_
#define _KVI_DLGSCRIPT_H_INCLUDED_

//
//   File : kvi_dlgscript.h (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_dlgscript.h)
//   Last major modification : Sun Mar 28 1999 15:31:45 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//   Copyright (C) 1999-2000 Till Busch (buti@geocities.com)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include <qtabdialog.h>
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qlistbox.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qframe.h>
#include <qlistview.h>
#include <qpopupmenu.h>

#include "kvi_string.h"

class KviAliasManager;
class KviEventManager;
class KviScriptEditor;
class KviPopupMenu;
class KviUserToolBar;

//================ class KviAliasNameValidator ==============

class KviAliasNameValidator : public QValidator
{
	Q_OBJECT
public:
	KviAliasNameValidator(QWidget * parent):QValidator(parent){};
	~KviAliasNameValidator(){};
	QValidator::State validate(QString & str,int &) const;
};

//================ class KviPopupListViewItem ==============

class KviPopupListViewItem : public QListViewItem
{
public:
	enum PopupItemType { Submenu , Separator , Item };
	KviPopupListViewItem(PopupItemType t,KviPopupListViewItem * parent,const char * name,const char *type,const char *buffer = 0,const char *icon = 0);
	KviPopupListViewItem(PopupItemType t,KviPopupListViewItem * parent,KviPopupListViewItem * after,const char * name,const char *type,const char *buffer = 0,const char *icon = 0);
	KviPopupListViewItem(PopupItemType t,QListView * parent,const char *name,const char *type,const char *buffer = 0,const char *icon = 0);
	KviPopupListViewItem(PopupItemType t,QListView * parent,KviPopupListViewItem * after,const char *name,const char *type,const char *buffer = 0,const char *icon = 0);
	~KviPopupListViewItem();
public:
	KviStr m_buffer;
	KviStr m_icon;
	PopupItemType m_type;
	int m_id;
public:
	void tryLoadImage();
};

// ===================== class KviToolBarListViewItem ======================

class KviToolBarListViewItem : public QListViewItem
{
public:
	enum ToolButtonType { Separator , Button, Menu, Submenu, Item };
	// Add item
	KviToolBarListViewItem(ToolButtonType t,QListView * parent,const char *name,const char *type,const char *buffer = 0,const char *icon = 0, const char *sbar = 0);
	// Add item below
	KviToolBarListViewItem(ToolButtonType t,QListView * parent,KviToolBarListViewItem * after,const char *name,const char *type,const char *buffer=0,const char *icon=0, const char *sbar=0);
	// Add menu
	KviToolBarListViewItem(ToolButtonType t,KviToolBarListViewItem * parent,const char * name,const char *type,const char *buffer,const char *icon, const char *sbar=0);
	// Add menu below
	KviToolBarListViewItem(ToolButtonType t,KviToolBarListViewItem * parent,KviToolBarListViewItem * after,const char * name,const char *type,const char *buffer,const char *icon, const char *sbar=0);

	~KviToolBarListViewItem();
public:
	KviStr	m_buffer;
	KviStr	m_icon;
	KviStr	m_sbar;
	int		m_id;
	ToolButtonType m_type;
public:
	void tryLoadImage();
};


// ================ class KviDlgScript ===================

class KviDlgScript : public QTabDialog
{
	Q_OBJECT
public:
	KviDlgScript();
	~KviDlgScript();
private:
	// ALIASES TAB
	enum KviAliasNameEditMode { None , NewAlias , RenameAlias };
	QLineEdit       * m_pAliasName;
	QComboBox       * m_pAliasBox;
	KviScriptEditor * m_pAliasEditor;
	QPushButton     * m_pAliasNew;
	QPushButton     * m_pAliasRename;
	QPushButton     * m_pAliasDelete;
	KviAliasManager * m_pAliasManager;
	KviAliasNameEditMode m_editMode;
	// EVENTS TAB
	QListBox        * m_pEventBox;
	QLabel          * m_pEventLabel;
	QCheckBox       * m_pEventEnable;
	KviScriptEditor * m_pEventEditor;
	KviEventManager * m_pEventManager;
	bool              m_bInitialSetupPhase;
	// POPUPS TAB
	QComboBox       * m_pPopupsBox;
	KviPopupMenu    * m_pConsolePopup;
	KviPopupMenu    * m_pChannelPopup;
	KviPopupMenu    * m_pQueryPopup;
	KviPopupMenu    * m_pUserlistPopup;
	KviPopupMenu    * m_pNotifylistPopup;
	KviPopupMenu    * m_pDccChatPopup;
	QListView       * m_pPopupView;
	KviScriptEditor * m_pPopupEditor;
	QLineEdit       * m_pPopupIconEditor;
	QLineEdit       * m_pPopupNameEditor;
	KviPopupListViewItem * m_pEditedPopupItem;
	QPopupMenu      * m_pPopupPopup;
	QPopupMenu      * m_pIconPopup;
	KviPopupMenu    * m_pEditedMenu;
	KviStr            m_szClipboardItemBuffer;
	KviStr            m_szClipboardItemIcon;
	KviStr            m_szClipboardItemName;
	KviStr            m_szClipboardItemSbar;
	// TOOLBAR TAB
	KviUserToolBar  * m_pToolBar;
	KviUserToolBar  * m_pEditedToolBar;
	QListView       * m_pToolBarView;
	QLineEdit       * m_pToolBarIconEditor;
	QLineEdit       * m_pToolBarNameEditor;
	QLineEdit       * m_pToolBarStatusBarEditor;
	QPopupMenu      * m_pToolBarPopup;
	KviScriptEditor * m_pToolBarEditor;
	KviToolBarListViewItem * m_pEditedToolButton;
	// VARIABLES TAB
	KviScriptEditor * m_pVariablesEditor;
	bool              m_bVariablesChanged;
private:
	// ALIASES TAB
	void fillAliasesBox();
	void saveLastSelectedAlias();
	void selectAlias(int index);
	void enterEditMode(const char *startText);
	// EVENTS TAB
	void fillEventsBox();
	void selectEvent(int index);
	void saveLastSelectedEvent();
	// POPUPS TAB
	void startEditPopup(KviPopupMenu * p);
	void appendChildPopup(KviPopupListViewItem * parent,KviPopupMenu *p);
	void disablePopupEditors();
	void savePopupItem(KviPopupListViewItem * p);
	void saveMenu(KviPopupMenu *p);
	void saveSubmenu(KviPopupMenu *p,KviPopupListViewItem *par);
	// TOOLBAR TAB
	void startEditToolBar(KviUserToolBar * p);
	void appendToolButton(KviUserToolBar *p);
	void appendToolBarMenu(KviToolBarListViewItem *parent, KviPopupMenu *p);
//	void appendChildPopup(KviToolBarListViewItem * parent,KviUserToolBar *p);
	void disableToolBarEditors();
	void saveToolButton(KviToolBarListViewItem * p);
	void saveToolBar(KviUserToolBar *p);
	void updateToolBar(KviUserToolBar *p,KviToolBarListViewItem *par);
	void saveToolBarMenu(KviPopupMenu *p,KviToolBarListViewItem *par);
//	void saveSubmenu(KviPopupMenu *p,KviPopupListViewItem *par);
	// VARIABLES TAB
	void fillVariablesEditor();
signals:
	void finished(bool bAccepted);
protected slots:
	// ALIASES TAB
	void leaveEditMode();
	void newAlias();
	void deleteCurrentAlias();
	void renameCurrentAlias();
	void aliasSelected(int index);
	// EVENTS TAB
	void eventSelected(int index);
	void eventEnableClicked();
	// POPUPS TAB
	void popupItemSelected(QListViewItem *i);
	void popupViewRightButtonPressed(QListViewItem *i,const QPoint &p,int);
	void popupComboSelectionChanged(int index);
	void removeEditedPopupItem();
	void addEmptyPopupItem();//Below
	void addSubmenuItem();   //Below
	void addSeparatorItem(); //Below
	void addEmptyPopupItemAbove();
	void addSubmenuItemAbove();
	void addSeparatorItemAbove();
	void addSubmenuItemInside();
	void addEmptyPopupItemInside();
	void copyItem();
	void pasteItemBelow();
	void pasteItemAbove();
	void pasteItemInside();
	void testCurrentPopup();
	void testPopupSlot(int id);
	bool selectPopupItemById(KviPopupListViewItem * par,int id);
	void iconPopupActivated(int id);
	// TOOLBAR TAB
	void toolButtonSelected(QListViewItem *i);
	void toolBarViewRightButtonPressed(QListViewItem *i,const QPoint &p,int);
	void removeEditedToolButton();
	void addToolBarMenu();
	void addToolBarMenuAbove();
	void addEmptyToolButton();//Below
	void addToolBarSeparatorItem(); //Below
	void addEmptyToolButtonAbove();
	void addToolBarSeparatorItemAbove();
	void copyToolButton();
	void pasteToolButtonBelow();
	void pasteToolButtonAbove();
	bool selectToolButtonById(KviToolBarListViewItem * par,int id);
	// VARIABLES TAB
	void variablesTextChanged();
	// other virtuals
	virtual void done(int r);
	virtual void closeEvent(QCloseEvent *);
	virtual bool eventFilter(QObject * o,QEvent *e);
};

#endif //!_KVI_DLGSCRIPT_H_INCLUDED_
