//
//   File : kvi_frame.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_frame.cpp)
//   Last major modification : Tue Jul 6 1999 14:43:32 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CHECK_RANGE_
//#define _KVI_DEBUG_CLASS_NAME_ "KviFrame"

#include "kvi_debug.h"

#include "kvi_app.h"
#include "kvi_locale.h"
#include "kvi_frame.h"

#include "kvi_menubar.h"
#include "kvi_toolbar.h"
#include "kvi_utoolbar.h"
#include "kvi_statusbar.h"
#include "kvi_taskbar.h"

#include "kvi_mdi.h"
#include "kvi_ircserver.h"
#include "kvi_netutils.h"
#include "kvi_userlist.h"

#include "kvi_window.h"
#include "kvi_ircview.h"
#include "kvi_console.h"
#include "kvi_channel.h"
#include "kvi_query.h"
#include "kvi_uwindow.h"
#include "kvi_dcc_chat.h"
#include "kvi_dcc_send.h"
#include "kvi_dcc_voice.h"
#include "kvi_links.h"
#include "kvi_list.h"

#include "kvi_ircsocket.h"
#include "kvi_error.h"

#include "kvi_dns.h"

#include "kvi_options.h"

#include "kvi_sparser.h"
#include "kvi_uparser.h"

#include "kvi_settings.h"
#include "kvi_madness.h"
#include "kvi_ircview.h"
#include "kvi_systray.h"

#include "kvi_winproperties.h"

#include "kvi_dcc_manager.h"

#include "kvi_regusersdb.h"
#include "kvi_listbox.h"

#include "kvi_event.h"

#include "kvi_process.h"

#include "kvi_asyncwhois.h"

#include <qcursor.h>
#include <qclipboard.h>
#include <qobjcoll.h>
#include <qtoolbutton.h>

//###########################################
//
// CREATION PROCESS
//
//###########################################


//This comes from kvi_app.cpp
extern QPopupMenu *g_pTaskBarPopup;
extern QPopupMenu *g_pViewSelectPopup;
extern QPopupMenu *g_pWindowPopup;

extern KviUserToolBarTemplate *g_pUserToolBarTemplate;


extern KviEventManager * g_pEventManager;

//============ constructor ============//

KviFrame::KviFrame()
:QMainWindow()
{
	// Register with the app
	g_pApp->insertNewFrame(this);
	// Create the local list of windows
	m_pWinList = new QList<KviWindow>;
	m_pWinList->setAutoDelete(false);
	// This seems to be needed (re-check it after Qt2.0 comed out)
	setFocusPolicy(ClickFocus);
	// Reset socket state
	m_state = Ready;
	m_pDns  = 0;
	m_pConsole = 0; //temp hack , so resetGlobals will not segfault
	m_global.pDnsWhoisPending = new QList<KviStr>;
	m_global.pDnsWhoisPending->setAutoDelete(true);
	m_global.pMultimediaFileOfferts = new QList<KviStr>;
	m_global.pMultimediaFileOfferts->setAutoDelete(true);
//	m_global.pAsyncWhoisPending = new QList<KviStr>; //was duplicated, and not deleted
//	m_global.pAsyncWhoisPending->setAutoDelete(true);

	m_pReconnectTimer = new QTimer();
	connect(m_pReconnectTimer,SIGNAL(timeout()),this,SLOT(reconnectToServer()));

	m_pMultimediaFileOffertTimer = new QTimer();
	connect(m_pMultimediaFileOffertTimer,SIGNAL(timeout()),this,SLOT(cleanupMultimediaFileOffertList()));

	m_pRecentServersPopup = new QPopupMenu();
	m_pRecentChannelsPopup = new QPopupMenu();
	m_pRecentNicknamesPopup = new QPopupMenu();

	connect(m_pRecentServersPopup,SIGNAL(activated(int)),this,SLOT(connectToRecentServer(int)));
	connect(m_pRecentChannelsPopup,SIGNAL(activated(int)),this,SLOT(joinRecentChannel(int)));
	connect(m_pRecentNicknamesPopup,SIGNAL(activated(int)),this,SLOT(setRecentNickname(int)));

	// Reset global struct
	m_pUserParser = 0;

	resetGlobals();
	// And start creating self
	createMdiManager();
	createMenuBar();
	createToolBar();
	createStatusBar();
	createTaskBar();
	createUserToolBar();

	// IRC parsers and socket
	m_pServerParser = new KviServerParser(this);
	m_pUserParser   = new KviUserParser(this);
	m_pSysTrayIoLed = (KviSysTrayIoLed * )(m_pToolBar->m_pSysTray->findSysTrayWidget("KviSysTrayIoLed"));
	m_pSysTrayOnLineTimer = (KviSysTrayOnLineTimer *)(m_pToolBar->m_pSysTray->findSysTrayWidget("KviSysTrayOnLineTimer"));
	m_pSocket       = new KviIrcSocket(this,m_pServerParser,m_pSysTrayIoLed);
	m_pDccManager   = new KviDccManager(this);
	// Global user list
	m_pUserList     = new KviIrcUserList();
	// Apply options for the MDI manager
	applyOptions();
	createConsole();
	// Now everything is created...set up the remaining things...
	m_pServerParser->setup(m_pSocket,m_pConsole,m_pUserParser);
	m_pUserParser->setup(m_pSocket,m_pConsole);

	// No active window yet (when shown , the console should be the active one)
	m_global.pCurrentActiveWindow = 0;
	m_iUserListUpdateTimer = 0;
	m_iNotifyListUpdateTimer = 0;
	updateCaption();

	fillRecentServersPopup();
	fillRecentChannelsPopup();
	fillRecentNicknamesPopup();

	// Set the geometry
	setGeometry(g_pOptions->m_rectFrameGeometry); //does not work with KWM... ???

	bool bHalt = false;

	if(g_pEventManager->eventEnabled(KviEvent_OnStartup)){
		bHalt = m_pUserParser->callEvent(KviEvent_OnStartup,m_pConsole,KviStr(KVI_VERSION));
	}

	if(!bHalt)m_pConsole->output(KVI_OUT_KVIRC,KVI_VERSION);
}

//============ applyOptions ============//
void KviFrame::applyOptions()
{
	m_pMdi->setMdiCaptionInactiveBackColor(g_pOptions->m_clrMdiCaptionInactiveBack);
	m_pMdi->setMdiCaptionInactiveForeColor(g_pOptions->m_clrMdiCaptionInactiveFore);
	m_pMdi->setMdiCaptionActiveBackColor(g_pOptions->m_clrMdiCaptionActiveBack);
	m_pMdi->setMdiCaptionActiveForeColor(g_pOptions->m_clrMdiCaptionActiveFore);
	m_pMdi->setMdiCaptionFont(g_pOptions->m_fntMdiCaption);
	m_pMdi->setOpaqueMove(g_pOptions->m_bMdiOpaqueMove);
	m_pMdi->setOpaqueResize(g_pOptions->m_bMdiOpaqueResize);

	if(!g_pOptions->m_pixMdiBack->isNull())m_pMdi->setBackgroundPixmap(*(g_pOptions->m_pixMdiBack));
	else m_pMdi->setBackgroundColor(g_pOptions->m_clrMdiBack);

	for(KviWindow *w = m_pWinList->first();w;w= m_pWinList->next()){
		w->applyOptions();
		QWidget *wdgt = w;
		if(w->mdiParent())wdgt =w->mdiParent();
		// Really ugly hack to FORCE the resize event
		// a resize(width(),height()) won't work...
		wdgt->resize(wdgt->width()+1,wdgt->height()+1);
		wdgt->resize(wdgt->width()-1,wdgt->height()-1);
	}

	m_pToolBar->m_pSysTray->repaint();
}

void KviFrame::createConsole()
{
	//Create the console window.
	m_pConsole = new KviConsole(this);
	addWindow(m_pConsole);
	if(g_pOptions->m_bAutoLogConsole)m_pConsole->m_pView->toggleLogging();
}

//============ createMdiManager ============//
void KviFrame::createMdiManager()
{
	m_pMdi=new KviMdiManager(this);
	setCentralWidget(m_pMdi);
}

//============ createMenuBar ============//
void KviFrame::createMenuBar()
{
	//Here we create the menu bar
	m_pMenuBar = new KviMenuBar(this);
	//QMainWindow does not provide a public setMenuBar() function
	//so we check validity here
	__range_valid(m_pMenuBar==menuBar());
}

//============ createToolBar ============//
void KviFrame::createToolBar()
{
	//Here we create the tool bar
	m_pToolBar = new KviToolBar(this,QMainWindow::Top);
	if(!g_pOptions->m_bToolBarVisible)m_pToolBar->hide();
	setRightJustification(true);
	applyToolbarOptions();
}

//============ createToolBar ============//
void KviFrame::createUserToolBar()
{
	//Here we create the tool bar
	m_pUserToolBar=0;
	updateUserToolBar();
}

void KviFrame::updateUserToolBar()
{
	if(!g_pUserToolBarTemplate->isEmpty())
	{
		if(!m_pUserToolBar) {
			m_pUserToolBar = new KviUserToolBar(this,QMainWindow::Top);
//			addToolBar(m_pUserToolBar,Top, true);
		}
		m_pUserToolBar->copyFromTemplate(g_pUserToolBarTemplate);
		m_pUserToolBar->hide();
		m_pUserToolBar->show();
	} else if (m_pUserToolBar) {
		delete m_pUserToolBar;
		m_pUserToolBar=0;
	}
}

void KviFrame::applyToolbarOptions()
{
//	setUsesBigPixmaps(g_pOptions->m_bUseBigToolbarPixmaps);
	QObjectList * l = m_pToolBar->queryList("QToolButton",0,false);
	QObjectListIt it(*l);
	while(it.current()){
		((QToolButton *)it.current())->setUsesTextLabel(g_pOptions->m_bUseTextToolbarComment);
		((QToolButton *)it.current())->setUsesBigPixmap(g_pOptions->m_bUseBigToolbarPixmaps);
		++it;
	}
	delete l;
}

void KviFrame::slot_toggleToolBar()
{
	if(m_pToolBar->isVisible()){
		m_pToolBar->hide();
		g_pOptions->m_bToolBarVisible = false;
	} else {
		m_pToolBar->show();
		g_pOptions->m_bToolBarVisible = true;
	}
}

//============ createStatusBar ============//
void KviFrame::createStatusBar()
{
	m_pStatusBar = new KviStatusBar(this);
	if(g_pOptions->m_bStatusBarVisible)m_pStatusBar->show();
	else m_pStatusBar->hide();
	__range_valid(m_pStatusBar==statusBar());
}

void KviFrame::slot_toggleStatusBar()
{
	if(m_pStatusBar->isVisible()){
		m_pStatusBar->hide();
		g_pOptions->m_bStatusBarVisible = false;
	} else {
		m_pStatusBar->show();
		g_pOptions->m_bStatusBarVisible = true;
	}
}

//============ createTaskBar ==============//
void KviFrame::createTaskBar()
{
	QMainWindow::ToolBarDock d;
	switch(g_pOptions->m_iTaskBarDock){
		case KVI_TOOL_BAR_DOCK_RIGHT: d = QMainWindow::Right; break;
		case KVI_TOOL_BAR_DOCK_LEFT: d = QMainWindow::Left; break;
		case KVI_TOOL_BAR_DOCK_BOTTOM: d = QMainWindow::Bottom; break;
		default: d = QMainWindow::Top; break;
	}
	m_pTaskBar = new KviTaskBar(this,d);
	if(!g_pOptions->m_bTaskBarVisible)m_pTaskBar->hide();
}

void KviFrame::slot_toggleTaskBar()
{
	if(m_pTaskBar->isVisible()){
		m_pTaskBar->hide();
		g_pOptions->m_bTaskBarVisible = false;
	} else {
		m_pTaskBar->show();
		g_pOptions->m_bTaskBarVisible = true;
	}
}

void KviFrame::fillRecentServersPopup()
{ fillRecentPopup(m_pRecentServersPopup,g_pOptions->m_szRecentServers); }
void KviFrame::fillRecentChannelsPopup()
{ fillRecentPopup(m_pRecentChannelsPopup,g_pOptions->m_szRecentChannels); }
void KviFrame::fillRecentNicknamesPopup()
{ fillRecentPopup(m_pRecentNicknamesPopup,g_pOptions->m_szRecentNicknames); }

void KviFrame::fillRecentPopup(QPopupMenu * popup,KviStr &data)
{
	popup->clear();
	const char *aux = data.ptr();
	KviStr token;
	while(*aux){
		aux = kvi_extractToken(token,aux,',');
		if(token.hasData()){
			popup->insertItem(token.ptr());
		}
	}
}

void KviFrame::addRecentServer(KviStr &serverName,KviStr &port)
{
	KviStr entry = serverName;
	entry.stripWhiteSpace();
	entry.append(':');
	if(port.isUnsignedNum())entry.append(port);
	else entry.append("6667");

	if(addRecentEntry(g_pOptions->m_szRecentServers,entry)){
		for(KviFrame * f=g_pApp->m_pFrameList->first();f;f=g_pApp->m_pFrameList->next())
			fillRecentServersPopup();
	}
}

void KviFrame::addRecentChannel(const char *chan)
{
	KviStr channel(chan);
	channel.stripWhiteSpace();
	if(addRecentEntry(g_pOptions->m_szRecentChannels,channel)){
		for(KviFrame * f=g_pApp->m_pFrameList->first();f;f=g_pApp->m_pFrameList->next())
			fillRecentChannelsPopup();
	}
}

void KviFrame::addRecentNickname(KviStr &nickname)
{
	nickname.stripWhiteSpace();
	if(addRecentEntry(g_pOptions->m_szRecentNicknames,nickname)){
		for(KviFrame * f=g_pApp->m_pFrameList->first();f;f=g_pApp->m_pFrameList->next())
			fillRecentNicknamesPopup();
	}
}

bool KviFrame::addRecentEntry(KviStr &list,KviStr &entry)
{
	if(list.findFirstIdx(entry.ptr(),false) >= 0)return false;
	int numEntries = list.contains(',') + 1;
	if(numEntries >= KVI_MAX_RECENT_ENTRIES){
		KviStr dummy;
		list.getToken(dummy,',');
	}
	if(!list.isEmpty())list.append(',');
	list.append(entry);
	return true;
}

void KviFrame::connectToRecentServer(int id)
{
	KviStr serverAndPort = m_pRecentServersPopup->text(id);
	serverAndPort.stripWhiteSpace();
	if(serverAndPort.isEmpty())return;
	KviStr server;
	serverAndPort.getToken(server,':');
	if(server.isEmpty())return;
	g_pOptions->m_pServerManager->iWantThisServerToBeCurrent(server.ptr(),serverAndPort.ptr());
	// Jump out of this handler , call it from the main event loop...
	doAsyncServerCommandCall();
}

void KviFrame::joinRecentChannel(int id)
{
	KviStr channel = m_pRecentChannelsPopup->text(id);
	channel.stripWhiteSpace();
	if(channel.isEmpty())return;
	if(m_state != Connected)return;
	if(!m_pSocket->sendFmtData("JOIN %s",channel.ptr()))m_pConsole->output(KVI_OUT_ERROR,"You're not connected to a server");
}

void KviFrame::setRecentNickname(int id)
{
	KviStr nick = m_pRecentNicknamesPopup->text(id);
	nick.stripWhiteSpace();
	if(nick.isEmpty())return;
	if(m_state != Connected)return;
	if(!m_pSocket->sendFmtData("NICK %s",nick.ptr()))m_pConsole->output(KVI_OUT_ERROR,"You're not connected to a server");
}

//============ ~KviFrame ============//
KviFrame::~KviFrame()
{
	if(g_pEventManager->eventEnabled(KviEvent_OnShutdown)){
		m_pUserParser->callEvent(KviEvent_OnShutdown,m_pConsole,KviStr(""));
	}

	// Save the last geometry
	if(g_pOptions->m_bUseHackedFrameGeometry){
		g_pOptions->m_rectFrameGeometry.moveTopLeft(frameGeometry().topLeft());
		g_pOptions->m_rectFrameGeometry.setSize(size());
	} else g_pOptions->m_rectFrameGeometry = geometry();

	if(m_state == Connected){
		// Send a quit....in extremis
		if(g_pOptions->m_szQuitMessage.hasData()){
			m_pSocket->sendFmtData("QUIT :%s",g_pOptions->m_szQuitMessage.ptr());
		} else m_pSocket->sendData("QUIT");
	}
	if(m_pReconnectTimer->isActive())m_pReconnectTimer->stop();
	delete m_pReconnectTimer;
	if(m_pMultimediaFileOffertTimer->isActive())m_pMultimediaFileOffertTimer->stop();
	delete m_pMultimediaFileOffertTimer;

	stopNotifyOrWatchList();
	clearConsoleListBox(); // Do it before deleting the global userlist
	__debug_1arg("KviFrame : Child list with %d items",m_pWinList->count());
	// safely close the windows so properties are saved...
	KviWindow *pWnd = 0;
	while((pWnd = m_pWinList->first()))closeWindow(pWnd);
	delete m_pWinList;
	delete m_pServerParser;
	delete m_pUserParser;
	delete m_pSocket;
	delete m_pDccManager;
	if(m_pDns)delete m_pDns;
	delete m_pUserList;
	delete m_global.pDnsWhoisPending;
	delete m_global.pMultimediaFileOfferts;
	delete m_pRecentServersPopup;
	delete m_pRecentNicknamesPopup;
	delete m_pRecentChannelsPopup;
	delete m_pUserToolBar;
	g_pApp->removeFrame(this);
}

void KviFrame::clearConsoleListBox()
{
	if(!m_pConsole)return;
	if(m_pConsole->m_pListBox->count() > 0)m_pConsole->m_pListBox->partAll();
}

//============ closeEvent ============//
void KviFrame::closeEvent(QCloseEvent *e)
{
	e->accept();
	delete this;
}

//============= createChannel ===============//
KviChannel * KviFrame::createChannel(const char *name)
{
	KviChannel * c= new KviChannel(this,name);
	addWindow(c);
	if(g_pOptions->m_bAutoLogChannels)c->m_pView->toggleLogging();
	return c;
}
//============= createLinksWindow ===============//
KviLinksWindow * KviFrame::createLinksWindow()
{
	__range_invalid(findWindow(KVI_LINKS_WINDOW_NAME));
	KviLinksWindow * c= new KviLinksWindow(this);
	addWindow(c);
	return c;
}
KviLinksWindow * KviFrame::getLinksWindow()
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_LINKS)return ((KviLinksWindow *)w);
	}
	return createLinksWindow();
}
//============= createListWindow ===============//
KviListWindow * KviFrame::createListWindow()
{
	__range_invalid(findWindow(KVI_LIST_WINDOW_NAME));
	KviListWindow * c= new KviListWindow(this);
	addWindow(c);
	return c;
}
KviListWindow * KviFrame::getListWindow()
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_LIST)return ((KviListWindow *)w);
	}
	return createListWindow();
}
void KviFrame::activateListWindow()
{
	KviListWindow * w = getListWindow();
	if(activeWindow() != w){
		w->raise();
		w->setFocus();
	}
}
//============= createQuery ===============//
KviQuery * KviFrame::createQuery(const char *name)
{
	KviQuery * q=new KviQuery(this,name);
	addWindow(q,!g_pOptions->m_bCreateIconifiedQuery);
	if(g_pOptions->m_bCreateIconifiedQuery)q->minimize(false); //do not animate
	if(g_pOptions->m_bAutoLogQueries)q->m_pView->toggleLogging();
	return q;
}

KviQuery * KviFrame::raiseOrCreateQuery(const char *name)
{
	KviQuery * query = findQuery(name);
	if(!query)query = createQuery(name);
	query->delayedAutoRaise();
	return query;
}

//============= createUWindow ===============//
KviUWindow * KviFrame::createUWindow(const char *name)
{
	KviUWindow * u=new KviUWindow(this,name);
	addWindow(u,true);
	return u;
}
//============= createDccChat ===============//
KviDccChat * KviFrame::createDccChat(const char *name)
{
	KviDccChat * q=new KviDccChat(this,name);
	addWindow(q,!g_pOptions->m_bCreateIconifiedDccChat);
	if(g_pOptions->m_bCreateIconifiedDccChat)q->minimize(false); //do not animate
	if(g_pOptions->m_bAutoLogChats)q->m_pView->toggleLogging();
	return q;
}

//============= createDccChat ===============//
KviDccSend * KviFrame::createDccSend(const char *name)
{
	KviDccSend * q=new KviDccSend(this,name);
	addWindow(q,!g_pOptions->m_bCreateIconifiedDccSend);
	if(g_pOptions->m_bCreateIconifiedDccSend)q->minimize(false); //do not animate
	if(g_pOptions->m_bAutoLogSends)q->m_pView->toggleLogging();
	return q;
}
//============= createDccVoice ===============//
KviDccVoice * KviFrame::createDccVoice(const char *name)
{
	KviDccVoice * q=new KviDccVoice(this,name);
	addWindow(q,!g_pOptions->m_bCreateIconifiedDccVoice);
	if(g_pOptions->m_bCreateIconifiedDccVoice)q->minimize(false); //do not animate
//	if(g_pOptions->m_bAutoLogSends)q->m_pView->toggleLogging();
	return q;
}
//================ addWindow ================//
void KviFrame::addWindow(KviWindow *pWnd,bool bShow)
{
	//The window can be added only once :)
	__range_valid(m_pWinList->findRef(pWnd) == -1);
	m_pWinList->append(pWnd);
	pWnd->setTaskBarButton(m_pTaskBar->addWinButton(pWnd));
	KviWindowPackedProperty * prop = g_pOptions->m_pWinPropertiesList->findProperty(pWnd->caption());
	if(prop){
		KviWindowProperty p;
		g_pOptions->m_pWinPropertiesList->unpackProperty(&p,prop);
		pWnd->setProperties(&p);
		if(p.isDocked){
			attachWindow(pWnd,bShow,false,&(p.rect)); //attach , do not cascade
			if(p.isMaximized)pWnd->maximize();
		} else {
			pWnd->setGeometry(p.rect);
			if(bShow)pWnd->show();
			pWnd->youAreDetached();
			if(bShow)pWnd->setFocus();
		}
	} else attachWindow(pWnd,bShow);
}

//============ attachWindow ============//
void KviFrame::attachWindow(KviWindow *pWnd,bool bShow,bool overrideGeometry,QRect *r)
{
	//Creates a new KviMdiChild in m_pMdi manager
	//Attacches the KviWindow to that child and starts the manager
	__enter("attachWindow");
	__range_valid(pWnd->parent() == 0);
	KviMdiChild *lpC=new KviMdiChild(m_pMdi);
	lpC->setClient(pWnd);
	pWnd->youAreAttacched(lpC);
	m_pTaskBar->windowAttacched(pWnd,true);
	if(!overrideGeometry && r)lpC->setGeometry(*r);
	m_pMdi->manageChild(lpC,bShow,overrideGeometry);
	m_pMdi->setFocus();
	__leave("attachWindow");
}

//============= detachWindow ==============//
void KviFrame::detachWindow(KviWindow *pWnd)
{
	//Reparents pWnd to 0
	//Destroys the KviMdiChild that contained it
	__range_valid(pWnd->parent());
	if(!pWnd->parent())return;
	__range_valid(pWnd->parent()->inherits("KviMdiChild"));
	KviMdiChild *lpC=pWnd->mdiParent();
	lpC->unsetClient();
	pWnd->youAreDetached();
	m_pTaskBar->windowAttacched(pWnd,false);
	m_pMdi->destroyChild(lpC,false); //Do not focus the new top child , we loose focus...
	pWnd->setFocus();
}

//============== closeWindow ==============//
void KviFrame::closeWindow(KviWindow *pWnd)
{
	//Closes a child window. sends no close event : simply deletes it
//	__debug_1arg("KviFrame : deleting window %s",pWnd->caption());
	pWnd->saveProperties();
#ifdef _KVI_DEBUG_CHECK_RANGE_
	__range_valid(m_pWinList->removeRef(pWnd));
#else
	m_pWinList->removeRef(pWnd);
#endif
	if(pWnd == m_global.pCurrentActiveWindow)
		m_global.pCurrentActiveWindow = ((pWnd == m_pConsole) ? 0 : m_pConsole);
	pWnd->setTaskBarButton(0);
	m_pTaskBar->removeWinButton(pWnd);
	if(pWnd->isAttacched())m_pMdi->destroyChild(pWnd->mdiParent());
	else delete pWnd;
}

//================== findWindow =================//
KviWindow * KviFrame::findWindow(const char *caption)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(kvi_strEqualCI(w->caption(),caption))return w;
	}
	return 0;
}
//================== findChannel =================//
KviChannel * KviFrame::findChannel(const char *chanName)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_CHANNEL){
			if(kvi_strEqualCI(w->caption(),chanName))return ((KviChannel *)w);
		}
	}
	return 0;
}
//================== findQuery =================//
KviQuery * KviFrame::findQuery(const char *queryName)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_QUERY){
			if(kvi_strEqualCI(w->caption(),queryName))return ((KviQuery *)w);
		}
	}
	return 0;
}
//================== findUWindow =================//
KviUWindow * KviFrame::findUWindow(const char *uwindowName)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_UWINDOW){
			if(kvi_strEqualCI(w->caption(),uwindowName))return ((KviUWindow *)w);
		}
	}
	return 0;
}
//================== findWaitingDccSend =================//
KviDccSend * KviFrame::findWaitingDccSend(unsigned short uPort,const char *nick)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_SEND){
			if(!((KviDccSend *)w)->m_bThreadRunning){
				if(((KviDccSend *)w)->m_dccSendStruct != 0){
					if((uPort == ((KviDccSend *)w)->m_dccSendStruct->uPort) || g_pOptions->m_bIgnorePortNumberInResumeRequests){
						if(kvi_strEqualCI(((KviDccSend *)w)->m_dccSendStruct->nick.ptr(),nick))return ((KviDccSend *)w);
					}
				}
			}
		}
	}
	return 0;
}
//================== findListeningDccSend =================//
KviDccSend * KviFrame::findListeningDccSend(unsigned short uPort,const char *nick)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_SEND){
			if(((KviDccSend *)w)->youAreListening(uPort,nick))return ((KviDccSend *)w);
		}
	}
	return 0;
}
//================== activeWindow ===================//
KviWindow * KviFrame::activeWindow()
{
	__range_valid(windowExists(m_global.pCurrentActiveWindow));
	return m_global.pCurrentActiveWindow ? m_global.pCurrentActiveWindow : m_pConsole;
}
//================== windowExists ? =================//
bool KviFrame::windowExists(KviWindow *pWnd)
{
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w == pWnd)return true;
	}
	return false;
}

QPopupMenu * KviFrame::windowPopup(KviIrcView * pView,KviWindow * pWnd,bool bIncludeTaskbarPopup)
{
	__range_valid(pWnd);
	__range_valid(pView);
	g_pWindowPopup->clear();
	if(bIncludeTaskbarPopup){
		g_pWindowPopup->insertItem(__tr("Window"),taskBarPopup(pWnd,false));
		g_pWindowPopup->insertSeparator();
	}
	if(pView->isLogging())g_pWindowPopup->insertItem(__tr("Stop logging"),pView,SLOT(toggleLogging()));
	else g_pWindowPopup->insertItem(__tr("Start logging (default file)"),pView,SLOT(toggleLogging()));
	g_pWindowPopup->insertItem(__tr("Log to file..."),pView,SLOT(logToFile()));
	g_pWindowPopup->insertSeparator();
	if(pView->timestamp())g_pWindowPopup->insertItem(__tr("Timestamp off"),pView,SLOT(toggleTimestamp()));
	else g_pWindowPopup->insertItem(__tr("Timestamp on"),pView,SLOT(toggleTimestamp()));
	if(pView->imagesVisible())g_pWindowPopup->insertItem(__tr("Hide output images"),pView,SLOT(toggleImages()));
	else g_pWindowPopup->insertItem(__tr("Show output images"),pView,SLOT(toggleImages()));
	g_pWindowPopup->insertSeparator();
	g_pWindowPopup->insertItem(__tr("Clear text buffer"),pView,SLOT(clearBuffer()));
	g_pWindowPopup->insertItem(__tr("Save buffer to file"),pView,SLOT(saveBufferToFile()));
//#warning "missing setBackgroundImage & setFont , then save these options too in winproperties"
	return g_pWindowPopup;
}

//================ taskBarPopup =================//
QPopupMenu * KviFrame::taskBarPopup(KviWindow *pWnd,bool bIncludeWindowPopup)
{
	//returns the g_pTaskBarPopup filled according to the KviWindow state
	g_pTaskBarPopup->clear();
	if(pWnd->isAttacched()){
		g_pTaskBarPopup->insertItem(_i18n_("Undock"),pWnd,SLOT(detach()));
		g_pTaskBarPopup->insertSeparator();
		if(pWnd->isMinimized() || pWnd->isMaximized())
			g_pTaskBarPopup->insertItem(_i18n_("Restore"),pWnd,SLOT(restore()));
		if(!pWnd->isMaximized())g_pTaskBarPopup->insertItem(_i18n_("Maximize"),pWnd,SLOT(maximize()));
		if(!pWnd->isMinimized())g_pTaskBarPopup->insertItem(_i18n_("Minimize"),pWnd,SLOT(minimize()));
	} else g_pTaskBarPopup->insertItem(_i18n_("Dock"),pWnd,SLOT(attach()));
	if(pWnd->type() != KVI_WND_TYPE_CONSOLE){
		g_pTaskBarPopup->insertSeparator();
		g_pTaskBarPopup->insertItem(_i18n_("Close"),pWnd,SLOT(close()));
	}
	if(pWnd->m_pView && bIncludeWindowPopup){
		// the window has a view...get the window popup
		g_pTaskBarPopup->insertSeparator();
		g_pTaskBarPopup->insertItem(__tr("Operations"),windowPopup(pWnd->m_pView,pWnd,false));  //alvoid recursion
	}
	return g_pTaskBarPopup;
}

//################################################
// The following functions are called DIRECTLY
// from the KviInput class.
//################################################
void KviFrame::switchWindows(bool bRight)
{
	KviMdiChild * pActMdi = m_pMdi->topChild();
	if(!pActMdi)return;
	KviWindow * pAct = (KviWindow *)pActMdi->m_pClient;
	if(!pAct)return;
	KviTaskBarButton * pNext = m_pTaskBar->getNextWindowButton(bRight,pAct);
	if(!pNext)return;
	if(pNext->m_pWindow->isAttacched())taskbarButtonLeftClicked(pNext->m_pWindow,pNext);
	else pNext->m_pWindow->attach();
	
}
//################################################
// The following functions are called DIRECTLY
// from the KviTaskBar class.
//################################################
void KviFrame::taskbarButtonLeftClicked(KviWindow *pWnd,KviTaskBarButton *btn)
{
//	__enter("taskbarButtonLeftClicked");
	if(pWnd->isAttacched()){
		if(pWnd->hasFocus() && (pWnd->mdiParent() == m_pMdi->topChild())){
			btn->setOn(false);
			pWnd->minimize();
		} else {
			btn->setOn(true);
			pWnd->restore();
			pWnd->setFocus();
		}
	} else { //not so cool...bu can not do more...
		btn->setOn(true);
		g_pApp->raiseTopLevelWidget(pWnd);
	}
//	__leave("taskbarButtonLeftClicked");
}
void KviFrame::taskbarButtonRightClicked(KviWindow *pWnd,KviTaskBarButton *btn)
{
//	__enter("taskbarButtonRightClicked");
	taskBarPopup(pWnd,true)->popup(QCursor::pos());
//	__leave("taskbarButtonRightClicked");
}

//################################################
// The following functions are called DIRECTLY
// from the KviWindow class.
//################################################

void KviFrame::childWindowGainFocus(KviWindow *pWnd)
{
//#warning "TEST"
//	debug("GainedFocus : %s",pWnd->caption());
//#warning "TEST"
	m_global.pCurrentActiveWindow = pWnd;
	m_pTaskBar->setActiveButton(pWnd);
}

void KviFrame::childWindowCloseRequest(KviWindow *pWnd)
{
//	__debug_1arg("Child window %s requested close",pWnd->m_szCaption.ptr());
	//Ignore console close requests
	if(pWnd==m_pConsole){
		//Get some fun
#ifdef COMPILE_MADNESS
		for(int i=0;i<5;i++){
			if(pWnd->isAttacched())kvi_madnessForWidget(m_pMdi);
			else kvi_madnessForWidget(pWnd->m_pView);
		}
		m_pStatusBar->tempText(__tr("And now ?"),5000);
#else
		m_pStatusBar->tempText(__tr("Sorry...no madness available."),5000);
#endif
	} else {
		// Other window types
		switch(pWnd->type()){
			case KVI_WND_TYPE_CHANNEL:
				partChannel(pWnd);
			break;
			case KVI_WND_TYPE_QUERY:
				closeWindow(pWnd);
			break;
			case KVI_WND_TYPE_CHAT:
				closeWindow(pWnd);
			break;
			case KVI_WND_TYPE_SEND:
				closeWindow(pWnd);
			break;
			case KVI_WND_TYPE_VOICE:
				closeWindow(pWnd);
			break;
			default:
				closeWindow(pWnd);
			break;
		}
	}
}

void KviFrame::partChannel(KviWindow *pWnd)
{
	__range_valid(pWnd);
	__range_valid(pWnd->type() == KVI_WND_TYPE_CHANNEL);
	__range_valid(m_pWinList->findRef(pWnd));
	pWnd->outputNoFmt(KVI_OUT_INTERNAL,__tr("Part message sent to server , waiting for reply..."));
	if(g_pOptions->m_szPartMessage.isEmpty())m_pSocket->sendFmtData("PART %s",pWnd->caption());
	else m_pSocket->sendFmtData("PART %s :%s",pWnd->caption(),g_pOptions->m_szPartMessage.ptr());
}

void KviFrame::slot_requestLinks()
{
	if(!m_pSocket->sendData("LINKS"))activeWindow()->outputNoFmt(KVI_OUT_ERROR,__tr("Oops..cannot request LINKS: socket write failed"));
}

bool KviFrame::checkHighlight(const char *msg)
{
	for(KviStr * s=g_pOptions->m_pHighlightWords->first();s;s=g_pOptions->m_pHighlightWords->next()){
		const char *aux = msg;
		char c = tolower(*(s->ptr())); //newer \0 !
		// Run thru the string
		while(*aux){
			while(*aux  && (tolower(*aux) != c))aux++; //run until we find a probable match
			if(*aux){
				// probable match...check
				if(kvi_strEqualCIN(s->ptr(),aux,s->len()))return true;
				else aux++;
			} // else finished the string
		}
	}
	return false;
}

void KviFrame::outputPrivmsg(KviWindow *wnd,int type,const char *talker,
		const char *talker_mask,const char *msg)
{
	__range_valid(wnd);
	if(g_pOptions->m_bWordHighlighting){
		if(checkHighlight(msg))type = KVI_OUT_HIGHLIGHT;
	}
	if(g_pOptions->m_bUseExtendedPrivmsgView){
		// <PREFIX>nick[!user@host]<POSTFIX>This is a test message
		if(g_pOptions->m_bShowUserAndHostInPrivmsgs){
			wnd->output(type,__tr("%s%s%s%s"),g_pOptions->m_szExtPrivmsgPrefix.ptr(),
				talker_mask,g_pOptions->m_szExtPrivmsgPostfix.ptr(),msg);
		} else {
			wnd->output(type,__tr("%s%s%s%s"),g_pOptions->m_szExtPrivmsgPrefix.ptr(),
				talker,g_pOptions->m_szExtPrivmsgPostfix.ptr(),msg);
		}
	} else wnd->output(type,__tr("<%s> %s"),talker,msg);
}

//################################################
// Userlist updates...
//################################################

void KviFrame::startUpdatingUserList()
{
	if(m_iUserListUpdateTimer != 0)return; //Already running
	if(m_state != Connected)return; //Can not do it man...
	if(g_pOptions->m_bDisableUpdateUserList)return; //User do not want this...Lag ?
	KviStr buffer;
	m_pUserList->getFirstFiveNicksToUpdate(buffer);
	if(buffer.isEmpty())return; //None to update
	// Do the first update now...
	if(!m_pSocket->sendFmtData("USERHOST %s",buffer.ptr()))return; //Oppps...something wrong ?
	m_global.bUserHostInProgress = true;
	if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Starting user list updates (%s)"),buffer.ptr());
	__debug_1arg("Updating user list : %s",buffer.ptr());
	if(g_pOptions->m_iUserListUpdateTimeoutInSecs < 5)g_pOptions->m_iUserListUpdateTimeoutInSecs = 5; //Minimum...avoid self-lag
	m_iUserListUpdateTimer = startTimer(g_pOptions->m_iUserListUpdateTimeoutInSecs * 1000);

}

void KviFrame::stopUpdatingUserList()
{
	if(m_iUserListUpdateTimer == 0)return; //Already stopped
	if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("User list updates stopped"));
	killTimer(m_iUserListUpdateTimer);
	m_global.bUserHostInProgress = false;
	m_iUserListUpdateTimer = 0;
}

void KviFrame::timerEvent(QTimerEvent *e)
{
	if(e->timerId() == m_iUserListUpdateTimer){
		if(m_state == Connected){
			if(m_global.bUserHostInProgress)return; //Need to wait  , the last call has not returned yet...
			KviStr buffer;
			m_pUserList->getFirstFiveNicksToUpdate(buffer);
			if(buffer.hasData()){
				if(m_pSocket->sendFmtData("USERHOST %s",buffer.ptr())){
					if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Updating user list (%s)"),buffer.ptr());
					m_global.bUserHostInProgress = true;
					return; //all ok
				}
			}
		}
		stopUpdatingUserList();
	} else if(e->timerId() == m_iNotifyListUpdateTimer){
		if(m_global.szNotifyListString.hasData()){
			m_pSocket->sendFmtData("ISON %s",m_global.szNotifyListString.ptr());
		} else stopNotifyList();
	} else QMainWindow::timerEvent(e);
}

//=====================================================================================================
// Start NOTIFY or WATCH LIST
// If g_pOptions->m_bEnableDalNetStyleNotifyMethod is true
// use the WATCH list , otherwise use the old ISON ping.
//

void KviFrame::startNotifyOrWatchList()
{
	__range_valid(m_global.szNotifyListString.isEmpty());
	__range_valid(m_iNotifyListUpdateTimer == 0);

	if(g_pOptions->m_bEnableNotifyList && (m_state == Connected)){
		// Ok....should run...
		if(g_pOptions->m_bEnableDalNetStyleNotifyMethod && m_global.bServerSupportsWatchList){
			if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Notify : watch list is enabled"));
			startWatchList();
		} else {
			if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Notify : notify list is enabled"));
			startNotifyList();
		}
	}
}

void KviFrame::startWatchList()
{
	__range_valid(m_global.szNotifyListString.isEmpty());
	__range_valid(m_iNotifyListUpdateTimer == 0);
	__range_valid(g_pOptions->m_bEnableDalNetStyleNotifyMethod);

	if(g_pOptions->m_bEnableNotifyList && g_pOptions->m_bEnableDalNetStyleNotifyMethod &&
		(m_state == Connected) && m_global.szNotifyListString.isEmpty() && m_global.bServerSupportsWatchList)
	{
		// was not running
		g_pOptions->m_pRegUsersDb->getWatchString(m_global.szNotifyListString);
		if(m_global.szNotifyListString.hasData()){
			if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Starting watch list..."));
			m_pSocket->sendFmtData("WATCH %s",m_global.szNotifyListString.ptr());
		}
	}
}

void KviFrame::startNotifyList()
{
	__range_valid(m_global.szNotifyListString.isEmpty());
	__range_valid(m_iNotifyListUpdateTimer == 0);
	__range_invalid(g_pOptions->m_bEnableDalNetStyleNotifyMethod);

	if(g_pOptions->m_bEnableNotifyList && (m_iNotifyListUpdateTimer == 0) && (m_state == Connected)){ //Not started yet...
		// Try to start
		g_pOptions->m_pRegUsersDb->getNotifyString(m_global.szNotifyListString);
		if(m_global.szNotifyListString.hasData()){
			// Ok...users to notify
			m_pSocket->sendFmtData("ISON %s",m_global.szNotifyListString.ptr());
			if(g_pOptions->m_iNotifyListCheckTimeoutInSecs < 3)g_pOptions->m_iNotifyListCheckTimeoutInSecs = 3;
			m_iNotifyListUpdateTimer = startTimer(g_pOptions->m_iNotifyListCheckTimeoutInSecs * 1000);
			if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Starting notify list..."));
			return;
		}
	}
	// If we're still here...we could not start it for some reason...clear the vars...
	m_global.szNotifyListString = "";
	m_iNotifyListUpdateTimer = 0;
}

void KviFrame::stopNotifyOrWatchList()
{
	if(!m_pConsole)return; // this might be called from the constructor
	if(m_iNotifyListUpdateTimer)stopNotifyList();
	else if(m_global.szNotifyListString.hasData())stopWatchList();
	// Just to be sure...clear everything...
	clearConsoleListBox();
	m_global.szNotifyListString = "";
	m_iNotifyListUpdateTimer = 0;
}

void KviFrame::stopNotifyList()
{
//	__range_valid(m_iNotifyListUpdateTimer != 0);
//	__range_valid(m_global.szNotifyListString.hasData());

	if(m_iNotifyListUpdateTimer != 0){
		// was running
		killTimer(m_iNotifyListUpdateTimer);
		if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Notify list stopped"));
	}
	clearConsoleListBox();
	m_global.szNotifyListString = "";
	m_iNotifyListUpdateTimer = 0;
}

void KviFrame::stopWatchList()
{
	__range_valid(m_iNotifyListUpdateTimer == 0);
	__range_valid(m_global.szNotifyListString.hasData());
	if(m_global.szNotifyListString.hasData() && (m_state == Connected)){
		if(m_global.bServerSupportsWatchList)m_pSocket->sendFmtData("WATCH clear"); //avoid clearing is unsupported
		if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Watch list stopped"));
	}
	clearConsoleListBox();
	m_global.szNotifyListString = "";
}

void KviFrame::updateNotifyOrWatchList()
{
	if((!g_pOptions->m_bEnableNotifyList) || (m_state != Connected)){
		// kill everything...
		if(m_iNotifyListUpdateTimer)stopNotifyList();
		if(m_global.szNotifyListString.hasData())stopWatchList();
		clearConsoleListBox();
		m_global.szNotifyListString = "";
		m_iNotifyListUpdateTimer = 0;
		return;
	}
	// Something is enabled now...
	if(g_pOptions->m_bEnableDalNetStyleNotifyMethod && m_global.bServerSupportsWatchList){
		// Watch list is enabled now!
		if(m_iNotifyListUpdateTimer){
			// Notify list was running
			stopNotifyList(); // Kill the notify list
			startWatchList(); // and start the Watch list
		} else {
			// Notify list was not running
			if(m_global.szNotifyListString.isEmpty()){
				// Watch list was not running yet...start it
				startWatchList();
			} else {
				// Watch list was already running...update it
				KviStr tmp;
				g_pOptions->m_pRegUsersDb->getWatchString(tmp);
				// Calculate the Diff :)
				if(kvi_strEqualCI(tmp.ptr(),m_global.szNotifyListString.ptr()))return; //no changes
				// Watch list changed!
				// For now just resend the new list to the server
				if(tmp.isEmpty()){
					// ooops..no more nicks to watch...
					if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Watch list changed ... no users to watch"));
					stopWatchList();
					return;
				}
				m_pSocket->sendFmtData("WATCH clear");
				clearConsoleListBox();
				m_pSocket->sendFmtData("WATCH %s",tmp.ptr());
				m_global.szNotifyListString = tmp;
				if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Watch list changed ... sending modifications to server"));
			}
		}
	} else {
		// Notify list is enabled now!
		if((m_iNotifyListUpdateTimer == 0) && m_global.szNotifyListString.hasData()){
			// The watch list was running before
			stopWatchList();
			startNotifyList();
		} else {
			if(m_iNotifyListUpdateTimer == 0){
				// Nothing was running before
				startNotifyList();
			} else {
				// Notify list was already running...
				KviStr tmp;
				g_pOptions->m_pRegUsersDb->getNotifyString(tmp);
				if(kvi_strEqualCI(tmp.ptr(),m_global.szNotifyListString.ptr()))return; // No changes...
				if(tmp.isEmpty()){
					if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Notify list changed ... no users to look for"));
					stopNotifyList();
					return;
				}
				// Ok...users to notify
				m_global.szNotifyListString = tmp;
				m_pSocket->sendFmtData("ISON %s",m_global.szNotifyListString.ptr());
				if(g_pOptions->m_iNotifyListCheckTimeoutInSecs < 3)g_pOptions->m_iNotifyListCheckTimeoutInSecs = 3;
				killTimer(m_iNotifyListUpdateTimer);
				clearConsoleListBox();
				m_iNotifyListUpdateTimer = startTimer(g_pOptions->m_iNotifyListCheckTimeoutInSecs * 1000);
				if(g_pOptions->m_bBeVerbose)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Notify list changed ... sending modifications to server"));
			}
		}
	}
}


//################################################
// The following functions are called DIRECTLY
// from the KviIrcView class.
//################################################


//================================================
// TEXT SELECTION
//================================================

void KviFrame::ircViewTextSelected(KviIrcView *pView,KviWindow *pWnd,const char *text)
{
	// The user selected some text in a view
	__range_valid(text);
	__range_valid(pWnd);
	__range_valid(pView);
	if(!(*text)){
		m_global.szCurrentSelectionText = ""; //selected nothing ?
		return;
	}
	m_global.szCurrentSelectionText = text; //remember it now...

	if(g_pOptions->m_bDoNotShowThePopupOnTextSelected){
		copyCurrentSelectionTextToClipboard();
		return;
	}

	// Extract the first token
	KviStr firstLine;
	kvi_extractToken(firstLine,text,'\n');
	firstLine.stripWhiteSpace();
	KviStr firstToken;
	kvi_extractToken(firstToken,firstLine.ptr(),' ');

	g_pViewSelectPopup->clear();
	g_pViewSelectPopup->insertItem(__tr("Copy to clipboard"),this,SLOT(copyCurrentSelectionTextToClipboard()));

	firstToken.stripWhiteSpace();

	if(firstToken.hasData()){
		g_pViewSelectPopup->insertSeparator();
		KviStr tmp(KviStr::Format,"Help on %s",firstToken.ptr());
		g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(doHelpOnCurrentlySelectedTextToken()));
		tmp.sprintf(__tr("Search help for '%s ...'"),firstToken.ptr());
		g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(doHelpSearchOnCurrentlySelectedTextToken()));
		g_pViewSelectPopup->insertSeparator();
		if(tmp.contains('.')){
			tmp.sprintf("/HOST %s",firstToken.ptr());
			g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(hostCurrentlySelectedToken()));
		}
		tmp.sprintf("/EXEC -s %s ...",firstToken.ptr());
		g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(execCurrentlySelectedText()));

		if(isConnected()){
			g_pViewSelectPopup->insertSeparator();
			firstToken.stripWhiteSpace();
			tmp.sprintf("/WHOIS %s",firstToken.ptr());
			g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(whoisCurrentlySelectedToken()));
			tmp.sprintf("/QUERY %s",firstToken.ptr());
			g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(queryCurrentlySelectedToken()));
			tmp.sprintf("/DCC CHAT %s",firstToken.ptr());
			g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(dccChatCurrentlySelectedToken()));
			tmp.sprintf("/DNS %s",firstToken.ptr());
			g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(dnsCurrentlySelectedToken()));
			if((*(firstToken.ptr()) == '#') || (*(firstToken.ptr()) == '&')
				|| (*(firstToken.ptr()) == '!')){
				// A channel name ?
				tmp.sprintf("/JOIN %s",firstToken.ptr());
				g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(joinCurrentlySelectedToken()));				
			}
		}
		if(pWnd->type() == KVI_WND_TYPE_CHANNEL){
			tmp.sprintf("/TOPIC %s %s",pWnd->caption(),firstToken.ptr());
			g_pViewSelectPopup->insertItem(tmp.ptr(),this,SLOT(topicCurrentlySelectedToken()));			
		}
	}

	m_global.szCurrentSelectionToken = firstToken;
	m_global.pCurrentSelectionWindow = pWnd;
	g_pViewSelectPopup->popup(QCursor::pos());
}

void KviFrame::actionForCurrentlySelectedTextToken(const char *action)
{
	if(!m_global.pCurrentSelectionWindow)return;
	KviStr token = m_global.szCurrentSelectionToken;
	if(token.hasData()){
		// Just redirect to the user parser
		token.prepend(' ');
		token.prepend(action);
		token.prepend('/');
		m_pUserParser->parseUserCommand(token,m_global.pCurrentSelectionWindow);
	}
}


void KviFrame::whoisCurrentlySelectedToken()
{ actionForCurrentlySelectedTextToken("WHOIS"); }

void KviFrame::queryCurrentlySelectedToken()
{ actionForCurrentlySelectedTextToken("QUERY"); }

void KviFrame::dccChatCurrentlySelectedToken()
{ actionForCurrentlySelectedTextToken("DCC CHAT"); }

void KviFrame::hostCurrentlySelectedToken()
{ actionForCurrentlySelectedTextToken("HOST"); }

void KviFrame::dnsCurrentlySelectedToken()
{ actionForCurrentlySelectedTextToken("DNS"); }

void KviFrame::joinCurrentlySelectedToken()
{ actionForCurrentlySelectedTextToken("JOIN"); }

void KviFrame::doHelpOnCurrentlySelectedTextToken()
{ actionForCurrentlySelectedTextToken("HELP"); }

void KviFrame::topicCurrentlySelectedToken()
{
	if(!m_global.pCurrentSelectionWindow)return;
	KviStr token = m_global.szCurrentSelectionToken;
	if(token.hasData()){
		// Just redirect to the user parser
		token.prepend(" ");
		token.prepend(m_global.pCurrentSelectionWindow->caption());
		token.prepend("/TOPIC ");
		m_pUserParser->parseUserCommand(token,m_global.pCurrentSelectionWindow);
	}
}

void KviFrame::doHelpSearchOnCurrentlySelectedTextToken()
{
	if(m_global.szCurrentSelectionText.hasData()){
		// Just redirect to the user parser
		KviStr tmp = m_global.szCurrentSelectionText;
		tmp.prepend("/HELP -s ");
		m_pUserParser->parseUserCommand(tmp,m_global.pCurrentSelectionWindow);
	}	
}

void KviFrame::execCurrentlySelectedText()
{
	if(m_global.szCurrentSelectionText.hasData()){
		// Just redirect to the user parser
		KviStr tmp = m_global.szCurrentSelectionText;
		tmp.prepend("/EXEC -s ");
		m_pUserParser->parseUserCommand(tmp,m_global.pCurrentSelectionWindow);
	}	
}

void KviFrame::copyCurrentSelectionTextToClipboard()
{
	QApplication::clipboard()->setText(m_global.szCurrentSelectionText.ptr());
}

void KviFrame::windowPopupRequested(KviIrcView * pView,KviWindow *pWnd)
{
	// true includes taskBarPopup !
	windowPopup(pView,pWnd,true)->popup(QCursor::pos());
}

void KviFrame::urlDoubleClicked(const KviStr &url)
{
	KviStr tmp = g_pOptions->m_szBrowserCommandline;
	tmp.stripWhiteSpace();
	if(tmp.hasData()){
		tmp.replaceAll('%',url.ptr());
		tmp.stripWhiteSpace();
		if(tmp.hasData()){
			KviStr stat(KviStr::Format,__tr("Opening url %s"),tmp.ptr());
			m_pStatusBar->tempText(stat.ptr(),5000);
			KviProcess proc;
			if(!proc.run(tmp.ptr())){
				g_pApp->warningBox(__tr("Could not execute %s"),tmp.ptr());
			}
			return;
		}
	}
	g_pApp->warningBox(__tr("Could not open %s\nNo browser commandline specified"),url.ptr());
}

//
// CONNECTION STUFF
//

//##################################################
// Socket handlers
// Called directly from KviIrcSocket class
//##################################################

void KviFrame::socketEvent(int sockEventId)
{
	switch(sockEventId){
		case KVI_SOCKEVENT_Connected:
			handleConnectedToIrcServer();
			m_pSysTrayOnLineTimer->run();
			break;
		case KVI_SOCKEVENT_ConnectionFailed:
			connectionAttemptFailed(m_pSocket->getError());
			break;
		case KVI_SOCKEVENT_Disconnected:
			m_pConsole->output(KVI_OUT_SOCKET,__tr("Disconnected : %s"),kvi_getErrorString(m_pSocket->m_error));
			handleDisconnectedFromServer();
			break;
//		case KVI_SOCKEVENT_OneMessageSent: //Newer sent
//			m_pConsole->output(KVI_OUT_INTERNAL,"OneMessageSent");
//			break;
		case KVI_SOCKEVENT_PartialSocketWrite:
			if(g_pOptions->m_bBeVerbose)m_pConsole->outputNoFmt(KVI_OUT_INTERNAL,__tr("Partial socket write"));
			break;
		case KVI_SOCKEVENT_MessageTruncated:
			if(g_pOptions->m_bBeVerbose)m_pConsole->outputNoFmt(KVI_OUT_INTERNAL,__tr("Outgoing data buffer truncated to 510 bytes"));
			break;
		case KVI_SOCKEVENT_UsingDefaultPort6667:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("Invalid server port %s , using default: 6667"),m_global.szCurrentServerPort.ptr());
			m_global.szCurrentServerPort = "6667";
			break;
		case KVI_SOCKEVENT_UsingDefaultPort1080:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("Invalid proxy port %s , using default: 1080"),m_global.szCurrentProxyPort.ptr());
			m_global.szCurrentProxyPort = "1080";
			break;

		case KVI_SOCKEVENT_ConnectedToProxy:
			m_pConsole->output(KVI_OUT_SOCKET,
				_i18n_("Connected to proxy : %s (%s:%s)"),
				m_global.szCurrentProxyHost.ptr(),
				m_global.szCurrentProxyIp.ptr(),
				m_global.szCurrentProxyPort.ptr());
			break;
		case KVI_SOCKEVENT_UsingProxyProtoV4:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("Using proxy protocol SOCKSV4"));
			break;
		case KVI_SOCKEVENT_UsingProxyProtoV5:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("Using proxy protocol SOCKSV5"));
			break;
		case KVI_SOCKEVENT_ProxySentV4Request:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV4 : Sent proxy connection request: waiting for acknowledgement..."));
			break;
		case KVI_SOCKEVENT_ProxyReply90RequestGranted:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV4 : Received proxy ack : 90 Request granted"));
			break;
		case KVI_SOCKEVENT_ProxySentV5MethodRequest:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Sent proxy authentication method request: waiting for acknowledgement..."));
			break;
		case KVI_SOCKEVENT_ProxyMethodSelectedNoAuth:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Proxy : No auth selected"));
			break;
		case KVI_SOCKEVENT_ProxyMethodSelectedUserPass:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Proxy : User/Pass auth method selected"));
			break;
		case KVI_SOCKEVENT_ProxySentUserAndPass:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Sent proxy user/pass: waiting for acknowledgement..."));
			break;
		case KVI_SOCKEVENT_ProxyAuthOK:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Proxy authentication succesfull"));
			break;
		case KVI_SOCKEVENT_ProxySentTargetData:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Sent target connection data: waiting for acknowledgement..."));
			break;
		case KVI_SOCKEVENT_ProxyReply00Success:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("SOCKSV5 : Received proxy ack : 00 Request granted"));
			break;
		case KVI_SOCKEVENT_ContactingHost:
			m_pConsole->output(KVI_OUT_SOCKET,_i18n_("Connection in progress..."));
			break;
		default: //Huh?
			break;
	}
}

// #################################################
// Connected to server
// #################################################

void KviFrame::handleConnectedToIrcServer()
{
	// Fire the event...
	bool bHalt = false;
	if(g_pEventManager->eventEnabled(KviEvent_OnConnect)){
		KviStr tmp(KviStr::Format,"%s %s %s %s",
			m_global.szCurrentServerHost.ptr(),
			m_global.szCurrentServerIp.ptr(),
			m_global.szCurrentServerPort.ptr(),
			m_global.szCurrentNetwork.ptr());
		bHalt = m_pUserParser->callEvent(KviEvent_OnConnect,m_pConsole,tmp);
	}
	// Output
	if(!bHalt){
		m_pConsole->output(KVI_OUT_INTERNAL,
			_i18n_("Connected to server : %s (%s:%s) [%s]"),
			m_global.szCurrentServerHost.ptr(),
			m_global.szCurrentServerIp.ptr(),
			m_global.szCurrentServerPort.ptr(),
			m_global.szCurrentNetwork.ptr());
	}
	// And start logging in
	resolveLocalHost();
	setState(LoggingIn);
	loginToIrcServer();
	addRecentServer(m_global.szCurrentServerHost,m_global.szCurrentServerPort);
}

void KviFrame::handleDisconnectedFromServer()
{
	// Save the "cause" of the disconnect
	// If the user caused the connection termination m_global.bSentQuit is true
	// otherwise if g_pOptions->m_bAutoReconnectOnDisconnect is true
	// try reconnecting to the same server
	bool bEventuallyReconnect = (!m_global.bSentQuit);

	stopUpdatingUserList();
	QList<KviWindow> l(*m_pWinList);
	l.setAutoDelete(false);
	
	for(KviWindow *w=l.first();w;w=l.next()){
		if((w->type() == KVI_WND_TYPE_QUERY) ||
			(w->type() == KVI_WND_TYPE_CHANNEL))closeWindow(w);
		else if(w->type() == KVI_WND_TYPE_LIST)((KviListWindow *)w)->notifyDisconnected();
	}

	// Fire the event...
	bool bHalt = false;
	if(g_pEventManager->eventEnabled(KviEvent_OnDisconnect)){
		KviStr tmp(KviStr::Format,"%s %s %s %s",
			m_global.szCurrentServerHost.ptr(),
			m_global.szCurrentServerIp.ptr(),
			m_global.szCurrentServerPort.ptr(),
			m_global.szCurrentNetwork.ptr());
		bHalt = m_pUserParser->callEvent(KviEvent_OnDisconnect,m_pConsole,tmp);
	}
	// Output
	if(!bHalt){
		m_pConsole->output(KVI_OUT_INTERNAL,__tr("Connection terminated : %s (%s:%s) [%s]"),
			m_global.szCurrentServerHost.ptr(),
			m_global.szCurrentServerIp.ptr(),
			m_global.szCurrentServerPort.ptr(),
			m_global.szCurrentNetwork.ptr());
	}

	resetGlobals();

	m_pUserList->clearList(); //remove all nicks...
	m_pSysTrayOnLineTimer->stop();
	m_pSysTrayIoLed->clear();
	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
		if(w->type() == KVI_WND_TYPE_LINKS)((KviLinksWindow *)w)->reset();
	}

	setState(Ready);

	if(bEventuallyReconnect){
		if(g_pOptions->m_bAutoReconnectOnDisconnect){
			setState(WaitingForReconnect);
			m_pConsole->output(KVI_OUT_INTERNAL,__tr("Reconnect attempt in %d seconds: hit the 'connect' button to abort."),KVI_RECONNECT_TIMEOUT_IN_SECS);
			__range_invalid(m_pReconnectTimer->isActive());
			// And do a single shot
			m_pReconnectTimer->start(KVI_RECONNECT_TIMEOUT_IN_SECS * 1000,true);
		}
	}
}

void KviFrame::reconnectToServer()
{
	__range_valid(m_state == WaitingForReconnect);
//	m_pReconnectTimer->stop();
	connectToServer();
}

void KviFrame::getArtificialNickname(KviStr &nickToChange)
{
	// This is useful when the two nicknames are
	// both refused by the server.
	// We "reinvent" the nickname by increasing the first character.
	__range_valid(nickToChange.hasData());
	char c = *(nickToChange.ptr());
	__range_valid(c != '\0');
	c++;
	if(!isalpha(c))c ='a';  // ran over z ?
	*(nickToChange.ptr()) = c;
}

void KviFrame::loginToIrcServer()
{
	// Find the nickname
	g_pOptions->m_szNickName1.stripWhiteSpace();
	g_pOptions->m_szNickName2.stripWhiteSpace();
	if(g_pOptions->m_szNickName1.isEmpty())g_pOptions->m_szNickName1 = KVI_DEFAULT_NICKNAME1;
	if(g_pOptions->m_szNickName2.isEmpty())g_pOptions->m_szNickName2 = KVI_DEFAULT_NICKNAME2;
	if(kvi_strEqualCS(g_pOptions->m_szNickName1.ptr(),g_pOptions->m_szNickName2.ptr())){
		// Refuse equal nicknames
		getArtificialNickname(g_pOptions->m_szNickName2);
	}
	m_global.szCurrentNick = g_pOptions->m_szNickName1;
	// Find the username
	if(g_pOptions->m_szUserName.hasData())m_global.szCurrentUserName = g_pOptions->m_szUserName;
	else m_global.szCurrentUserName = KVI_DEFAULT_USERNAME;
	// Find the realname
	if(g_pOptions->m_szRealName.hasData())m_global.szCurrentRealName = g_pOptions->m_szRealName;
	else m_global.szCurrentRealName = KVI_DEFAULT_REALNAME;
	// Say that we're logging in
	m_pConsole->output(KVI_OUT_INTERNAL,__tr("Logging in as %s (%s : %s)"),
		m_global.szCurrentNick.ptr(),
		m_global.szCurrentUserName.ptr(),
		m_global.szCurrentRealName.ptr());
	if(m_global.szCurrentPassword.hasData()){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Sending %s as password"),
			m_global.szCurrentPassword.ptr());
		m_pSocket->sendFmtData("PASS %s",m_global.szCurrentPassword.ptr());
	}
	// Login
	m_pSocket->sendFmtData("USER %s %s %s :%s",
		m_global.szCurrentUserName.ptr(),
		m_global.szLocalHostIp.ptr(),
		m_global.szCurrentServerHost.ptr(),
		m_global.szCurrentRealName.ptr());
	m_pSocket->sendFmtData("NICK %s",m_global.szCurrentNick.ptr());
	// And wait for reply...
}

void KviFrame::loginNickRefused()
{
	// This one is called from the KviServerParser class
	// when we're still logging in and we receive an ERR_NICKNAMEINUSE
	// or a similar error message that says that the server
	// cannot accept our login nickname
	if(kvi_strEqualCS(m_global.szCurrentNick.ptr(),g_pOptions->m_szNickName1.ptr())){
		// First nickname refused , use the second one
		m_pConsole->output(KVI_OUT_INTERNAL,
			__tr("Login nickname %s refused , using alternate nick %s"),
			m_global.szCurrentNick.ptr(),g_pOptions->m_szNickName2.ptr());
			m_global.szCurrentNick = g_pOptions->m_szNickName2;
	} else {
		// Second or artificial nickname refused. Use another artificial one
		KviStr tmp = m_global.szCurrentNick;
		getArtificialNickname(tmp);
		m_pConsole->output(KVI_OUT_INTERNAL,
			__tr("Login nickname %s refused , using artificial alternate nick %s"),
			m_global.szCurrentNick.ptr(),tmp.ptr());
		m_global.szCurrentNick = tmp;
	}
	m_pSocket->sendFmtData("NICK %s",m_global.szCurrentNick.ptr());
	// And wait for reply...
}

void KviFrame::userModeChange(const char *umode)
{
	while((*umode)&&(*umode == ' '))umode++;

	bool bAdd = true;
	while(*umode && (*umode != ' ')){
		if((*umode == '+')||(*umode == '-')){
			bAdd = (*umode == '+');
			umode++;
			continue;
		}
		// else an user mode
		int idx = m_global.szCurrentUserMode.findFirstIdx(*umode);
		if(idx != -1){
			if(!bAdd)m_global.szCurrentUserMode.cut(idx,1);
		} else {
			if(bAdd)m_global.szCurrentUserMode += (*umode);
		}
		umode++;
	}
	updateCaption();
	if(m_global.szCurrentUserMode.hasData()){
		KviStr tmp(KviStr::Format,__tr("Changed umode to +%s"),m_global.szCurrentUserMode.ptr());
		m_pStatusBar->tempText(tmp.ptr(),KVI_TEMP_TEXT_TIMEOUT);
	} else m_pStatusBar->tempText(__tr("Umode reset"),KVI_TEMP_TEXT_TIMEOUT);
}

//##################################################
// Toolbar slots
//##################################################

void KviFrame::slot_connectToServer()
{
	//The button 'connect' was pressed...
	switch(m_state){
		case Ready:               connectToServer();      break;
		case LoggingIn:           disconnectFromServer(); break;
		case Connected:           disconnectFromServer(); break;
		case WaitingForConnect:   abortConnection();      break;
		case WaitingForDns:       abortDns();             break;
		case WaitingForReconnect: abortReconnect();       break;
	}
}

//##################################################
// Connection management
//##################################################

//
// Connect stuff
//

// Flow chart of function calls up to the connect() call

// connectToServer()
//    |
// have a serve name ? ---No--> connectionAttemptFailed()
//    |Yes
// use a proxy ? --------Yes--> have a proxy to contact ? -----Yes--> lookupProxyHostname()
//    |                          |                                     |     |
//    | <-do not use proxy <---No- <---------------Failed to run kvidns-   kvidns
//    |                                                                      |
//    | <------------------- haveProxyIp() <----Resolved proxy host--------proxyNameResolved()
//    |                                                                      |
// lookupServerHostname() <-do not use proxy-------------Proxy not found------
//    |         |
//    |         ----Failed to run kvidns and to resolve host "by hand"->conncetionAttemptFailed()
// kvidns
//    |
//    |
// serverNameResolved() ---Host not found----> connectionAttemptFailed()
//    |
// haveServerIp()
//    |
// still use a proxy? ---Yes---> connect(proxyIp)-----
//    |                                              |
//    -------------------No----> connect(serverIp)----
//                                                   |
//                                        handleConnectedToIrcServer();

void KviFrame::connectToServer()
{
	// We always start from here.
	__range_valid((m_state == Ready) || (m_state == WaitingForReconnect));
	// Reset everything...
	resetGlobals();
	// Find the server that we want to contact
	if(!findCurrentServerToContact()){
		connectionAttemptFailed(KVI_ERROR_NoServerEntries);
		return;
	}
	// Here we go!
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Attempting connection to %s (%s) on port %s"),
		m_global.szCurrentServerHost.ptr(),m_global.szCurrentNetwork.ptr(),m_global.szCurrentServerPort.ptr());
	// Check if we have to use a proxy
	if(g_pOptions->m_bUseProxy){
		// Yes...
		if(findCurrentProxyHost()){
			// OK...have a proxy to try...
			m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Proxy host: %s on port %s"),
				m_global.szCurrentProxyHost.ptr(),m_global.szCurrentProxyPort.ptr());
			// Lookup the proxy hostname (the secondary way will rejoin the normal
			// process in lookupServerHostname())
			lookupProxyHostname();
			return;
		} else {
			// No proxy entry...resume direct connection.
			m_pConsole->output(KVI_OUT_ERROR,
				_i18n_("Can not find the proxy host to contact: resuming direct connection"));
			g_pOptions->m_bUseProxy = false;
		}
	}
	// OK...time to lookup the server hostname
	lookupServerHostname();
}

void KviFrame::lookupProxyHostname()
{
	// Alternative way to lookupServerHostname()
	// Check if the ip was cached
	if(kvi_isValidStringIp(m_global.szCurrentProxyIp.ptr())){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Using cached proxy IP address..."));
		haveProxyIp();
		return;
	}
	// Check if the hostname is an IP
	if(kvi_isValidStringIp(m_global.szCurrentProxyHost.ptr())){
		m_global.szCurrentProxyIp = m_global.szCurrentProxyHost;
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Using absolute proxy IP address..."));
		haveProxyIp();
		return;
	}
	// Nope...must resolve the host
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Looking up proxy hostname..."));
	__range_invalid(m_pDns); //must be zero!
	m_pDns = new KviAsyncDns();
	connect(m_pDns,SIGNAL(dnsFinished(KviDnsStruct *)),this,SLOT(proxyNameResolved(KviDnsStruct *)));
	if(!m_pDns->resolve(m_global.szCurrentProxyHost.ptr())){
		// Failed to start?
		delete m_pDns;
		m_pDns = 0;
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Can not start the DNS thread: trying blocking Dns query"));
		if(!m_pSocket->getHostByName(m_global.szCurrentProxyHost.ptr(),m_global.szCurrentProxyIp)){
			// Invalid host...or something
			m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Can not resolve proxy hostname: resuming direct connection"));
			g_pOptions->m_bUseProxy = false;
			// Forget proxy names...
			resetProxyGlobals();
			// Jump back to normal connection process
			lookupServerHostname();
			return;
		}
		haveProxyIp();
	} else {
		if(g_pOptions->m_bBeVerbose)m_pConsole->outputNoFmt(KVI_OUT_INTERNAL,__tr("DNS thread started"));
	}
	// Now wait for kvidns to return
	setState(WaitingForDns);
}

void KviFrame::lookupServerHostname()
{
	// Check if the ip was cached
	if(kvi_isValidStringIp(m_global.szCurrentServerIp.ptr())){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Using cached server IP address..."));
		haveServerIp();
		return;
	}
	// Check if the hostname is an ip address
	if(kvi_isValidStringIp(m_global.szCurrentServerHost.ptr())){
		m_global.szCurrentServerIp = m_global.szCurrentServerHost;
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Using absolute server IP address..."));
		haveServerIp();
		return;
	}
	// Nope  , must resolve the host
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Looking up server hostname..."));
	__range_invalid(m_pDns); //must be zero!
	m_pDns = new KviAsyncDns();
	connect(m_pDns,SIGNAL(dnsFinished(KviDnsStruct *)),this,SLOT(serverNameResolved(KviDnsStruct *)));
	if(!m_pDns->resolve(m_global.szCurrentServerHost.ptr())){
		delete m_pDns;
		m_pDns = 0;
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Can not start the DNS thread: trying blocking Dns query"));
		if(!m_pSocket->getHostByName(m_global.szCurrentServerHost.ptr(),m_global.szCurrentServerIp)){
			connectionAttemptFailed(m_pSocket->getError());
			return;
		}
		haveServerIp();
	} else {
		if(g_pOptions->m_bBeVerbose)m_pConsole->outputNoFmt(KVI_OUT_INTERNAL,__tr("DNS thread started"));
	}
	setState(WaitingForDns);
}

bool KviFrame::findCurrentServerToContact()
{
	KviIrcNetwork * net = g_pOptions->m_pServerManager->getCurrentNetwork();
	if(!net)return false;
	KviIrcServer * srv = net->currentServer();
	if(!srv)return false;
	m_global.szCurrentNetwork    = net->name();
	m_global.szCurrentServerHost = srv->szHost;
	m_global.szCurrentServerIp   = srv->szIp;
	m_global.szCurrentServerPort = srv->szPort;
	m_global.szCurrentPassword   = srv->szPassword;
	return true;
}

bool KviFrame::findCurrentProxyHost()
{
	KviIrcProxy * prx = g_pOptions->m_pProxyManager->currentProxy();
	if(!prx)return false;
	m_global.szCurrentProxyHost     = prx->szHost;
	m_global.szCurrentProxyIp       = prx->szIp;
	m_global.szCurrentProxyPort     = prx->szPort;
	m_global.szCurrentProxyPassword = prx->szPassword;
	m_global.szCurrentProxyUsername = prx->szUsername;
	return true;
}

bool KviFrame::findNextServerToContact()
{
	KviIrcNetwork * net = g_pOptions->m_pServerManager->getCurrentNetwork();
	if(!net)return false;
	KviIrcServer * srv = net->nextServer();
	if(!srv)return false;
	m_global.szCurrentNetwork    = net->name();
	m_global.szCurrentServerHost = srv->szHost;
	m_global.szCurrentServerIp   = srv->szIp;
	m_global.szCurrentServerPort = srv->szPort;
	m_global.szCurrentPassword   = srv->szPassword;
	return true;
}

void KviFrame::connectionAttemptFailed(int errNum)
{
	//Dead....reset the entire state
	m_pConsole->output(KVI_OUT_ERROR,_i18n_("[Socket error] : %s"),kvi_getErrorString(errNum));
	m_pConsole->output(KVI_OUT_ERROR,_i18n_("Connection attempt failed (%s)"),m_global.szCurrentServerHost.ptr());
	resetGlobals();
	setState(Ready);

	if(g_pOptions->m_bTryNextServerOnConnectFailed){
		if(findNextServerToContact()){
			setState(WaitingForReconnect);
			m_pConsole->output(KVI_OUT_INTERNAL,__tr("Trying next server in the current network: %s"),m_global.szCurrentServerHost.ptr());
			m_pConsole->output(KVI_OUT_INTERNAL,__tr("New connect attempt in %d seconds: hit the 'connect' button to abort."),KVI_RECONNECT_TIMEOUT_IN_SECS);
			__range_invalid(m_pReconnectTimer->isActive());
			// And do a single shot
			m_pReconnectTimer->start(KVI_RECONNECT_TIMEOUT_IN_SECS * 1000,true);
		}
	}

}

void KviFrame::serverNameResolved(KviDnsStruct * dns)
{
	//Slot called by the KviAsyncDns class.
	//We resolved the hostname.
	//If we get a DNS error here , it has no sense to try a blocking DNS call
	//we will get the same error.
	__range_valid(dns->dnsParent = m_pDns);
	if(dns->iError != KVI_ERROR_Success){
//		int error;
//		ptr->getError(query,&error);
		connectionAttemptFailed(dns->iError);
		delete m_pDns;
		m_pDns = 0;
		return;
	}
	m_global.szCurrentServerIp = dns->szIp;
//	ptr->getResult(query,hostname,m_global.szCurrentServerIp,alias1,alias2);
	if(dns->szHostname != m_global.szCurrentServerHost){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Server real hostname is %s"),dns->szHostname.ptr());
		m_global.szCurrentServerHost = dns->szHostname;
	}
	//Save the server IP for next time...
	g_pOptions->m_pServerManager->updateServerIp(m_global.szCurrentNetwork.ptr(),
			dns->szHostToResolve.ptr(),m_global.szCurrentServerIp.ptr());
	//Curious part...just some fun
	if((!dns->szAlias1.isEmpty()) && (dns->szAlias1 != "*")){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Server primary alias is %s"),dns->szAlias1.ptr());
		if((!dns->szAlias2.isEmpty()) && (dns->szAlias2 != "*")){
			m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Server secondary alias is %s"),dns->szAlias2.ptr());
		}
	}
	//ok...go ahead
	delete m_pDns;
	m_pDns = 0;
	haveServerIp();
}

void KviFrame::proxyNameResolved(KviDnsStruct *dns)
{
	//Slot called by the KviDns class.
	//We resolved the hostname.
	//If we get a DNS error here , it has no sense to try a blocking DNS call
	//we will get the same error.
	__range_valid(dns->dnsParent == m_pDns);
//	KviStr query,hostname,alias1,alias2;
	if(dns->iError != KVI_ERROR_Success){
//		int error;
//		ptr->getError(query,&error);
		m_pConsole->output(KVI_OUT_ERROR,_i18n_("[Socket error] : %s"),kvi_getErrorString(dns->iError));
		m_pConsole->output(KVI_OUT_INTERNAL,
			_i18n_("Can not resolve proxy hostname: resuming direct connection"));
		g_pOptions->m_bUseProxy = false;
		delete m_pDns;
		m_pDns = 0;
		resetProxyGlobals();
		lookupServerHostname();
		return;
	}
	m_global.szCurrentProxyIp = dns->szIp;
//	ptr->getResult(query,hostname,m_global.szCurrentProxyIp,alias1,alias2);
	if(dns->szHostname != m_global.szCurrentProxyHost){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Proxy real hostname is %s"),dns->szHostname.ptr());
		m_global.szCurrentProxyHost = dns->szHostname;
	}
	//Save the server IP for next time...
	g_pOptions->m_pProxyManager->updateProxyIp(dns->szHostToResolve.ptr(),m_global.szCurrentProxyIp.ptr());
	//Curious part...just some fun
	if((!dns->szAlias1.isEmpty()) && (dns->szAlias1 != "*")){
		m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Proxy primary alias is %s"),dns->szAlias1.ptr());
		if((!dns->szAlias2.isEmpty()) && (dns->szAlias2 != "*")){
			m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Proxy secondary alias is %s"),dns->szAlias2.ptr());
		}
	}
	//ok...go ahead
	delete m_pDns;
	m_pDns = 0;
	haveProxyIp();
}

void KviFrame::resolveLocalHost()
{
	//Finds the local hostname
	//In the worst case sets m_global.szLocalHostIp to 127.0.0.1
	//On IRC it is not so important , but on DCC it will be.
	if(!m_pSocket->getLocalHostIp(m_global.szLocalHostIp)){
		m_global.szLocalHostIp = g_pOptions->m_szLocalHostIp.isEmpty() ? "127.0.0.1" : g_pOptions->m_szLocalHostIp.ptr();
		m_pConsole->output(KVI_OUT_ERROR,_i18n_("Unable to resolve localhost : %s , using %s"),kvi_getErrorString(m_pSocket->getError()),
			m_global.szLocalHostIp.ptr());
		return;
	}
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Local host is %s"),m_global.szLocalHostIp.ptr());
}

void KviFrame::haveProxyIp()
{
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Proxy hostname resolved to %s"),m_global.szCurrentProxyIp.ptr());
	// Ok...go back standard way now...
	lookupServerHostname();
}

void KviFrame::haveServerIp()
{
	// The last in the connection sequence...
	// We have a good server Ip here...
	// and we can try to contact a server.
	// Next function called in this class will be socketEvent() (from KviIrcSocket)
	// or abortConnection();
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Server hostname resolved to %s"),m_global.szCurrentServerIp.ptr());

	// If m_global.szCurrentProxyIp is a valid Ip we are going to connect
	// to the server using a proxy.
	KviProxyStruct prx;
	KviProxyStruct *ptr=0;
	if(m_global.szCurrentProxyIp.hasData() && kvi_isValidStringIp(m_global.szCurrentProxyIp.ptr())){
		prx.szIp       = m_global.szCurrentProxyIp;
		prx.szPort     = m_global.szCurrentProxyPort;
		prx.szPassword = m_global.szCurrentProxyPassword;
		prx.szUsername = m_global.szCurrentProxyUsername;
		prx.bProtoV5   = g_pOptions->m_bProxyProtoV5;
		ptr            = &prx;
	}
	if(m_pSocket->connectTo(m_global.szCurrentServerIp.ptr(),m_global.szCurrentServerPort.ptr(),ptr)){
		setState(WaitingForConnect);
	} else {
		connectionAttemptFailed(m_pSocket->getError());
	}
}

void KviFrame::ircLoginSuccesful(KviStr &szAcceptedNickname)
{
	// This one is called from the KviServerParser class
	// when the first numeric message is received.
	if(szAcceptedNickname.hasData()){
		if(!kvi_strEqualCI(szAcceptedNickname.ptr(),m_global.szCurrentNick.ptr())){
			// The server changed our nickname in some way...update it
			m_pConsole->output(KVI_OUT_INTERNAL,__tr("The server changed your nickname to %s"),
				szAcceptedNickname.ptr());
			m_global.szCurrentNick = szAcceptedNickname;
		}
	}
	// Temp hack...updated when we join the first channel
	m_global.szCurrentMaskFromServer.sprintf("%s!%s@%s",m_global.szCurrentNick.ptr(),
		m_global.szCurrentUserName.ptr(),m_global.szLocalHostIp.ptr());

	setState(Connected);
	// Set the requested user modes now.
	KviStr szModes;
	if(g_pOptions->m_bModeWOnConnect)szModes+="w";
	if(g_pOptions->m_bModeIOnConnect)szModes+="i";
	if(g_pOptions->m_bModeSOnConnect)szModes+="s";
	if(szModes.hasData()){
		m_pConsole->output(KVI_OUT_INTERNAL,__tr("Setting umode +%s"),szModes.ptr());
		m_pSocket->sendFmtData("MODE %s %s",szAcceptedNickname.ptr(),szModes.ptr());
	} else m_pSocket->sendFmtData("USERHOST %s",szAcceptedNickname.ptr()); //Need to know exactly our mask
	startNotifyOrWatchList();

	// Fire the event...
	bool bHalt = false;
	if(g_pEventManager->eventEnabled(KviEvent_OnIrc)){
		bHalt = m_pUserParser->callEvent(KviEvent_OnIrc,m_pConsole,m_global.szCurrentNick);
	}
	if(!bHalt)m_pConsole->output(KVI_OUT_INTERNAL,__tr("Login operations complete: happy ircing :)"));

//	// Allow requesting the channel list now
//	for(KviWindow *w=m_pWinList->first();w;w=m_pWinList->next()){
//		if(w->type() == KVI_WND_TYPE_LIST){
//			((KviListWindow *)w)->notifyConnected();
//			return;
//		}
//	}
}

//
// Disconnect stuff
//

void KviFrame::disconnectFromServer()
{
	__range_valid(m_state == Connected || m_state == LoggingIn );

	if(g_pOptions->m_bBrutalQuit || m_global.bSentQuit){
		m_pSocket->abortConnection();
		m_global.bSentQuit = true;

		handleDisconnectedFromServer();
	} else {

		if(g_pOptions->m_szQuitMessage.hasData()){
			m_pSocket->sendFmtData("QUIT :%s",g_pOptions->m_szQuitMessage.ptr());
		} else m_pSocket->sendData("QUIT");

		m_pConsole->output(KVI_OUT_INTERNAL,__tr("Sent QUIT (%s)"),g_pOptions->m_szQuitMessage.ptr());
		m_pConsole->output(KVI_OUT_INTERNAL,__tr("Waiting for the server to close the connection..."));
//		m_pConsole->output(KVI_OUT_INTERNAL,__tr("Press the 'disconnect' button again to force a 'brutal quit'"));

		m_global.bSentQuit = true;
	}
}

void KviFrame::abortDns()
{
	__range_valid(m_state == WaitingForDns);
	__range_valid(m_pDns);
//	__range_valid(m_pDns->isActive());
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Connection aborted"));
	if(m_pDns)delete m_pDns;
	m_pDns = 0;
	resetGlobals();
	setState(Ready);
}

void KviFrame::abortConnection()
{
	__range_valid(m_state == WaitingForConnect);
	m_pSocket->abortConnection();
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Connection aborted"));
	resetGlobals();
	setState(Ready);
}

void KviFrame::abortReconnect()
{
	__range_valid(m_state == WaitingForReconnect);
	m_pConsole->output(KVI_OUT_INTERNAL,_i18n_("Reconnect aborted"));
	__range_valid(m_pReconnectTimer->isActive());
	if(m_pReconnectTimer->isActive())m_pReconnectTimer->stop();
	resetGlobals();
	setState(Ready);
}

//##################################################
// Utils
//##################################################

void KviFrame::addMultimediaFileOffert(const KviStr &offert)
{
	KviStr *s = new KviStr(offert);
	s->stripWhiteSpace();
	m_global.pMultimediaFileOfferts->append(s);
	if(m_pMultimediaFileOffertTimer->isActive())m_pMultimediaFileOffertTimer->stop();
	m_pMultimediaFileOffertTimer->start(120000); // 2 mins offert
}

void KviFrame::cleanupMultimediaFileOffertList()
{
	while(!m_global.pMultimediaFileOfferts->isEmpty())m_global.pMultimediaFileOfferts->removeFirst();
	__range_valid(m_pMultimediaFileOffertTimer->isActive());
	m_pMultimediaFileOffertTimer->stop();
}

void KviFrame::findMultimediaFileOffert(KviStr &retPath,KviStr &filename)
{
	for(KviStr *s=m_global.pMultimediaFileOfferts->first();s;s=m_global.pMultimediaFileOfferts->next())
	{
		if(s->len() >= filename.len())
		{
			// xoom.mp3 len = 8
			// /usr/build/xoom.mp3 len = 19
			if(kvi_strEqualCI(filename.ptr(),(char *)(s->ptr() + (s->len() - filename.len()))))
			{
				retPath = s->ptr();
				return;
			}
		}
	}
}


void KviFrame::resetGlobals()
{
	m_global.szCurrentServerHost    = "";
	m_global.szCurrentNetwork       = "";
	m_global.szCurrentServerIp      = "";
	m_global.szCurrentServerPort    = "";
	m_global.szCurrentPassword      = "";
	m_global.szLocalHostIp          = "127.0.0.1";
	m_global.szCurrentNick          = "";
	m_global.szCurrentUserName      = "";
	m_global.szCurrentRealName      = "";
	m_global.szCurrentSelectionText = "";
	m_global.szCurrentMaskFromServer= "";
	m_global.szLocalHostName        = "";
	m_global.szCurrentUserMode      = "";
	m_global.bServerSupportsEMode   = true; //Assume that it supports it until we receive an error
	m_global.bServerSupportsWatchList = true; //Same as above
	m_global.bUserHostInProgress    = false;
	m_global.bSentQuit              = false;
	m_global.szNotifyListString     = "";
	while(!m_global.pDnsWhoisPending->isEmpty())m_global.pDnsWhoisPending->removeFirst();
	while(!m_global.pMultimediaFileOfferts->isEmpty())m_global.pMultimediaFileOfferts->removeFirst();
	if(m_pMultimediaFileOffertTimer->isActive())
	{
		m_pMultimediaFileOffertTimer->stop();
	}

	if(m_pUserParser)m_pUserParser->m_pAsyncWhoisController->clearList();

	stopNotifyOrWatchList();

	resetProxyGlobals();
}

void KviFrame::resetProxyGlobals()
{
	m_global.szCurrentProxyHost     = "";
	m_global.szCurrentProxyIp       = "";
	m_global.szCurrentProxyPort     = "";
	m_global.szCurrentProxyPassword = "";
	m_global.szCurrentProxyUsername = "";
}

void KviFrame::setState(KviIrcConnectionState state)
{
//	__range_valid(m_state != state); //sanity check , to be removed later (at least on connect errors it is false)
	m_state = state;
	switch(state){
		case Ready:
			m_pToolBar->setConnectButtonState(KVI_CONNECT_BUTTON_STATE_READY);
			m_pMenuBar->enableOnLinePopup(false);
			break;
		case WaitingForDns:
			m_pToolBar->setConnectButtonState(KVI_CONNECT_BUTTON_STATE_BUSY);
			m_pMenuBar->enableOnLinePopup(false);
			break;
		case WaitingForConnect:
			m_pToolBar->setConnectButtonState(KVI_CONNECT_BUTTON_STATE_BUSY);
			m_pMenuBar->enableOnLinePopup(false);
			break;
		case LoggingIn:
			m_pToolBar->setConnectButtonState(KVI_CONNECT_BUTTON_STATE_BUSY);
			m_pMenuBar->enableOnLinePopup(false);
			break;
		case Connected:
			m_pToolBar->setConnectButtonState(KVI_CONNECT_BUTTON_STATE_CONNECTED);
			m_pMenuBar->enableOnLinePopup(true);
			break;
		case WaitingForReconnect:
			m_pToolBar->setConnectButtonState(KVI_CONNECT_BUTTON_STATE_BUSY);
			m_pMenuBar->enableOnLinePopup(false);
			break;
	}
	updateCaption();
}

void KviFrame::updateCaption()
{
	KviStr tmp;
	const char *captionText = g_pOptions->m_szCaptionMessage.hasData() ? g_pOptions->m_szCaptionMessage.ptr() : "KVirc";
	switch(m_state){
		case Ready:
			setCaption(captionText);
			break;
		case WaitingForDns:
			tmp.sprintf(__tr("%s : Waiting for DNS"),captionText);
			setCaption(tmp.ptr());
			break;
		case WaitingForConnect:
			tmp.sprintf(__tr("%s : Connection in progress"),captionText);
			setCaption(tmp.ptr());
			break;
		case WaitingForReconnect:
			tmp.sprintf(__tr("%s : Waiting for reconnect"),captionText);
			setCaption(tmp.ptr());
			break;
		case LoggingIn:
			tmp.sprintf(__tr("%s : Login in progress"),captionText);
			setCaption(tmp.ptr());
			break;
		case Connected:
			if(m_global.szCurrentUserMode.hasData()){
				tmp.sprintf(__tr("[%s (+%s) on %s] - %s"),m_global.szCurrentNick.ptr(),
					m_global.szCurrentUserMode.ptr(),m_global.szCurrentServerHost.ptr(),captionText);
			} else {
				tmp.sprintf(__tr("[%s on %s] - %s"),m_global.szCurrentNick.ptr(),
					m_global.szCurrentServerHost.ptr(),captionText);
			}
			setCaption(tmp.ptr());
			break;
	}
}

void KviFrame::doAsyncServerCommandCall()
{
	// This is called from the /SERVER command parsing routine
	// It needs to "jump out" of that code and call the
	// slot_connectedToServer from the main event loop;
	// just to avoid return segfaults :)
	QTimer::singleShot(100,this,SLOT(asyncServerCommand()));
}

void KviFrame::asyncServerCommand()
{
	if(m_state != Ready){
		// Still connected...Force a brutal QUIT...
		m_pSocket->sendFmtData("QUIT :%s",__tr("Changing server..."));
		m_global.bSentQuit = true;
		slot_connectToServer(); //will call disconnect...
	}
	slot_connectToServer();
}

//##################################################
// Internal handlers
//##################################################

void KviFrame::focusInEvent(QFocusEvent *)
{
	m_pMdi->setFocus();
}

#include "m_kvi_frame.moc"
