//
//   File : kvi_helpwindow.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_helpwindow.cpp)
//   Last major modification : Thu Mar 25 1999 03:17:13 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_CHECK_RANGE_
#include "kvi_debug.h"

#include "kvi_helpwindow.h"
#include "kvi_app.h"

#include "kvi_string.h"
#include "kvi_malloc.h"
#include "kvi_locale.h"
#include "kvi_defines.h"
#include "kvi_options.h"

#include "kvi_process.h"

#include <qmime.h>
#include <qstylesheet.h>
#include <qtextbrowser.h>
#include <qtooltip.h>

//Declared in kvi_app.cpp and managed by KviApp class
extern QPixmap * g_pWindowIcon[KVI_WND_NUM_ICONS];
extern QPixmap * g_pHelpIcons[KVI_HELP_ICONS_NUM_IMAGES];

//============ KviHelpWindow ============//

KviHelpWindow::KviHelpWindow()
:QWidget(0,"help_window",WStyle_StaysOnTop)
{
	setCaption(KVI_HELPWINDOW_WINDOW_NAME);

	m_pBrowser = new QTextBrowser(this);
	m_pBrowser->setFrameStyle(QFrame::Panel|QFrame::Sunken);
//	m_pBrowser->setFactory(QMimeSourceFactory::defaultFactory());

	m_pHome = new QPushButton(this);
	m_pHome->setPixmap(*g_pHelpIcons[KVI_HELP_ICON_INDEX]);
	QToolTip::add(m_pHome,QString(__tr("Show the help index")));
	m_pBack = new QPushButton(this);
	m_pBack->setPixmap(*g_pHelpIcons[KVI_HELP_ICON_BACK]);
	QToolTip::add(m_pBack,QString(__tr("Go back in the history")));
	m_pBack->setEnabled(false);
	m_pForward = new QPushButton(this);
	m_pForward->setPixmap(*g_pHelpIcons[KVI_HELP_ICON_FORWARD]);
	QToolTip::add(m_pForward,QString(__tr("Go forward in the history")));
	m_pForward->setEnabled(false);
	m_pSearchString = new QLineEdit(this);
	QToolTip::add(m_pSearchString,QString(__tr("Search words")));
	m_pExactSearch = new QPushButton(this);
	m_pExactSearch->setPixmap(*g_pHelpIcons[KVI_HELP_ICON_SEARCH]);
	QToolTip::add(m_pExactSearch,QString(__tr("Search documentation")));

	m_pSearchJob = 0;

	connect(m_pHome,SIGNAL(clicked()),this,SLOT(showIndex()));
	connect(m_pBack,SIGNAL(clicked()),m_pBrowser,SLOT(backward()));
	connect(m_pForward,SIGNAL(clicked()),m_pBrowser,SLOT(forward()));
	connect(m_pBrowser,SIGNAL(backwardAvailable(bool)),this,SLOT(backwardAvailable(bool)));
	connect(m_pBrowser,SIGNAL(forwardAvailable(bool)),this,SLOT(forwardAvailable(bool)));
	connect(m_pExactSearch,SIGNAL(clicked()),this,SLOT(doExactSearch()));

	setIcon(*(g_pWindowIcon[KVI_WND_ICON_HELPWINDOW]));

	setDocument(g_pApp->m_szLastHelpDocument.ptr());

	setGeometry(g_pOptions->m_rectHelpWinGeometry);
}

//============ ~KviHelpWindow ============//

KviHelpWindow::~KviHelpWindow()
{
	if(m_pSearchJob)delete m_pSearchJob;
	if(g_pOptions->m_bUseHackedFrameGeometry){
		g_pOptions->m_rectHelpWinGeometry.moveTopLeft(frameGeometry().topLeft());
		g_pOptions->m_rectHelpWinGeometry.setSize(size());
	} else g_pOptions->m_rectHelpWinGeometry = geometry();
}

void KviHelpWindow::reloadDocument()
{
	KviStr tmp = g_pApp->m_szLastHelpDocument.ptr();
	showIndex(); //Hack to make it reload the doc
	setDocument(tmp.ptr());
}

void KviHelpWindow::resetStyleSheet()
{
	m_pBrowser->setStyleSheet(QStyleSheet::defaultSheet());
}

void KviHelpWindow::setDocument(const char *topic)
{ m_pBrowser->setSource(QString(topic)); }

void KviHelpWindow::showIndex()
{ setDocument(KVI_HELP_QML_MAIN_TOPIC_FILE); }

void KviHelpWindow::backwardAvailable(bool bAvailable)
{ m_pBack->setEnabled(bAvailable); }

void KviHelpWindow::forwardAvailable(bool bAvailable)
{ m_pForward->setEnabled(bAvailable); }

void KviHelpWindow::doExactSearch()
{
	KviStr txt = m_pSearchString->text();
	if(txt.hasData())doExactSearchForString(txt.ptr());
}

void KviHelpWindow::doExactSearchForString(const char *str)
{
	if(m_pSearchJob)delete m_pSearchJob;
	m_pSearchJob = new KviProcess();
	connect(m_pSearchJob,SIGNAL(terminated(KviProcess *,int)),this,SLOT(searchJobTerminated(KviProcess *,int)));
	KviStr filesToSearch;
	g_pApp->getGlobalKVircDirectory(filesToSearch,KviApp::Help,"*.kvihelp");
	KviStr fileToProduce;
	g_pApp->getLocalKVircDirectory(fileToProduce,KviApp::Help,"qh_last_search_result.kvihelp");

	KviStr tmp;
	tmp.sprintf("kvi_search_help.sh %s %s %s",filesToSearch.ptr(),fileToProduce.ptr(),str);
	if(!m_pSearchJob->run(tmp.ptr(),false,true)){
		delete m_pSearchJob;
		m_pSearchJob = 0;
		g_pApp->warningBox(__tr("Could not execute command\n%s"),tmp.ptr());
	} else m_pExactSearch->setEnabled(false);
}

void KviHelpWindow::searchJobTerminated(KviProcess * proc,int)
{
	__range_valid(proc == m_pSearchJob);
	delete m_pSearchJob;
	m_pSearchJob = 0;
	showIndex(); //HACK HACK HACK
	KviStr fileToProduce;
	g_pApp->getLocalKVircDirectory(fileToProduce,KviApp::Help,"qh_last_search_result.kvihelp");
	setDocument(fileToProduce.ptr());
	m_pExactSearch->setEnabled(true);
}

//================ resizeEvent ===============//

void KviHelpWindow::resizeEvent(QResizeEvent *)
{
	int hght = m_pBack->sizeHint().height();
	m_pHome->setGeometry(0,0,hght,hght);
	m_pBack->setGeometry(hght,0,hght,hght);
	m_pForward->setGeometry(hght*2,0,hght,hght);
	m_pExactSearch->setGeometry(hght*3,0,hght,hght);
	int lineWdth = width() - (hght * 4);
	if(lineWdth < hght)lineWdth = hght;
	m_pSearchString->setGeometry(hght*4,0,lineWdth,hght);
	m_pBrowser->setGeometry(0,hght,width(),height()-hght);
}

//============== closeEvent ================//

void KviHelpWindow::closeEvent(QCloseEvent *e)
{
	e->ignore(); //we ignore the event , and then close later if needed.
	g_pApp->helpWindowCloseRequest();
}

#include "m_kvi_helpwindow.moc"
