//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//


#include "kvi_app.h"
#include "kvi_string.h"
#include "kvi_defines.h"
#include <qglobal.h> //for debug()


#define KVI_ARGS_RETCODE_OK 0
#define KVI_ARGS_RETCODE_ERROR 1
#define KVI_ARGS_RETCODE_STOP 2

int parseArgs(int argc,char **argv,char ** configFile,char ** server,char ** port,bool *createFile)
{
	if(argc > 1){
		int idx = 1;
		while(idx < argc)
		{
			if(kvi_strEqualCI("-v",argv[idx]) || kvi_strEqualCI("--version",argv[idx])){
				debug("%s",KVI_VERSION);
				debug("Build date: %s",BUILD_DATE);
				debug("Script language tag: %s",KVI_SCRIPT_LANGUAGE_TAG);
				debug("Home page: http://www.kvirc.org");
				debug("Bug reports to: <kvirc@tin.it>");
				return KVI_ARGS_RETCODE_STOP;
			} else if(kvi_strEqualCI("-h",argv[idx]) || kvi_strEqualCI("--help",argv[idx])){
				debug("Usage:");
				debug("              %s [options] [server [port]]",argv[0]);
				debug("Available options:");
				debug("    -h        : Print this help and exit");
				debug("    -v        : Print the version informations and exit");
				debug("    -c <file> : Use <file> as config file instead of ~/%s",KVI_HOME_CONFIG_FILE_NAME);
				debug("                (defaults to ~/%s if <file> does not exist)",KVI_HOME_CONFIG_FILE_NAME);
				debug("    -n <file> : Use <file> as config file instead of ~/%s",KVI_HOME_CONFIG_FILE_NAME);
				debug("                (create <file> if it does not exist)");
				debug("    [server]  : Connect to this server after startup");
				debug("    [port]    : Use this port for connection");
				return KVI_ARGS_RETCODE_STOP;
			} else if(kvi_strEqualCI("-c",argv[idx])){
				idx++;
				if(idx >= argc){
					debug("Option -c requires a config file name");
					return KVI_ARGS_RETCODE_ERROR;
				}
				*configFile = argv[idx];
				debug("Using file %s as config",argv[idx]);
			} else if(kvi_strEqualCI("-n",argv[idx])){
				idx++;
				if(idx >= argc){
					debug("Option -n requires a config file name");
					return KVI_ARGS_RETCODE_ERROR;
				}
				*configFile = argv[idx];
				*createFile=true;
				debug("Using file %s as config",argv[idx]);
			} else {
				if((*(argv[idx]) != '-') && (!(*port))){
					// no dash
					KviStr tmp(argv[idx]);
					if(tmp.hasData()){
						if(*server){
							if(tmp.isUnsignedNum())*port = argv[idx];
						} else (*server) = argv[idx];
					}
				}
			}
			idx++;
		}
	}
	return KVI_ARGS_RETCODE_OK;
}

int main(int argc,char **argv)
{
	char * configFile         = 0;
	char * port               = 0;
	char * server             = 0;
	bool createFile           =false;

	int retCode = parseArgs(argc,argv,&configFile,&server,&port, &createFile);

	if(retCode != KVI_ARGS_RETCODE_OK)return ((retCode == KVI_ARGS_RETCODE_ERROR) ? (-1) : 0);

	g_pApp=new KviApp(argc,argv);
	g_pApp->m_pCreateConfig = createFile;
	g_pApp->m_pConfigFile = configFile;
	g_pApp->setup();
	if(server)g_pApp->connectAfterStartup(server,port);
	retCode=g_pApp->exec();
	delete g_pApp;
	return retCode;
}
