//
//   File : kvi_menubar.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_menubar.cpp)
//   Last major modification : Tue Jul 6 1999 14:46:33 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//


#define _KVI_DEBUG_CLASS_NAME_ "KviMenuBar"
#include "kvi_debug.h"

#include "kvi_menubar.h"
#include "kvi_frame.h"
#include "kvi_mdi.h"
#include "kvi_app.h"
#include "kvi_locale.h"

#include <qpopupmenu.h>
#include <qpainter.h>

//============ KviMenuBar ============//

KviMenuBar::KviMenuBar(KviFrame *parent)
:QMenuBar(parent)
{
//	setFont(g_pOptions->m_fntApplication);
//	setFontPropagation(QWidget::SameFont);
	//Create the menu
	m_pMainPopup = new QPopupMenu(this);
	m_pMainPopup->insertItem(__tr("&New server window"),g_pApp,SLOT(createNewFrame()));
	m_pMainPopup->insertItem(__tr("&Close this server window"),parent,SLOT(close()));
	m_pMainPopup->insertSeparator();
	m_pMainPopup->insertItem(__tr("Connect/Disconnect"),parent,SLOT(slot_connectToServer()));
	m_pMainPopup->insertItem(__tr("Recent servers"),parent->m_pRecentServersPopup);
	m_pMainPopup->insertSeparator();
	m_iDockWidgetId = m_pMainPopup->insertItem(__tr("Show dock widget"),g_pApp,SLOT(slot_createWMDockWidget()));
//	m_pMainPopup->insertItem(__tr("Undock"),g_pApp,SLOT(slot_WMUndock()));
	m_pMainPopup->insertSeparator();
	m_pMainPopup->insertItem(__tr("&Quit KVIrc"),g_pApp,SLOT(slot_safeQuit()));

	QPopupMenu * opts = new QPopupMenu(this);
	opts->insertItem(__tr("&Nickname , username..."),g_pApp,SLOT(slot_doUserOptionsDialog()));
	opts->insertItem(__tr("&Servers and proxy hosts"),g_pApp,SLOT(slot_doServerOptionsDialog()));
	opts->insertItem(__tr("&User interface settings"),g_pApp,SLOT(slot_doGuiOptionsDialog()));
	opts->insertItem(__tr("&Misc options"),g_pApp,SLOT(slot_doMiscOptionsDialog()));
	opts->insertItem(__tr("&Registered users database"),g_pApp,SLOT(slot_doRegUsersDialog()));
	opts->insertItem(__tr("&Dcc options"),g_pApp,SLOT(slot_doDccOptionsDialog()));
	opts->insertSeparator();
	opts->insertItem(__tr("&Save options now"),g_pApp,SLOT(slot_saveSettings()));

	QPopupMenu * tool = new QPopupMenu(this);
	tool->insertItem(__tr("&Script center"),g_pApp,SLOT(slot_doScriptDialog()));
	tool->insertItem(__tr("&Directory browser"),g_pApp,SLOT(slot_toggleDirBrowser()));
	tool->insertItem(__tr("Channel &list window"),parent,SLOT(activateListWindow()));
	tool->insertSeparator();
	tool->insertItem(__tr("Save &current script"),g_pApp,SLOT(slot_doScriptWizard()));
	tool->insertItem(__tr("&Load a script"),g_pApp,SLOT(slot_doLoadScriptDialog()));
	tool->insertSeparator();
	tool->insertItem(__tr("Toggle status &bar"),parent,SLOT(slot_toggleStatusBar()));
	tool->insertItem(__tr("Toggle task b&ar"),parent,SLOT(slot_toggleTaskBar()));
	tool->insertItem(__tr("Toggle tool ba&r"),parent,SLOT(slot_toggleToolBar()));

	m_pOnlinePopup = new QPopupMenu();
	m_pOnlinePopup->insertItem(__tr("Recent &channels"),parent->m_pRecentChannelsPopup);
	m_pOnlinePopup->insertItem(__tr("Recent &nicknames"),parent->m_pRecentNicknamesPopup);
	m_pOnlinePopup->insertSeparator();
	m_pOnlinePopup->insertItem(__tr("Request &links from server"),parent,SLOT(slot_requestLinks()));

	QPopupMenu * help = new QPopupMenu(this);
	help->insertItem(__tr("&Help window"),g_pApp,SLOT(slot_toggleHelpWindow()));
	help->insertItem(__tr("&What is this mode"),g_pApp,SLOT(slot_whatIsThisRequest()));
	help->insertSeparator();
	help->insertItem(__tr("&Tip of the Day"),g_pApp,SLOT(slot_doTipOfTheDay()));
	help->insertSeparator();
	help->insertItem(__tr("&About KVirc"),g_pApp,SLOT(slot_doAboutDialog()));

	insertItem(__tr("&KVIrc"),m_pMainPopup);
	insertItem(__tr("&Options"),opts);
	insertItem(__tr("&Tools"),tool);
	m_iOnlinePopupId = insertItem(__tr("On-&Line"),m_pOnlinePopup);
	setItemEnabled(m_iOnlinePopupId,false);
	insertItem(__tr("&Window"),parent->m_pMdi->getWindowMenu());
	insertSeparator();
	insertItem(__tr("&Help"),help);
}

//============ ~KviMenuBar ============//

KviMenuBar::~KviMenuBar()
{
//	__debug("~KviMenuBar");
	delete m_pOnlinePopup;
}

void KviMenuBar::enableOnLinePopup(bool bEnable)
{
	setItemEnabled(m_iOnlinePopupId,bEnable);
}

void KviMenuBar::wmDockWidgetStateChanged(bool bIsVisible)
{
	m_pMainPopup->removeItem(m_iDockWidgetId);
	if(bIsVisible){
		m_iDockWidgetId = m_pMainPopup->insertItem(__tr("Hide dock widget"),g_pApp,SLOT(slot_destroyWMDockWidget()),0,m_iDockWidgetId,6);
	} else {
		m_iDockWidgetId = m_pMainPopup->insertItem(__tr("Show dock widget"),g_pApp,SLOT(slot_createWMDockWidget()),0,m_iDockWidgetId,6);
	}
}

void KviMenuBar::setBackgroundPixmap(const QPixmap &pm)
{
	QMenuBar::setBackgroundPixmap(pm);
	setPalette(palette());
}

void KviMenuBar::setPalette(const QPalette &pal)
{
	if(backgroundPixmap()){
		if(!backgroundPixmap()->isNull()){
			QPalette p=pal;
			QColorGroup g=p.normal();
			g.setBrush(QColorGroup::Button,QBrush(NoBrush));
			p.setNormal(g);
			g = p.disabled();
			g.setBrush(QColorGroup::Button,QBrush(NoBrush));
			p.setDisabled(g);			
			QMenuBar::setPalette(p);
			return;
		}
	}
	QMenuBar::setPalette(pal);
}

void KviMenuBar::drawContents(QPainter *p)
{
	// It is horrible , I know it...but there is no other human way.
	const QPixmap * pix = backgroundPixmap();
	if(pix){
		if(!pix->isNull()){
			p->drawTiledPixmap(0,0,width(),height(),*pix,0,0);
			QMenuBar::drawContents(p);
			return;
		}
	}
	QMenuBar::drawContents(p);
}

#include "m_kvi_menubar.moc"
