//
//   File : kvi_scripteditor.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_scripteditor.cpp)
//   Last major modification : Sun Mar 28 1999 16:12:41 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
#define _KVI_DEBUG_CHECK_RANGE_
#include "kvi_debug.h"

#include "kvi_scripteditor.h"

#include <qpainter.h>
#include <qnamespace.h>
#include "kvi_locale.h"

#include <X11/Xlib.h>

#include <ctype.h>

#define KVI_SCRIPTEDITOR_TAB_WIDTH 32
#define KVI_SCRIPTEDITOR_TAB_SIZE 4
#define KVI_SCRIPTEDITOR_TAB_WIDTH_DIV_TAB_SIZE 8
#define KVI_SCRIPTEDITOR_BORDER 3

#define KVI_SCRIPTEDITOR_STATE_NEWCOMMAND 0
#define KVI_SCRIPTEDITOR_STATE_INCOMMAND 1
#define KVI_SCRIPTEDITOR_STATE_INSTRING 2
#define KVI_SCRIPTEDITOR_STATE_INCOMMENT 3

KviScriptEditor::KviScriptEditor(QWidget *parent)
:QMultiLineEdit(parent)
{
	m_pFm = new QFontMetrics(font());

	m_pMemBuffer = new QPixmap(cellWidth(),cellHeight());
	m_pContextPopup = new QPopupMenu();

	m_pContextPopup->insertItem(__tr("Copy"),this,SLOT(copy()));
	m_pContextPopup->insertItem(__tr("Paste"),this,SLOT(paste()));
	m_pContextPopup->insertItem(__tr("Cut"),this,SLOT(cut()));
	m_pContextPopup->insertSeparator();
	m_pContextPopup->insertItem(__tr("Select all"),this,SLOT(selectAll()));

	setBackgroundColor(Qt::black);
}

KviScriptEditor::~KviScriptEditor()
{
	delete m_pFm;
	delete m_pMemBuffer;
	delete m_pContextPopup;
}

int KviScriptEditor::extractNextWord(const char *s)
{
	int len=0;

	// spaces | tabs
	if((*s==' ')||(*s=='\t')){
		len++; s++;
		while((*s==' ') || (*s=='\t')){ len++; s++; }
		return len; 
	}

	if((*s == '#') && m_textState == KVI_SCRIPTEDITOR_STATE_NEWCOMMAND){
		len++; s++;
		while((*s) && (*s != ' ') && (*s != '\t') && (*s != ';') && (*s != '\n')){ len++; s++; }
		return len;
	}

	if(m_textState == KVI_SCRIPTEDITOR_STATE_INCOMMENT){
		while((*s) && (*s != ' ') && (*s != '\t') && (*s != ';') && (*s != '\n')){ len++; s++; }
		return len;		
	}

	if(*s == '\\'){
		// escape!
		len++; s++;
		if(*s && (*s != '\n') && (*s != ' ') && (*s != '\t'))len++;
		return len;
	}

	if(*s == '"')return 1;

	if(isalnum(*s) || (*s== '_') || (*s=='.')){
		// command or normal text
		s++; len++;
		while(*s && (isalnum(*s) || (*s == '_') || (*s == '.'))){ s++; len++; }
		return len;
	}

	if((*s == '%')||(*s == '$')){
		// Variable or identifier
		s++; len++;
		while(*s && (isalnum(*s) || (*s == '_') || (*s == '.'))){ s++; len++; }
		return len;
	}

	if((*s=='{')||(*s=='}'))return 1;

	if((*s=='(')||(*s==')')||(*s=='[')||
		(*s==']')||(*s==',')||(*s==':')||
		(*s==';')||(*s=='=')||(*s=='-')||
		(*s=='<')||(*s=='>')||(*s=='&')||
		(*s=='|')||(*s=='+')||(*s=='*')||
		(*s=='/')||(*s=='!')||(*s=='^')){
		len++; s++;
		while((*s=='(')||(*s==')')||(*s=='[')||
				(*s==']')||(*s==',')||(*s==':')||
				(*s==';')||(*s=='=')||(*s=='-')||
				(*s=='<')||(*s=='>')||(*s=='&')||
				(*s=='|')||(*s=='+')||(*s=='*')||
				(*s=='/')||(*s=='!')||(*s=='^')){ len++; s++; }
		return len;
	}

	//senseless string
	s++; len++;
	while((*s) && (isalnum(*s))){ len++; s++; }
	return len;
}

QColor KviScriptEditor::getColor(const char *s,int len)
{
	if((*s==' ')||(*s=='\t'))return QColor();

	if(m_textState == KVI_SCRIPTEDITOR_STATE_NEWCOMMAND){
		if(*s == '#'){
			m_textState = KVI_SCRIPTEDITOR_STATE_INCOMMENT;
			return Qt::darkGreen;
		} else {
			m_textState = KVI_SCRIPTEDITOR_STATE_INCOMMAND;
			if(isalnum(*s) || (*s== '_') || (*s=='.'))return Qt::green; //Qt::gray <---broken!
		}
	} else if(m_textState == KVI_SCRIPTEDITOR_STATE_INCOMMENT){
		if((*s != ';')&&(*s != '\n'))return Qt::darkGreen;
	}


	if(*s == '"'){
		if(m_textState != KVI_SCRIPTEDITOR_STATE_INSTRING)m_textState = KVI_SCRIPTEDITOR_STATE_INSTRING;
		else m_textState = KVI_SCRIPTEDITOR_STATE_INCOMMAND;
		return Qt::cyan;
	}

	if(m_textState == KVI_SCRIPTEDITOR_STATE_INSTRING)return Qt::cyan;


	if(*s == ';')m_textState = KVI_SCRIPTEDITOR_STATE_NEWCOMMAND;

	if(*s == '\\')return Qt::magenta;

	if((*s=='{')||(*s=='}'))return Qt::red;
	if(*s == '%'){
		s++;
		if((!*s) && (!(isalnum(*s))) && (*s != '_') && (*s != '.'))return QColor(255,150,0);
		if(islower(*s))return Qt::lightGray; //local variable
		return Qt::white;
	}
	if(*s == '$')return Qt::yellow;

	if((*s=='(')||(*s==')')||(*s=='[')||
		(*s==']')||(*s==',')||(*s==':')||
		(*s==';')||(*s=='=')||(*s=='-')||
		(*s=='<')||(*s=='>')||(*s=='&')||
		(*s=='|')||(*s=='+')||(*s=='*')||
		(*s=='/')||(*s=='!')||(*s=='^'))return QColor(255,150,0);

	return Qt::green;
}

int KviScriptEditor::getSpacesAndTabsLen(const char *s,int nChars,int curNChars,int *width)
{
	int lastChars=curNChars;
	(*width)=0;
	while(nChars && *s){
		if(*s==' '){
			curNChars++;
			(*width)+=m_spaceWidth;
		} else { //A TAB!!!!
			int upToNextTab=KVI_SCRIPTEDITOR_TAB_SIZE-(curNChars % KVI_SCRIPTEDITOR_TAB_SIZE);
			if(upToNextTab==0){
				curNChars+=KVI_SCRIPTEDITOR_TAB_SIZE;
				(*width)+=KVI_SCRIPTEDITOR_TAB_WIDTH;
			} else {
				curNChars+=upToNextTab;
				(*width)+=(KVI_SCRIPTEDITOR_TAB_WIDTH_DIV_TAB_SIZE*upToNextTab);
			}
		}
		s++;
		nChars--;
	}
#ifdef _KVI_DEBUG_CHECK_RANGE_
	if(nChars)debug("ooops...nChars was wrong!");
#endif
	return (curNChars-lastChars);
}

int KviScriptEditor::textWidthWithTabs(const char *s,int nChars)
{
    if(!s)return 0;
    if(nChars == -1)nChars = strlen(s);
    int dist = - m_pFm->minLeftBearing();
    const char *t = s;
    while ( *t && t - s < nChars ){
		if ( *t == '\t')dist=(dist/KVI_SCRIPTEDITOR_TAB_WIDTH+1)*KVI_SCRIPTEDITOR_TAB_WIDTH;
		else dist+=m_pFm->width(t,1);
		t++;
    }
	return dist;
}

void KviScriptEditor::paintCell(QPainter *painter,int row,int col)
{
	QString s=textLine(row);
    if(s.isNull())return;

    QRect updateR   = cellUpdateRect();

	int rowY;
	if(!rowYPos(row,&rowY))return; //invisible row

	if(m_pMemBuffer->width() != cellWidth())m_pMemBuffer->resize(cellWidth(),cellHeight());

	Display *dspy	=m_pMemBuffer->x11Display();
	HANDLE hnd		=m_pMemBuffer->handle();
	GC gc_aux       =XCreateGC(dspy,hnd,0,0);
	XSetFont(dspy,gc_aux,font().handle());

	//Draw the background
	XSetLineAttributes(dspy,gc_aux,0,LineSolid,CapButt,JoinMiter);

 	XSetForeground(dspy,gc_aux,Qt::black.pixel());
	XSetBackground(dspy,gc_aux,Qt::black.pixel());
	XSetFillStyle(dspy,gc_aux,FillSolid);
  	XFillRectangle(dspy,hnd,gc_aux,0,0,cellWidth(),cellHeight());

	m_spaceWidth = m_pFm->width(' ');

	int markX1 = 0;
	int markX2 = 0;

    if(hasMarkedText()){
		int markBeginX,markBeginY,markEndX,markEndY;
		getMarkedRegion(&markBeginY,&markBeginX,&markEndY,&markEndX);
		if ( row >= markBeginY && row <= markEndY ) {
			if ( row == markBeginY ) {
				markX1 = markBeginX;
				if ( row == markEndY )markX2 = markEndX;
				else markX2 = s.length();
			} else {
				if ( row == markEndY ) {
					markX1 = 0;
					markX2 = markEndX;
				} else {
					markX1 = 0;
					markX2 = s.length();
				}
			}
		}
	}


	const char *pC=s.ascii();
	int len=0;
	int xLeft=KVI_SCRIPTEDITOR_BORDER;

	int totLen=0;
	int myyPos=m_pFm->ascent();


	QColor clr;

	m_textState = KVI_SCRIPTEDITOR_STATE_NEWCOMMAND;

	while((*pC) && (*pC != '\n')){

		len=extractNextWord(pC);
		clr=getColor(pC,len);

		if(clr.isValid()){
			// no spaces or tabs
			XSetForeground(dspy,gc_aux,clr.pixel());
			XDrawString(dspy,hnd,gc_aux,xLeft,myyPos,pC,len);
			xLeft+=m_pFm->width(pC,len);
			totLen+=len;
		} else {
			int addWidth;
			int nSpaces=getSpacesAndTabsLen(pC,len,totLen,&addWidth);
			xLeft+=addWidth;
			totLen+=nSpaces;
		}
		pC+=len;
	}

    if(markX1!=markX2){
		int sLength   = s.length();
		int xpos1     = KVI_SCRIPTEDITOR_BORDER + textWidthWithTabs(s.data(),markX1);
		int xpos2     = KVI_SCRIPTEDITOR_BORDER + textWidthWithTabs(s.data(),markX2) - 1;
		int fillxpos1 = xpos1;
		int fillxpos2 = xpos2;
		if (markX1 == 0)fillxpos1 -= 2;
		if (markX2 == sLength)fillxpos2 += 3;

		XSetFunction(dspy,gc_aux,GXinvert);
	  	XFillRectangle(dspy,hnd,gc_aux,fillxpos1,0,fillxpos2-fillxpos1,cellHeight());
		XSetFunction(dspy,gc_aux,GXcopy);

	}
	int cursorY,cursorX;
	getCursorPosition(&cursorY,&cursorX);
    if (row==cursorY /*&& m_bCursorOn && cursorOn FIXME* blinking text*/ ){
		int cursorPos = QMIN((int)s.length(),cursorX);
		int cXPos     = KVI_SCRIPTEDITOR_BORDER + textWidthWithTabs(s.data(),cursorPos)-1;
		int cYPos     = 0;
		if(hasFocus()){
			XSetForeground(dspy,gc_aux,Qt::red.pixel());
			XDrawLine(dspy,hnd,gc_aux,cXPos-2,cYPos,cXPos + 2,cYPos );
			XDrawLine(dspy,hnd,gc_aux,cXPos,cYPos,cXPos,cYPos+m_pFm->height()-2);
			XDrawLine(dspy,hnd,gc_aux,cXPos-2,cYPos+m_pFm->height()-2,cXPos+2,cYPos+m_pFm->height()-2);

		}
    }

//	debug("rowY(%d),update.top(%d),update.height(%d),yOffset(%d)",rowY,updateR.top(),updateR.height(),yOffset());

	XCopyArea(dspy,hnd,this->handle(),gc_aux,
			updateR.left(),
			updateR.top(),
			updateR.width(),updateR.height(),
			frameWidth() + updateR.left() - xOffset(),
			rowY + updateR.top());
	XFreeGC(dspy,gc_aux);
}

void KviScriptEditor::resizeEvent(QResizeEvent *e)
{
	m_pMemBuffer->resize(cellWidth(),cellHeight());
	QMultiLineEdit::resizeEvent(e);
}

void KviScriptEditor::mousePressEvent(QMouseEvent *e)
{
	if(e->button() & RightButton){
		m_pContextPopup->popup(mapToGlobal(e->pos()));
	} else QMultiLineEdit::mousePressEvent(e);
}

void KviScriptEditor::setFont(const QFont &font)
{
	delete m_pFm;
	m_pFm = new QFontMetrics(font);
	QMultiLineEdit::setFont(font);
	m_pMemBuffer->resize(cellWidth(),cellHeight());
}

#include "m_kvi_scripteditor.moc"
