//
//   File : kvi_statusbar.cpp (/usr/build/NEW_kvirc/kvirc/src/kvirc/kvi_statusbar.cpp)
//   Last major modification : Tue Jul 6 1999 14:49:18 by Szymon Stefanek
//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#define _KVI_DEBUG_THIS_FILE_
#define _KVI_DEBUG_CLASS_NAME_ "KviStatusBar"
#include "kvi_debug.h"
//#include "kvi_options.h"
#include "kvi_statusbar.h"
#include "kvi_frame.h"
#include "kvi_style.h"

#include <qlist.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtimer.h>
#include <qdrawutil.h>
#include <qapplication.h>
#include <qsizegrip.h>

// ---------------

#define FUCKED_UP_QSTATUSBAR


#ifdef FUCKED_UP_QSTATUSBAR

class QStatusBarPrivate
{
public:
    QStatusBarPrivate() {}

    struct StatusBarPrivateItem {
	StatusBarPrivateItem( QWidget * widget, int stretch, bool permanent )
	    : s( stretch ), w( widget ), p(permanent) {}
	int s;
	QWidget * w;
	bool p;
    };

    QList<StatusBarPrivateItem> items;
    StatusBarPrivateItem * firstPermanent;

    QString temporary;

    QBoxLayout * box;
    QTimer * timer;

    QSizeGrip * resizer;
};

QStatusBar::QStatusBar( QWidget * parent, const char *name )
    : QWidget( parent, name )
{
    d = new QStatusBarPrivate;
    d->items.setAutoDelete( TRUE );
    d->box = 0;
    d->resizer = new QSizeGrip( this );
    d->timer = 0;
    reformat();
}

QStatusBar::~QStatusBar()
{
    delete d;
    d = 0;
}

void QStatusBar::addWidget( QWidget * widget, int stretch, bool permanent )
{
    QStatusBarPrivate::StatusBarPrivateItem * item
	= new QStatusBarPrivate::StatusBarPrivateItem( widget, stretch,
						       permanent );

    d->items.last();
    while( !permanent && d->items.current() && d->items.current()->p )
	d->items.prev();

    d->items.insert( d->items.at() >= 0 ? d->items.at()+1 : 0, item );

    reformat();
}

void QStatusBar::removeWidget( QWidget * widget )
{
    d->items.first();
    do {
	if ( d->items.current() &&
	     d->items.current()->w == widget ) {
	    d->items.remove();
	    reformat();
	    return;
	}
    } while( d->items.next() );
}

void QStatusBar::reformat()
{
    if ( d->box ) {
	delete d->box;
	d->box = 0;
    }

    d->box = new QVBoxLayout( this );
    d->box->addSpacing( 3 );
    QBoxLayout * l = new QHBoxLayout( d->box );

    QStatusBarPrivate::StatusBarPrivateItem * i;
    d->items.first();
    int space = 1;
    int h = 0;
    if ( space == 1 ) {
	l->addStretch( 1 );
	h = QMAX( h,  3 + fontMetrics().height() + 3 );
    }
    while( (i=d->items.current()) != 0 ) {
	d->items.next();
	l->addSpacing( space );
	space = 4;
	l->addWidget( i->w, i->s, AlignRight );
	h = QMAX( h, i->w->sizeHint().height() );
    }
    l->addStrut( h );
    l->addSpacing( 2 );
    l->addWidget( d->resizer, 0, AlignBottom );
    d->box->addSpacing( 2 );
    d->box->activate();
}

void QStatusBar::message( const QString &message )
{
    if ( d->temporary == message )
	return;
    d->temporary = message; // ### clip and add ellipsis if necessary
    if ( d->timer ) {
	delete d->timer;
	d->timer = 0;
    }
    hideOrShow();
}

void QStatusBar::message( const QString &message, int ms )
{
    d->temporary = message; // ### clip and add ellipsis if necessary

    if ( !d->timer ) {
	d->timer = new QTimer( this );
	connect( d->timer, SIGNAL(timeout()), this, SLOT(clear()) );
    }
    d->timer->start( ms );
    hideOrShow();
}

void QStatusBar::clear()
{
    if ( d->temporary.isEmpty() )
	return;
    if ( d->timer ) {
	delete d->timer;
	d->timer = 0;
    }
    d->temporary = QString::fromLatin1("");
    hideOrShow();
}

void QStatusBar::hideOrShow()
{
    update();
}

void QStatusBar::paintEvent( QPaintEvent * )
{
    QStatusBarPrivate::StatusBarPrivateItem * i;
    bool b = (d->temporary.length() == 0);
    QPainter p( this );
    d->items.first();
    while( (i=d->items.current()) != 0 ) {
	d->items.next();
    }
    if ( !b ) {
	p.setPen( colorGroup().text() );
	p.drawText( 6, 0, width() - 12, height(), AlignVCenter + SingleLine,
		    d->temporary );
    }
}

#endif



KviDockableWidget::KviDockableWidget(void * plugin_handle,KviStatusBar * parent)
: QWidget(parent)
{
	m_plugin_handle = plugin_handle;
}

KviDockableWidget::~KviDockableWidget()
{
	// debug("Dockable widget dying");
}


//============ KviStatusBar ============//

KviStatusBar::KviStatusBar(KviFrame *parent)
:QStatusBar(parent)
{
//	setFont(g_pOptions->m_fntApplication);
	// HORRIBLE HACK FOR THE QT STATUSBAR LAYOUT MANAGEMENT
//	KviTransparentWidget *tr = new KviTransparentWidget(this);
//	QWidget *qw = new QWidget(this);
//	addWidget(tr,1,false);
//	qw->hide();
//	m_pDockedWidgetList = new QList<KviDockableWidget>;
//	m_pDockedWidgetList->setAutoDelete(false);
}


//============ ~KviStatusBar ============//

KviStatusBar::~KviStatusBar()
{
//	__debug("~KviStatusBar");
//	delete m_pDockedWidgetList;
}

void KviStatusBar::tempText(const char *msg,int msecs)
{
	if(msecs > 0) message(QString(msg),msecs);
	else message(QString(msg));
}
/*
void KviStatusBar::dockWidget(KviDockableWidget *w)
{
	m_pDockedWidgetList->append(w);
	recalcHeight();
	layoutDockedWidgets();
}


void KviStatusBar::undockWidget(KviDockableWidget *w)
{
	m_pDockedWidgetList->removeRef(w);
	recalcHeight();
	layoutDockedWidgets();
}

void KviStatusBar::recalcHeight()
{
	int min = 16;
	int x;
	for(KviDockableWidget *w=m_pDockedWidgetList->first();w;w=m_pDockedWidgetList->next()){
		x = w->minimumHeight() + 2;
		if(min < x)min = x;
	}
	// trick
	setMinimumHeight(min);
	setMaximumHeight(min);
	((QWidget *)parent())->resize(((QWidget *)parent())->width(),((QWidget *)parent())->height());
}

void KviStatusBar::layoutDockedWidgets()
{
	int x = width() - 13; //13 is QSizeGrip::sizeHint()
	for(KviDockableWidget *w=m_pDockedWidgetList->first();w;w=m_pDockedWidgetList->next()){
		x -= (w->minimumWidth() + 3);
		if(x < 3)x = 3;
		w->setGeometry(x,2,w->minimumWidth(),height() - 4);
	}
}

void KviStatusBar::killDockedWidgetsByPluginHandle(void * plugin_handle)
{
	QList<KviDockableWidget> *l = new QList<KviDockableWidget>;
	l->setAutoDelete(true);
	for(KviDockableWidget *w=m_pDockedWidgetList->first();w;w=m_pDockedWidgetList->next()){
		if(w->m_plugin_handle == plugin_handle)l->append(w);
	}
	for(KviDockableWidget *wdg=l->first();wdg;wdg=l->next()){
		m_pDockedWidgetList->removeRef(wdg);
	}
	delete l;
	recalcHeight();
	layoutDockedWidgets();
}

void KviStatusBar::resizeEvent(QResizeEvent *e)
{
	QStatusBar::resizeEvent(e);
	layoutDockedWidgets();
}
*/
#include "m_kvi_statusbar.moc"
