//
//   This file is part of the KVirc irc client distribution
//   Copyright (C) 1999-2000 Szymon Stefanek (stefanek@tin.it)
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc. ,59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

#include "kvi_wmdock.h"
#include "kvi_app.h"

#include "kvi_locale.h"

#include "kvi_imagelib.h"

#include <qcursor.h>
#include <qpainter.h>

#include <X11/Xlib.h>

extern Display * g_display;

KviWMDockWidget::KviWMDockWidget()
: QWidget(0,"kvirc_wm_dock_widget")
{
	m_bFramesVisible = true;
	m_bHighlight = false;
	m_bAltColor = false;
	m_pContextPopup = new QPopupMenu(this);
	m_pContextPopup->insertItem(__tr("Hide this widget"),g_pApp,SLOT(slot_destroyWMDockWidget()));
	m_pContextPopup->insertSeparator();
	m_pContextPopup->insertItem(__tr("Quit KVIrc"),g_pApp,SLOT(slot_safeQuit()));

	KviStr szP1;
	KviStr szP2;
	g_pApp->getLocalKVircDirectory(szP1,KviApp::Pics,"kvi_dock.png");
	g_pApp->getGlobalKVircDirectory(szP2,KviApp::Pics,"kvi_dock.png");
	KviImageLibrary lib5(szP1.ptr(),szP2.ptr(),16,16);
	m_pNormalIcon=new QPixmap(lib5.getImage(0));
	m_pHighlightIcon=new QPixmap(lib5.getImage(1));
	m_pAltColorIcon=new QPixmap(lib5.getImage(2));

	setCaption("kvirc_wm_dock_widget");

	Atom a = XInternAtom(g_display,"KWM_DOCKWINDOW",false);
	long data = 1;
	XChangeProperty(g_display,winId(),a,a,32,PropModeReplace,(unsigned char *)&data,1);
	show();
}

KviWMDockWidget::~KviWMDockWidget()
{
	delete m_pNormalIcon;
	delete m_pHighlightIcon;
	delete m_pAltColorIcon;
	if(g_pApp->m_pWMDockWidget != 0)g_pApp->wmDockWidgetDestroyed();
}

void KviWMDockWidget::setFramesVisible(bool bVisible)
{
	m_bFramesVisible = bVisible;
	if(bVisible){
		m_bHighlight = false;
		m_bAltColor = false;
		update();
	}
}

void KviWMDockWidget::mousePressEvent(QMouseEvent *e)
{
	if(e->button() & RightButton){
		m_pContextPopup->popup(QCursor::pos());
	} else {
		if(m_bFramesVisible)g_pApp->hideAllFrames();
		else g_pApp->showAllFrames();
		
	}
}

void KviWMDockWidget::highlight(bool bAltColor)
{
	if((m_bAltColor != bAltColor) || (!m_bHighlight)){
		m_bHighlight = true;
		m_bAltColor = bAltColor;
		update();
	}
}

void KviWMDockWidget::paintEvent(QPaintEvent *)
{
	QPainter p(this);
	int x = (width() - 16) / 2;
	int y = (height() - 16) / 2;
	if(m_bHighlight){
		if(m_bAltColor)p.drawPixmap(x,y,*m_pAltColorIcon,0,0,16,16);
		else p.drawPixmap(x,y,*m_pHighlightIcon,0,0,16,16);
	} else p.drawPixmap(x,y,*m_pNormalIcon,0,0,16,16);
}

#include "m_kvi_wmdock.moc"
